;
define("lib/page_publish", function(){});

/**
 * @license RequireJS text 2.0.7 Copyright (c) 2010-2012, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/requirejs/text for details
 */
/*jslint regexp: true */
/*global require, XMLHttpRequest, ActiveXObject,
  define, window, process, Packages,
  java, location, Components, FileUtils */

define('text',['module'], function (module) {
    'use strict';

    var text, fs, Cc, Ci,
        progIds = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'],
        xmlRegExp = /^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im,
        bodyRegExp = /<body[^>]*>\s*([\s\S]+)\s*<\/body>/im,
        hasLocation = typeof location !== 'undefined' && location.href,
        defaultProtocol = hasLocation && location.protocol && location.protocol.replace(/\:/, ''),
        defaultHostName = hasLocation && location.hostname,
        defaultPort = hasLocation && (location.port || undefined),
        buildMap = {},
        masterConfig = (module.config && module.config()) || {};

    text = {
        version: '2.0.7',

        strip: function (content) {
            //Strips <?xml ...?> declarations so that external SVG and XML
            //documents can be added to a document without worry. Also, if the string
            //is an HTML document, only the part inside the body tag is returned.
            if (content) {
                content = content.replace(xmlRegExp, "");
                var matches = content.match(bodyRegExp);
                if (matches) {
                    content = matches[1];
                }
            } else {
                content = "";
            }
            return content;
        },

        jsEscape: function (content) {
            return content.replace(/(['\\])/g, '\\$1')
                .replace(/[\f]/g, "\\f")
                .replace(/[\b]/g, "\\b")
                .replace(/[\n]/g, "\\n")
                .replace(/[\t]/g, "\\t")
                .replace(/[\r]/g, "\\r")
                .replace(/[\u2028]/g, "\\u2028")
                .replace(/[\u2029]/g, "\\u2029");
        },

        createXhr: masterConfig.createXhr || function () {
            //Would love to dump the ActiveX crap in here. Need IE 6 to die first.
            var xhr, i, progId;
            if (typeof XMLHttpRequest !== "undefined") {
                return new XMLHttpRequest();
            } else if (typeof ActiveXObject !== "undefined") {
                for (i = 0; i < 3; i += 1) {
                    progId = progIds[i];
                    try {
                        xhr = new ActiveXObject(progId);
                    } catch (e) {}

                    if (xhr) {
                        progIds = [progId];  // so faster next time
                        break;
                    }
                }
            }

            return xhr;
        },

        /**
         * Parses a resource name into its component parts. Resource names
         * look like: module/name.ext!strip, where the !strip part is
         * optional.
         * @param {String} name the resource name
         * @returns {Object} with properties "moduleName", "ext" and "strip"
         * where strip is a boolean.
         */
        parseName: function (name) {
            var modName, ext, temp,
                strip = false,
                index = name.indexOf("."),
                isRelative = name.indexOf('./') === 0 ||
                             name.indexOf('../') === 0;

            if (index !== -1 && (!isRelative || index > 1)) {
                modName = name.substring(0, index);
                ext = name.substring(index + 1, name.length);
            } else {
                modName = name;
            }

            temp = ext || modName;
            index = temp.indexOf("!");
            if (index !== -1) {
                //Pull off the strip arg.
                strip = temp.substring(index + 1) === "strip";
                temp = temp.substring(0, index);
                if (ext) {
                    ext = temp;
                } else {
                    modName = temp;
                }
            }

            return {
                moduleName: modName,
                ext: ext,
                strip: strip
            };
        },

        xdRegExp: /^((\w+)\:)?\/\/([^\/\\]+)/,

        /**
         * Is an URL on another domain. Only works for browser use, returns
         * false in non-browser environments. Only used to know if an
         * optimized .js version of a text resource should be loaded
         * instead.
         * @param {String} url
         * @returns Boolean
         */
        useXhr: function (url, protocol, hostname, port) {
            var uProtocol, uHostName, uPort,
                match = text.xdRegExp.exec(url);
            if (!match) {
                return true;
            }
            uProtocol = match[2];
            uHostName = match[3];

            uHostName = uHostName.split(':');
            uPort = uHostName[1];
            uHostName = uHostName[0];

            return (!uProtocol || uProtocol === protocol) &&
                   (!uHostName || uHostName.toLowerCase() === hostname.toLowerCase()) &&
                   ((!uPort && !uHostName) || uPort === port);
        },

        finishLoad: function (name, strip, content, onLoad) {
            content = strip ? text.strip(content) : content;
            if (masterConfig.isBuild) {
                buildMap[name] = content;
            }
            onLoad(content);
        },

        load: function (name, req, onLoad, config) {
            //Name has format: some.module.filext!strip
            //The strip part is optional.
            //if strip is present, then that means only get the string contents
            //inside a body tag in an HTML string. For XML/SVG content it means
            //removing the <?xml ...?> declarations so the content can be inserted
            //into the current doc without problems.

            // Do not bother with the work if a build and text will
            // not be inlined.
            if (config.isBuild && !config.inlineText) {
                onLoad();
                return;
            }

            masterConfig.isBuild = config.isBuild;

            var parsed = text.parseName(name),
                nonStripName = parsed.moduleName +
                    (parsed.ext ? '.' + parsed.ext : ''),
                url = req.toUrl(nonStripName),
                useXhr = (masterConfig.useXhr) ||
                         text.useXhr;

            //Load the text. Use XHR if possible and in a browser.
            if (!hasLocation || useXhr(url, defaultProtocol, defaultHostName, defaultPort)) {
                text.get(url, function (content) {
                    text.finishLoad(name, parsed.strip, content, onLoad);
                }, function (err) {
                    if (onLoad.error) {
                        onLoad.error(err);
                    }
                });
            } else {
                //Need to fetch the resource across domains. Assume
                //the resource has been optimized into a JS module. Fetch
                //by the module name + extension, but do not include the
                //!strip part to avoid file system issues.
                req([nonStripName], function (content) {
                    text.finishLoad(parsed.moduleName + '.' + parsed.ext,
                                    parsed.strip, content, onLoad);
                });
            }
        },

        write: function (pluginName, moduleName, write, config) {
            if (buildMap.hasOwnProperty(moduleName)) {
                var content = text.jsEscape(buildMap[moduleName]);
                write.asModule(pluginName + "!" + moduleName,
                               "define(function () { return '" +
                                   content +
                               "';});\n");
            }
        },

        writeFile: function (pluginName, moduleName, req, write, config) {
            var parsed = text.parseName(moduleName),
                extPart = parsed.ext ? '.' + parsed.ext : '',
                nonStripName = parsed.moduleName + extPart,
                //Use a '.js' file name so that it indicates it is a
                //script that can be loaded across domains.
                fileName = req.toUrl(parsed.moduleName + extPart) + '.js';

            //Leverage own load() method to load plugin value, but only
            //write out values that do not have the strip argument,
            //to avoid any potential issues with ! in file names.
            text.load(nonStripName, req, function (value) {
                //Use own write() method to construct full module value.
                //But need to create shell that translates writeFile's
                //write() to the right interface.
                var textWrite = function (contents) {
                    return write(fileName, contents);
                };
                textWrite.asModule = function (moduleName, contents) {
                    return write.asModule(moduleName, fileName, contents);
                };

                text.write(pluginName, nonStripName, textWrite, config);
            }, config);
        }
    };

    if (masterConfig.env === 'node' || (!masterConfig.env &&
            typeof process !== "undefined" &&
            process.versions &&
            !!process.versions.node)) {
        //Using special require.nodeRequire, something added by r.js.
        fs = require.nodeRequire('fs');

        text.get = function (url, callback, errback) {
            try {
                var file = fs.readFileSync(url, 'utf8');
                //Remove BOM (Byte Mark Order) from utf8 files if it is there.
                if (file.indexOf('\uFEFF') === 0) {
                    file = file.substring(1);
                }
                callback(file);
            } catch (e) {
                errback(e);
            }
        };
    } else if (masterConfig.env === 'xhr' || (!masterConfig.env &&
            text.createXhr())) {
        text.get = function (url, callback, errback, headers) {
            var xhr = text.createXhr(), header;
            xhr.open('GET', url, true);

            //Allow plugins direct access to xhr headers
            if (headers) {
                for (header in headers) {
                    if (headers.hasOwnProperty(header)) {
                        xhr.setRequestHeader(header.toLowerCase(), headers[header]);
                    }
                }
            }

            //Allow overrides specified in config
            if (masterConfig.onXhr) {
                masterConfig.onXhr(xhr, url);
            }

            xhr.onreadystatechange = function (evt) {
                var status, err;
                //Do not explicitly handle errors, those should be
                //visible via console output in the browser.
                if (xhr.readyState === 4) {
                    status = xhr.status;
                    if (status > 399 && status < 600) {
                        //An http 4xx or 5xx error. Signal an error.
                        err = new Error(url + ' HTTP status: ' + status);
                        err.xhr = xhr;
                        errback(err);
                    } else {
                        callback(xhr.responseText);
                    }

                    if (masterConfig.onXhrComplete) {
                        masterConfig.onXhrComplete(xhr, url);
                    }
                }
            };
            xhr.send(null);
        };
    } else if (masterConfig.env === 'rhino' || (!masterConfig.env &&
            typeof Packages !== 'undefined' && typeof java !== 'undefined')) {
        //Why Java, why is this so awkward?
        text.get = function (url, callback) {
            var stringBuffer, line,
                encoding = "utf-8",
                file = new java.io.File(url),
                lineSeparator = java.lang.System.getProperty("line.separator"),
                input = new java.io.BufferedReader(new java.io.InputStreamReader(new java.io.FileInputStream(file), encoding)),
                content = '';
            try {
                stringBuffer = new java.lang.StringBuffer();
                line = input.readLine();

                // Byte Order Mark (BOM) - The Unicode Standard, version 3.0, page 324
                // http://www.unicode.org/faq/utf_bom.html

                // Note that when we use utf-8, the BOM should appear as "EF BB BF", but it doesn't due to this bug in the JDK:
                // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4508058
                if (line && line.length() && line.charAt(0) === 0xfeff) {
                    // Eat the BOM, since we've already found the encoding on this file,
                    // and we plan to concatenating this buffer with others; the BOM should
                    // only appear at the top of a file.
                    line = line.substring(1);
                }

                if (line !== null) {
                    stringBuffer.append(line);
                }

                while ((line = input.readLine()) !== null) {
                    stringBuffer.append(lineSeparator);
                    stringBuffer.append(line);
                }
                //Make sure we return a JavaScript string and not a Java string.
                content = String(stringBuffer.toString()); //String
            } finally {
                input.close();
            }
            callback(content);
        };
    } else if (masterConfig.env === 'xpconnect' || (!masterConfig.env &&
            typeof Components !== 'undefined' && Components.classes &&
            Components.interfaces)) {
        //Avert your gaze!
        Cc = Components.classes,
        Ci = Components.interfaces;
        Components.utils['import']('resource://gre/modules/FileUtils.jsm');

        text.get = function (url, callback) {
            var inStream, convertStream,
                readData = {},
                fileObj = new FileUtils.File(url);

            //XPCOM, you so crazy
            try {
                inStream = Cc['@mozilla.org/network/file-input-stream;1']
                           .createInstance(Ci.nsIFileInputStream);
                inStream.init(fileObj, 1, 0, false);

                convertStream = Cc['@mozilla.org/intl/converter-input-stream;1']
                                .createInstance(Ci.nsIConverterInputStream);
                convertStream.init(inStream, "utf-8", inStream.available(),
                Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);

                convertStream.readString(inStream.available(), readData);
                convertStream.close();
                inStream.close();
                callback(readData.value);
            } catch (e) {
                throw new Error((fileObj && fileObj.path || '') + ': ' + e);
            }
        };
    }
    return text;
});

define('text!component/noItemIcon/index.html',[],function () { return '<section>\n    <!-- 团队 -->\n    <!-- ko if: params().itemType == \'noTeamList\'  -->\n        <div class="noItemIconTeam">\n            <img src="Content/images/noDataIcons/default-team.svg" alt="">\n            <div class="noItemIconTeamTitle" data-bind="text:lang.noItemIconTeamTitle"></div>\n            <div class="noItemIconTeamText" data-bind="text:lang.noItemIconTeamText1"></div>\n            <div class="noItemIconTeamText" data-bind="text:lang.noItemIconTeamText2"></div>\n            <div class="noItemIconCreateTeam" data-bind="visible:constInfo.ShowCreateTeam(),text:lang.noItemIconCreateTeam,event:{click:handleClickCreateTeam}"></div>\n        </div>\n        <!-- <div class="noItemIcon" >\n            将 文件/文件夹 拽到此处可以上传 <p>或</p>\n            <ul class="noItemList">\n                <li data-bind="click: isNewAddOperation.bind($root)">\n                    <p>新建文件夹</p>\n                    <img src="Content/images/noDataIcons/newFolder.svg" alt="">\n                </li>\n                <li data-bind="click:$root.doCommand.bind($root)">\n                    <p>从模板新建</p>\n                    <img src="Content/images/noDataIcons/templateIcon.svg" alt="">\n                </li>\n                <li>\n                    <p>上传文件</p>\n                    <img src="Content/images/noDataIcons/uploadFile.svg" alt="">\n                </li>\n                <li>\n                    <p>上传文件夹</p>\n                    <img src="Content/images/noDataIcons/uploadFolder.svg" alt="">\n                </li>\n            </ul>\n        </div> -->\n    <!--/ko-->\n    <!-- ko ifnot: params().itemType == \'noTeamList\'   -->\n        <div class="noItemIcon"><img data-bind="attr:{src:icon}" /> <br /><span data-bind="text:msg" ></span></div>\n    <!--/ko-->\n    \n</section>\n\n\n';});

define('component/noItemIcon/index',[lang(), "knockout","durandal/app",'text!./index.html'], function (lang, ko,app, noItemIconView) {
    ko.components.register('noItemIcon', {
        viewModel: function (params) {
            this.isDefault = lang[params.itemType] ? false : true;
            // this.icon = ko.observable("Content/images/noDataIcons/" +(this.isDefault?"noData":params.itemType) + ".png");
            this.icon = ko.observable("Content/images/noDataIcons/" +(this.isDefault?"noData":params.itemType) + ".svg");
            this.msg = ko.observable(lang[params.itemType] ? lang[params.itemType] : lang['noData']);
            this.params = ko.observable(params);
            this.lang = lang;
            this.handleClickCreateTeam = function(){
                app.trigger("component:show", {
                    moduleId: "doc/team/create/index",
                    moduleName: "team",
                    datas: '',
                    message: ""
                });
            }
        },
        template: noItemIconView
        // template: '<div class="noItemIcon"><img data-bind="attr:{src:icon}" /> <br /><span data-bind="text:msg" ></span></div> '
    });

});
//从message模块中抽离出loading部分
define('component/loading/index',['require','exports','module','knockout'],function (require, exports, module) {
    var ko = require('knockout');
    //延迟id
    var delayId = 0;
    module.exports = function () {
        this.isShowLoading = ko.observable(false);
        this.counter+=1;
    }
    var fn = module.exports.prototype;
    fn.getView = function () {
        var view = '<div class="container-loading" data-unique-id="'+this.counter+'">' +
            '<span data-bind="visible:isShowLoading" class="icon-loading">' +
            '<em></em> ' +
            '</span>' +
            '</div>';
        return $(view).get(0);
    }
    fn.counter=0;
    fn.activate=function() {};
    fn.show = function (isdelay) {
        //show新的loading时先去除以前的loading;   liuning.yao--->bug:5076;
        clearTimeout(delayId);
        if (isdelay) {
            this.isShowLoading(true);
        }
        else {
            var self = this;
            //loading延迟出现(如果两秒钟还未加成完成就show出loading)
            delayId = setTimeout(function () {
                self.isShowLoading(true);
            }, 2000)
        }
        
    }
    fn.hide = function () {
        //取消即将出现的loading;
        clearTimeout(delayId);
        //取消loading;
        this.isShowLoading(false);
    }
});
//优化 by huang.hao@macrowing.com
//2018年1月18日
define('component/message/index',["durandal/app", "knockout", "jquery", "./../loading/index"], function (app, ko, $, Loading) {
    window.durandalApp = app;
    var constinfo = require("header/userinfo/constinfo");
    var messageObj = null;
    var timer = null;
    var content = ko.observable();
    var isShowMessage = ko.observable(false);
    var showClose = ko.observable(false);
    var msgCss = {
        success: ko.observable(false),
        error: ko.observable(false),
        warning: ko.observable(false)
    };
    var loading = new Loading();
    //消息类型数据
    var msgTypeData = {
        error: 'error',
        success: 'success',
        warn: 'warn'
    }
    //加载状态数据
    var loadingTypeData = {
        loading: 'loading',
        loaded: 'loaded'
    }

    function resetMsgCss() {
        msgCss.success(false);
        msgCss.error(false);
        msgCss.warning(false);
    }
    //usage: app.trigger("message:show", {type:'error',message:folderInfo.name});
    function init() {
        app.on('message:show').then(function (message) {
            var defaultMsg = {
                message: '',
                type: msgTypeData.success,
                delay: 1500,
                handler:null,
                showClose:true,
                modal:true //显示蒙板层（不让点击）
            }
            if (typeof message == "string") {
                message = {
                    message: message
                }
            }

            message = $.extend({}, defaultMsg, message);
            var m = message;
            if (loadingTypeData[m.type] !== undefined && loadingTypeData[m.type] !== null) {
                if(m.type === loadingTypeData.loading){
                    loading.show(m.delay == -1 ? true : false);
                }
                if(m.type === loadingTypeData.loaded){
                    loading.hide();
                }
            } else {
                messageObj=m;
                isShowMessage(true);
                // 系统管理页面提示不显示
                if (constInfo.userInfo.NavigationMenuNoShow) {
                    $(".message-box").css("margin-top","50px");
                }else if(window.location.hash.indexOf("#system")>-1){
                    $(".message-box").css("margin-top","50px");
                }
                if (!message.modal) {
                    $('.container-messager').css({
                        width:$(window).width(),
                        height:$(window).height()
                    });
                }
                showClose(m.showClose);
                content(m.message);
                resetMsgCss();
                //兼容已有代码
                if (m.type === msgTypeData.warn) {
                    msgCss[m.type + 'ing'](true);
                } else {
                    msgCss[m.type](true);
                }
                clearTimeout(timer);
                timer = setTimeout(function () {
                    $('.container-messager').css({
                        width:'auto',
                        height:'auto'
                    });
                    isShowMessage(false);
                }, m.delay);
            }
        });
        app.on('message:hide').then(function (message) {
            isShowMessage(false);
        });
    }


    return {
        loading:loading,
        content: content,
        isShowMessage: isShowMessage,
        showClose: showClose,
        msgCss: msgCss,
        compositionComplete: function (child, parent, settings) {
            init();
        },
        //提示关闭按钮
        onCloseClick: function () {
            $('.container-messager').css({
                width:'auto',
                height:'auto'
            });
            isShowMessage(false);
            clearTimeout(timer);
        },
        //单击弹出消息的事件响应
        onMessageContentClick: function (data, event) {
            if ($(event.target).hasClass('metadata-batchAdds')) {
                if (messageObj.handler) {
                    messageObj.handler(true);
                }
                event.stopPropagation();
            }
            if($(event.target).hasClass('security-batchAdds')) {
                if (messageObj.handler) {
                    messageObj.handler(false);
                }
                event.stopPropagation();
            }
        }
    }
});
//cookie操作,因appscan安全扫描固拆分于此处
define('header/userinfo/cookie',[],function () {
    return function (name, value, options) {
        if (typeof value != 'undefined') {
            options = options || {};
            if (value === null) {
                value = '';
                options = $.extend({},options);
                options.expires = -1;
            }
            var expires = '';
            if (options.expires && (typeof options.expires == 'number' || options.expires.toUTCString)) {
                var date;
                if (typeof options.expires == 'number') {
                    date = new Date();
                    date.setTime(date.getTime() + (options.expires * 24 * 60 * 60 * 1000));
                }
                else {
                    date = options.expires;
                }
                expires = '; expires=' + date.toUTCString();
            }
            var path = options.path ? '; path=' + (options.path) : '';
            var domain = options.domain ? '; domain=' + (options.domain) : '';
            var secure = options.secure ? '; secure' : '';
            window.document["cookie"] = [name, '=', encodeURIComponent(value), expires, path, domain, secure].join('');
        }
        else {
            var cookieValue = null;
            if (window.document["cookie"] && window.document["cookie"] != '') {
                var cookiesArray = window.document["cookie"].split(';');
                for (var i = 0; i < cookiesArray.length; i++) {
                    var cookie = $.trim(cookiesArray[i]);
                    if (cookie.substring(0, name.length + 1) == (name + '=')) {
                        cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                        break;
                    }
                }
            }
            return cookieValue;
        }
    };
});
define('doc/navigation/service',['durandal/app'], function (app) {

    /*
    * 服务类
    * 本为团队路径提供路径返加上一级的能力，但与navigation/index.js功能重复了，暂且去掉
    */
    function Service() {
        //路径变量
        this.paths;
        //当前所在位置
        this.current;
        this.pathObj;
    }
    var pt = Service.prototype;
    /*
    转到指定路径位置
    */
    pt.gotoPath = function (data) {
        this.current = data;
        app.trigger(this.pathObj.message, data);
        app.trigger('PreviewModelViewPanelShow:close');
    },
    //返回上一级
    pt.gotoParent = function () {
        var path = this.paths || [];
        if (path.length) {
            this.current = path.pop();
        }
        var hash=window.location.hash;
        // modify 2022-09-07 liuchunli #32515团队库搜索结果页面，点击返回按钮，会直接退出该团队库
        if(hash.indexOf('/search?searchkey') == -1){
            //modify 2022-09-13 liuchunli #37273企业/团队/个人库搜索结果页面，点击搜索标签后，再搜索结果页面选中文件点击返回按钮，应返回到搜索之前的层级目录
            if(hash.indexOf('fromInfobar')<0 && hash.indexOf('/search?searchType=3') == -1){//右侧面板搜索标签时,不返回上一级,而是返回搜索起始位置
                if (path.length) {
                    this.current = path.pop();
                }
            }
        }else{
            if (this.pathObj.message == 'team:tree'){
                this.pathObj.message = 'teamlist:load';
            }
        }
        app.trigger(this.pathObj.message, this.current,{});
        app.trigger('PreviewModelViewPanelShow:close');
    },
    //刷新
    pt.refresh = function () {
        var path = this.paths;
        this.current = this.paths[this.paths.length - 1];
        app.trigger(this.pathObj.message, this.current, true);
    }

    var service = new Service();

    app.on("navigation:init").then(function (messager) {
        //过滤掉非团队通知事件的消息 modify:2017-11-08 haibin.long
        if (messager.message.indexOf('team') < 0) return;
        service.pathObj = messager;
        service.paths = messager.paths;
        service.current = service.paths[service.paths.length - 1];
    });

    return {
        gotoPath: function (data) {
            service.gotoPath(data);
        },
        gotoParent: function () {
            service.gotoParent();
        },
        refresh: function () {
            service.refresh();
        }
    }
});
/*!
 * Cropper v0.9.2
 * https://github.com/fengyuanchen/cropper
 *
 * Copyright (c) 2014-2015 Fengyuan Chen and contributors
 * Released under the MIT license
 *
 * Date: 2015-04-18T04:35:01.500Z
 */

define('Cropper/cropper',['jquery'], function($) {
    var $window = $(window),
        $document = $(document),
        location = window.location,
  
        // Constants
        CROPPER_NAMESPACE = '.cropper',
        CROPPER_PREVIEW = 'preview' + CROPPER_NAMESPACE,
  
        // RegExps
        REGEXP_DRAG_TYPES = /^(e|n|w|s|ne|nw|sw|se|all|crop|move|zoom)$/,
  
        // Classes
        CLASS_MODAL = 'cropper-modal',
        CLASS_HIDE = 'cropper-hide',
        CLASS_HIDDEN = 'cropper-hidden',
        CLASS_INVISIBLE = 'cropper-invisible',
        CLASS_MOVE = 'cropper-move',
        CLASS_CROP = 'cropper-crop',
        CLASS_DISABLED = 'cropper-disabled',
        CLASS_BG = 'cropper-bg',
  
        // Events
        EVENT_MOUSE_DOWN = 'mousedown touchstart',
        EVENT_MOUSE_MOVE = 'mousemove touchmove',
        EVENT_MOUSE_UP = 'mouseup mouseleave touchend touchleave touchcancel',
        EVENT_WHEEL = 'wheel mousewheel DOMMouseScroll',
        EVENT_DBLCLICK = 'dblclick',
        EVENT_RESIZE = 'resize' + CROPPER_NAMESPACE, // Bind to window with namespace
        EVENT_BUILD = 'build' + CROPPER_NAMESPACE,
        EVENT_BUILT = 'built' + CROPPER_NAMESPACE,
        EVENT_DRAG_START = 'dragstart' + CROPPER_NAMESPACE,
        EVENT_DRAG_MOVE = 'dragmove' + CROPPER_NAMESPACE,
        EVENT_DRAG_END = 'dragend' + CROPPER_NAMESPACE,
        EVENT_ZOOM_IN = 'zoomin' + CROPPER_NAMESPACE,
        EVENT_ZOOM_OUT = 'zoomout' + CROPPER_NAMESPACE,
  
        // Supports
        SUPPORT_CANVAS = $.isFunction($('<canvas>')[0].getContext),
  
        // Others
        sqrt = Math.sqrt,
        min = Math.min,
        max = Math.max,
        abs = Math.abs,
        sin = Math.sin,
        cos = Math.cos,
        num = parseFloat,
  
        // Prototype
        prototype = {};
  
    function isNumber(n) {
      return typeof n === 'number';
    }
  
    function isUndefined(n) {
      return typeof n === 'undefined';
    }
  
    function toArray(obj, offset) {
      var args = [];
  
      if (isNumber(offset)) { // It's necessary for IE8
        args.push(offset);
      }
  
      return args.slice.apply(obj, args);
    }
  
    // Custom proxy to avoid jQuery's guid
    function proxy(fn, context) {
      var args = toArray(arguments, 2);
  
      return function () {
        return fn.apply(context, args.concat(toArray(arguments)));
      };
    }
  
    function isCrossOriginURL(url) {
      var parts = url.match(/^(https?:)\/\/([^\:\/\?#]+):?(\d*)/i);
  
      return parts && (parts[1] !== location.protocol || parts[2] !== location.hostname || parts[3] !== location.port);
    }
  
    function addTimestamp(url) {
      var timestamp = 'timestamp=' + (new Date()).getTime();
  
      return (url + (url.indexOf('?') === -1 ? '?' : '&') + timestamp);
    }
  
    function inRange(source, target) {
      return target.left < 0 && source.width < (target.left + target.width) && target.top < 0 && source.height < (target.top + target.height);
    }
  
    function getRotateValue(degree) {
      return degree ? 'rotate(' + degree + 'deg)' : 'none';
    }
  
    function getRotatedSizes(data, reverse) {
      var deg = abs(data.degree) % 180,
          arc = (deg > 90 ? (180 - deg) : deg) * Math.PI / 180,
          sinArc = sin(arc),
          cosArc = cos(arc),
          width = data.width,
          height = data.height,
          aspectRatio = data.aspectRatio,
          newWidth,
          newHeight;
  
      if (!reverse) {
        newWidth = width * cosArc + height * sinArc;
        newHeight = width * sinArc + height * cosArc;
      } else {
        newWidth = width / (cosArc + sinArc / aspectRatio);
        newHeight = newWidth / aspectRatio;
      }
  
      return {
        width: newWidth,
        height: newHeight
      };
    }
  
    function getSourceCanvas(image, data) {
      var canvas = $('<canvas>')[0],
          context = canvas.getContext('2d'),
          width = data.naturalWidth,
          height = data.naturalHeight,
          rotate = data.rotate,
          rotated = getRotatedSizes({
            width: width,
            height: height,
            degree: rotate
          });
  
      if (rotate) {
        canvas.width = rotated.width;
        canvas.height = rotated.height;
        context.save();
        context.translate(rotated.width / 2, rotated.height / 2);
        context.rotate(rotate * Math.PI / 180);
        context.drawImage(image, -width / 2, -height / 2, width, height);
        context.restore();
      } else {
        canvas.width = width;
        canvas.height = height;
        context.drawImage(image, 0, 0, width, height);
      }
  
      return canvas;
    }
  
    function Cropper(element, options) {
      this.$element = $(element);
      this.options = $.extend({}, Cropper.DEFAULTS, $.isPlainObject(options) && options);
  
      this.ready = false;
      this.built = false;
      this.rotated = false;
      this.cropped = false;
      this.disabled = false;
      this.canvas = null;
      this.cropBox = null;
  
      this.load();
    }
  
    prototype.load = function (url) {
      var options = this.options,
          $this = this.$element,
          crossOrigin,
          bustCacheUrl,
          buildEvent,
          $clone;
  
      if (!url) {
        if ($this.is('img')) {
          if (!$this.attr('src')) {
            return;
          }
  
          url = $this.prop('src');
        } else if ($this.is('canvas') && SUPPORT_CANVAS) {
          url = $this[0].toDataURL();
        }
      }
  
      if (!url) {
        return;
      }
  
      buildEvent = $.Event(EVENT_BUILD);
      $this.one(EVENT_BUILD, options.build).trigger(buildEvent); // Only trigger once
  
      if (buildEvent.isDefaultPrevented()) {
        return;
      }
  
      if (options.checkImageOrigin && isCrossOriginURL(url)) {
        crossOrigin = 'anonymous';
  
        if (!$this.prop('crossOrigin')) { // Only when there was not a "crossOrigin" property
          bustCacheUrl = addTimestamp(url); // Bust cache (#148)
        }
      }
  
      this.$clone = $clone = $('<img>');
  
      $clone.one('load', $.proxy(function () {
        var naturalWidth = $clone.prop('naturalWidth') || $clone.width(),
            naturalHeight = $clone.prop('naturalHeight') || $clone.height();
  
        this.image = {
          naturalWidth: naturalWidth,
          naturalHeight: naturalHeight,
          aspectRatio: naturalWidth / naturalHeight,
          rotate: 0
        };
  
        this.url = url;
        this.ready = true;
        this.build();
      }, this)).one('error', function () {
        $clone.remove();
      }).attr({
        /* #134: Seems like the order is important for Chrome. */
        crossOrigin: crossOrigin,
        src: bustCacheUrl || url
      });
  
      // Hide and insert into the document
      $clone.addClass(CLASS_HIDE).insertAfter($this);
    };
  
    prototype.build = function () {
      var $this = this.$element,
          $clone = this.$clone,
          options = this.options,
          $cropper,
          $cropBox;
  
      if (!this.ready) {
        return;
      }
  
      if (this.built) {
        this.unbuild();
      }
  
      // Create cropper elements
      this.$cropper = $cropper = $(Cropper.TEMPLATE);
  
      // Hide the original image
      $this.addClass(CLASS_HIDDEN);
  
      // Show the clone iamge
      $clone.removeClass(CLASS_HIDE);
  
      this.$container = $this.parent().append($cropper);
      this.$canvas = $cropper.find('.cropper-canvas').append($clone);
      this.$dragBox = $cropper.find('.cropper-drag-box');
      this.$cropBox = $cropBox = $cropper.find('.cropper-crop-box');
      this.$viewBox = $cropper.find('.cropper-view-box');
  
      this.addListeners();
      this.initPreview();
  
      // Format aspect ratio
      options.aspectRatio = num(options.aspectRatio) || NaN; // 0 -> NaN
  
      if (options.autoCrop) {
        this.cropped = true;
  
        if (options.modal) {
          this.$dragBox.addClass(CLASS_MODAL);
        }
      } else {
        $cropBox.addClass(CLASS_HIDDEN);
      }
  
      if (options.background) {
        $cropper.addClass(CLASS_BG);
      }
  
      if (!options.highlight) {
        $cropBox.find('.cropper-face').addClass(CLASS_INVISIBLE);
      }
  
      if (!options.guides) {
        $cropBox.find('.cropper-dashed').addClass(CLASS_HIDDEN);
      }
  
      if (!options.movable) {
        $cropBox.find('.cropper-face').data('drag', 'move');
      }
  
      if (!options.resizable) {
        $cropBox.find('.cropper-line, .cropper-point').addClass(CLASS_HIDDEN);
      }
  
      this.setDragMode(options.dragCrop ? 'crop' : 'move');
  
      this.built = true;
      this.render();
      $this.one(EVENT_BUILT, options.built).trigger(EVENT_BUILT); // Only trigger once
    };
  
    prototype.unbuild = function () {
      if (!this.built) {
        return;
      }
  
      this.built = false;
      this.container = null;
      this.canvas = null;
      this.cropBox = null; // This is necessary when replace
      this.removeListeners();
  
      this.resetPreview();
      this.$preview = null;
  
      this.$viewBox = null;
      this.$cropBox = null;
      this.$dragBox = null;
      this.$canvas = null;
      this.$container = null;
  
      this.$cropper.remove();
      this.$cropper = null;
    };
  
    $.extend(prototype, {
      render: function () {
        this.initContainer();
        this.initCanvas();
        this.initCropBox();
  
        this.renderCanvas();
  
        if (this.cropped) {
          this.renderCropBox();
        }
      },
  
      initContainer: function () {
        var $this = this.$element,
            $container = this.$container,
            $cropper = this.$cropper,
            options = this.options;
  
        $cropper.addClass(CLASS_HIDDEN);
        $this.removeClass(CLASS_HIDDEN);
  
        $cropper.css((this.container = {
          width: max($container.width(), num(options.minContainerWidth) || 200),
          height: max($container.height(), num(options.minContainerHeight) || 100)
        }));
  
        $this.addClass(CLASS_HIDDEN);
        $cropper.removeClass(CLASS_HIDDEN);
      },
  
      // image box (wrapper)
      initCanvas: function () {
        var container = this.container,
            containerWidth = container.width,
            containerHeight = container.height,
            image = this.image,
            aspectRatio = image.aspectRatio,
            canvas = {
              aspectRatio: aspectRatio,
              width: containerWidth,
              height: containerHeight
            };
  
        if (containerHeight * aspectRatio > containerWidth) {
          canvas.height = containerWidth / aspectRatio;
        } else {
          canvas.width = containerHeight * aspectRatio;
        }
  
        canvas.oldLeft = canvas.left = (containerWidth - canvas.width) / 2;
        canvas.oldTop = canvas.top = (containerHeight - canvas.height) / 2;
  
        this.canvas = canvas;
        this.limitCanvas(true, true);
        this.initialImage = $.extend({}, image);
        this.initialCanvas = $.extend({}, canvas);
      },
  
      limitCanvas: function (size, position) {
        var options = this.options,
            strict = options.strict,
            container = this.container,
            containerWidth = container.width,
            containerHeight = container.height,
            canvas = this.canvas,
            aspectRatio = canvas.aspectRatio,
            cropBox = this.cropBox,
            cropped = this.cropped && cropBox,
            minCanvasWidth,
            minCanvasHeight;
  
        if (size) {
          minCanvasWidth = num(options.minCanvasWidth) || 0;
          minCanvasHeight = num(options.minCanvasHeight) || 0;
  
          if (minCanvasWidth) {
            if (strict) {
              minCanvasWidth = max(cropped ? cropBox.width : containerWidth, minCanvasWidth);
            }
  
            minCanvasHeight = minCanvasWidth / aspectRatio;
          } else if (minCanvasHeight) {
  
            if (strict) {
              minCanvasHeight = max(cropped ? cropBox.height : containerHeight, minCanvasHeight);
            }
  
            minCanvasWidth = minCanvasHeight * aspectRatio;
          } else if (strict) {
            if (cropped) {
              minCanvasWidth = cropBox.width;
              minCanvasHeight = cropBox.height;
  
              if (minCanvasHeight * aspectRatio > minCanvasWidth) {
                minCanvasWidth = minCanvasHeight * aspectRatio;
              } else {
                minCanvasHeight = minCanvasWidth / aspectRatio;
              }
            } else {
              minCanvasWidth = containerWidth;
              minCanvasHeight = containerHeight;
  
              if (minCanvasHeight * aspectRatio > minCanvasWidth) {
                minCanvasHeight = minCanvasWidth / aspectRatio;
              } else {
                minCanvasWidth = minCanvasHeight * aspectRatio;
              }
            }
          }
  
          $.extend(canvas, {
            minWidth: minCanvasWidth,
            minHeight: minCanvasHeight,
            maxWidth: Infinity,
            maxHeight: Infinity
          });
        }
  
        if (position) {
          if (strict) {
            if (cropped) {
              canvas.minLeft = min(cropBox.left, (cropBox.left + cropBox.width) - canvas.width);
              canvas.minTop = min(cropBox.top, (cropBox.top + cropBox.height) - canvas.height);
              canvas.maxLeft = cropBox.left;
              canvas.maxTop = cropBox.top;
            } else {
              canvas.minLeft = min(0, containerWidth - canvas.width);
              canvas.minTop = min(0, containerHeight - canvas.height);
              canvas.maxLeft = max(0, containerWidth - canvas.width);
              canvas.maxTop = max(0, containerHeight - canvas.height);
            }
          } else {
            canvas.minLeft = -canvas.width;
            canvas.minTop = -canvas.height;
            canvas.maxLeft = containerWidth;
            canvas.maxTop = containerHeight;
          }
        }
      },
  
      renderCanvas: function (changed) {
        var options = this.options,
            canvas = this.canvas,
            image = this.image,
            aspectRatio,
            rotated;
  
        if (this.rotated) {
          this.rotated = false;
  
          // Computes rotatation sizes with image sizes
          rotated = getRotatedSizes({
            width: image.width,
            height: image.height,
            degree: image.rotate
          });
  
          aspectRatio = rotated.width / rotated.height;
  
          if (aspectRatio !== canvas.aspectRatio) {
            canvas.left -= (rotated.width - canvas.width) / 2;
            canvas.top -= (rotated.height - canvas.height) / 2;
            canvas.width = rotated.width;
            canvas.height = rotated.height;
            canvas.aspectRatio = aspectRatio;
            this.limitCanvas(true, false);
          }
        }
  
        if (canvas.width > canvas.maxWidth || canvas.width < canvas.minWidth) {
          canvas.left = canvas.oldLeft;
        }
  
        if (canvas.height > canvas.maxHeight || canvas.height < canvas.minHeight) {
          canvas.top = canvas.oldTop;
        }
  
        canvas.width = min(max(canvas.width, canvas.minWidth), canvas.maxWidth);
        canvas.height = min(max(canvas.height, canvas.minHeight), canvas.maxHeight);
  
        this.limitCanvas(false, true);
  
        canvas.oldLeft = canvas.left = min(max(canvas.left, canvas.minLeft), canvas.maxLeft);
        canvas.oldTop = canvas.top = min(max(canvas.top, canvas.minTop), canvas.maxTop);
  
        this.$canvas.css({
          width: canvas.width,
          height: canvas.height,
          left: canvas.left,
          top: canvas.top
        });
  
        this.renderImage();
  
        if (this.cropped && options.strict && !inRange(this.container, canvas)) {
          this.limitCropBox(true, true);
        }
  
        if (changed) {
          this.output();
        }
      },
  
      renderImage: function () {
        var canvas = this.canvas,
            image = this.image,
            reversed;
  
        if (image.rotate) {
          reversed = getRotatedSizes({
            width: canvas.width,
            height: canvas.height,
            degree: image.rotate,
            aspectRatio: image.aspectRatio
          }, true);
        }
  
        $.extend(image, reversed ? {
          width: reversed.width,
          height: reversed.height,
          left: (canvas.width - reversed.width) / 2,
          top: (canvas.height - reversed.height) / 2
        } : {
          width: canvas.width,
          height: canvas.height,
          left: 0,
          top: 0
        });
  
        this.$clone.css({
          width: image.width,
          height: image.height,
          marginLeft: image.left,
          marginTop: image.top,
          transform: getRotateValue(image.rotate)
        });
      },
  
      initCropBox: function () {
        var options = this.options,
            canvas = this.canvas,
            aspectRatio = options.aspectRatio,
            autoCropArea = num(options.autoCropArea) || 0.8,
            cropBox = {
              width: canvas.width,
              height: canvas.height
            };
  
        if (aspectRatio) {
          if (canvas.height * aspectRatio > canvas.width) {
            cropBox.height = cropBox.width / aspectRatio;
          } else {
            cropBox.width = cropBox.height * aspectRatio;
          }
        }
  
        this.cropBox = cropBox;
        this.limitCropBox(true, true);
  
        // Initialize auto crop area
        cropBox.width = min(max(cropBox.width, cropBox.minWidth), cropBox.maxWidth);
        cropBox.height = min(max(cropBox.height, cropBox.minHeight), cropBox.maxHeight);
  
        // The width of auto crop area must large than "minWidth", and the height too. (#164)
        cropBox.width = max(cropBox.minWidth, cropBox.width * autoCropArea);
        cropBox.height = max(cropBox.minHeight, cropBox.height * autoCropArea);
        cropBox.oldLeft = cropBox.left = canvas.left + (canvas.width - cropBox.width) / 2;
        cropBox.oldTop = cropBox.top = canvas.top + (canvas.height - cropBox.height) / 2;
  
        this.initialCropBox = $.extend({}, cropBox);
      },
  
      limitCropBox: function (size, position) {
        var options = this.options,
            strict = options.strict,
            container = this.container,
            containerWidth = container.width,
            containerHeight = container.height,
            canvas = this.canvas,
            cropBox = this.cropBox,
            aspectRatio = options.aspectRatio,
            minCropBoxWidth,
            minCropBoxHeight;
  
        if (size) {
          minCropBoxWidth = num(options.minCropBoxWidth) || 0;
          minCropBoxHeight = num(options.minCropBoxHeight) || 0;
  
          // min/maxCropBoxWidth/Height must less than conatiner width/height
          cropBox.minWidth = min(containerWidth, minCropBoxWidth);
          cropBox.minHeight = min(containerHeight, minCropBoxHeight);
          cropBox.maxWidth = min(containerWidth, strict ? canvas.width : containerWidth);
          cropBox.maxHeight = min(containerHeight, strict ? canvas.height : containerHeight);
  
          if (aspectRatio) {
            // compare crop box size with container first
            if (cropBox.maxHeight * aspectRatio > cropBox.maxWidth) {
              cropBox.minHeight = cropBox.minWidth / aspectRatio;
              cropBox.maxHeight = cropBox.maxWidth / aspectRatio;
            } else {
              cropBox.minWidth = cropBox.minHeight * aspectRatio;
              cropBox.maxWidth = cropBox.maxHeight * aspectRatio;
            }
          }
  
          // The "minWidth" must be less than "maxWidth", and the "minHeight" too.
          cropBox.minWidth = min(cropBox.maxWidth, cropBox.minWidth);
          cropBox.minHeight = min(cropBox.maxHeight, cropBox.minHeight);
        }
  
        if (position) {
          if (strict) {
            cropBox.minLeft = max(0, canvas.left);
            cropBox.minTop = max(0, canvas.top);
            cropBox.maxLeft = min(containerWidth, canvas.left + canvas.width) - cropBox.width;
            cropBox.maxTop = min(containerHeight, canvas.top + canvas.height) - cropBox.height;
          } else {
            cropBox.minLeft = 0;
            cropBox.minTop = 0;
            cropBox.maxLeft = containerWidth - cropBox.width;
            cropBox.maxTop = containerHeight - cropBox.height;
          }
        }
      },
  
      renderCropBox: function () {
        var options = this.options,
            container = this.container,
            containerWidth = container.width,
            containerHeight = container.height,
            $cropBox = this.$cropBox,
            cropBox = this.cropBox;
  
        if (cropBox.width > cropBox.maxWidth || cropBox.width < cropBox.minWidth) {
          cropBox.left = cropBox.oldLeft;
        }
  
        if (cropBox.height > cropBox.maxHeight || cropBox.height < cropBox.minHeight) {
          cropBox.top = cropBox.oldTop;
        }
  
        cropBox.width = min(max(cropBox.width, cropBox.minWidth), cropBox.maxWidth);
        cropBox.height = min(max(cropBox.height, cropBox.minHeight), cropBox.maxHeight);
  
        this.limitCropBox(false, true);
  
        cropBox.oldLeft = cropBox.left = min(max(cropBox.left, cropBox.minLeft), cropBox.maxLeft);
        cropBox.oldTop = cropBox.top = min(max(cropBox.top, cropBox.minTop), cropBox.maxTop);
  
        if (options.movable) {
          $cropBox.find('.cropper-face').data('drag', (cropBox.width === containerWidth && cropBox.height === containerHeight) ? 'move' : 'all');
        }
  
        $cropBox.css({
          width: cropBox.width,
          height: cropBox.height,
          left: cropBox.left,
          top: cropBox.top
        });
  
        if (this.cropped && options.strict && !inRange(container, this.canvas)) {
          this.limitCanvas(true, true);
        }
  
        if (!this.disabled) {
          this.output();
        }
      },
  
      output: function () {
        var options = this.options;
  
        this.preview();
  
        if (options.crop) {
          options.crop.call(this.$element, this.getData());
        }
      }
    });
  
    prototype.initPreview = function () {
      var url = this.url;
  
      this.$preview = $(this.options.preview);
      this.$viewBox.html('<img src="' + url + '">');
  
      // Override img element styles
      // Add `display:block` to avoid margin top issue (Occur only when margin-top <= -height)
      this.$preview.each(function () {
        var $this = $(this);
  
        $this.data(CROPPER_PREVIEW, {
          width: $this.width(),
          height: $this.height(),
          original: $this.html()
        }).html('<img src="' + url + '" style="display:block;width:100%;min-width:0!important;min-height:0!important;max-width:none!important;max-height:none!important;image-orientation: 0deg!important">');
      });
    };
  
    prototype.resetPreview = function () {
      this.$preview.each(function () {
        var $this = $(this);
  
        $this.html($this.data(CROPPER_PREVIEW).original).removeData(CROPPER_PREVIEW);
      });
    };
  
    prototype.preview = function () {
      var image = this.image,
          canvas = this.canvas,
          cropBox = this.cropBox,
          width = image.width,
          height = image.height,
          left = cropBox.left - canvas.left - image.left,
          top = cropBox.top - canvas.top - image.top,
          rotate = image.rotate;
  
      if (!this.cropped || this.disabled) {
        return;
      }
  
      this.$viewBox.find('img').css({
        width: width,
        height: height,
        marginLeft: -left,
        marginTop: -top,
        transform: getRotateValue(rotate)
      });
  
      this.$preview.each(function () {
        var $this = $(this),
            data = $this.data(CROPPER_PREVIEW),
            ratio = data.width / cropBox.width,
            newWidth = data.width,
            newHeight = cropBox.height * ratio;
  
        if (newHeight > data.height) {
          ratio = data.height / cropBox.height;
          newWidth = cropBox.width * ratio;
          newHeight = data.height;
        }
  
        $this.width(newWidth).height(newHeight).find('img').css({
          width: width * ratio,
          height: height * ratio,
          marginLeft: -left * ratio,
          marginTop: -top * ratio,
          transform: getRotateValue(rotate)
        });
      });
    };
  
    prototype.addListeners = function () {
      var options = this.options;
  
      this.$element.on(EVENT_DRAG_START, options.dragstart).on(EVENT_DRAG_MOVE, options.dragmove).on(EVENT_DRAG_END, options.dragend).on(EVENT_ZOOM_IN, options.zoomin).on(EVENT_ZOOM_OUT, options.zoomout);
      this.$cropper.on(EVENT_MOUSE_DOWN, $.proxy(this.dragstart, this)).on(EVENT_DBLCLICK, $.proxy(this.dblclick, this));
  
      if (options.zoomable && options.mouseWheelZoom) {
        this.$cropper.on(EVENT_WHEEL, $.proxy(this.wheel, this));
      }
  
      $document.on(EVENT_MOUSE_MOVE, (this._dragmove = proxy(this.dragmove, this))).on(EVENT_MOUSE_UP, (this._dragend = proxy(this.dragend, this)));
  
      if (options.responsive) {
        $window.on(EVENT_RESIZE, (this._resize = proxy(this.resize, this)));
      }
    };
  
    prototype.removeListeners = function () {
      var options = this.options;
  
      this.$element.off(EVENT_DRAG_START, options.dragstart).off(EVENT_DRAG_MOVE, options.dragmove).off(EVENT_DRAG_END, options.dragend).off(EVENT_ZOOM_IN, options.zoomin).off(EVENT_ZOOM_OUT, options.zoomout);
      this.$cropper.off(EVENT_MOUSE_DOWN, this.dragstart).off(EVENT_DBLCLICK, this.dblclick);
  
      if (options.zoomable && options.mouseWheelZoom) {
        this.$cropper.off(EVENT_WHEEL, this.wheel);
      }
  
      $document.off(EVENT_MOUSE_MOVE, this._dragmove).off(EVENT_MOUSE_UP, this._dragend);
  
      if (options.responsive) {
        $window.off(EVENT_RESIZE, this._resize);
      }
    };
  
    $.extend(prototype, {
      resize: function () {
        var $container = this.$container,
            container = this.container,
            canvasData,
            cropBoxData,
            ratio;
  
        if (this.disabled) {
          return;
        }
  
        ratio = $container.width() / container.width;
  
        if (ratio !== 1 || $container.height() !== container.height) {
          canvasData = this.getCanvasData();
          cropBoxData = this.getCropBoxData();
  
          this.render();
          this.setCanvasData($.each(canvasData, function (i, n) {
            canvasData[i] = n * ratio;
          }));
          this.setCropBoxData($.each(cropBoxData, function (i, n) {
            cropBoxData[i] = n * ratio;
          }));
        }
      },
  
      dblclick: function () {
        if (this.disabled) {
          return;
        }
  
        if (this.$dragBox.hasClass(CLASS_CROP)) {
          this.setDragMode('move');
        } else {
          this.setDragMode('crop');
        }
      },
  
      wheel: function (event) {
        var e = event.originalEvent,
            delta = 1;
  
        if (this.disabled) {
          return;
        }
  
        event.preventDefault();
  
        if (e.deltaY) {
          delta = e.deltaY > 0 ? 1 : -1;
        } else if (e.wheelDelta) {
          delta = -e.wheelDelta / 120;
        } else if (e.detail) {
          delta = e.detail > 0 ? 1 : -1;
        }
  
        this.zoom(-delta * 0.1);
      },
  
      dragstart: function (event) {
        var options = this.options,
            originalEvent = event.originalEvent,
            touches = originalEvent && originalEvent.touches,
            e = event,
            dragType,
            dragStartEvent,
            touchesLength;
  
        if (this.disabled) {
          return;
        }
  
        if (touches) {
          touchesLength = touches.length;
  
          if (touchesLength > 1) {
            if (options.zoomable && options.touchDragZoom && touchesLength === 2) {
              e = touches[1];
              this.startX2 = e.pageX;
              this.startY2 = e.pageY;
              dragType = 'zoom';
            } else {
              return;
            }
          }
  
          e = touches[0];
        }
  
        dragType = dragType || $(e.target).data('drag');
  
        if (REGEXP_DRAG_TYPES.test(dragType)) {
          event.preventDefault();
  
          dragStartEvent = $.Event(EVENT_DRAG_START, {
            originalEvent: originalEvent,
            dragType: dragType
          });
  
          this.$element.trigger(dragStartEvent);
  
          if (dragStartEvent.isDefaultPrevented()) {
            return;
          }
  
          this.dragType = dragType;
          this.cropping = false;
          this.startX = e.pageX;
          this.startY = e.pageY;
  
          if (dragType === 'crop') {
            this.cropping = true;
            this.$dragBox.addClass(CLASS_MODAL);
          }
        }
      },
  
      dragmove: function (event) {
        var options = this.options,
            originalEvent = event.originalEvent,
            touches = originalEvent && originalEvent.touches,
            e = event,
            dragType = this.dragType,
            dragMoveEvent,
            touchesLength;
  
        if (this.disabled) {
          return;
        }
  
        if (touches) {
          touchesLength = touches.length;
  
          if (touchesLength > 1) {
            if (options.zoomable && options.touchDragZoom && touchesLength === 2) {
              e = touches[1];
              this.endX2 = e.pageX;
              this.endY2 = e.pageY;
            } else {
              return;
            }
          }
  
          e = touches[0];
        }
  
        if (dragType) {
          event.preventDefault();
  
          dragMoveEvent = $.Event(EVENT_DRAG_MOVE, {
            originalEvent: originalEvent,
            dragType: dragType
          });
  
          this.$element.trigger(dragMoveEvent);
  
          if (dragMoveEvent.isDefaultPrevented()) {
            return;
          }
  
          this.endX = e.pageX;
          this.endY = e.pageY;
  
          this.change();
        }
      },
  
      dragend: function (event) {
        var dragType = this.dragType,
            dragEndEvent;
  
        if (this.disabled) {
          return;
        }
  
        if (dragType) {
          event.preventDefault();
  
          dragEndEvent = $.Event(EVENT_DRAG_END, {
            originalEvent: event.originalEvent,
            dragType: dragType
          });
  
          this.$element.trigger(dragEndEvent);
  
          if (dragEndEvent.isDefaultPrevented()) {
            return;
          }
  
          if (this.cropping) {
            this.cropping = false;
            this.$dragBox.toggleClass(CLASS_MODAL, this.cropped && this.options.modal);
          }
  
          this.dragType = '';
        }
      }
    });
  
    $.extend(prototype, {
      reset: function () {
        if (!this.built || this.disabled) {
          return;
        }
  
        this.image = $.extend({}, this.initialImage);
        this.canvas = $.extend({}, this.initialCanvas);
        this.renderCanvas();
  
        if (this.cropped) {
          this.cropBox = $.extend({}, this.initialCropBox);
          this.renderCropBox();
        }
      },
  
      clear: function () {
        if (!this.cropped || this.disabled) {
          return;
        }
  
        $.extend(this.cropBox, {
          left: 0,
          top: 0,
          width: 0,
          height: 0
        });
  
        this.cropped = false;
        this.renderCropBox();
  
        this.limitCanvas();
        this.renderCanvas(); // Render canvas after render crop box
  
        this.$dragBox.removeClass(CLASS_MODAL);
        this.$cropBox.addClass(CLASS_HIDDEN);
      },
  
      destroy: function () {
        var $this = this.$element;
  
        if (this.ready) {
          this.unbuild();
          $this.removeClass(CLASS_HIDDEN);
        } else {
          this.$clone.off('load').remove();
        }
  
        $this.removeData('cropper');
      },
  
      replace: function (url) {
        if (!this.disabled && url) {
          this.load(url);
        }
      },
  
      enable: function () {
        if (this.built) {
          this.disabled = false;
          this.$cropper.removeClass(CLASS_DISABLED);
        }
      },
  
      disable: function () {
        if (this.built) {
          this.disabled = true;
          this.$cropper.addClass(CLASS_DISABLED);
        }
      },
  
      move: function (offsetX, offsetY) {
        var canvas = this.canvas;
  
        if (this.built && !this.disabled && isNumber(offsetX) && isNumber(offsetY)) {
          canvas.left += offsetX;
          canvas.top += offsetY;
          this.renderCanvas(true);
        }
      },
  
      zoom: function (delta) {
        var canvas = this.canvas,
            zoomEvent,
            width,
            height;
  
        delta = num(delta);
  
        if (delta && this.built && !this.disabled && this.options.zoomable) {
          zoomEvent = delta > 0 ? $.Event(EVENT_ZOOM_IN) : $.Event(EVENT_ZOOM_OUT);
          this.$element.trigger(zoomEvent);
  
          if (zoomEvent.isDefaultPrevented()) {
            return;
          }
  
          delta = delta <= -1 ? 1 / (1 - delta) : delta <= 1 ? (1 + delta) : delta;
          width = canvas.width * delta;
          height = canvas.height * delta;
          canvas.left -= (width - canvas.width) / 2;
          canvas.top -= (height - canvas.height) / 2;
          canvas.width = width;
          canvas.height = height;
          this.renderCanvas(true);
          this.setDragMode('move');
        }
      },
  
      rotate: function (degree) {
        var image = this.image;
  
        degree = num(degree);
  
        if (degree && this.built && !this.disabled && this.options.rotatable) {
          image.rotate = (image.rotate + degree) % 360;
          this.rotated = true;
          this.renderCanvas(true);
        }
      },
  
      getData: function () {
        var cropBox = this.cropBox,
            canvas = this.canvas,
            image = this.image,
            ratio,
            data;
  
        if (this.built && this.cropped) {
          data = {
            x: cropBox.left - canvas.left,
            y: cropBox.top - canvas.top,
            width: cropBox.width,
            height: cropBox.height
          };
  
          ratio = image.width / image.naturalWidth;
  
          $.each(data, function (i, n) {
            n = n / ratio;
            data[i] = n;
          });
  
        } else {
          data = {
            x: 0,
            y: 0,
            width: 0,
            height: 0
          };
        }
  
        data.rotate = image.rotate;
  
        return data;
      },
  
      getContainerData: function () {
        return this.built ? this.container : {};
      },
  
      getImageData: function () {
        return this.ready ? this.image : {};
      },
  
      getCanvasData: function () {
        var canvas = this.canvas,
            data;
  
        if (this.built) {
          data = {
            left: canvas.left,
            top: canvas.top,
            width: canvas.width,
            height: canvas.height
          };
        }
  
        return data || {};
      },
  
      setCanvasData: function (data) {
        var canvas = this.canvas,
            aspectRatio = canvas.aspectRatio;
  
        if (this.built && !this.disabled && $.isPlainObject(data)) {
          if (isNumber(data.left)) {
            canvas.left = data.left;
          }
  
          if (isNumber(data.top)) {
            canvas.top = data.top;
          }
  
          if (isNumber(data.width)) {
            canvas.width = data.width;
            canvas.height = data.width / aspectRatio;
          } else if (isNumber(data.height)) {
            canvas.height = data.height;
            canvas.width = data.height * aspectRatio;
          }
  
          this.renderCanvas(true);
        }
      },
  
      getCropBoxData: function () {
        var cropBox = this.cropBox,
            data;
  
        if (this.built && this.cropped) {
          data = {
            left: cropBox.left,
            top: cropBox.top,
            width: cropBox.width,
            height: cropBox.height
          };
        }
  
        return data || {};
      },
  
      setCropBoxData: function (data) {
        var cropBox = this.cropBox,
            aspectRatio = this.options.aspectRatio;
  
        if (this.built && this.cropped && !this.disabled && $.isPlainObject(data)) {
  
          if (isNumber(data.left)) {
            cropBox.left = data.left;
          }
  
          if (isNumber(data.top)) {
            cropBox.top = data.top;
          }
  
          if (aspectRatio) {
            if (isNumber(data.width)) {
              cropBox.width = data.width;
              cropBox.height = cropBox.width / aspectRatio;
            } else if (isNumber(data.height)) {
              cropBox.height = data.height;
              cropBox.width = cropBox.height * aspectRatio;
            }
          } else {
            if (isNumber(data.width)) {
              cropBox.width = data.width;
            }
  
            if (isNumber(data.height)) {
              cropBox.height = data.height;
            }
          }
  
          this.renderCropBox();
        }
      },
  
      getCroppedCanvas: function (options) {
        var originalWidth,
            originalHeight,
            canvasWidth,
            canvasHeight,
            scaledWidth,
            scaledHeight,
            scaledRatio,
            aspectRatio,
            canvas,
            context,
            data;
  
        if (!this.built || !this.cropped || !SUPPORT_CANVAS) {
          return;
        }
  
        if (!$.isPlainObject(options)) {
          options = {};
        }
  
        data = this.getData();
        originalWidth = data.width;
        originalHeight = data.height;
        aspectRatio = originalWidth / originalHeight;
  
        if ($.isPlainObject(options)) {
          scaledWidth = options.width;
          scaledHeight = options.height;
  
          if (scaledWidth) {
            scaledHeight = scaledWidth / aspectRatio;
            scaledRatio = scaledWidth / originalWidth;
          } else if (scaledHeight) {
            scaledWidth = scaledHeight * aspectRatio;
            scaledRatio = scaledHeight / originalHeight;
          }
        }
  
        canvasWidth = scaledWidth || originalWidth;
        canvasHeight = scaledHeight || originalHeight;
  
        canvas = $('<canvas>')[0];
        canvas.width = canvasWidth;
        canvas.height = canvasHeight;
        context = canvas.getContext('2d');
  
        if (options.fillColor) {
          context.fillStyle = options.fillColor;
          context.fillRect(0, 0, canvasWidth, canvasHeight);
        }
  
        // https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D.drawImage
        context.drawImage.apply(context, (function () {
          var source = getSourceCanvas(this.$clone[0], this.image),
              sourceWidth = source.width,
              sourceHeight = source.height,
              args = [source],
              srcX = data.x, // source canvas
              srcY = data.y,
              srcWidth,
              srcHeight,
              dstX, // destination canvas
              dstY,
              dstWidth,
              dstHeight;
  
          if (srcX <= -originalWidth || srcX > sourceWidth) {
            srcX = srcWidth = dstX = dstWidth = 0;
          } else if (srcX <= 0) {
            dstX = -srcX;
            srcX = 0;
            srcWidth = dstWidth = min(sourceWidth, originalWidth + srcX);
          } else if (srcX <= sourceWidth) {
            dstX = 0;
            srcWidth = dstWidth = min(originalWidth, sourceWidth - srcX);
          }
  
          if (srcWidth <= 0 || srcY <= -originalHeight || srcY > sourceHeight) {
            srcY = srcHeight = dstY = dstHeight = 0;
          } else if (srcY <= 0) {
            dstY = -srcY;
            srcY = 0;
            srcHeight = dstHeight = min(sourceHeight, originalHeight + srcY);
          } else if (srcY <= sourceHeight) {
            dstY = 0;
            srcHeight = dstHeight = min(originalHeight, sourceHeight - srcY);
          }
  
          args.push(srcX, srcY, srcWidth, srcHeight);
  
          // Scale destination sizes
          if (scaledRatio) {
            dstX *= scaledRatio;
            dstY *= scaledRatio;
            dstWidth *= scaledRatio;
            dstHeight *= scaledRatio;
          }
  
          // Avoid "IndexSizeError" in IE and Firefox
          if (dstWidth > 0 && dstHeight > 0) {
            args.push(dstX, dstY, dstWidth, dstHeight);
          }
  
          return args;
        }).call(this));
  
        return canvas;
      },
  
      setAspectRatio: function (aspectRatio) {
        var options = this.options;
  
        if (!this.disabled && !isUndefined(aspectRatio)) {
          options.aspectRatio = num(aspectRatio) || NaN; // 0 -> NaN
  
          if (this.built) {
            this.initCropBox();
  
            if (this.cropped) {
              this.renderCropBox();
            }
          }
        }
      },
  
      setDragMode: function (mode) {
        var $dragBox = this.$dragBox,
            cropable = false,
            movable = false;
  
        if (!this.ready || this.disabled) {
          return;
        }
  
        switch (mode) {
          case 'crop':
            if (this.options.dragCrop) {
              cropable = true;
              $dragBox.data('drag', mode);
            } else {
              movable = true;
            }
  
            break;
  
          case 'move':
            movable = true;
            $dragBox.data('drag', mode);
  
            break;
  
          default:
            $dragBox.removeData('drag');
        }
  
        $dragBox.toggleClass(CLASS_CROP, cropable).toggleClass(CLASS_MOVE, movable);
      }
    });
  
    prototype.change = function () {
      var dragType = this.dragType,
          options = this.options,
          canvas = this.canvas,
          container = this.container,
          cropBox = this.cropBox,
          width = cropBox.width,
          height = cropBox.height,
          left = cropBox.left,
          top = cropBox.top,
          right = left + width,
          bottom = top + height,
          minLeft = 0,
          minTop = 0,
          maxWidth = container.width,
          maxHeight = container.height,
          renderable = true,
          aspectRatio = options.aspectRatio,
          range = {
            x: this.endX - this.startX,
            y: this.endY - this.startY
          },
          offset;
  
      if (options.strict) {
        minLeft = cropBox.minLeft;
        minTop = cropBox.minTop;
        maxWidth = minLeft + min(container.width, canvas.width);
        maxHeight = minTop + min(container.height, canvas.height);
      }
  
      if (aspectRatio) {
        range.X = range.y * aspectRatio;
        range.Y = range.x / aspectRatio;
      }
  
      switch (dragType) {
        // Move cropBox
        case 'all':
          left += range.x;
          top += range.y;
          break;
  
        // Resize cropBox
        case 'e':
          if (range.x >= 0 && (right >= maxWidth || aspectRatio && (top <= minTop || bottom >= maxHeight))) {
            renderable = false;
            break;
          }
  
          width += range.x;
  
          if (aspectRatio) {
            height = width / aspectRatio;
            top -= range.Y / 2;
          }
  
          if (width < 0) {
            dragType = 'w';
            width = 0;
          }
  
          break;
  
        case 'n':
          if (range.y <= 0 && (top <= minTop || aspectRatio && (left <= minLeft || right >= maxWidth))) {
            renderable = false;
            break;
          }
  
          height -= range.y;
          top += range.y;
  
          if (aspectRatio) {
            width = height * aspectRatio;
            left += range.X / 2;
          }
  
          if (height < 0) {
            dragType = 's';
            height = 0;
          }
  
          break;
  
        case 'w':
          if (range.x <= 0 && (left <= minLeft || aspectRatio && (top <= minTop || bottom >= maxHeight))) {
            renderable = false;
            break;
          }
  
          width -= range.x;
          left += range.x;
  
          if (aspectRatio) {
            height = width / aspectRatio;
            top += range.Y / 2;
          }
  
          if (width < 0) {
            dragType = 'e';
            width = 0;
          }
  
          break;
  
        case 's':
          if (range.y >= 0 && (bottom >= maxHeight || aspectRatio && (left <= minLeft || right >= maxWidth))) {
            renderable = false;
            break;
          }
  
          height += range.y;
  
          if (aspectRatio) {
            width = height * aspectRatio;
            left -= range.X / 2;
          }
  
          if (height < 0) {
            dragType = 'n';
            height = 0;
          }
  
          break;
  
        case 'ne':
          if (aspectRatio) {
            if (range.y <= 0 && (top <= minTop || right >= maxWidth)) {
              renderable = false;
              break;
            }
  
            height -= range.y;
            top += range.y;
            width = height * aspectRatio;
          } else {
            if (range.x >= 0) {
              if (right < maxWidth) {
                width += range.x;
              } else if (range.y <= 0 && top <= minTop) {
                renderable = false;
              }
            } else {
              width += range.x;
            }
  
            if (range.y <= 0) {
              if (top > 0) {
                height -= range.y;
                top += range.y;
              }
            } else {
              height -= range.y;
              top += range.y;
            }
          }
  
          if (width < 0 && height < 0) {
            dragType = 'sw';
            height = 0;
            width = 0;
          } else if (width < 0) {
            dragType = 'nw';
            width = 0;
          } else if (height < 0) {
            dragType = 'se';
            height = 0;
          }
  
          break;
  
        case 'nw':
          if (aspectRatio) {
            if (range.y <= 0 && (top <= minTop || left <= minLeft)) {
              renderable = false;
              break;
            }
  
            height -= range.y;
            top += range.y;
            width = height * aspectRatio;
            left += range.X;
          } else {
            if (range.x <= 0) {
              if (left > 0) {
                width -= range.x;
                left += range.x;
              } else if (range.y <= 0 && top <= minTop) {
                renderable = false;
              }
            } else {
              width -= range.x;
              left += range.x;
            }
  
            if (range.y <= 0) {
              if (top > 0) {
                height -= range.y;
                top += range.y;
              }
            } else {
              height -= range.y;
              top += range.y;
            }
          }
  
          if (width < 0 && height < 0) {
            dragType = 'se';
            height = 0;
            width = 0;
          } else if (width < 0) {
            dragType = 'ne';
            width = 0;
          } else if (height < 0) {
            dragType = 'sw';
            height = 0;
          }
  
          break;
  
        case 'sw':
          if (aspectRatio) {
            if (range.x <= 0 && (left <= minLeft || bottom >= maxHeight)) {
              renderable = false;
              break;
            }
  
            width -= range.x;
            left += range.x;
            height = width / aspectRatio;
          } else {
            if (range.x <= 0) {
              if (left > 0) {
                width -= range.x;
                left += range.x;
              } else if (range.y >= 0 && bottom >= maxHeight) {
                renderable = false;
              }
            } else {
              width -= range.x;
              left += range.x;
            }
  
            if (range.y >= 0) {
              if (bottom < maxHeight) {
                height += range.y;
              }
            } else {
              height += range.y;
            }
          }
  
          if (width < 0 && height < 0) {
            dragType = 'ne';
            height = 0;
            width = 0;
          } else if (width < 0) {
            dragType = 'se';
            width = 0;
          } else if (height < 0) {
            dragType = 'nw';
            height = 0;
          }
  
          break;
  
        case 'se':
          if (aspectRatio) {
            if (range.x >= 0 && (right >= maxWidth || bottom >= maxHeight)) {
              renderable = false;
              break;
            }
  
            width += range.x;
            height = width / aspectRatio;
          } else {
            if (range.x >= 0) {
              if (right < maxWidth) {
                width += range.x;
              } else if (range.y >= 0 && bottom >= maxHeight) {
                renderable = false;
              }
            } else {
              width += range.x;
            }
  
            if (range.y >= 0) {
              if (bottom < maxHeight) {
                height += range.y;
              }
            } else {
              height += range.y;
            }
          }
  
          if (width < 0 && height < 0) {
            dragType = 'nw';
            height = 0;
            width = 0;
          } else if (width < 0) {
            dragType = 'sw';
            width = 0;
          } else if (height < 0) {
            dragType = 'ne';
            height = 0;
          }
  
          break;
  
        // Move image
        case 'move':
          canvas.left += range.x;
          canvas.top += range.y;
          this.renderCanvas(true);
          renderable = false;
          break;
  
        // Scale image
        case 'zoom':
          this.zoom(function (x1, y1, x2, y2) {
            var z1 = sqrt(x1 * x1 + y1 * y1),
                z2 = sqrt(x2 * x2 + y2 * y2);
  
            return (z2 - z1) / z1;
          }(
            abs(this.startX - this.startX2),
            abs(this.startY - this.startY2),
            abs(this.endX - this.endX2),
            abs(this.endY - this.endY2)
          ));
  
          this.startX2 = this.endX2;
          this.startY2 = this.endY2;
          renderable = false;
          break;
  
        // Crop image
        case 'crop':
          if (range.x && range.y) {
            offset = this.$cropper.offset();
            left = this.startX - offset.left;
            top = this.startY - offset.top;
            width = cropBox.minWidth;
            height = cropBox.minHeight;
  
            if (range.x > 0) {
              if (range.y > 0) {
                dragType = 'se';
              } else {
                dragType = 'ne';
                top -= height;
              }
            } else {
              if (range.y > 0) {
                dragType = 'sw';
                left -= width;
              } else {
                dragType = 'nw';
                left -= width;
                top -= height;
              }
            }
  
            // Show the cropBox if is hidden
            if (!this.cropped) {
              this.cropped = true;
              this.$cropBox.removeClass(CLASS_HIDDEN);
            }
          }
  
          break;
  
        // No default
      }
  
      if (renderable) {
        cropBox.width = width;
        cropBox.height = height;
        cropBox.left = left;
        cropBox.top = top;
        this.dragType = dragType;
  
        this.renderCropBox();
      }
  
      // Override
      this.startX = this.endX;
      this.startY = this.endY;
    };
  
    $.extend(Cropper.prototype, prototype);
  
    Cropper.DEFAULTS = {
      // Defines the aspect ratio of the crop box
      // Type: Number
      aspectRatio: NaN,
  
      // Defines the percentage of automatic cropping area when initializes
      // Type: Number (Must large than 0 and less than 1)
      autoCropArea: 0.8, // 80%
  
      // Outputs the cropping results.
      // Type: Function
      crop: null,
  
      // Add extra containers for previewing
      // Type: String (jQuery selector)
      preview: '',
  
      // Toggles
      strict: true, // strict mode, the image cannot zoom out less than the container
      responsive: true, // Rebuild when resize the window
      checkImageOrigin: true, // Check if the target image is cross origin
  
      modal: true, // Show the black modal
      guides: true, // Show the dashed lines for guiding
      highlight: true, // Show the white modal to highlight the crop box
      background: true, // Show the grid background
  
      autoCrop: true, // Enable to crop the image automatically when initialize
      dragCrop: true, // Enable to create new crop box by dragging over the image
      movable: true, // Enable to move the crop box
      resizable: true, // Enable to resize the crop box
      rotatable: true, // Enable to rotate the image
      zoomable: true, // Enable to zoom the image
      touchDragZoom: true, // Enable to zoom the image by wheeling mouse
      mouseWheelZoom: true, // Enable to zoom the image by dragging touch
  
      // Dimensions
      minCanvasWidth: 0,
      minCanvasHeight: 0,
      minCropBoxWidth: 0,
      minCropBoxHeight: 0,
      minContainerWidth: 200,
      minContainerHeight: 100,
  
      // Events
      build: null, // Function
      built: null, // Function
      dragstart: null, // Function
      dragmove: null, // Function
      dragend: null, // Function
      zoomin: null, // Function
      zoomout: null // Function
    };
  
    Cropper.setDefaults = function (options) {
      $.extend(Cropper.DEFAULTS, options);
    };
  
    // Use the string compressor: Strmin (https://github.com/fengyuanchen/strmin)
    Cropper.TEMPLATE = (function (source, words) {
      words = words.split(',');
      return source.replace(/\d+/g, function (i) {
        return words[i];
      });
    })('<0 6="5-container"><0 6="5-canvas"></0><0 6="5-2-9" 3-2="move"></0><0 6="5-crop-9"><1 6="5-view-9"></1><1 6="5-8 8-h"></1><1 6="5-8 8-v"></1><1 6="5-face" 3-2="all"></1><1 6="5-7 7-e" 3-2="e"></1><1 6="5-7 7-n" 3-2="n"></1><1 6="5-7 7-w" 3-2="w"></1><1 6="5-7 7-s" 3-2="s"></1><1 6="5-4 4-e" 3-2="e"></1><1 6="5-4 4-n" 3-2="n"></1><1 6="5-4 4-w" 3-2="w"></1><1 6="5-4 4-s" 3-2="s"></1><1 6="5-4 4-ne" 3-2="ne"></1><1 6="5-4 4-nw" 3-2="nw"></1><1 6="5-4 4-sw" 3-2="sw"></1><1 6="5-4 4-se" 3-2="se"></1></0></0>', 'div,span,drag,data,point,cropper,class,line,dashed,box');
  
    // Save the other cropper
    Cropper.other = $.fn.cropper;
  
    // Register as jQuery plugin
    $.fn.cropper = function (options) {
      var args = toArray(arguments, 1),
          result;
  
      this.each(function () {
        var $this = $(this),
            data = $this.data('cropper'),
            fn;
  
        if (!data) {
          $this.data('cropper', (data = new Cropper(this, options)));
        }
  
        if (typeof options === 'string' && $.isFunction((fn = data[options]))) {
          result = fn.apply(data, args);
        }
      });
  
      return isUndefined(result) ? this : result;
    };
  
    $.fn.cropper.Constructor = Cropper;
    $.fn.cropper.setDefaults = Cropper.setDefaults;
  
    // No conflict
    $.fn.cropper.noConflict = function () {
      $.fn.cropper = Cropper.other;
      return this;
    };
  
  return {
    Cropper
  }
})
;
define('Cropper/main',['jquery'], function($) {
  // 配置参数
  function init(image, callback, onOkId){
    var $image = image,
        $dataX = $('#dataX'),
        $dataY = $('#dataY'),
        $dataHeight = $('#dataHeight'),
        $dataWidth = $('#dataWidth'),
        $dataRotate = $('#dataRotate'),
        options = {
          // strict: false,
          // responsive: false,
          // checkImageOrigin: false

          // modal: false,
          // guides: false,
          // highlight: false,
          // background: false,

          // autoCrop: false,
          // autoCropArea: 0.5,
          // dragCrop: false,
          // movable: false,
          // resizable: false,
          // rotatable: false,
          // zoomable: false,
          // touchDragZoom: false,
          // mouseWheelZoom: false,

          // minCanvasWidth: 320,
          // minCanvasHeight: 180,
          // minCropBoxWidth: 160,
          // minCropBoxHeight: 90,
          // minContainerWidth: 320,
          // minContainerHeight: 180,

          // build: null,
          // built: null,
          // dragstart: null,
          // dragmove: null,
          // dragend: null,
          // zoomin: null,
          // zoomout: null,
          // modify: 2022-03-10 jiangyang.li 缺陷[27606]。
          autoCropArea: 0.6,
          // modify: 2022-03-08 jiangyang.li 按罗大要求，截图去掉等比缩放。
          // aspectRatio: 1,
          preview: '.img-preview',
          crop: function (data) {
            $dataX.val(Math.round(data.x));
            $dataY.val(Math.round(data.y));
            $dataHeight.val(Math.round(data.height));
            $dataWidth.val(Math.round(data.width));
            $dataRotate.val(Math.round(data.rotate));
          }
        };
        $image.on({
          'build.cropper': function (e) {
            console.log(e.type);
          },
          'built.cropper': function (e) {
            console.log(e.type);
          },
          'dragstart.cropper': function (e) {
            console.log(e.type, e.dragType);
          },
          'dragmove.cropper': function (e) {
            console.log(e.type, e.dragType);
          },
          'dragend.cropper': function (e) {
            console.log(e.type, e.dragType);
          },
          'zoomin.cropper': function (e) {
            console.log(e.type);
          },
          'zoomout.cropper': function (e) {
            console.log(e.type);
          }
        }).cropper(options);
        $image.cropper(options);

        // modify: 2022-02-28 jiangyang.li 新增：修复截图弹窗点击确定发送多次请求。
        $(onOkId).off('click');
        $(onOkId).on('click', function (e) {
          var data = $(this).data(),
              $target,
              result;
          if (data.method) {
          data = $.extend({}, data); // Clone a new one
  
          if (typeof data.target !== 'undefined') {
              $target = $(data.target);
              if (typeof data.option === 'undefined') {
                  try {
                      data.option = JSON.parse($target.val());
                  } catch (e) {
                      console.log(e.message);
                  }
              }
          }
  
          result = $image.cropper(data.method, data.option);
  
          // 获取图片信息
          if (data.method === 'getCroppedCanvas') {
            if (result.toDataURL) {
              var base64Img = result.toDataURL('image/jpg');
              callback(base64Img);
            }
  
              // canvas转为blob
              // result.toBlob(function (blob) {
              //     console.log(blob, 'blob');
              //     callback(imgObject)
              // }, 'image/jpeg');
  
          }
        }});
  };

  return {
    init: init,
  };
});

/*
 author:mikahuang
 create:2015-5-10
 desc:导航栏
*/
define('doc/navigation/index',['require', './service', 'underscore', '../' + lang(),'Cropper/cropper', 'Cropper/main', "plugins/router",], function (require, navService, _, lang,cropper, cropperMain, router) {
    var ko = require("knockout");
    var app = require("durandal/app");
    // var doc = require("doc/index");
    var constInfo = require("header/userinfo/constinfo")
    var subscription;
    var pathObj;
    var sharePerm;
    var pathNodes = ko.observableArray();
    var orgPathNodes = ko.observableArray();
    var omitHideList = ko.observableArray();
    var _sourceNodes = null;
    var selectedItem = ko.observable('');
    //modify 2022-03-24 zhengjie 28410面包屑路径收缩至只展示最后一项 企业库点击返回按钮报错，共享给我和个人库点击返回按钮无反应，且个人库点击...下的文件夹无反应  +backId ,+shareId
    var omit = { id: -2, backId: '', shareId:'',text: '···', crumbsOmit: true };
    var currentPath;
    var currentNode = ko.observable(1);
    var omitHideLeft = ko.observable();
    var currentPageMsg = ko.observable(); // 当前页标识
    var omitIs = ko.observable(false);
    var isShowInnerChain = ko.observable(false);//v6是否展示面包屑内链（ture:展示，false不展示）
    var isShowNavTab = ko.observable(false);//是否展示tab切换样式,最近
    var searchNavigationType = ko.observable('curfolder');
    var searchNavigationNodes = ko.observableArray();

    var isShowAtlasIcon = ko.observable(false);//面包屑图谱按钮是否显示
    var searchAtlasWidth = ko.observable(560);//面板宽度
    var searchAtlasHeight = ko.observable(400);//面板高度
    var searchNavigationAtlasTextType = ko.observable('entity');//图谱类型
    var searchInfo = ko.observable('');//面包屑展示搜索面板用户搜索内容
    var isShowSearchNavInfo = ko.observable(false);//面包屑图谱按钮是否显示
    var isAllSeacrch = ko.observable(false);//是否全库搜索

    function textSize(fontSize, fontFamily, text, textNum, isReturnSize = false) {
        var span = document.createElement("span");
        var result = {};
        var textWidth = 0;
        var additionalWidth = (textNum * 16) + ((textNum - 1) * 18) || 0;
        result.width = span.offsetWidth;
        result.height = span.offsetHeight;
        span.style.visibility = "hidden";
        span.style.fontSize = fontSize;
        span.style.fontFamily = fontFamily;
        span.style.display = "inline-block";
        document.body.appendChild(span);
        if (typeof span.textContent != "undefined") {
            span.textContent = text;
        } else {
            span.innerText = text;
        }
        textWidth = window.getComputedStyle(span).width;
        //modify by haiwei.cui 20220309  动态增加的span标签用完及时清理
        if (isReturnSize) {
            // 直接返回文字长度
            span.remove();
            return parseInt(textWidth);
        }
        result.width = parseFloat(textWidth) + additionalWidth - result.width;
        result.height = parseFloat(window.getComputedStyle(span).height) - result.height;
        span.remove();
        return result;
    }

    //视图设置图标 是否显示
    function showViewSetting() {
        //视图设置要根据权限显示
        //if (window.location.hash.indexOf("enterprise") > -1) {
        //    app.trigger('docsetbar:showsetup', { isneed: true });
        //} else {
        //    app.trigger('docsetbar:showsetup', { isneed: false });
        //}
    }

    function deDuplicationArr(allList = [], remList = []) {
        var cloneAllList = _.clone(allList);
        var cloneRemList = _.clone(remList);
        for (var i = 0; i < cloneAllList.length; i++) {
            for (var j = 0; j < cloneRemList.length; j++) {
                if (cloneAllList[i].id == cloneRemList[j].id && !cloneRemList[j].crumbsOmit) {
                    cloneAllList.splice(i, 1);
                }
            }
        };
        return cloneAllList;
    }

    // 计算显示 4 个面包屑
    function calculateCrumbsShow(crumbsList = [], message, omitLeft = 0, omitList = [], nwidth = 0) {
        var cloneCrumbsList = _.clone(crumbsList); 
        var textLenWidth = 0;
        var truncateIndex = 0;
        var itemTxtWidth = 0;
        var thresholdNum = 0;
        if(!cloneCrumbsList || (cloneCrumbsList && cloneCrumbsList.length == 0)) {
            // 面包屑数据异常时重置其他参数
            omitHideList([]);
            pathNodes([]);
        };
        // modify：shuchen.hu 2022-04-27  #29229 共享给我中多级文件夹 面包屑出现...的时候，用户头像会变成文件夹图标（KB也有此问题）
        if(message == 'teamlist:load' || message == 'shareNav') { // 团队库需要显示前两条所以做额外处理
            truncateIndex = 2;
        } else { // 非团队库时截取第二项
            truncateIndex = 1;
        };
        for (var index = 0; index < cloneCrumbsList.length; index++) {
            itemTxtWidth = textSize("12px", '"Microsoft YaHei", arial', cloneCrumbsList[index].text, index, true);
            // 团队库时动态获取团队库名称长度已计算更多面包屑容器的距左距离 判断omitList长度控制只计算一次
            if(message == 'teamlist:load' && omitList.length < 1 && (index == 0 || index == 1)) {
                // omitLeft += (itemTxtWidth + (18 * 2));
                omitLeft += ((itemTxtWidth > 400 ? 400 : itemTxtWidth) + (18 * 2));
            };

            // modify：shuchen.hu 2022-04-27  #29229 共享给我中多级文件夹 面包屑出现...的时候，用户头像会变成文件夹图标（KB也有此问题）
            if(message == 'shareNav' && omitList.length < 1 && (index == 0 || index == 1)) {
                // omitLeft += (itemTxtWidth + (18 * 2));
                omitLeft += ((itemTxtWidth > 400 ? 400 : itemTxtWidth) + (18 * 2));
            };

            if(message != 'teamlist:load' && index == 0 && !omitLeft){
                // 非团队库距左距离取第一个面包屑就可以了 判断omitLeft是否存在控制只计算一次
                omitLeft += (itemTxtWidth + (18 * 2))
            };
            // 加上每一项的内边距宽度
            textLenWidth += 16;
            // 加上文字宽度
            textLenWidth += (itemTxtWidth > 400 ? 400 : itemTxtWidth);
            // 加上箭头宽度
            if(index > 0){
                textLenWidth += 18;
            };
            // 企业团队加上内链宽度
            if(["enterprise", "teamlist"].findIndex(item => message.indexOf(item) != -1) != -1 && !index){
                textLenWidth += 34;
            };

            // 不够完美
            // // 当面包屑内容宽度大于可容纳宽度时显示省略面包屑
            // if(textLenWidth > nwidth && !thresholdNum){
            //     thresholdNum = index;
            //     // 团队库特殊处理，可容纳宽度等于3时显示4个
            //     if(message == 'teamlist:load' && thresholdNum == 3){
            //         thresholdNum = 4;
            //     };
            //     if(cloneCrumbsList.findIndex(item => item.id == -2) == -1){
            //         // 显示内容中新增省略面包屑
            //         cloneCrumbsList.splice(truncateIndex, 0, omit);
            //     };
            //     // 赋值边距
            //     omitHideLeft(omitLeft);
            //     // 省略面包屑数组
            //     omitList.push(...cloneCrumbsList.splice(truncateIndex + 1, cloneCrumbsList.length - thresholdNum));
            //     omitHideList(omitList);
            //     pathNodes(cloneCrumbsList);
            //     continue;
            // } else {
            //     pathNodes(cloneCrumbsList);
            // };
        };
        // 递归形式性能损耗厉害先不用
        // 加上箭头宽度
        // textLenWidth += (18 * (cloneCrumbsList.length - 1));
        // // 企业团队加上内链宽度
        // if(["enterprise", "teamlist"].findIndex(item => message.indexOf(item) != -1) != -1){
        //     textLenWidth += 34;
        // };
        // 当面包屑内容宽度大于可容纳宽度时显示省略面包屑
        if((nwidth - textLenWidth) < 50){
            if(cloneCrumbsList.findIndex(item => item.id == -2) == -1){
                // 显示内容中新增省略面包屑
                cloneCrumbsList.splice(truncateIndex, 0, omit);
            };
            omitList.push(...cloneCrumbsList.splice(truncateIndex + 1, 1));
            // 进入回调继续循环判断  轻文档除外
            if (window.location.href.indexOf("doc/quick/mydocflow") === -1) {
                calculateCrumbsShow(cloneCrumbsList, message, omitLeft, omitList, nwidth);
            }
        } else {
            // 赋值边距
            //modify 2022-03-24 zhengjie 28410面包屑路径收缩至只展示最后一项 企业库点击返回按钮报错，共享给我和个人库点击返回按钮无反应，且个人库点击...下的文件夹无反应  +backId ,+shareId
            var indexOmit = cloneCrumbsList.findIndex(res => res.id == -2);
            if(cloneCrumbsList && cloneCrumbsList.length > 0 && indexOmit != -1 && omitList && omitList.length > 0){
                cloneCrumbsList[indexOmit].backId = omitList[omitList.length - 1].id;
                cloneCrumbsList[indexOmit].shareId = omitList[omitList.length - 1].shareId;
            };
            omitHideLeft(omitLeft);
            // 单独处理我的标签模块面包屑，给一个标识用来添加类名 (PD-2017 cuizhichao （UI）我的标签优化)
            var curentModule = require('knockout').$store.state.curentModule();
            if (curentModule && curentModule.hash == "quick/mytag") {
                //modify 2022-06-22 tangning  32447 标签"我的标签"字体和其他左导航模块大小显示不一致  已确认可以进行修改
                cloneCrumbsList[0]["tagFlag"] = false//true
                pathNodes(cloneCrumbsList);
            }else{
                pathNodes(cloneCrumbsList);
            }
            // modify: 2022-08-20 cuizhichao #36307 图文搜索结果页面，点击产品自带刷新后，面包屑不再显示；
            $.cookie('searchImageNav',JSON.stringify(pathNodes()))
            omitHideList(omitList);
        };
    };

    // modify: 2022-02-23 jiangyang.li 新增截图搜图功能。
    var screenshot = {
        retrievalVisible: ko.observable(false), // 检索内容是否可见。
        visible: ko.observable(false), // 截图框是否可见
        // 打开&关闭截图弹窗回调
        showScreenshot: function (type, o, e) {
            e.stopPropagation();
            screenshot.visible(type);
        },
        //优先从url获取token，其次从cooike里面读取token的值
        getToken: function () {
            var token = $.getQueryString("token");
            if (token.length !== 0) {
                return token;
            }
            token = $.cookie('token');
            if (token) {
                return token;
            }
            return token;
        },
        // 序列化url参数
        urlObj: function (url) {
            var o = {};
            if (url.includes('?')) {
                var list = url.split('&');
                for (var i=0; i<list.length; i++) {
                    if (list[i].includes('=')) {
                        var clist = list[i].split('=');
                        o[clist[0]] = clist[1];
                    }
                }
            }
            return o;
        },
        init: function (mark, base64) {
            // modify: 2022-03-07 jiangyang.li 新增：初始化前应该关闭截图弹窗。
            screenshot.visible(false);
            var o = screenshot.urlObj(decodeURI(window.location.href));
            var licInfo = constInfo.getLicInfo();
            // 根据权限和url参数判断检索内容是否可见。
            // modify: 2022-03-01 jiangyang.li 搜文不显示截图搜图功能。
            if (!(licInfo && licInfo.EnabledModules && licInfo.EnabledModules.includes('SearchImage') && o.type == 'searchForPictures' && decodeURI(window.location.href).includes('enterprise'))) {
                screenshot.retrievalVisible(false);
                return;
            };
            // modify: 2022-03-07 jiangyang.li 缺陷[27336] 搜图结果页，进行截图搜图，截图后还是按照原图搜图。
            var imgSrc = base64 || '/imageSearch/GetStream?&token=' + screenshot.getToken() + '&mark=' + (mark || o.fileid || o.mark || '') +  "&time" + Math.random();
            // modify: 2022-03-01 jiangyang.li 搜图列表截图添加防错：如果接口查询错误不显示截图弹窗。
            var ruleImgUrl = document.createElement('img');
            ruleImgUrl.setAttribute('src', imgSrc);
            ruleImgUrl.setAttribute('id', 'ruleImgUrl');
            ruleImgUrl.onload = function (res) {
                if (res) {
                    screenshot.retrievalVisible(true);
                    document.getElementById('ruleImgUrl') && document.getElementById('ruleImgUrl').remove();
                    $('#screenshotImg').attr("src", imgSrc);
                    // modify: 2022-03-14 jiangyang.li 新增：截图图片变形的问题。
                    $('#listImg').remove();
                    $('#img-container').prepend('<img id="listImg" alt="Picture" class="cropper-hidden" src="' + imgSrc + '"></img>');
                    $('.cropper-container').remove();
                    cropperMain.init($('#listImg'), function (imgBase64) {
                        function base64ToFile(baseData) {
                            var arr = baseData.split(',');
                            var type = arr[0].match(/:(.*?);/)[1];
                            var bytes = atob(arr[1]);
                            var n = bytes.length;
                            var bufferArray = new Uint8Array(n);
                            while (n--) {
                                bufferArray[n] = bytes.charCodeAt(n);
                            }
                            return new File([bufferArray], 'test' + Math.random() + '.jpg', {type: type});
                        };
                        var file = base64ToFile(imgBase64);
                        var formData = new FormData();
                        formData.append('file', file);
                        $.mask.show();
                        $.ajax({
                            url: 'ImageSearch/Upload',
                            type: 'POST',
                            processData: false,
                            contentType: false,
                            dataType: 'json',
                            data: formData,
                        }).done(function (data) {
                            $('.page-loading').hide();
                            if (data.nResult == 0) {
                                screenshot.visible(false);
                                // modify: 2022-03-07 jiangyang.li 缺陷[27336] 搜图结果页，进行截图搜图，截图后还是按照原图搜图。
                                imgBase64 && screenshot.init(null, imgBase64);                     
                                // modify: 2022-03-02 jiangyang.li 修复截图弹窗点击确定后列表未发送请求。
                                var url = 'doc/enterprise/1/search?searchkey=&type=searchForPictures' + '&mark=' + data.mark;
                                // modify: 2022-03-15 jiangyang.li 缺陷[26226] 搜图结果页默认略缩图模式
                                document.cookie = "searchThumb=true";
                                // modify: 2022-03-01 jiangyang.li 对象未初始话时点击无效。
                                router.navigate(url + ('&val=' + new Date().getTime()), { trigger: true,replace: true}); 
                                // modify: 2022-02-24 jiangyang.li 新增列表截图后自动复制图片内链, 可以粘贴到搜索框进行搜图。
                                var http = window.location.protocol + '//' + window.location.host + "/preview.html?mark=";
                                var tag = document.createElement('input');
                                tag.setAttribute('id', 'cp_screenshot_input');
                                tag.value = http + data.mark;
                                document.getElementsByTagName('body')[0].appendChild(tag);
                                document.getElementById('cp_screenshot_input').select();
                                document.execCommand('copy');
                                document.getElementById('cp_screenshot_input').remove();
                            } else {
                                app.trigger("message:show", { type: 'error', message:  lang.searchImgErr[data.nResult || 59999] });
                            }
                        })
                    }, '#getCroppedCanvas');
                }
            }
            ruleImgUrl.onerror = function (err) {
                screenshot.retrievalVisible(false);
                document.getElementById('ruleImgUrl') && document.getElementById('ruleImgUrl').remove();
            }
        },
    };

    return {
        compositionComplete: function (element) {
            /*
             反调用消息  例如:app.trigger("navigation:init", { message: "enterprise:tree", paths: [{id:1,text:""}] })
            */
            subscription = app.on("navigation:init").then(function (messager) {
                // modify: 2022-08-20 cuizhichao #36307 图文搜索结果页面，点击产品自带刷新后，面包屑不再显示；
                if (window.location.hash.indexOf("type=searchForPictures") > -1 || window.location.hash.indexOf("type=searchByText") > -1) {
                    messager.paths = JSON.parse($.cookie('searchImageNav'))
                }
                //页面刷新后会走次方法，再次设置页面刷新之前的面包屑状态
                // modify: #33833 cuizhicaho 以文搜文结果页面，F5刷新后，面包屑显示搜索内容
                if (window.location.hash.indexOf("search?") > -1 && window.location.hash.indexOf('type=searchForPictures') <= -1 && window.location.hash.indexOf('type=searchByText') <= -1) {
                    var pageModule = require('knockout').$store.state.curentModule();
                    var licInfo = constInfo.getLicInfo();
                    // 该处判断页面刷新时是否在搜索结果页面
                    isShowSearchNavInfo(true)
                    // QC 18177 2022-10-19 cuizhichao 搜索結果頁面上方，顯示所搜索的關鍵詞与搜索的内容标签不匹配
                    var getUrlParms = decodeURIComponent(window.location.href).split("search?")[1].split("&");
                    for (let i = 0; i < getUrlParms.length; i++) {
                        var element = getUrlParms[i];
                        // modify: 2022-12-26 by hsc [缺陷:41512]高级搜索后，面包屑中不显示搜索内容
                        if (element.indexOf("searchkey") > -1 && element.split("searchkey=")[1] != '' && JSON.stringify(getUrlParms).indexOf("searchmoreconds=") < 0) {
                            //modify: 2023-01-04 by zj 41781点击文件右侧面板中的标签进行搜索后，搜索结果页中的搜索内容会多显示一个引号
                            var newstr = ''
                            if (element.split("searchkey=")[1][0] == '"' && element.split("searchkey=")[1][element.split("searchkey=")[1].length - 1] == '"' ) {
                                newstr = element.split("searchkey=")[1].substr(1)
                                newstr = newstr.substring(0,newstr.length - 1)
                                $.cookie("searchNavParmasInfo", newstr);
                            }else{
                                $.cookie("searchNavParmasInfo", element.split("searchkey=")[1]);
                            }
                        }
                    }
                    searchInfo($.cookie("searchNavParmasInfo"))
                    if (licInfo.EnabledModules.includes("IntelligentSearch") && pageModule.module == 'enterprise' && $.cookie("isEnterprise") == 'true') {
                        isShowAtlasIcon(true)
                    }else{
                        isShowAtlasIcon(false)
                    }
                }else if (window.location.hash.indexOf("type=searchByText") > -1 ) {
                    // QC 18177 2022-10-19 cuizhichao 搜索結果頁面上方，顯示所搜索的關鍵詞与搜索的内容标签不匹配
                    var getUrlParms = decodeURIComponent(window.location.href).split("search?")[1].split("&");
                    for (let i = 0; i < getUrlParms.length; i++) {
                        var element = getUrlParms[i];
                        if (element.indexOf("searchkey") > -1 && element.split("searchkey=")[1] != '') {
                            //modifyby-gcg-2023-1-3 缺陷：41633 以文搜文搜索结果页面中，不显示以文搜文的搜索文件
                            //    $.cookie("searchNavParmasInfo", element.split("searchkey=")[1]);
                            //modify: 2023-01-04 by zj 41781点击文件右侧面板中的标签进行搜索后，搜索结果页中的搜索内容会多显示一个引号
                            var newstr = ''
                            if (element.split("searchkey=")[1][0] == '"' && element.split("searchkey=")[1][element.split("searchkey=")[1].length - 1] == '"' ) {
                                newstr = element.split("searchkey=")[1].substr(1)
                                newstr = newstr.substring(0,newstr.length - 1)
                                $.cookie("searchNavParmasInfo", newstr);
                            }else{
                                $.cookie("searchNavParmasInfo", element.split("searchkey=")[1]);
                            }
                        }
                    }
                    searchInfo($.cookie("searchNavParmasInfo"))
                    isShowSearchNavInfo(true)
                }else{
                    app.trigger("closeNavAtlas",true)
                    isShowSearchNavInfo(false)
                }
                //modify: 2022-03-16 by shuanbao.wei [缺陷:27937]复制文件后粘贴，弹出粘贴成功弹窗后立刻切换界面，文件路径显示错误
                var surentModel = ko.$store.state.curentModule();
                if (surentModel.module == "enterprise" && messager.message.indexOf("person") > -1) {
                    return;
                }
                // modify: 2022-0908 hsc #37187 共享文件夹，包含子文件夹，在共享给我进入子文件夹中，当前路径未高亮显示
                // modify: 2022-12-27 by hsc [缺陷:#40645]共享中收缩左侧目录框，左侧目录收缩失效，导致有左侧目录框的模块也失效
                if(surentModel.id == "myshareme" && messager.paths){
                    messager.paths = messager.paths.filter(function(e){
                       return e.text != undefined
                    })
                }
                if (messager.message != currentPageMsg()) { // 切换页面清除选中状态
                    selectedItem("");
                };

                if (messager.message == "myvisit:reload"){
                    isShowNavTab(true);
                    isAllSeacrch(false)   // 进入最近页面隐藏高级搜索面包屑
                    return;
                }
                else{
                    isShowNavTab(false);
                }
                
                ele_font_size = $('input').css('font-size');
                ele_font_family = $('input').css('font-family');
                //左侧面板拖动大小走此逻辑
                if (messager.message == "leftPanelResize") {
                    var nodes = _.clone(orgPathNodes());
                    var timeName = setTimeout(function(){
                        clearTimeout(timeName);
                        calculateCrumbsShow(nodes, messager.message, 0, [], $(".navigation-list").width());
                    })
                    return;
                }
                pathObj = messager;
                sharePerm = messager && messager.sharePerm;
                var nodes = messager.paths;
                var nwidth = $(".navigation-list").width();
                orgPathNodes(messager.paths);
                _sourceNodes = _.clone(nodes);

                //加载面包屑数据
                calculateCrumbsShow(nodes, messager.message, 0, [], nwidth);

                currentPageMsg(messager.message);
                currentPath = _sourceNodes[_sourceNodes.length - 1];
                currentNode(currentPath);
                //视图设置图标 是否 显示
                showViewSetting();
                // v6 只有企业和团队库展示面包屑内链
                if (messager.message.indexOf("enterprise") > -1 || messager.message.indexOf("teamlist") > -1) {
                    isShowInnerChain(true)
                } else {
                    isShowInnerChain(false)
                }

                // 26141企业库回收站的面包屑不应显示复制内链按钮
                if (messager.paths.length > 0) {
                    for (var i = 0; i < messager.paths.length; i++) {
                        if (messager.paths[i].id == -1) {
                            isShowInnerChain(false)
                        }
                    }
                }

                if(window.location.hash.indexOf("search?") < 0){
                    isAllSeacrch(false)
                }

            });

            // modify: 2022-02-23 jiangyang.li 新增截图搜图功能。
            screenshot.init();
            $(window).bind('hashchange', function() {
                screenshot.init();
            });
            app.on("navigation:searchImgCallBack").then(function (o) {
                // modify: 2022-03-02 jiangyang.li 修复截图弹窗图片变形。
                screenshot.init(o.mark);
            });
            // 监听鼠标右键点击，关闭截图弹窗。
            $(document).mousedown(function(e){
                if(e.which == 3){
                    screenshot.visible(false);
                }
            });

            // modify: 2022-02-23 jiangyang.li 新增：控制截图显示。
            app.on("navigation:searchImgHide").then(function (o) {
                if (o.type) screenshot.init();
                if (o.type) {
                    isShowSearchNavInfo(false)
                }
                screenshot.retrievalVisible(o.type);
                // searchNavigationNodes([pathNodes()[0]])
                // searchNavigationType('allFolder')
            });
            // modify 2022-09-06 liuchunli #37029【搜索】搜索结果页点击路径跳转后，再点击浏览器返回按钮，路径未刷新
            app.on("navigationBack:SearchNavInfo").then(function () {
                    isShowSearchNavInfo(true)
                
            });
            app.on("navigation:changeSearchHide").then(function (search_parm) {
                // searchNavigationNodes([pathNodes()[0]])
                // searchNavigationType(o.type)
                isShowSearchNavInfo(true)//用户执行搜索，显示面包屑搜索内容信息
                searchInfo(search_parm.searchText)
                $.cookie("searchNavParmasInfo",search_parm.searchText)//搜索面板输入词语存入cookie
                var licInfo = constInfo.getLicInfo();//获取lic授权信息
                var pageModule = require('knockout').$store.state.curentModule();//获取当前模块信息，只企业库支持图谱相关展示
                if (licInfo.EnabledModules.includes("IntelligentSearch") && pageModule.module == 'enterprise' && search_parm.isEnterprise) {
                    // 根据lic授权文件、是否在企业库、是否选择的内容 文件名 标签来显示面包屑图谱icon，
                    isShowAtlasIcon(true)
                    $.cookie("isEnterprise",true)//设置cookie，设置面包屑是否显示图谱icon
                }else{
                    isShowAtlasIcon(false)
                    $.cookie("isEnterprise",false)//设置cookie，设置面包屑是否显示图谱icon
                }
                // modify: cuizhichao 2022-09-14 #37413 企业、团队、个人库回收站，点击右侧面板的标签后，面包屑中显示出搜索内容；
                if (window.location.href.indexOf("enterprise/-1") > -1 || window.location.href.indexOf("?recycle=-1") > -1 || window.location.href.indexOf("person/-1") > -1) {
                    isShowSearchNavInfo(false)//用户执行搜索，显示面包屑搜索内容信息
                }
            });
            // modify: cuizhichao 2022-09-08 #37119 未被索引的文件以文搜文后，不应该显示出搜索内容；
            app.on("fileNotIndexed").then(function (isFileNotIndexed) {
                if (isFileNotIndexed) {
                    isShowSearchNavInfo(false)
                }
            })

            // modify: hsc20230404 QC19998 跨库搜索 面包屑显示--显示检索全库
            app.on("navigation:isSankuCrumbsInfo").then(function(msg){
                if(msg.isAdv){ // 高级搜索
                    if(!msg.isAllSearch){
                        isAllSeacrch(false) 

                    }else{
                        isAllSeacrch(true)

                    }
                }else{
                    if(msg.isAllSearchKey !== 'allFolder'){
                        isAllSeacrch(false) 
                    }else{
                        isAllSeacrch(true)
                    }
                }
            })
        },
        lang: lang,
        currentNode: currentNode,
        pathNodes: pathNodes,
        orgPathNodes: orgPathNodes,
        selectedItem: selectedItem,
        omitHideLeft: omitHideLeft,
        omitHideList: omitHideList,
        currentPageMsg: currentPageMsg,
        isShowInnerChain: isShowInnerChain,
        isShowNavTab: isShowNavTab,
        searchNavigationType:searchNavigationType,
        searchNavigationNodes:searchNavigationNodes,
        isShowAtlasIcon:isShowAtlasIcon,
        searchInfo:searchInfo,
        searchAtlasWidth:searchAtlasWidth,
        searchAtlasHeight:searchAtlasHeight,
        searchNavigationAtlasTextType:searchNavigationAtlasTextType,
        // ----
        isShowSearchNavInfo:isShowSearchNavInfo,
        isAllSeacrch:isAllSeacrch,
        handleMouseoverOmit: function () {
            selectedItem('omit-2');
            omitIs(true);
        },
        handleMouseover: function (data) {
            if (data == "omit") {
                selectedItem('');
                return;
            };
            if (omitIs()) {
                omitIs(false);
            };
            if (data.crumbsOmit && !selectedItem()) {
                selectedItem('omit' + data.id);
            };
            if (!data.crumbsOmit && selectedItem()) {
                selectedItem("");
            };
        },
        handleMouseout: function () {
            if (selectedItem()) {
                var timeName = setTimeout(function () {
                    if (!omitIs()) {
                        selectedItem("");
                    };
                    clearTimeout(timeName);
                }, 100)
            };
        },
        // 复制内链
        copyUrl: function () {
            var tag = document.createElement('input');
            // modify hsc 20220908 #37221 搜索结果页的地址制给其他人打开后，搜索内容显示错误
            var matchReg = /(\S*)\/search/;
            tag.setAttribute('id', 'cp_hgz_input');
            tag.value = window.location.href;
            if(window.location.href.includes("/search")){
                tag.value = window.location.href.match(matchReg)[1];
            }
            document.getElementsByTagName('body')[0].appendChild(tag);
            document.getElementById('cp_hgz_input').select();
            document.execCommand('copy');
            document.getElementById('cp_hgz_input').remove();
            app.trigger("message:show", { type: 'success', message: lang.innerChainCopyUrl });
        },
        //转到
        gotoPath: function (data) {

            app.trigger('PreviewModelViewPanelShow:close');
            // modify:2022-09-01 cuizhichao #36744 点击面包屑返回时，所点击的面包屑高亮显示；
            setTimeout(() => {
                selectedItem("")
            }, 100);
            // 我的共享 ##gs  我的共享面包屑优化
            if (currentPageMsg() == 'shareNav' && data.id <= 0 && data.type == 'shareBase') {
                return false;
            }

            // 我的收藏
            if (currentPageMsg() == 'favoritenavigation' && data.id <= 0) {
                return false;
            }
            // 我的轻文档
            if (currentPageMsg() == 'docflownavigation') {
                return false;
            }
            //modify by hanshunlong,2022-05-05,QC14413[GMP]齐鲁GMP项目全文检索界面部分功能隐藏需求
            if (constInfo.systemInfo.instanceConfigInfo.EnableGMP && window.location.href.indexOf("search?") > 0) {
                return;
            }


            if (data.id == -1) {//回收站点了也不跳它
                return;
            };
            // 选中后再次点击去掉选中状态
            if (((selectedItem() || selectedItem() === 0) && selectedItem() === data.id)
                || (data.crumbsOmit && selectedItem() === 'omit' + data.id)
                // ||(data.moreTeams && selectedItem() === 'teamlist:load' + data.id)
            ) {
                selectedItem("");
                return;
            };
            // 更多
            if (data.crumbsOmit) {
                selectedItem('omit' + data.id);
                return;
            };
             // modify 2022-03-31 liuchunli #28408 团队库搜索结果页面，F5刷新后，右侧面板和面包屑显示空白
             // modify 2022-08-30 liuchunli #36648 企业/个人库中点击面包屑，再切换到团队库时，显示为企业/个人库中的文件
            if(window.location.hash.indexOf('#doc/team') >-1){
                app.trigger("teamCurrentFolderId:id",data.id);
            }
            // 团队库更多团队额外处理
            // if(data.moreTeams){
            //     selectedItem('teamlist:load' + data.id);
            //     return;
            // };
            selectedItem(data.id);
            currentNode(data);
            data.sharePerm = sharePerm;
            app.trigger(pathObj.message, data);
            app.trigger('PreviewModelViewPanelShow:close');
        },
        //返回上一级
        gotoParent: function () {
            currentPath = pathNodes();
            if (currentPath.length) {
                currentNode(currentPath.pop());
            }
            var hash = window.location.hash;
            // modify 2022-09-07 liuchunli #32515企业/团队/个人库搜索结果页面，点击返回按钮，应返回到搜索之前的层级目录
            if(hash.indexOf('/search?searchkey') == -1){
                //modify 2022-09-13 liuchunli #37273企业/团队/个人库搜索结果页面，点击搜索标签后，再搜索结果页面选中文件点击返回按钮，应返回到搜索之前的层级目录
                if (hash.indexOf('fromInfobar') < 0 && hash.indexOf('/search?searchType=3') == -1) {//右侧面板搜索标签时,不返回上一级,而是返回搜索起始位置
                    if (currentPath.length) {
                        currentNode(currentPath.pop());
                    }
                }
            }
            currentNode().sharePerm = sharePerm;
            app.trigger(pathObj.message, currentNode(), {});
            app.trigger('PreviewModelViewPanelShow:close');
        },
        // 搜索内容关闭
        clearSearch: function () {
            selectedItem(857)
            searchNavigationType('curfolder')
            searchNavigationNodes([])
            if ($.cookie("clearSearchToggleHash") != null && $.cookie("clearSearchToggleHash").length > 0) {
                // modify:2022-09-01 cuizhichao #36798 点击右侧面板中标签进行搜索后，搜索结果页面异常；
                if ($.cookie("clearSearchToggleHash").indexOf('/search?') > -1) {
                    window.location.replace($.cookie("clearSearchToggleHash").split('/search?')[0]);
                }else{
                    window.location.replace($.cookie("clearSearchToggleHash"));
                }
            }else{
                return;
            }
        },
        // 点击图谱按钮事件，显示图谱弹窗
        clearSearchAtlas: function () {
            // $(".searchAtlasModal").css({
            //     display:'block',
            //     // left:`-${parseInt($(".searchAtlasIcon").offset().left) - 300}px`
            // })
            if ($('.easy-atlas-modal-entity').css('display') == 'block') return;
            let fileId = 427;
            let createData=[
                {
                    module: 'atlsaModal',
                    fileId: fileId,
                    testParms:{
                        atlasName:'bigKeyAtlas',
                        type: searchNavigationAtlasTextType(),
                        showOp:0,
                        // width:$root.searchAtlasWidth(),
                        // height:$root.searchAtlasHeight(),
                        fontSize:12,
                        nodeSize:38,
                        level:2,
                        anchor:searchInfo(),
                        onSearch:22,
                        // searchNode:function(data){$root.searchMoreAtlas(data) }
                    }
                }
            ]
            app.trigger("component:show", {
                moduleId: 'doc/atlsaModal/index',
                datas: createData,
                message: 'atlsaModal:reload',
                moduleName:'favoritelist'
            });
        },
        //刷新
        refresh: function () {
            currentPath = pathNodes();
            currentNode(currentPath[currentPath.length - 1]);
            app.trigger(pathObj.message, currentNode(), true);
        },
        /*
         * yinlu创建于20170227
         * 获取用户头像
         * modify: 2022-04-21 shuchen.hu 缺陷 【29229】 共享给我中多级文件夹 面包屑出现...的时候，用户头像会变成文件夹图标（KB也有此问题）;
        */
         getUserIcon: function (data) {
            if (data != 0) {
                return constInfo.getAvatar(60, data, Math.random());
            }
            return "Content/images/fileIcons/folder32.png";
        },
        // modify: 2022-02-23 jiangyang.li 新增截图搜图功能。
        screenshot: screenshot,
    }
});
define('doc/tactic/processStrategy/processStrategy',["underscore", "knockout", "durandal/app", "header/userinfo/constinfo"], function (_, ko, app, constinfo) {
    var moduleName;
    var selectData;
    var message;
    var Process = ko.observableArray();
    var folders = "";
    var files = "";
    var folderId;
    var type;
    /*
        * mikahuang创建于2016年09月
        * 获取选中
        * @data  页面传递的选中项
        */
    function getSelectedEntry(data) {
        folders = "";
        files = "";
        if (data && data.data) {
            if (data.folderType) {
                folders = data.id;
                type = data.folderType;
            }
            if (data.fileType) {
                files = data.id;
                type = data.fileType;
            }
            if (data.data.folderType) {
                folders = data.data.id;
                type = data.data.folderType;
            }
            if (data.data.fileType) {
                files = data.data.id;
                type = data.data.fileType;
            }
            return;
        }
        $.each(data, function (n, obj) {
            //如果没有data，就从本身取
            var doc;
            if (obj.data) {
                doc = obj.data;
            } else {
                doc = obj;
            }
            if (doc.folderType) {
                folders += doc.id + ",";
            }
            //201610mikahuang 引用等无法删除,去除fileType==2的判断
            //else if (doc.fileType && doc.fileType == 2) {
            else if (doc.fileType) {
                files += doc.id + ",";
            }
        });
        folders = rTrim(folders, ",");
        files = rTrim(files, ",");
    }
    /*
        * mikahuang创建于2016年09月
        * 去除右侧多余字符
        * @str 待处理字符串
        * @flag 需去除字符
        */
    function rTrim(str, flag) {
        return str.replace(new RegExp((flag || " ") + "$"), "");
    }
    function getProcessStrategy(currentDate, action) {
        var processStrategy;
        var openTheThirdPartyProcess;
        getSelectedEntry(currentDate);
        if (action == "create") {
            if (currentDate.length == 1) {
                if (currentDate[0].folderType || currentDate[0].data.folderType) {
                    folderId = currentDate[0].id ? currentDate[0].id : currentDate[0].data.id;
                } else {
                    folderId = currentDate[0].data.parentFolderId;

                }
            } else {
                folderId = currentDate[0].data.parentFolderId;
            }
        }
        if (action == "delete") {
           // getSelectedEntry(currentDate);
            if (currentDate.length == 1) {
                if (files != "") {
                    folderId = currentDate[0].data.parentFolderId;
                } else {
                    folderId = currentDate[0].data.id;
                }
            } else {
                folderId = currentDate[0].data.parentFolderId;
            }
        }
        if (action == "update") {
            folderId = currentDate.data.parentFolderId;
        }
        if (action == "attribute") {
            if (currentDate[0].data.folderType) {
                type = currentDate[0].data.folderType
                folderId = currentDate[0].data.id;
            }
            if (currentDate[0].data.fileType) {
                type = currentDate[0].data.fileType
                folderId = currentDate[0].data.parentFolderId;
            }
        }
        if (action == "share") {
            if (currentDate[0].data.folderType) {
                type = currentDate[0].data.folderType
                folderId = currentDate[0].data.id;
            }
            if (currentDate[0].data.fileType) {
                type = currentDate[0].data.fileType
                folderId = currentDate[0].data.parentFolderId;
            }
            
        }
        if (action == "public") {
            if (currentDate[0].data.folderType) {
                type = currentDate[0].data.folderType
                folderId = currentDate[0].data.id;
            }
            if (currentDate[0].data.fileType) {
                type = currentDate[0].data.fileType
                folderId = currentDate[0].data.parentFolderId;
            }
          
        }
        if (action == "outerNetPublish") {//流程外网外发
            folderId = currentDate[0].data.parentFolderId;
            type = currentDate[0].data.fileType;
        }
        if(action == "SecretLevel"){
            if(currentDate.folderType){
                type = currentDate.folderType
                folderId = currentDate.data.id;
            }
            if(currentDate.fileType){
                type = currentDate.fileType
                folderId = currentDate.parentFolderId
            }
            else if (!currentDate.isFolder && currentDate.isFolder != undefined) {
                type = currentDate[0].data.fileType
                folderId = currentDate[0].data.parentFolderId
            }
            if(!folderId && currentDate.length > 0){
                folderId = currentDate[0].data.parentFolderId;
                type = currentDate[0].data.fileType;
            }
        }
        if(action == "DataFerry"){
            if (currentDate.length == 1) {
                if (files != "") {
                    folderId = currentDate[0].data.parentFolderId;
                } else {
                    folderId = currentDate[0].data.id;
                }
            } else {
                folderId = currentDate[0].data.parentFolderId;
            }
        }

        if (action == "FexFerry") {
            if (currentDate.length == 1) {
                if (files != "") {
                    folderId = currentDate[0].data.parentFolderId;
                } else {
                    folderId = currentDate[0].data.id;
                }
            } else {
                folderId = currentDate[0].data.parentFolderId;
            }
        }


        $.ajax({
            data: {
                module: "WebClient",
                fun: "GetProcessStrategyByKey",
                entryId: folderId,
                entryType: "1",
            },
            async: false,
        }).done(function (result) {
            if (result.result == 0) {
                processStrategy = result.processStrategy;
                openTheThirdPartyProcess = result.openTheThirdPartyProcess;
            }
        });
        var process=[];
        switch (action) {
            case "create":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([1], d.procType);
                });
                break;
            case "update":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([2], d.procType);
                });
                break;
            case "delete":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([3], d.procType);
                });
                break;
            case "attribute":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([4], d.procType);
                });
                break;
            case "share":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([5], d.procType);
                });
                break;
            case "public":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([6], d.procType);
                });
                break;
            case "SecretLevel":
                process = _.filter(processStrategy,function(d){
                    return _.contains([7],d.procType);
                });
                break;
            case "DataFerry":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([8], d.procType);
                });
                break;
            case "FexFerry":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([9], d.procType);
                });
                break;
            case "outerNetPublish":
                process = _.filter(processStrategy, function (d) {
                    return _.contains([10], d.procType);
                });
                break;
        }
        if (process.length > 0) {
            var ph;
            var wfdocId = "";
            var wfdocType = "";
            if (action == "create") {
                if (currentDate[0] && currentDate[0].id) {
                    wfdocId = currentDate[0].id;
                } else if (currentDate[0] && currentDate[0].data && currentDate[0].data.id) {
                    wfdocId = currentDate[0].data.id;
                }
                if (currentDate[0] && currentDate[0].folderType) {
                    wfdocType = currentDate[0].folderType;
                } else if (currentDate[0] && currentDate[0].data && currentDate[0].data.folderType) {
                    wfdocType = currentDate[0].data.folderType;
                }
            }
            if (openTheThirdPartyProcess == true) {
                var spt = "?";
                if (process[0].stgProcAddress.indexOf("?") > -1) {
                    spt = "&";
                }
                switch (action) {
                    case "create":
                        ph = process[0].stgProcAddress + spt + "entryId=" +
                            wfdocId + "&entryType=" + wfdocType + "&entryName=" + encodeURI(currentDate[0].name) + "&folders=&files=&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "update":
                        ph = process[0].stgProcAddress + spt + "entryId=" +
                            currentDate.data.id + "&entryType=" + currentDate.data.fileType + "&entryName=" + encodeURI(currentDate.data.name) + "&folders=&files=&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "delete":
                        ph = process[0].stgProcAddress + spt + "entryId=& entryType=& entryName=& folders="
                            + folders + "&files=" + files + "&orgToken=" + constInfo.getTokenCookie();
                        break;
                    case "attribute":
                        ph = process[0].stgProcAddress + spt + "entryId=&entryType="
                            + type + "&entryName=" + encodeURI(currentDate[0].data.name) + "&folders=" + folders + "&files=" + files + "&orgToken="
                            + constInfo.getTokenCookie();
                        break;

                    case "share":
                        if (type == 128) {
                            ph = process[0].stgProcAddress + spt + "entryId=" + folders + files + "&entryType=2&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        else {
                            ph = process[0].stgProcAddress + spt + "entryId=" + folders + files + "&entryType=" + type + "&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                       
                        break;
                    case "public":
                        if (type == 128) {
                            ph = process[0].stgProcAddress + spt + "entryId=" + folders + files + "&entryType=2&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        else {
                            ph = process[0].stgProcAddress + spt + "entryId=" + folders + files + "&entryType=" + type + "&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        break;
                    case "SecretLevel":
                        if (currentDate.data == undefined) {
                            ph = process[0].stgProcAddress + spt + "entryId=" + currentDate[0].data.id + "&entryType=" + constinfo.systemInfo.instanceConfigInfo.SecretEdition
                                + "&entryName=" + encodeURI(currentDate[0].data.name) + "&secretlevelId=" + currentDate[0].data.securityLevelId + "&secretleveName=" + encodeURI(currentDate[0].data.securityLevelName) + "&entryName=&folders=&files=" + files + "&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        else {
                            ph = process[0].stgProcAddress + spt + "entryId=" + currentDate.data.id + "&entryType=" + constinfo.systemInfo.instanceConfigInfo.SecretEdition
                                + "&entryName=" + encodeURI(currentDate.data.name) + "&secretlevelId=" + currentDate.data.securityLevelId + "&secretleveName=" + encodeURI(currentDate.data.securityLevelName) + "&entryName=&folders=&files=" + files + "&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        break;
                    case "DataFerry":
                    ph = process[0].stgProcAddress + spt + "entryId=&entryType=&entryName=&folders="
                    + folders + "&files=" + files + "&orgToken=" + constInfo.getTokenCookie();
                        break;
                    case "FexFerry":
                        ph = process[0].stgProcAddress + spt + "entryId=&entryType=&entryName=&folders="
                            + folders + "&files=" + files + "&orgToken=" + constInfo.getTokenCookie();
                        break;
                    case "outerNetPublish":
                        ph = process[0].stgProcAddress + spt + "entryId=" + folders + files + "&entryType=" + type + "&entryName=&folders=&files=&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                }

            } else {
                switch (action) {
                    case "create":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" +
                            wfdocId + "&entryType=" + wfdocType + "&entryName=" + encodeURI(currentDate[0].name) + "&folders=&files=&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "update":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" +
                            currentDate.data.id + "&entryType=" + currentDate.data.fileType + "&entryName=" + encodeURI(currentDate.data.name) + "&folders=&files=&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "delete":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=&entryType=&entryName=&folders="
                            + folders + "&files=" + files + "&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "attribute":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=&entryType="
                            + type + "&entryName=" + encodeURIComponent(currentDate[0].data.name) + "&folders=" + folders + "&files=" + files+"&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "share":
                        if (type == 128) {
                            ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" + folders + files + "&entryType=2&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        else {
                            ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" + folders + files + "&entryType="
                                + type + "&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        break;
                    case "public":
                        if (type == 128) {
                            ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" + folders + files + "&entryType=2&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                        else {
                            ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" + folders + files + "&entryType="
                                + type + "&entryName=&folders=&files=&orgToken="
                                + constInfo.getTokenCookie();
                        }
                       
                        break;
                    case "SecretLevel":
                        // ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" + currentDate.data.id+ "&entryType="
                        //     + currentDate.data.fileType + "&entryName=" + encodeURI(currentDate.data.name) + "&secretlevelId=" + currentDate.data.securityLevelId + "&secretleveName=" + encodeURI(currentDate.data.securityLevelName) + "&entryName=&folders=&files=" +files+"&orgToken="
                        //     +constInfo.getTokenCookie();
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId="+constinfo.userInfo.SecLevel+"&entryType="+constinfo.systemInfo.instanceConfigInfo.SecretEdition+"&entryName=&secretlevelId="+ constinfo.userInfo.SecLevel+"&secretleveName=&entryName=&folders=&files=" + files + "&orgToken="
                            +constInfo.getTokenCookie();
                        break;
                    case "DataFerry":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=&entryType=&entryName=&folders="
                            + folders + "&files=" + files + "&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "FexFerry":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=&entryType=&entryName=&folders="
                            + folders + "&files=" + files + "&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                    case "outerNetPublish":
                        ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].procId + "&entryId=" + folders + files + "&entryType="
                            + type + "&entryName=&folders=&files=&orgToken="
                            + constInfo.getTokenCookie();
                        break;
                }
            }
            window.open(ph);
            return true;
        }
        return false;
    }

    function IsProcessStrategy(currentDate, processType) {
        var data = {};
        getSelectedEntry(currentDate);
        $.ajax({
            type: "GET",
            url: "/inbiz/api/services/WorkFlowDoc/IsProcessStrategy",
            data: {
                token: constInfo.getTokenCookie(),
                fileIds: files,
                folderIds: folders,
                processType: processType,
            },
            async: false,
        }).done(function (result) {
            data = {
                IsProcessStrategy: result.IsProcessStrategy,
                IsNoDocument: result.IsNoDocument,
                NoProcessStrategy: result.NoProcessStrategy
            };
        });
        return data;
    }
    return {
        getProcessStrategy: getProcessStrategy,
        IsProcessStrategy: IsProcessStrategy,
        getView: function () {
            var view = '<div></div>';
            return $(view).get(0);
        },
        compositionComplete: function (view, parent) {
            var $parent = ko.dataFor(parent);
            var datas = $parent.datas();
            var action = $parent.datas().action;
            //$.ajax({
            //    type: "GET",
            //    url: "/edoc2Flow-web/edoc-task/getProcessApply",
            //    data: {
            //        userId: constInfo.userInfo.guid,
            //        orgToken: constInfo.getTokenCookie(),
            //    },
            //    async: false,
            //}).done(function (result) {
            //    if (result.length > 0) {
            //        Process(result);
            //    }
            //})
            //var process;
            //if (action == "create") {
            //    process = _.filter(Process(), d => _.contains(["文件新增"], d.Process_Name))
            //}
            //if (action == "update") {
            //    process = _.filter(Process(), d => _.contains(["文件更新"], d.Process_Name))
            //}
            //if (action == "delete") {
            //    process = _.filter(Process(), d => _.contains(["文件删除"], d.Process_Name))
            //}
            //if (action == "attribute") {
            //    process = _.filter(Process(), d => _.contains(["权限申请"], d.Process_Name))
            //}
            //if (process != null && process.length > 0) {
            //    //var ph = "/edoc2Flow-web/edoc2-form/jumpStartForm?processId=" + process[0].Process_ID + "&entryId=" + datas[0].id + "&entryType="
            //    //    + datas[0].type + "&entryName=" + datas[0].name + "&folders=" + "" + "&files=" + "" + "&orgToken="
            //    //    + constInfo.getTokenCookie();
            //    //window.open(ph); 
            //}
        }
    };
});
/*
   lizheng创建于2015年6月8日
   描述:文件上传模块
*/

define('component/upload/index',["jquery", 'durandal/app', 'knockout', "komapping", "WebUploader",
    "header/userinfo/constinfo", "header/userinfo/cookie", "doc/navigation/index", lang(), "edoc2ErrorCode", "doc/tactic/processStrategy/processStrategy"],//"css!../style/1.css" 可以使用 require-css来加载样式
    function (jquery, app, ko, komapping,
        //WebUploader引用
        webUploader,
        //通用方法模块
        constInfo,
        $cookie,
        navigation,
        //多语言
        lang,
        errorCode, process
    ) {
        //false=显示 true=销毁 其他值隐藏（要写onClose）
        var showWindow = ko.observable(1);
        //当前文件夹
        var currentFolder = ko.observable();
        //被调用时，保存前一个文件夹
        var previousFolder = ko.observable();
        //文件列表
        var fileList = ko.observableArray();
        //运行时 html5 或 flash
        var runtime = ko.observable();
        //是否支持图片Base64
        var isSupportBase64 = true;
        var isProcess = false;
        var uploader;
        var acceptError = "";
        //组件状态 ready,finish,paused,uploading
        var state = ko.observable();
        //recordId --uuid 上传到临时文件夹时使用
        //var recordId = "";
        //function S4() {
        //    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
        //}
        //function NewGuid() {
        //    return (S4() + S4() + "-" + S4() + "-" + S4() + "-" + S4() + "-" + S4() + S4() + S4());
        //}
        //重命名组件
        var renameComponent = ko.observable();
        //重命名组件参数
        var renameData = ko.observable();
        //保存冲突策略
        var renameStrategy = {};
        //后台上传时的显示信息
        var miniInfo = ko.observable();
        //单文件最大值
        var fileSingleSizeLimit = 6 * 1024 * 1024 * 1024; //6G
        var eformEnvSizeLimit;
        var person = ko.observable(false);
        //是否外发列表中上传文件
        var isOuPublish = false;
        //是否是附件文件上传
        var isAttachFile = false;
        //用于文件夹上传时，忽略本地选择的根目录名称，使用此目录名称代替
        var rootFolderName = "";
        //当前窗口的提示信息
        //var messageTip = ko.observable();
        var headMd5Size = 1024 * 1024;
        var minChunkSize = 5 * 1024 * 1024;// 5M
        var _fileNumLimit = 500; // 一次选择或拖拽的文件个数上限
        var $uploadContainer, $pickContainer, $placeholder, $renameComponent, $fileList, $filePicker, $mini, $folderPicker, $buttonFolder, $inputFolderDiv, $inputFolder, $labelFolder;
        //保存是否eform上传
        var isEformEnv = false;
        //是否跨域
        var isCrossDomain = false;
        //是否支持多文件选择，流程只支持单文件
        var uploadMultiple = ko.observable(true);
        // 入队前回调函数
        var onBeforeFileQueued = undefined;
        //只允许单文件上传
        if ($.getQueryString("singleuploadmode") === "true") {
            _fileNumLimit = 1;
        }
        if (_fileNumLimit === 1) {
            uploadMultiple = ko.observable(false);
        }
        // 文件上传数量限制，默认500
        var uploadFilesNum = ko.observable(_fileNumLimit);
        var uploadAccept = {};
        //eform控件允许还是禁止上传（true为允许）
        var isAllow = true;
        //元数据是否启用强制策略
        var isPrevInherit = false;
        //编辑文件元数据权限
        var updateMetaPerm = false;
        //设置密级权限
        var setLvlPerm = false;
        var hideWindow = function () {
            showWindow(1);
            //如果前一个文件夹有数据
            if (previousFolder()) {
                //切换文件夹
                switchFolder(previousFolder(), currentFolder());
                //清空前一个文件夹
                previousFolder = ko.observable();
            }
            //后台上传时更新一下信息
            updateMiniInfo();
            //如果上传全部完成，移除列表
            if (state() === 'finish') {
                fileList.removeAll();
                uploader.reset();
            }
            if (/^#system/.test(window.location.hash)) {
                app.trigger('templatelist:reload', 'Refresh');
            }

            // 处理门户取消上传后不能关闭窗口问题
            if (!isCrossDomain) {
                if (window.parent.activeUploaderHost && window.parent.activeUploaderHost.onHideWindow) {
                    window.parent.activeUploaderHost.onHideWindow();
                }
            }

        };

        //更新后台上传时的显示信息
        var updateMiniInfo = function () {
            //设置上传信息
            if (showWindow()) {
                //cq.wuyongwen修改于2018-2-1 个人内容库内存变化
                app.trigger('person:updateCapacity');
                //var img = '<img src ="scripts/lib/webuploader/' + state() + '.gif"/>';
                if (state() === 'uploading') {
                    //modify: 2017-11-21 haibin.long 需要统计 队列中，暂定两个状态的数据

                    miniInfo(lang.miniInfoUploading + (uploader.getFiles("queued", "interrupt").length));
                } else if (state() === 'paused') {
                    miniInfo(lang.miniInfoPaused);
                }
            }
        };
        //最后一批文件（同文件夹算一批）
        var lastBatch = { folderId: 0, fileIds: [] };

        //切换文件夹，将两个文件夹相同的属性交换
        var switchFolder = function (folder1, folder2) {

            for (var key in folder2) {
                //ko数据不处理
                if (key === '__ko_mapping__') {
                    continue;
                }
                if (folder1[key]) {
                    var temp = folder2[key]();
                    folder2[key](folder1[key]());
                    folder1[key](temp);
                }
            }
        };

        var supportWebkitdirectory = undefined;
        var testWebkitdirectory = function () {
            if (supportWebkitdirectory) {
                return supportWebkitdirectory;
            }
            var tester = document.createElement('input');
            tester.type = 'file';
            if ('multiple' in tester && 'webkitdirectory' in tester) {
                supportWebkitdirectory = true;
            } else {
                supportWebkitdirectory = false;
            }
            return supportWebkitdirectory;
        };

        //获取限速配置
        function GetSpeedLimit(userId, tkn) {
            var uploadLimitResult = 0;
            $.ajax({ data: { token: tkn, module: "SystemManager", fun: "GetCurrentUserSpeedLimit", userId: userId }, async: false }).done(function (r) {
                if (r && r.result == 0 && r.speedLimit && r.speedLimit != '') {
                    var speedLimitData = JSON.parse(r.speedLimit);
                    uploadLimitResult = speedLimitData.UploadLimit;

                    if (speedLimitData.DownloadLimit > 0) {
                        $.cookie('maxspeed', speedLimitData.DownloadLimit);
                    } else {
                        $.cookie('maxspeed', null);
                    }
                }
            });
            return uploadLimitResult;
        }
        //初始化
        var init = function (options) {
            //ie9关闭上传框后第二次点击上传又会出现flash.exec报错的问题
            if (options && options.forceInit) {
                uploader = null;
            }
            /* 没用到
            var fileNumLimit = 300;
            if ($.getQueryString("singleuploadmode") === "true") { fileNumLimit = 1; }
            */
            if (!uploader) {
                //判断浏览器是否支持Base64
                var data = new Image();
                data.onload = data.onerror = function () {
                    if (this.width != 1 || this.height != 1) {
                        isSupportBase64 = false;
                    }
                };
                data.src = "data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///ywAAAAAAQABAAACAUwAOw==";
                //var serverurl = constInfo.systemInfo.uploadUrl + (constInfo.isPubilsh ? ("?code=" + constInfo.code) : "?token=" + getTokenByCondition());
                var serverurl = getUploadUrl({
                    RegionType: 1
                });

                constInfo.reloadSysConfigInfo();
                var confInfo = constInfo.getSysConfigInfo();
                var isLimit = false;
                var currentChunkSize = confInfo.ChunkSize * 1024 * 1024;
                var uploadLimit = GetSpeedLimit(constInfo.userInfo.id, constInfo.userInfo.token);
                if (uploadLimit > 0) {
                    isLimit = true;
                    currentChunkSize = uploadLimit;
                }
                // 实例化
                uploader = webUploader.create({
                    //添加文件按钮
                    pick: {
                        id: $filePicker,
                        label: lang.addFile,
                        multiple: uploadMultiple()
                    },
                    //formData: {
                    //    token: 123
                    //},
                    //上传框内部拖拽
                    dnd: $placeholder,
                    //粘贴处理事件元素
                    paste: document.body,
                    // 去重， 根据文件名字、文件大小和最后修改时间来生成hash Key
                    duplicate: true,
                    //粘贴前事件，自定义添加，非自带
                    pasteBefore: function () {
                        var event = arguments.callee.caller.caller.caller.caller.caller.arguments[0];
                        //不是文本框并且上传窗口显示时才可以粘贴文件
                        if (event.target == document.body && !showWindow()) {
                            return true;
                        }
                        return false;
                    },
                    //flash路径
                    swf: 'scripts/lib/webuploader/Uploader.swf',
                    //分片传输
                    chunked: true,
                    //分片大小5M
                    chunkSize: currentChunkSize,
                    //chunkSize: 512 * 1024,
                    //上传接受的服务器端
                    server: serverurl,

                    //runtimeOrder: 'flash',
                    accept: uploadAccept,

                    // 禁掉全局的拖拽功能。这样不会出现图片拖进页面的时候，把图片打开。
                    disableGlobalDnd: true,
                    //最大文件数
                    fileNumLimit: uploadFilesNum(),
                    //不使用图片压缩
                    compress: null,
                    //同时只有一个文件在传输
                    threads: 1,
                    //有文件自动开始传输
                    auto: true,
                    //限制大小2g，单文件
                    fileSingleSizeLimit: fileSingleSizeLimit,
                    //prepareNextFile:true
                    //所有文件大小
                    //fileSizeLimit: 200 * 1024 * 1024,   
                    isSpeedLimit: isLimit,
                    isAllow: isAllow
                });
                //保存当前运行时
                runtime(uploader.request('predict-runtime-type'));
                //当有文件添加进来时
                uploader.onFileQueued = function (file) {
                    if (!currentFolder()) return;
                    if (isEformEnv) {
                        if (!isCrossDomain) {
                            if (window.parent.activeUploaderHost && window.parent.activeUploaderHost.onBeforeFileQueued) {
                                if (window.parent.activeUploaderHost.onBeforeFileQueued(file) == false) {
                                    uploader.cancelFile(file.id);
                                    return;
                                }
                            }
                        }
                    }
                    //属性赋值
                    file.folderId = currentFolder().id();
                    var obj = {};
                    obj.id = file.id;
                    //根据文件名 最后修改时间 大小来生成的hash
                    obj.hash = file.__hash;
                    //隐藏文件夹上传后，获取传的recordId
                    //if (currentFolder().id() === 6) {
                    //    if ($.getQueryString("recordId") != "" && $.getQueryString("recordId") != undefined && $.getQueryString("recordId") != null) {
                    //        recordId = $.getQueryString("recordId");
                    //    } else {
                    //        recordId = NewGuid();
                    //    }
                    //    console.log("recordId:" + recordId);
                    //    if (recordId != "" && recordId != undefined && recordId != null) {
                    //        obj.name = recordId + "@@" + file.name;
                    //    } else {
                    //        obj.name = file.name;
                    //    }
                    //} else {
                    //    obj.name = file.name;
                    //}
                    obj.name = file.name;
                    obj.size = constInfo.formatSize(file.size);
                    obj.ext = file.ext;
                    obj.type = file.type;
                    if (currentFolder().masterFile() && currentFolder().masterFile().id()) {
                        obj.masterFileId = currentFolder().masterFile().id();
                    }
                    obj.folderId = currentFolder().id();
                    obj.folderName = currentFolder().name();
                    obj.folderPath = currentFolder().path();
                    obj.fileId = 0; // 后台返回的fileId
                    //文件夹相对路径
                    obj.folderRelativePath = "";
                    obj.regionHash = "";
                    //进度% 
                    obj.percent = 0;
                    //速度 字节
                    obj.speed = 0;
                    var ico = constInfo.getFileIcon(file.ext);
                    if (!ico) {
                        ico = "unknown";
                    }
                    obj.icon = $.format("external/icon-file-type/32/{0}.png", ico);
                    //状态信息
                    obj.statusText = "queued";
                    //文件错误信息的原因
                    obj.reason = "";
                    //文件冲突的策略
                    obj.strategy = "";
                    obj.uploadId = constInfo.getGuid(); // 进度条缓存id
                    obj.fullPath = file.source.fullPath || "";
                    obj.regionId = 1;
                    obj.approved = true; // checkAndCreateDocInfo是否通过
                    var koObj = komapping.fromJS(obj);

                    //速度信息 63%(1.26 MB/s)
                    koObj.speedInfo = ko.computed(function () {
                        if (/*this.percent() == 0 &&*/ this.speed() === 0) {
                            return "";
                        }
                        return $.format("{0}/s", constInfo.formatSize(this.speed())); //({0}%){1}/s
                    }, koObj);

                    fileList.push(komapping.fromJS(koObj));
                    //文件状态改变的处理事件
                    file.on('statuschange', function (cur, prev) {
                        var obj = findFile(file.id);
                        if (file.size == 0) {
                            if (obj.statusText() !== "complete" && obj.statusText() !== "error" && obj.statusText() !== "cancelled")
                                obj.statusText(cur);
                            else
                                cur = obj.statusText();
                            if (obj.statusText() === "error") {
                                file.setStatus("error");
                            } else if (obj.statusText() === "cancelled") {
                                file.setStatus("complete");
                            }
                        }
                        else {
                            if (obj.statusText() !== "cancelled")
                                obj.statusText(cur);
                        }
                        if (cur === 'progress') {
                            //如果开始上传了，把之前的错误信息清空
                            obj.reason('');
                            //记录
                            if (!file.startTime) {
                                file.startTime = new Date();
                            }
                        }

                    });
                    //生成缩略图
                    uploader.makeThumb(file, function (error, src) {
                        if (error) {
                            return;
                        }
                        if (isSupportBase64) {
                            koObj.icon(src);
                        }
                    }, 64, 64);
                };
                uploader.onFileDequeued = function (file) {
                    //alert(file.name);
                };
                uploader.onUploadProgress = function (file, percentage) {
                    //从fileList里找到和触发事件id相等的对象
                    var obj = findFile(file.id);
                    if (!obj) return;
                    //计算进度和速度
                    if (obj.percent() == 100) {
                        return;
                    }
                    var percentFixed = (percentage * 100).toFixed(2);
                    obj.percent(percentFixed);
                    var uploadSize = file.size * percentage;
                    var s = (new Date() - file.startTime) / 1000;
                    obj.speed(uploadSize / s);

                };
                // 文件被加入队列之前触发，返回值为false，文件不会被添加进入队列
                uploader.on("beforeFileQueued", function (file) {
                    if (onBeforeFileQueued) {
                        if (!onBeforeFileQueued(file)) {
                            return false;
                        }
                    }
                    //收集不支持上传类型
                    if (uploader.options.accept[0] &&
                        uploader.options.accept[0].extensionsButQueued &&
                        //统一大小写
                        uploader.options.accept[0].extensionsButQueued.indexOf("," + file.ext.toLowerCase() + ",") == -1 &&
                        acceptError.indexOf("," + file.ext.toLowerCase()) == -1 &&
                        uploader.options.isAllow) {
                        acceptError = acceptError + "," + file.ext.toLowerCase();
                    }
                    else if (uploader.options.accept[0] &&
                        uploader.options.accept[0].extensionsButQueued &&                   //统一大小写
                        uploader.options.accept[0].extensionsButQueued.indexOf("," + file.ext.toLowerCase() + ",") != -1 &&
                        acceptError.indexOf("," + file.ext.toLowerCase()) == -1 &&
                        !uploader.options.isAllow) {
                        acceptError = acceptError + "," + file.ext.toLowerCase();
                    }
                    //clog("fullPath: " + file.source.fullPath);
                    if (file.source.fullPath) {
                        if (rootFolderName) {
                            var arr = file.source.fullPath.split('/');
                            if (arr && arr.length > 2) {
                                file.source.fullPath = file.source.fullPath.replace("/" + arr[1] + "/", "/" + rootFolderName + "/");
                            }
                        }
                        // 判断有没有创建文件夹权限
                        if (!currentFolder()) return;
                        var folderId = currentFolder().id();
                        if (!checkCreateFolderPerm(folderId, null, false)) { // 同步获取创建[文件夹]权限
                            isShowNoCreateFolderTip = true;
                            return false; // 不加入列队
                        }
                    }
                });
                var isShowNoCreateFolderTip = false;
                // 当一批文件添加进队列以后触发，files个数为0也会触发
                uploader.on("filesQueued", function (files) {
                    if (isShowNoCreateFolderTip) {
                        isShowNoCreateFolderTip = false;
                        app.trigger("message:show", { type: 'error', message: lang.noPermCreateFolder });
                    }
                    //如果acceptError不为空 报个错 并设置isClose为false 不关闭文件上传窗口
                    if (acceptError != "") {
                        acceptError = acceptError.substr(1);
                        app.trigger("message:show", { type: 'error', message: lang.acceptError });
                        acceptError = "";
                    }
                    // 更新后台上传时的显示信息
                    updateMiniInfo();
                });
                //某个文件开始上传前触发，一个文件只会触发一次(其实retry后，还会触发)
                uploader.on('uploadStart', function (file) {
                    // 开始上传 tangbangguo 2018/6/25
                    if (isEformEnv && !isCrossDomain && parent.window.activeUploaderHost) {
                        // 如果是eform上传，会去校验名称是否存在，如果存在就提示文件已存在。 // tangbangguo 2018/6/25 
                        // 附件列表文件集合
                        var files = parent.window.activeUploaderHost.files;
                    }
                    var obj = findFile(file.id);
                    if (!obj) return;

                    // 上传0Bytes文件二次确认,每个文件上传前都会去读取一下系统设置里面的开关
                    constInfo.reloadSysConfigInfo();
                    var confInfo = constInfo.getSysConfigInfo();
                    var enableZeroBytesDoubleCheck = confInfo.ZeroBytesDoubleCheck;
                    if (file.size == 0 && enableZeroBytesDoubleCheck && file.doubleCheck === undefined) {
                        uploader.stop();
                        file.setStatus("error", 4320);
                        file.doubleCheck = 1; // 给file增加doubleCheck属性,在重传中判断是否有该属性
                        obj.approved(false); // 不通过
                        obj.reason(lang.ErrorCode4320);
                        return;
                    }

                    var strategy = "";
                    if (file.nResult && file.nResult === 610) {
                        if (obj.strategy && obj.strategy()) {
                            //单独处理过重名策略
                            strategy = obj.strategy();
                        }
                        if (!strategy && renameStrategy[obj.folderId()]) {
                            //应用处理过重名策略
                            strategy = renameStrategy[obj.folderId()];
                        }

                    }

                    // 判断是上传还是更新
                    var FILE_MODE = strategy ? "UPDATE" : "UPLOAD";
                    var masterFileId = "";
                    if (obj.masterFileId && obj.masterFileId()) {
                        masterFileId = obj.masterFileId();
                    }
                    var folderId = obj.folderId();
                    //var hash = buildHash(folderId, file);
                    var code = constInfo.isPubilsh ? constInfo.code : "";

                    var lastModifiedDate;
                    if (file.lastModifiedDate instanceof Date) {
                        lastModifiedDate = file.lastModifiedDate.format("yyyy-MM-dd hh:mm:ss");
                    } else {
                        try {
                            lastModifiedDate = new Date(file.source.source.lastModified).format("yyyy-MM-dd hh:mm:ss");
                        } catch (e) {
                            lastModifiedDate = "";
                        }
                    }
                    var _fullPath = file.source.fullPath;
                    if (file.source.fullPath)
                        _fullPath = file.source.fullPath.replace(/\+/g, "%2b");
                    var postCheckData = {
                        folderId: folderId,
                        masterFileId: masterFileId,
                        token: constInfo.userInfo.token,
                        fileName: obj.name(),
                        fileRemark: "",
                        size: file.size,
                        type: file.type,
                        attachType: 0,
                        fullPath: _fullPath,
                        code: code,
                        //hash: hash,
                        strategy: strategy,
                        lastModifiedDate: lastModifiedDate,
                        fileModel: FILE_MODE,
                        ext: file.ext
                    };
                    if (isEformEnv) {
                        //eform更新文件,通过fileid
                        try {
                            if (currentFolder().operateType() === "update" && currentFolder().updateFileId()) {
                                postCheckData.fileId = currentFolder().updateFileId();
                                postCheckData.strategy = currentFolder().strategy();
                                postCheckData.fileModel = "UPDATE";
                            }
                        }
                        catch (ex) { }
                    }
                    checkAndCreateDocInfo(postCheckData, false, function (result) {
                        if (isEformEnv && eformEnvSizeLimit) {
                            if (file.size > eformEnvSizeLimit) {
                                result.reason = lang.ErrorCode617;
                                result.result = 617;
                                app.trigger("message:show", { type: 'error', message: lang.ErrorCode617 });
                            }
                        }
                        //控件传过来的文件类型配置验证                                                                                                                            //统一大小写
                        if (uploader.options.accept[0] && uploader.options.accept[0].extensionsButQueued && uploader.options.accept[0].extensionsButQueued.indexOf("," + file.ext.toLowerCase() + ",") == -1 && uploader.options.isAllow) {
                            result.result = 611;
                            result.reason = lang.ErrorCode611;                                                                                                                          //统一大小写
                        } else if (uploader.options.accept[0] && uploader.options.accept[0].extensionsButQueued && uploader.options.accept[0].extensionsButQueued.indexOf("," + file.ext.toLowerCase() + ",") != -1 && !uploader.options.isAllow) {
                            result.reason = lang.ErrorCode611;
                            result = {
                                availableSizes: result.availableSizes,
                                result: 611,
                                reason: lang.ErrorCode611,
                            }
                        }
                        if (result.result === 0) {
                            obj.approved(true); // 通过验证
                            obj.regionHash(result.data.RegionHash); // 服务器返回的hash值
                            obj.fileId(result.data.FileId);// 服务器返回的FileId
                            obj.regionId(result.data.RegionId);
                            if (result.data.FileVerId) {
                                obj.updateFileVerId = ko.observable(result.data.FileVerId);
                            }
                            //obj.nResult(0);
                            var __uploadUrl = getUploadUrl(result.data);
                            uploader.option('server', __uploadUrl);
                            file.nResult = result.result;
                            if (file.size > 0) {
                                if (!isEformEnv) {
                                    md5Verification(uploader, file, result.data.RegionId); // 验证md5
                                }
                            }
                            else {
                                var fileId = result.data.FileId;
                                // modify :2022.02.22 mengfc 缺陷 26608
                                //uploader.skipFile(file);
                                obj.statusText("complete");
                                obj.percent(100); // 进度条百分百

                                if (lastBatch.folderId !== folderId) {
                                    lastBatch.folderId = folderId;
                                    lastBatch.fileIds = [fileId];
                                } else {
                                    //如果相等，把fileIds加入新id
                                    lastBatchPush(fileId);
                                }
                                if (isEformEnv && window.parent.activeUploaderHost) {
                                    window.parent.activeUploaderHost.isDrawRecord = true;
                                    if (window.parent.activeUploaderHost._filesArray == undefined) {
                                        window.parent.activeUploaderHost._filesArray = [];
                                    }
                                    //关闭时需要再绘制一次记录
                                    if (lastBatch.fileIds.length > 0) {
                                        window.parent.activeUploaderHost._filesArray.push({
                                            name: file.name,
                                            size: file.size,
                                            id: lastBatch.fileIds[lastBatch.fileIds.length - 1],
                                            fileType: 2,
                                            parentId: lastBatch.folderId,
                                            ext: file.ext
                                        });
                                    }
                                }
                            }
                        } else {
                            obj.approved(false); // 不通过
                            uploader.option('server', serverurl);
                            //obj.nResult = result.result;
                            file.nResult = result.result;
                            file.FILE_MODE = FILE_MODE;
                            if (result.result === 4302 || result.result === 2048 || result.result === 4096) {
                                file.reason = result.reason;
                            } else {
                                file.reason = result.result;
                            }
                            if (file.size == 0) {
                                if (result.result == 610) {
                                    uploader.stop();
                                    file.setStatus("error", result.result);
                                    obj.reason(lang.ErrorCode610);
                                    //showWindow610(obj, uploader, file);
                                    strategy610(obj, uploader, file);
                                } else {
                                    reason = convertReason(file.reason, FILE_MODE);
                                    obj.reason(reason);
                                    obj.statusText("error");
                                }
                            }
                        }

                    });

                });
                uploader.on('error', function (action, limit, file) {
                    switch (action) {
                        case "Q_EXCEED_NUM_LIMIT":
                            app.trigger("message:show", { type: 'error', message: $.format(lang.outnumlimit, limit) });
                            //messageTip({ type: 'error', message: $.format(lang.outnumlimit, limit) });
                            break;
                        case "F_EXCEED_SIZE":
                            app.trigger("message:show", { type: 'error', message: $.format(lang.uploadedFileSize, constInfo.formatSize(limit)) });
                            //messageTip({ type: 'error', message: $.format("上传的文件大小不能超过{0}", constInfo.formatSize(limit)) });
                            break;
                        default:
                    }

                });
                uploader.on('uploadError', function (file, reason) {
                    uploader.stop();
                    //出错误时，要把开始时间设置undefined，否则会有计算速度问题
                    file.startTime = undefined;
                    var obj = findFile(file.id);
                    if (!obj) return;
                    obj.uploadId(constInfo.getGuid());  // 更新进度条缓存id
                    reason = convertReason(reason, file.FILE_MODE);

                    obj.reason(reason);
                    obj.percent(0);
                    //如果出错原因是重名，需要继续处理    
                    if (file.nResult && file.nResult === 610) {
                        strategy610(obj, uploader, file);
                    } else if (parseInt(reason) === 4) {
                        //特殊处理，如果返回4，说明没有登录或登陆失效，直接跳转登陆页面
                        obj.reason(errorCode["ErrorCode4"]);
                        window.top.location.href = "api/auth/login?returnUrl=" + window.encodeURIComponent(window.location.href);
                    } else if (file.nResult === 5) {
                        // modify 222-04-27 liuchunli #29773没有上传权限时，拖拽上传文件的提示语不一致
                        obj.reason(lang.ErrorCode5Create);
                    }
                    else {
                        if (uploader.getFiles("queued", "interrupt").length > 0) {
                            upload();
                        }

                    }
                });
                //不管成功或者失败，文件上传完成时触发
                uploader.on('uploadComplete', function (file) {
                    updateMiniInfo();
                });
                //上传成功时触发
                uploader.on('uploadSuccess', function (file) {
                    if (file.nResult == 0) {
                        //modify: 2022-11-10 by shuanbao.wei [QC:18454]web组件预览跨域
                        if (!isCrossDomain && window.parent.activeUploaderHost) {
                            window.parent.activeUploaderHost.isDrawRecord = true;
                            if (window.parent.activeUploaderHost._filesArray == undefined) {
                                window.parent.activeUploaderHost._filesArray = [];
                            }
                            //关闭时需要再绘制一次记录
                            if (lastBatch.fileIds.length > 0) {
                                window.parent.activeUploaderHost._filesArray.push({
                                    name: file.name,
                                    size: file.size,
                                    id: lastBatch.fileIds[lastBatch.fileIds.length - 1],
                                    fileType: 2,
                                    parentId: lastBatch.folderId,
                                    ext: file.ext
                                });
                            }
                        }
                    }
                });
                //上传确认事件
                uploader.on('uploadAccept', function (obj, ret, cb) {

                    var file = findFile(obj.file.id); // obj.file是上传组件的file，findFile(obj.file.id)是obj
                    if (!file) {
                        return false;
                    }

                    if (ret && ret.status) {
                        //如果状态是Error，需要处理
                        if (ret.status === "Error") {
                            //if (cb) cb(ret.message);
                            if (cb) {
                                if (ret.errorCode === 4302) {
                                    cb(ret.message);
                                } else {
                                    cb(ret.errorCode);
                                }
                            }
                            return false;
                        } else {
                            /*
                            //"tag":"605|958|电子文件元数据标准设计框架研究.doc|2601|false"
                            var split = ret.tag.split('|');
                            if (split.length < 0) {
                                return false;
                            }*/
                            var fileId = file.fileId();
                            //if (split.length >= 5 && split[4] == "true") { // 是秒传
                            if (ret.tag === "true") { // reg.tag不在返回其他数据，只有"true"or"false"值，判断是否秒传
                                uploader.skipFile(obj.file);   // 剩下的块不再传输到后台
                                file.secondPass = true;
                                file.statusText("secondPass"); // 更改标识为秒传，用于显示“秒传”文字和“已完成”的icon
                                file.percent(100);             // 前端进度进百分百
                            }

                            if (file.secondPass || ret.status === "End") { // 秒传 或 正常完成
                                //如果最后一批文件的folderId 不等，把最新的赋值
                                if (lastBatch.folderId != obj.file.folderId) {
                                    lastBatch.folderId = obj.file.folderId;
                                    lastBatch.fileIds = [fileId];
                                } else {
                                    //如果相等，把fileIds加入新id
                                    lastBatchPush(fileId);
                                }

                                return true;
                            }
                        }

                        if (file.statusText && file.statusText() === 'pausing') {
                            //暂停修改为，传输完当前块再暂停
                            uploader.stop(true);
                            updateMiniInfo();
                        }

                        return true;

                    } else {
                        //没有返回正确信息
                        return false;
                    }
                });
                //开始上传前触发事件
                uploader.on('uploadBeforeSend', function (block, data) {
                    // block为分块数据。
                    //alert("uploadBeforeSend");
                    // file为分块对应的file对象。
                    var file = block.file;
                    var obj = findFile(file.id);
                    if (!obj) return;
                    if (obj.approved() === false) {
                        // 不通过
                        data.approved = false;
                        data.reason = file.reason;
                    }

                    // 修改data可以控制发送哪些携带数据。
                    //data.FILE_INFO = $.format("{0}\1{1}\1{2}\1{3}\1{4}\1{5}", obj.folderId(), obj.name(),
                    //    "", //file_remark,
                    //    masterFileId, //masterFileId,
                    //    "0", //attachType,
                    //    strategy);
                    data.fileName = obj.name();
                    //data.hash = buildHash(obj.folderId(), file);
                    data.regionHash = obj.regionHash();
                    data.regionId = obj.regionId();
                    data.uploadId = obj.uploadId(); // 进度条缓存参数
                    data.chunkSize = uploader.options.chunkSize;
                    data.blockSize = block.blob.size;
                    //data.fullPath = file.source.fullPath || "";
                    if (file.fileMd5) {
                        data.fileMd5 = file.fileMd5;
                    }

                    // 删除其他数据
                    // delete data.key;
                });
                //接受所有事件
                uploader.on('all', function (type) {
                    //  //test point 是否有完成时类型
                    //根据事件来改变上传状态  没那么难  
                    switch (type) {
                        case 'ready':
                            state('ready');
                            break;
                        case 'uploadFinished': // 当所有文件上传结束时触发
                            if (uploader.getFiles('error').length > 0) {
                                //Eform也需要此功能注释掉判断
                                //if (isEformEnv) {
                                //    //todo
                                //}
                                //else {
                                state('paused');
                                updateMiniInfo();
                                return;
                                //}
                            }

                            state('finish');
                            if (uploader.getFiles().length === 0) {
                                return;
                            }
                            //关闭时不需要再绘制一次记录
                            //modify: 2022-11-10 by shuanbao.wei [QC:18454]web组件预览跨域
                            if (!isCrossDomain && window.parent.activeUploaderHost) {
                                window.parent.activeUploaderHost.isDrawRecord = false;
                            }
                            if (lastBatch.fileIds.length > 0) {
                                //test point 看能否修改文件列表
                                var files = $.extend({}, lastBatch.fileIds);
                                //$.unique(files.sort());// 去重
                                //var sfiles = uploader.getFiles();
                                //不能取所有的状态文件,只取完成的,其他的顾虑掉的失败的等等不要,防止lastBatch.fileIds,sfiles数量不等,导致filesArray错误,mikahuang,202001
                                //文件上传状态:inited 初始状态,queued 已经进入队列,progress 上传中,complete 上传完成,error 上传出错，可重试,interrupt 上传中断,invalid 文件不合格，cancelled 文件被移除
                                var sfiles = uploader.getFiles("complete");
                                //0字节问题修复,complete状态无法获取,可取progress
                                var _sfiles_progress = uploader.getFiles("progress");
                                if (_sfiles_progress.length) {
                                    sfiles = sfiles.concat(_sfiles_progress);
                                }
                                var filesArray = [];

                                //modify：20220307 by xuying v6缺陷27344 上传列表文件如果均为秒传，上传列表不会自动关闭
                                var mfolderId = 0;
                                if (sfiles.length > 0) {
                                    mfolderId = sfiles[0].folderId;
                                }
                                // modify：2022-01-20 by mengfc v6缺陷25612【表单引擎/上传控件】策略设置“跳过重名文件”后，预览页展示已上传文件列表有问题-start
                                //modify: 20220407 by xuying 缺陷28365 秒传元数据强制必填
                                if (isEformEnv) {
                                    sfiles = sfiles.filter(f => !f.skipped);
                                }
                                $.each(sfiles, function (index, data) {
                                    // modify：2022-01-23 缺陷25612 上传文件后，切换到其他页面，待文件上传完成后，打开页面，上传弹窗不会关闭
                                    if (!sfiles[index]) { return true; }
                                    //modify:20220322 by xuying 缺陷28235 批量上传文件其中包含零字节文件，返回文件id与name不匹配
                                    var obj = findFile(sfiles[index].id);
                                    if (!obj) { return true; };
                                    filesArray.push({
                                        name: sfiles[index].name,
                                        size: sfiles[index].size,
                                        id: obj.fileId(),
                                        fileType: 2,
                                        parentId: lastBatch.folderId,
                                        ext: sfiles[index].ext
                                    });

                                });
                                //modify：2022-01-20 by mengfc v6缺陷25612【表单引擎/上传控件】策略设置“跳过重名文件”后，预览页展示已上传文件列表有问题-end

                                //判断是否有元数据编辑权限及设置密级权限
                                updateMetadataPerm(lastBatch.fileIds[0]);
                                var uploadMessage = $.format(lang.uploadFinishNorm, lastBatch.fileIds.length);
                                if (constInfo.LicMetaData() && !isAttachFile) {
                                    if (/^#doc\/enterprise/.test(window.location.hash)) {
                                        var metaMess = updateMetaPerm ? lang.uploadMetadata : "";
                                        var lvlMess = setLvlPerm ? lang.uploadLvlPerm : "";
                                        uploadMessage = $.format(isOuPublish === true ? lang.uploadFinishNoMetadata : lang.uploadFinish, lastBatch.fileIds.length, metaMess, lvlMess);
                                    }
                                    if (/^#doc\/team/.test(window.location.hash)) {
                                        var lvlMess = setLvlPerm ? lang.uploadLvlPerm : "";
                                        uploadMessage = $.format(isOuPublish === true ? lang.uploadFinishNoMetadata : lang.uploadFinish, lastBatch.fileIds.length, '', lvlMess);
                                    }
                                    if (/^#doc\/person/.test(window.location.hash)) {
                                        var lvlMess = setLvlPerm ? lang.uploadLvlPerm : "";
                                        uploadMessage = $.format(isOuPublish === true ? lang.uploadFinishNoMetadata : lang.uploadFinish, lastBatch.fileIds.length, '', lvlMess);
                                    }
                                }

                                //eform上传回调
                                if (isEformEnv) {
                                    if (isCrossDomain) {
                                        top.postMessage(JSON.stringify(filesArray), '*');
                                    }
                                    else {
                                        if (window.parent.activeUploaderHost && window.parent.activeUploaderHost.onUploadFinish) {
                                            //处理多次上传(如移除文件等操作会触发二次上传),会重复触发onUploadFinish事件导致数据重复问题,mikahuang,202001
                                            var _filesArray = [];
                                            window.__uploadFinishFiles = window.__uploadFinishFiles || {};
                                            $.each(filesArray, function (index, item) {
                                                if (!window.__uploadFinishFiles[item.fileType + "_" + item.id]) {
                                                    _filesArray.push(item);
                                                }
                                            });
                                            if (_filesArray.length) {
                                                window.parent.activeUploaderHost.uploadMessage = uploadMessage;
                                                parent.window.activeUploaderHost.onUploadFinish(_filesArray);
                                                //onUploadFinish事件后记录已经上传完成的文件ID
                                                $.each(_filesArray, function (index, item) {
                                                    window.__uploadFinishFiles[item.fileType + "_" + item.id] = 1;
                                                });
                                            }
                                        }
                                    }
                                } else {

                                    //modify：20220307 by xuying v6缺陷27344 上传列表文件如果均为秒传，上传列表不会自动关闭
                                    //判断上传成功后判断是否启用元数据强制应用策略
                                    getFldMetaTypes(mfolderId);
                                    ko.$store.dispatch('setUploadFiles', filesArray);
                                    if (isPrevInherit) {
                                        //得到所有文件的Id
                                        var filesIds = lastBatch.fileIds.join();
                                        isMetadataMaintenance(filesIds);
                                        if (updateMetaPerm) {
                                            app.trigger("component:show", {
                                                moduleId: "doc/metadata/index",
                                                message: message,
                                                datas: filesArray
                                            });
                                        } else {
                                            app.trigger("message:show", {
                                                type: 'success',
                                                message: uploadMessage,
                                                delay: 4000, handler: function (Ismetadata) {
                                                    if (Ismetadata) {
                                                        app.trigger('component:show', {
                                                            moduleId: "doc/metadata/index",
                                                            message: message,
                                                            datas: filesArray
                                                        });
                                                    } else {
                                                        app.trigger('component:show', {
                                                            moduleId: "doc/informationbar/basicinfo/singlefile/batchmoficationlevel/index",
                                                            message: message,
                                                            datas: filesArray
                                                        });
                                                    }
                                                }
                                            });
                                        }
                                    }
                                    else {
                                        app.trigger("message:show", {
                                            type: 'success',
                                            message: uploadMessage,
                                            delay: 4000, handler: function (Ismetadata) {
                                                if (Ismetadata) {
                                                    app.trigger('component:show', {
                                                        moduleId: "doc/metadata/index",
                                                        message: message,
                                                        datas: filesArray
                                                    });
                                                } else {
                                                    app.trigger('component:show', {
                                                        moduleId: "doc/informationbar/basicinfo/singlefile/batchmoficationlevel/index",
                                                        message: message,
                                                        datas: filesArray
                                                    });
                                                }
                                            }
                                        });
                                    }

                                }
                            } else {
                                //eform上传回调
                                if (isEformEnv) {
                                    if (uploader.getFiles("error").length == 0) {
                                        if (window.parent.activeUploaderHost && window.parent.activeUploaderHost.uploadSkip) {
                                            window.parent.activeUploaderHost.uploadSkip();
                                        }
                                    }

                                }
                            }

                            window.setTimeout(function () {//全部完成重新赋值
                                lastBatch = { folderId: 0, fileIds: [] };
                                if (message === "enterprise:reload") {
                                    setTimeout(function () { app.trigger('enterprise:reload', { force: true, needResetViewType: false, needSavePageNum: true, customSort: true }); }, 10);
                                }
                                else if (message === "person:reload") {
                                    setTimeout(function () { app.trigger('person:reload', { force: true, needResetViewType: false, needSavePageNum: true, customSort: true }); }, 10);
                                }
                                else if (message === "enterpriselist:reload") {
                                    setTimeout(function () { app.trigger('enterprise:reload', { force: true, needResetViewType: false }); }, 10);
                                }
                                else if (message === "personlist:reload") {
                                    setTimeout(function () { app.trigger('person:reload', { force: true, needResetViewType: false, needSavePageNum: true }); }, 10);
                                }
                                else if (message === "team:load" || message === "teamlist:reload") {
                                    setTimeout(function () { app.trigger(message, currentFolder()); }, 10);
                                }
                                else if (message === "teamdata:reload") {//团队列表右键上传时
                                    setTimeout(function () { message && app.trigger(message, { id: currentFolder().id() }); }, 10);
                                }
                                else if (message === "outpublish:reload") {
                                    setTimeout(function () { app.trigger('outpublish:reload'); }, 10);
                                }
                                else {
                                    setTimeout(function () { message && app.trigger(message); }, 10);
                                }
                                //}, 4000);//4秒太久了，改为1秒 cq.taiqi 2017年12月22日 11:22:30
                            }, 1000);

                            if (window.location.pathname === '/outpublish.html') {
                                hideWindow();
                            }
                            setTimeout(function () {
                                foldersPerm = undefined; // 重置文件夹权限
                                if (isEformEnv) {
                                    //eform环境会变成空白窗体
                                    //hideWindow();
                                }
                                else {
                                    hideWindow();
                                }
                                if (runtime() !== 'html5') {
                                    setTimeout(function () {
                                        init({ forceInit: true });
                                    }, 500);
                                }
                            }, 1200);
                            updateMiniInfo();
                            break;
                        case 'startUpload':
                            state('uploading');
                            break;
                        case 'stopUpload':
                            state('paused');
                            break;
                    }
                });

                //html5 运行时
                if (runtime() === 'html5') {
                    //全局拖拽
                    var $dropOverlay = jquery('<div id="dropOverlay"><h1>' + lang.dndGlobal + '</h1></div>').appendTo(document.body);
                    var dragenter = function (e) {
                        //解决vdrive端禁止拖拽
                        if (constInfo.parseQueryString().IsVDriver) {
                            e.preventDefault();
                            e.stopPropagation();
                            return;
                        }
                        //modify: 2017-11-22 haibin.long 判断拖入的是否为文件
                        var dTrans = e.originalEvent.dataTransfer;
                        var types = dTrans.types;
                        var fileCount = 0;
                        $.each(types, function (index, item) {
                            if (item.toLowerCase() === 'files') {
                                fileCount++;
                            }
                        });
                        if (fileCount === 0) { //modify: 2018-02-26 haibin.long 处理safari拖拽上传失效的Bug #5906
                            return;
                        }
                        e.preventDefault();
                        e.stopPropagation();
                        //快速导航不处理
                        if (/\/quick/.test(window.location.hash)) {
                            return;
                        }
                        if (/^#system/.test(window.location.hash)) {
                            return;
                        }
                        //团队根目录不触发文件拖拽上传.modify:2018-10-8 min.tian
                        if (/^#doc\/team/.test(window.location.hash) && currentFolder().id() === 7) {
                            return;
                        }
                        //如果上传窗口打开了，不处理外部的拖拽了
                        if (!showWindow()) return;
                        $dropOverlay.css("display", "block");
                    };
                    var dragover = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        //如果上传窗口打开了，不处理外部的拖拽了
                        if (!showWindow()) return;
                    };
                    var dragleave = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        //如果上传窗口打开了，不处理外部的拖拽了
                        if (!showWindow()) return;
                        $dropOverlay.css("display", "none");
                    };
                    var drop = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        //如果上传窗口打开了，不处理外部的拖拽了
                        if (!showWindow()) return;
                        $dropOverlay.css("display", "none");
                        if (runtime() === 'html5') {
                            ///uploader.request('predict-runtime-type');
                            //showWindow(false);
                            //不能直接显示，不然全路经可能为空
                            app.trigger("upload:show", { showWindow: true, localFolderSelect: true });
                            //app.trigger("upload:show", { showWindow: true });
                            if (isProcess) {
                                isProcess = false;
                                return;
                            }
                            //添加文件
                            uploader.request("enter", e);
                            //WebUploader.Runtime.Html5.Dnd._dropHandler(e);
                        }
                    };
                    //鼠标进入时让遮罩隐藏，ie下有bug不触发drop
                    var mouseover = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        $dropOverlay.css('display', 'none');
                    };
                    //清除遮罩
                    var dblclick = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        $dropOverlay.css("display", "none");
                    };
                    //全局拖入
                    jquery(document.body).on({
                        dragenter: dragenter
                    });
                    $dropOverlay.on({
                        dragover: dragover,
                        dragleave: dragleave,
                        drop: drop,
                        dblclick: dblclick
                    });
                }
            }
        };
        //开始上传
        var upload = function () {
            uploader.upload();
        };
        //找到对应的前台对象
        var findFile = function (id) {
            return $.grep(fileList(), function (cur, i) {
                return cur.id() == id;
            })[0];
        };
        var cancelFile = function (obj, uploader) {
            uploader.cancelFile(obj.id());
            //obj.size(lang.skip);// 这为什么是skip
            uploader.retry();
        };
        var skipFile = function (obj, uploader, file) {
            obj.statusText("cancelled");
            uploader.skipFile(file);
            uploader.upload();
        }
        var showWindow610 = function (obj, uploader, file) {
            renameData(null);
            renameData({
                fileName: obj.name(),
                folderId: obj.folderId(),
                folderName: obj.folderName(),
                attachmentRename: obj.folderId() === 3,//是否是附件文件夹
                callback: function (param) {
                    //只有重命名不能应用策略，先处理
                    if (param.processStrategy === "rename") {
                        obj.name(param.newFileName);
                    } else {
                        //判断是否应用策略
                        if (param.alwaysApplyCurStrategy) {
                            renameStrategy[obj.folderId()] = param.processStrategy;
                        } else {
                            obj.strategy(param.processStrategy);
                        }
                    }

                    // 如果选择跳过，就直接取消文件
                    if (param.processStrategy === "skip") {
                        skipFile(obj, uploader, file);
                    } else {
                        if (file.size == 0) {
                            obj.statusText("");
                            obj.reason("");
                        }
                        uploader.retry(file);
                    }
                }
            });
            //加载冲突窗口
            renameComponent(null);
            showWindow(false);
            renameComponent("doc/versionManagement/renamed/index");
        };
        var strategy610 = function (obj, uploader, file) {
            var strategy = "";
            if (obj.strategy && obj.strategy()) {
                //单独处理过重名策略
                strategy = obj.strategy();
            }
            if (!strategy && renameStrategy[obj.folderId()]) {
                //应用处理过重名策略
                strategy = renameStrategy[obj.folderId()];
            }
            if (isEformEnv) {
                strategy = currentFolder().strategy();
                if (strategy) {
                    if (strategy === "skip") {// 如果选择跳过，就直接取消文件
                        //cancelFile(obj, uploader);
                        skipFile(obj, uploader, file);
                    }
                    else {
                        if (strategy === "rename") { //只有重命名不能应用策略，先处理
                            obj.name(obj.name().substring(0, obj.name().lastIndexOf(".")) + new Date().valueOf() + obj.name().substring(obj.name().lastIndexOf(".")));
                            //不需要设置策略
                            //obj.strategy(strategy);
                        } else { // 更新策略
                            renameStrategy[obj.folderId()] = strategy;//设置策略
                            obj.strategy(strategy);
                        }
                        if (file.size == 0) {
                            obj.statusText("");
                            obj.reason("");
                        }
                        if (file.size > 0)
                            uploader.retry(file);
                        else {
                            setTimeout(function () {
                                uploader.retry(file);
                            }, 1);
                        }
                    }
                }
                else {//如果策略为空 取消文件
                    cancelFile(obj, uploader);
                }
                //uploader.retry();
            }
            //跳过的话，不再弹出窗口
            else if (strategy === "skip") {
                skipFile(obj, uploader, file);
            }
            else if (strategy) {//liuning.yao  检测策略是否存在
                if (file.size == 0) {
                    obj.statusText("");
                    obj.reason("");
                }
                if (file.size > 0)
                    uploader.retry(file);
                else {
                    setTimeout(function () {
                        uploader.retry(file);
                    }, 1);
                }
            } else {
                showWindow610(obj, uploader, file);
            }
        };
        var module;
        var selectData;
        var message;
        function removeFile(data) {
            var strategy = "";
            if (data.strategy && data.strategy()) {
                //单独处理过重名策略
                strategy = data.strategy();
            }
            if (!strategy && renameStrategy[data.folderId()]) {
                //应用处理过重名策略
                strategy = renameStrategy[data.folderId()];
            }

            var delFileId = data.fileId();
            if (delFileId > 0) {
                //只对上传进行处理
                if (!strategy) {
                    $.ajax({
                        data: {
                            module: "WebClient",
                            fun: "UndoCreateFileByFileId",
                            fileId: delFileId
                        }
                    }).done(function (data) {
                        if (uploader.getFiles("queued", "interrupt").length >= 0) {
                            upload();
                        }
                    });
                } else {
                    // 更新                     
                    var updateFileVerId = data.updateFileVerId();
                    $.ajax({
                        data: {
                            module: "WebClient",
                            fun: "UndoCreateFileVer",
                            fileId: delFileId,
                            fileVerId: updateFileVerId
                        }
                    }).done(function (data) {
                    });
                }
            }
            var statusText = data.statusText();
            var _uploadFileId = data.id();
            if (_uploadFileId) {
                var file = uploader.getFile(_uploadFileId);
                if (file && file.nResult == 610 || (statusText != "queued" && statusText != "interrupt")) {  // 如果是重名而暂停，删除后，后面的文件继续
                    if (uploader.getFiles("queued", "interrupt").length >= 0) {
                        upload();
                    }
                }
            }

            uploader.removeFile(data.id(), true);
            fileList.remove(data);
        }
        function getFldMetaTypes(folderId) {
            $.ajax({
                data: {
                    token: $.cookie('token'), //constInfo.userInfo.token,
                    module: "MetaDataManager",
                    fun: "GetFldMetaTypesAndAttr",
                    fldId: folderId,
                    objType: 4,//4 文件策略 9文件夹策略
                    isFillMetaAttr: true
                },
                async: false
            }).done(function (data) {
                if (data) {
                    //强制应用
                    isPrevInherit = data.isForce;
                }
            });
        }
        //元数据启用强制策略时候，上传文件状态全部改为元数据维护中
        function isMetadataMaintenance(fileIds) {
            $.ajax({
                data: {
                    module: "WebClient",
                    fun: "FilesMetadataMaintenance",
                    fileIds: fileIds,
                    isMetadataMaintenance: true
                }
            }).done(function (data) {
                if (data.result !== 0) {
                    app.trigger("message:show", { type: 'error', message: data.result });
                }
            });
        }
        //判断是否有元数据编辑权限
        function updateMetadataPerm(fileId) {
            $.ajax({
                data: {
                    module: "WebClient",
                    fun: "GetPermByFileId",
                    fileId: fileId,
                },
                async: false
            }).done(function (data) {
                if (data) {
                    updateMetaPerm = data.metadataPerm;
                    setLvlPerm = data.setLvlPerm;
                }
            });
        }
        function retryFile(data) {
            var _obj = uploader.getFile(data.id());
            if (_obj && _obj.doubleCheck) { // 判断该上传file是否有doubleCheck属性来确认是否为0Bytes文件可以重新上传
                data.statusText('queued'); // queued状态:已经进入队列,等待上传
            }
            uploader.retry(data.id());
        }
        function isExistsHeadMd5(headMd5, regionId, fn) {
            $.ajax({
                data: {
                    module: "UploadFileManager",
                    fun: "IsExistsHeadMd5",
                    headMd5: headMd5,
                    regionId: regionId // 区域编号
                }
            }).done(function (data) {
                var exists = data.exists === true;
                fn && fn(exists);
            });
        }
        //根据文件的hash值找到对应的前台对象
        function getFileByHash(hash) {
            return $.grep(fileList(), function (cur, i) {
                return cur.hash() === hash;
            })[0];
        };
        function paused(data) {
            if (data.statusText() === 'interrupt') {
                // uploader.upload(data.id());
                //if (uploader.getFiles('error').length == 0) {
                upload();
                //uploader.upload();
                //uploader.upload(data.id());
                //}

            }
            else if (data.statusText() === 'progress') {
                //开始暂停
                data.statusText('pausing');
                //uploader.stop();
            }
            updateMiniInfo();
        }
        function buildHash(folderId, file) {
            //return constInfo.userInfo.token + "_" + folderId + "_" + fileHash; // 存在重复隐患
            if (file.serverHash) {
                return file.serverHash;
            } else {
                var serverHash = constInfo.getGuid() + constInfo.userInfo.token + "_" + folderId + "_" + file.__hash;
                file.serverHash = serverHash;
                return serverHash;
            }
        }
        function convertReason(reason, fileModel) {
            var val = "";
            if (typeof (reason) === "number") {
                if (reason === 5) {
                    val = fileModel == "UPDATE" ? lang.ErrorCode5Update : lang.ErrorCode5Create;
                } else {
                    val = lang["ErrorCode" + reason];
                }
                if (val === undefined) {
                    val = lang.ErrorCodeOther + reason;
                }
            } else if (reason === "abort") {
                val = lang.uploaderEnd;
            } else
                val = reason;
            return val;
        }
        function md5Verification(uploader, file, regionId) {
            if (constInfo.systemInfo.instanceConfigInfo.SecondPass === false || file.size <= minChunkSize) {
                return;
            }
            // 如果文件上传出错后，又重新上传的，但fileMd5值已经计算完成
            if (file.fileMd5) {
                clog("之前算过：" + file.name + "'s md5 --> " + file.fileMd5);
                var _obj = findFile(file.id);
            } else {

                if (file.size > headMd5Size) { // 大于 1024 * 1024 的文件

                    uploader.md5File(file, 0, headMd5Size).then(function (headMd5) { // 请求后台查询headMd5是否存在

                        isExistsHeadMd5(headMd5, regionId, function (isExists) {

                            if (!isExists) { // 不存在相同headMd5，不秒传
                                return;
                            }

                            // 存在相同headMd5，继续计算全量md5来判断是否秒传
                            uploader.md5File(file).then(function (retMd5) {
                                clog(file.name + "-->" + retMd5);
                                file.fileMd5 = retMd5;
                            });

                        });
                    });

                } else {

                    // 计算全量md5来判断是否秒传
                    uploader.md5File(file).then(function (retMd5) {
                        clog(file.name + "-->" + retMd5);
                        file.md5 = retMd5;
                    });
                }

            }

        }

        var foldersPerm = undefined; // 缓存创建文件夹权限, 所有文件上传完成时清除
        function checkCreateFolderPerm(folderId, fn, async) {
            var havePerm = false;
            if (foldersPerm === undefined || foldersPerm[folderId] === undefined) {
                if (!foldersPerm)
                    foldersPerm = {};
                $.ajax({
                    async: async === undefined ? true : async,
                    data: {
                        module: "FolderOperationManager",
                        fun: "HaveCreateFolderPerm",
                        folderId: folderId
                    }
                }).done(function (data) {
                    havePerm = data.havePerm;
                    foldersPerm[folderId] = data.havePerm;
                    fn && fn(data.havePerm);
                });
            }
            else {
                havePerm = foldersPerm[folderId];
            }
            return havePerm;
        }
        function clog(info) {
            if (window.console) {
                console.log(info);
            }
        }

        function lastBatchPush(fileId) {
            var isExists = false;
            $.each(lastBatch.fileIds, function (index, _fileId) {
                if (_fileId === fileId) {
                    isExists = true;
                    return false;
                }
            });
            if (!isExists)
                lastBatch.fileIds.push(fileId);
        }
        function getUploadUrl(uploadRule) {
            var token = $.cookie("token");
            if (!token) {
                token = getTokenByCondition();
            }
            var regionUrl = uploadRule.RegionUrl;
            var storagePlatform = uploadRule.StoragePlatform;
            if (!regionUrl) {
                if (uploadRule.IsSupportMultiTd) {
                    return "/document/uploadMultiTd" + (constInfo.isPubilsh ? ("?code=" + constInfo.code) : "?token=" + token);
                }
                else {
                    return constInfo.systemInfo.uploadUrl + (constInfo.isPubilsh ? ("?code=" + constInfo.code) : "?token=" + token);
                }
            }
            else {
                regionUrl = regionUrl.lastIndexOf("/") === regionUrl.length - 1 ? regionUrl : regionUrl + "/";
                if (uploadRule.IsSupportMultiTd) {
                    return regionUrl + "document/uploadMultiTd" + (constInfo.isPubilsh ? ("?code=" + constInfo.code) : "?token=" + token);
                }
                else {
                    return regionUrl + "document/upload" + (constInfo.isPubilsh ? ("?code=" + constInfo.code) : "?token=" + token);
                }
            }
        }
        //Accept 格式整理
        //isAllow是否为允许上传
        function changeAccept(types, isAllow) {
            var accept = {}
            var extensions = "";
            var mimeTypes = ""
            for (var index in types) {
                var fileType = types[index].fileType.replace(".", '');
                if (fileType == "") {
                    break;
                }
                //统一大小写
                extensions = extensions + "," + fileType.toLowerCase();
                //常用mimeType格式转换
                var nMimeType = changeMimeType(fileType);
                if (nMimeType != "") {
                    mimeTypes = mimeTypes + "," + nMimeType;
                }
            }
            if (extensions == "") {
                return accept;
            }
            if (isAllow) {
                if (mimeTypes != "") {
                    mimeTypes = mimeTypes.substr(1);
                }
                accept = {
                    //加入队列 这里不做限制
                    extensionsButQueued: extensions + ",",
                    mimeTypes: mimeTypes
                }
            } else {
                //禁止上传的类型
                accept = {
                    //加入队列 这里不做限制
                    extensionsButQueued: extensions + ",",
                }
            }
            return accept;
        }
        //常用mimeType格式转换
        function changeMimeType(fileType) {
            //常用类型兼容ie
            switch (fileType) {
                case "doc":
                    return "application/msword";
                case "docx":
                    return "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
                case "xlsx":
                    return "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
                case "pptx":
                    return "application/vnd.openxmlformats-officedocument.presentationml.presentation";
                case "ppsx":
                    return "application/vnd.openxmlformats-officedocument.presentationml.slideshow";
                case "pdf":
                    return "application/pdf";
                case "swf":
                    return "application/x-shockwave-flash";
                case "exe":
                    return "application/octet-stream";
                case "tar":
                    return "application/x-tar";
                case "tgz":
                    return "application/x-compressed";
                case "zip":
                    return "application/x-zip-compressed";
                case "z":
                    return "application/x-compress";
                case "wav":
                    return "audio/wav";
                case "wma":
                    return "audio/x-ms-wma";
                case "wmv":
                    return "video/x-ms-wmv";
                case "mp3":
                    return "audio/mpeg";
                case "mid":
                    return "audio/mid";
                case "bmp":
                    return "image/bmp";
                case "gif":
                    return "image/gif";
                case "png":
                    return "image/png";
                case "tif":
                case "tiff":
                    return "image/tiff";
                case "jpeg":
                case "jpg":
                    return "image/jpeg";
                case "txt":
                    return "text/plain";
                case "xml":
                    return "text/xml";
                case "html":
                    return "text/html";
                case "css":
                    return "text/css";
                case "js":
                    return "text/javascript";
                //默认返回 .xxx格式
                default:
                    return "." + fileType;
            }

        }
        // 校验并获取规则
        function checkAndCreateDocInfo(parameter, async, fn) {                                                                                                                 //统一大小写
            if (isEformEnv && uploader.options.accept[0] && uploader.options.accept[0].extensionsButQueued && uploader.options.accept[0].extensionsButQueued.indexOf("," + parameter.ext.toLowerCase() + ",") == -1 && uploader.options.isAllow) {
                fn && fn({
                    availableSizes: 0,
                    result: 611,
                    reason: lang.ErrorCode611,
                });
                return;                                                                                                                                                                         //统一大小写
            } else if (isEformEnv && uploader.options.accept[0] && uploader.options.accept[0].extensionsButQueued && uploader.options.accept[0].extensionsButQueued.indexOf("," + parameter.ext.toLowerCase() + ",") != -1 && !uploader.options.isAllow) {
                fn && fn({
                    availableSizes: 0,
                    result: 611,
                    reason: lang.ErrorCode611,
                });
                return;
            }
            token = $.cookie("token");
            if (token && token !== parameter.token) {
                parameter.token = token;
            }
            $.ajax({
                async: async === undefined ? true : async,
                data: $.extend({
                    module: "RegionDocOperationApi",
                    fun: "CheckAndCreateDocInfo"
                }, parameter)
            }).done(function (result) {
                fn(result);
            });
        }
        return {
            //多语言
            lang: lang,
            //运行时
            runtime: runtime,
            //窗口显示变量
            showWindow: showWindow,
            //messageTip:messageTip,
            //点击显示
            clickShow: function () {
                var currentFolder = this.currentFolder();
                if (currentFolder) {
                    if (!currentFolder.relativePath() && currentFolder.folderType() !== -1) { // -1是回收站
                        constInfo.getFolderRelativePath(currentFolder.path()).done(function (path) {
                            currentFolder.relativePath(path);
                        });
                    }
                }
                showWindow(false);
            },
            //隐藏窗口
            hideWindow: hideWindow,
            //当前对应的文件夹对象
            currentFolder: currentFolder,
            //当前对应的文件夹对象的路径，相对路径
            path: ko.computed(function () {
                if (!currentFolder()) {
                    return "";
                }
                var relativePath = "";
                var _currentFolder = komapping.toJS(currentFolder);

                if (_currentFolder.masterFile) {
                    if (_currentFolder.relativePath) {
                        relativePath = _currentFolder.relativePath.split('\\');
                        relativePath = relativePath[relativePath.length - 1];
                    }
                    return lang.toAttachment + "'" + relativePath + "'" + lang.updateAttachment;
                }
                relativePath = _currentFolder.relativePath;
                //个人内容库的情况
                if (relativePath && ((_currentFolder.namePath && /^PersonalRoot/i.test(_currentFolder.namePath))
                    || _currentFolder.id === window.constInfo.userInfo.topPersonalFolderId //个人内容库根目录点空白，然后点上传
                    || _currentFolder.parentFolderId == window.constInfo.userInfo.topPersonalFolderId)) //个人内容库选中文件夹，然后点上传
                {
                    var arrRelativePath = relativePath.split("\\");
                    arrRelativePath.splice(1, 1);
                    relativePath = arrRelativePath.join("\\");
                }

                return lang.toFolder + relativePath;
            }),
            //上传方法
            upload: upload,
            //组件状态
            state: state,
            //要上传的文件的列表
            fileList: fileList,
            //冲突组件地址变量
            renameComponent: renameComponent,
            //冲突组件参数
            renameData: renameData,
            //后台上传时的显示信息
            miniInfo: miniInfo,
            pngConversionSvg: constInfo.pngConversionSvg,
            binding: function () {
                //绑定方法，动态创建元素
                $uploadContainer = jquery("#uploadContainer");
                //顶部容器
                $pickContainer = jquery('<div class="pickContainer"></div>').appendTo($uploadContainer);
                //目标文件夹路径
                //jquery('<span class="targetpath" data-bind="text:path,attr: {title: path }"></span>').appendTo($pickContainer);
                jquery('<div class="form" style="padding: 0px 20px; word-break: break-all;" data-bind="text:path,attr: {title: path }"></div>').appendTo($pickContainer);
                //开始上传按钮
                jquery('<button type="button" class="form-button start" style="display: none" data-bind="click:upload">start</button>').appendTo($pickContainer);
                //placeholder
                $placeholder = jquery('<div class="placeholder" data-bind="css:{solid:fileList().length>0}" ></div>').appendTo($uploadContainer);
                //$renameComponent
                $renameComponent = jquery('<div style="display:none" data-bind="compose:{model:renameComponent}"></div>').appendTo($uploadContainer);
                var _fileListHtml = '<ul class="filelist uploadModuleFlag">' +
                    '<!--ko foreach:fileList -->' +
                    '<li>' +
                    '<div class="inline-mask" data-bind="style: { width: $data.percent()+ \'%\'},visible:$data.statusText() != \'error\'"></div>' +
                    '<div class="inline-content">' +
                    '<img class="fileicon" data-bind="attr: { src: $root.pngConversionSvg($data.icon()).url}" onerror="this.src=\'external/icon-file-type/32/unknown.svg\'; this.onerror=null;"/>' +
                    '<span class="filename vertical-mid" data-bind="text:$data.fullPath()+$data.name(),attr: {title: $data.fullPath()+$data.name() }"></span>' +
                    '<span class="filesize vertical-mid" data-bind="text:$data.size"></span>' +
                    '<span class="speedinfo vertical-mid" data-bind="text:$data.speedInfo,visible:$data.statusText() == \'progress\'"></span>' +

                    '<span class="status " data-bind="visible:$data.statusText() != \'complete\'&&$data.statusText() != \'error\'&&$data.statusText() != \'pausing\',attr:{class:$data.statusText(),title:$data.reason()},click:$root.paused"></span>' +//右侧状态，正在上传
                    '<span class="status " data-bind="visible:$data.statusText() == \'paused\',attr:{class:$data.statusText(),title:$data.reason()},"></span>' +//右侧状态，开始
                    '<span class="status " data-bind="visible:$data.statusText() == \'uploading\',attr:{class:$data.statusText(),title:$data.reason()},"></span>' +//右侧状态,暂停
                    '<span class="status ic_retry" data-bind="visible:$data.statusText() != \'complete\'&&$data.statusText() == \'error\',attr:{title:$data.reason()},click:$root.retryFile"></span>' +//右侧状态，重传


                    '<span class="file-successinfo " data-bind="text:$data.folderRelativePath(),attr: {title: $data.name },visible:$data.statusText() == \'complete\'"></span>' +
                    '<span class="error" data-bind="text:$data.reason(),attr: {title: $data.reason() },visible:$data.statusText() == \'error\'"></span>' +//项中部，错误
                    '<span class="status_mid vertical-mid" style=\'width:48px\' data-bind="attr: {title: $data.name },text:$root.lang.suspend,visible:$data.statusText() == \'interrupt\'"></span>' +//项中部，已暂停
                    '<span class="status_mid vertical-mid" style=\'width:48px\' data-bind="attr: {title: $data.name },text:$root.lang.uploaded,visible:$data.statusText() == \'complete\'"></span>' +//项中部，已经传
                    '<span class="status_mid vertical-mid" style=\'width:48px\' data-bind="attr: {title: $data.name },text:$root.lang.queued,visible:$data.statusText() == \'queued\'"></span>' +//项中部，等待
                    '<span class="status_mid vertical-mid" style=\'width:48px\' data-bind="attr: {title: $data.name },text:$root.lang.secondPass,visible:$data.statusText() == \'secondPass\'"></span>' +//项中部，秒传
                    '<span class="status_mid vertical-mid" style=\'width:48px\' data-bind="attr: {title: $data.name },text:$root.lang.skip,visible:$data.statusText() == \'cancelled\'"></span>' +//项中部，已跳过
                    '</div>' +
                    '<span class="remove" data-bind="css:{ remove: ($data.statusText() != \'complete\'&&$data.statusText() != \'cancelled\'&&$data.statusText() != \'secondPass\'), success: ($data.statusText() == \'complete\'||$data.statusText() == \'cancelled\'||$data.statusText() == \'secondPass\')},click:$root.removeFile"></span>' +
                    '</li>' +
                    '<!--/ko-->';
                //ie9下flash.exec报错修复
                if ($.browser.msie9) {
                    _fileListHtml += '<li class="dnd-upload-button" data-bind="css:{\'webuploader-element-invisible\':fileList().length>0}"></li>';
                    _fileListHtml += '<li class="dnd-tip" data-bind="visible:runtime() != \'html5\' && fileList().length==0"><h1>' + $.format(lang.dndTip3, _fileNumLimit) + '</h1><p>' + lang.dndTip2 + '</p></li>';
                }
                else {
                    _fileListHtml += '<li class="dnd-upload-button" data-bind="visible:fileList().length==0" ></li>';
                    _fileListHtml += '<li class="dnd-tip" data-bind="visible:runtime() == \'html5\' && fileList().length==0"><h1>' + $.format(lang.dndTip1, _fileNumLimit) + '</h1></li>';
                }
                _fileListHtml += '</ul>';
                //文件列表
                $fileList = jquery(_fileListHtml).appendTo($placeholder);
                //文件选择器
                $filePicker = jquery('<div class="filePicker"></div>').appendTo($placeholder.find(".dnd-upload-button"));
                if (testWebkitdirectory()) {
                    $folderPicker = jquery('<div class="filePicker webuploader-container "></div>').appendTo($placeholder.find(".dnd-upload-button"));

                    $buttonFolder = jquery('<div class="webuploader-pick">' + lang.addFolder + '</div>').appendTo($folderPicker);
                    $inputFolderDiv = jquery('<div style="position: absolute; top: 0px; left: 0px; width: 120px; height: 36px; overflow: hidden; bottom: auto; right: auto;"></div>').appendTo($folderPicker);
                    $inputFolder = jquery('<input type="file" name="file" class="webuploader-element-invisible" style="display:none" webkitdirectory multiple />').appendTo($inputFolderDiv);
                    $labelFolder = jquery('<label style="opacity: 0; width: 100%; height: 100%; display: block; cursor: pointer; background: rgb(255, 255, 255);"></label>').appendTo($inputFolderDiv);

                    $labelFolder.on('click', function () {
                        $inputFolder.trigger('click');
                    });
                    $labelFolder.on('mouseenter mouseleave', function (e) {
                        switch (e.type) {
                            case 'mouseenter':
                                $buttonFolder.addClass('webuploader-pick-hover');
                                break;

                            case 'mouseleave':
                                $buttonFolder.removeClass('webuploader-pick-hover');
                                break;
                        }
                    });
                    $inputFolder.on('change', function (e) {
                        var files = e.target.files;
                        if (files && files.length > 0) {
                            files = $.map(files, function (file) {
                                var arr = file.webkitRelativePath.split('/');
                                if (arr && arr.length > 0) {
                                    arr = arr.slice(0, arr.length - 1);
                                    file.fullPath = "/" + arr.join('/') + "/";
                                }
                                return file;
                            });
                            uploader.addFile(files);
                            $inputFolder.val("");
                        }
                    });
                }
                //后台上传元素，先将元素附加到当前容器，让它和ko绑定，然后后边在把他移动到document.body上
                // $mini = jquery('<div class="mini-upload" data-bind="visible:showWindow()&&(state()==\'uploading\'||state()==\'paused\'),attr: {title: miniInfo },click:clickShow"><img data-bind="attr:{src:\'scripts/lib/webuploader/\'+state()+\'.gif\'}" style="vertical-align: middle;height:14px" /><span data-bind="text:miniInfo" style="vertical-align: middle;line-height: 14px;"></span></div>').appendTo($uploadContainer);
                $mini = jquery('<div class="mini-upload" data-bind="visible:showWindow()&&(state()==\'uploading\'||state()==\'paused\'),attr: {title: miniInfo },click:clickShow"><span data-bind="attr:{class:state()}"></span><span data-bind="text:miniInfo" style="vertical-align: middle;line-height: 14px;"></span></div>').appendTo($uploadContainer);


                $fileList.css("position", "relative");
                $fileList.perfectScrollbar('update').perfectScrollbar();
            },
            attached: function (view, parent) {
                //后台上传元素移动到document.body
                $mini.appendTo(document.body);
                var urlParam = $.getQueryString("moduledata");
                if (urlParam) {
                    var wfData = JSON.parse(urlParam);
                    if (wfData && wfData.datas && wfData.datas.multiple) {
                        uploadMultiple(wfData.datas.multiple);
                    }
                    if (wfData && wfData.datas && wfData.datas.fileNum) {
                        uploadFilesNum(wfData.datas.fileNum);
                    }
                    if (wfData && wfData.datas && wfData.datas.accept) {
                        uploadAccept = wfData.datas.accept;
                    }
                    if (wfData && wfData.isAllow != undefined) {
                        isAllow = wfData.isAllow;
                    }
                    if (wfData && wfData.uploadAccept && wfData.uploadAccept.fileTypes && !wfData.datas.accept) {
                        uploadAccept = changeAccept(wfData.uploadAccept.fileTypes, isAllow);
                    }
                    if (wfData && wfData.datas && wfData.datas.fileControlUnit) {
                        var fileControlUnit = 0;
                        var fileControlSize = wfData.datas.fileSize || 0;
                        if (wfData.datas.fileControlUnit == "KB") {
                            fileControlUnit = 1024;
                        }
                        else if (wfData.datas.fileControlUnit == "MB") {
                            fileControlUnit = 1024 * 1024;
                        }
                        else if (wfData.datas.fileControlUnit == "G") {
                            fileControlUnit = 1024 * 1024 * 1024;
                        }
                        //绘制提示
                        var sizeLimitTipTxt = "6G";
                        if (0 < (fileControlUnit * fileControlSize) && (fileControlUnit * fileControlSize) < fileSingleSizeLimit) {
                            eformEnvSizeLimit = fileControlUnit * fileControlSize;
                            sizeLimitTipTxt = fileControlSize + wfData.datas.fileControlUnit;
                        } else {
                            eformEnvSizeLimit = 6 * 1024 * 1024 * 1024;//6G
                        }
                        fileSingleSizeLimit = 0;//解除ECM不进队列的限制
                        var sizeLimitTip = '<h1>' + $.format(lang.dndTip4, sizeLimitTipTxt) + '</h1>';
                        $fileList.find('li.dnd-tip').append(sizeLimitTip);
                    }

                }
                //初始化
                init();
                $("ul.filelist.uploadModuleFlag").perfectScrollbar('update');
            },
            compositionComplete: function (view, parent) {
                if (/^#system/.test(window.location.hash)) {
                    app.trigger('templatelist:reload', 'Refresh');
                }
                var msg = {
                    module: "update",
                    message: "team"
                };
                app.trigger('upload:init', msg);
                //监控文件夹id改变
                app.on("upload:init").then(function (node) {
                    var selectEntry = node.select[0];
                    //忽略不是文件夹的消息
                    if (constInfo.getObjType(selectEntry) !== "folder") {
                        return;
                    }
                    //相对路径 声明，后边赋值
                    selectEntry.relativePath = "";

                    selectEntry.masterFile = ko.observable();
                    if (selectEntry.data) {
                        selectEntry = $.extend({}, selectEntry, selectEntry.data);
                    }
                    //得到消息中的文件夹对象，并保存下来
                    if (!selectEntry.name) {
                        selectEntry.name = "folder name empty";
                    }
                    var tmp = komapping.fromJS(selectEntry);
                    currentFolder(tmp);
                });

                //监控是否组件显示
                app.on("upload:show").then(function (node) {
                    var serverurl = getUploadUrl({
                        RegionType: 1
                    });
                    if (uploader.options.server !== serverurl) {
                        //uploader.options.server = serverurl;
                        uploader.option('server', serverurl);
                    }
                    if (node.isOutpublish) {
                        isOuPublish = true;
                    }
                    if (node.rootFolderName) {
                        rootFolderName = node.rootFolderName;
                    }
                    var $parent = ko.dataFor(parent);
                    module = $parent.moduleName();
                    selectData = $parent.datas();
                    message = $parent.message();
                    if (module === "favoritelist") {
                        var data = process.IsProcessStrategy(selectData, 1);
                        if (data.IsProcessStrategy) {
                            node.window = false;
                            isProcess = true;
                            app.trigger("message:show", { type: 'error', message: lang.ProcessEnabled });
                            return;
                        }
                    }
                    if (module === "enterprise") {
                        if (process.getProcessStrategy(selectData, "create")) {
                            node.window = false;
                            isProcess = true;
                            return;
                        }
                    }
                    isAttachFile = false;
                    //显示窗口
                    if (node.showWindow) {
                        //如果浏览器支持选择本地文件夹，并且通过参数传递，是需要显示文件夹选择功能
                        if (testWebkitdirectory() && node.localFolderSelect) {
                            $placeholder.find(".webuploader-container").css('display', 'inline-block').css('margin', '0 10px');
                        }
                        else if ($folderPicker) {
                            $folderPicker.css('display', 'none');
                        }
                        //附件
                        if (node.masterFile) {
                            isAttachFile = true;
                            var tempFolder = {};
                            tempFolder.id = 3;//附件文件夹id
                            tempFolder.name = "Attachment";
                            tempFolder.relativePath = "";
                            tempFolder.path = node.masterFile.path().substr(0, node.masterFile.path().length - 1);//去掉最后一个\
                            previousFolder.fromJS(tempFolder);
                            previousFolder().masterFile = ko.observable(node.masterFile);

                            // modify 20220317 by xuying 缺陷26428 附件上传弹窗显示上传路径 
                            $pickContainer.find(".form").html('').append(lang.toAttachment + "'" + node.masterFile.name() + "'" + lang.updateAttachment);

                            //modify: 2027-12-07 haibin.long 处理 预览界面 关联附加 功能没有currentFolder的Bug
                            if (currentFolder()) {
                                //切换文件夹
                                switchFolder(currentFolder(), previousFolder());
                            } else {
                                //currentFolder不存的情况，为预览界面 关联附加 功能触发
                                currentFolder = previousFolder;
                            }
                        }
                        //特定文件夹，被调用时|eform调用时不调用
                        if (!node.isEformEnv && node.folder) {
                            //app.trigger("upload:show", { showWindow: true, folder: { id:1, name:"企业内容库", path:"1" } });//至少有id name path
                            node.folder.relativePath = "";
                            previousFolder.fromJS(node.folder);

                            //切换文件夹
                            switchFolder(currentFolder(), previousFolder());
                        }
                        if (node.isEformEnv) {
                            var _eformFolder = {
                                id: node.folder.id,
                                name: node.folder.name,
                                path: node.folder.path,
                                relativePath: node.folder.relativePath,
                                masterFile: ko.observable(),
                                strategy: node.strategy,
                                operateType: node.operateType,
                                updateFileId: node.updateFileId,
                                namePath: node.folder.namePath, //个人库需要
                                parentFolderId: node.folder.parentFolderId
                            };
                            isEformEnv = node.isEformEnv;
                            isCrossDomain = node.isCrossDomain;
                            currentFolder(komapping.fromJS(_eformFolder));

                        }

                        if (!currentFolder() || !currentFolder().path()) {
                            return;
                        }
                        //将值取反转换成dialog需要的值
                        showWindow(!node.showWindow);
                        //20180906 liqi 流程上传控件显示问题
                        var urlParam = $.getQueryString("moduledata");
                        if (urlParam) {
                            var wfData = JSON.parse(urlParam);
                            if (wfData && wfData.datas && wfData.datas.isEformEnv) {
                                $('.panel.window').addClass('isEformEnvAttachmentList');
                            }
                        }

                        //modify:2017-11-22 haibin.long 更新滚动条
                        $("ul.filelist.uploadModuleFlag").perfectScrollbar('update');
                        //modify: 2018-07-09 haibin.long 对接eform的文档上传控件
                        if ($parent.moduleDataType && ko.unwrap($parent.moduleDataType) === 'custom') {
                            $('#uploadContainer').closest('.window').css({
                                top: 0
                            });
                        }
                        //只有显示窗口的时候才去取路径，减少请求
                        constInfo.getFolderRelativePath(currentFolder().path()).done(function (path) {
                            if (node.masterFile) {
                                currentFolder().relativePath($.format("{0}\\{1}", path, node.masterFile.name()));
                            }
                            else {
                                currentFolder().relativePath(path);
                            }
                        });
                    }
                });

                //	$fileList.perfectScrollbar('update').perfectScrollbar();
            },
            activate: function () {
                $("ul.filelist.uploadModuleFlag").perfectScrollbar('update');
            },
            //删除文件
            removeFile: function (data) {
                removeFile(data);
            },
            //gs:增加暂停ui
            paused: function (data) {
                paused(data);
            },
            //增加重试ui
            retryFile: function (data) {
                retryFile(data);
            }
        };
    });

define('doc/contextmenu/menuinfo',{});
/*
 *whimco创建于2015年6月10日
 *mikahuang更新于2015年6月15日
 描述：菜单配置数据
 command:组件所在的目录的js文件路径，无需后缀
 icon:暂时未使用
 text:菜单或者工具栏显示的文字
 entryType：数组，文件的类型，有file：单文件，folder：单文件夹，virtualfolder：虚拟文件夹，root:根目录，mix：混合类型，files：多文件，folders：多文件夹，form：表单，record：记录
 perm：权限的组合，或关系
 moduleType:支持显示的模块名称，用于标识该组件可显示的模块
 children：子菜单或者子工具栏按钮，二级定义
 */
//full buttons => back,refresh,download,upload,update,extedit,log
define('doc/contextmenu/publishMenuCfg',[],function () {
    return [
        { id: 'back', command: "outpublish/plugins/back/index", icon: "goback.png", text: "返回", lang: { "zhcn": "返回", "zhtw": "返回", "en": "Return", "jp": "戻る" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix"], moduleType: ['outpublish'] },
        { id: 'refresh', command: "outpublish/plugins/refresh/index", icon: "refresh.png", text: "刷新", lang: { "zhcn": "刷新", "zhtw": "重繪", "en": "Refresh", "jp": "リフレッシュ" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix", "docflow", "docflows"], moduleType: ['outpublish'] },
        { id:'download',command: "outpublish/plugins/download/index", icon: "download.png", text: "下载", lang: { "zhcn" : "下载" , "zhtw" : "下載" , "en" : "download" , "jp" : "ダウンロード" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix"], moduleType: ['outpublish'] },
        { id: 'downloadformat', command: "outpublish/plugins/downloadformat/index", icon: "downloadformat", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載" , "en" : "pdf download" , "jp" : "ダウンロード" }, entryType: ["publishroot", "file"], moduleType: ['outpublish'] },
        { id: 'upload', command: "component/upload/index", icon: "upload.png", text: "上传", lang: { "zhcn" : "上传" , "zhtw" : "上傳" , "en" : "Upload" , "jp" : "アップロード" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix"], moduleType: ['outpublish'] },
        { id: 'update',command: "doc/updatefile/index", icon: "update.png", text: "更新", lang: { "zhcn" : "更新" , "zhtw" : "更新" , "en" : "Update" , "jp" : "変更" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix"], moduleType: ['outpublish'] },
        //{ id: 'extedit', command: "doc/extedit/extedit", icon: "extedit.png", text: "扩展编辑", lang: { "zhcn" : "扩展编辑" , "zhtw" : "擴展編輯" , "en" : "Extended editor" , "jp" : "拡張編集" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix"], moduleType: ['outpublish'] },
        { id: 'log', command: "outpublish/plugins/outpublishlog/index", icon: "log.png", text: "外部日志", lang: { "zhcn": "外部日志", "zhtw": "外部日誌", "en": "External log", "jp": "外部ログ" }, entryType: ["publishroot", "file", "files", "folder", "folders", "subfolder", "mix", "docflow", "docflows"], moduleType: ['outpublish'] }
        
    ];
});

define('doc/contextmenu/menusort',{});
define('doc/contextmenu/menutemplateconfig',["./permtype"], function (permType) {
    return [
      //文件模板刷新按钮
      { command: "doc/refresh/index", icon: "refresh", text: "刷新", lang: { "zhcn" : "刷新" , "zhtw" : "重繪" , "en" : "Refresh" , "jp" : "リフレッシュ","ja":"リフレッシュ" }, entryType: ["folder","file", "files", "templateroot"], moduleType: ["template"] },
      { command: "component/upload/index", icon: "upload", text: "上传", lang: { "zhcn" : "上传" , "zhtw" : "上傳" , "en" : "upload" , "jp" : "アップロード" }, entryType: ["folder","teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', /*"folder",*/ "record", "record", "root", "templateroot", "mix"], perm:[permType.createFile], moduleType: ['template'] },
      { command: "doc/delete/index", icon: "delete", text: "删除", lang: { "zhcn" : "删除" , "zhtw" : "删除" , "en" : "delete" , "jp" : "削除" }, entryType: ["folder", "vitualfolder", "file", "files", "folders", "mix", "link", "links", "form", "record", "forms", "records"],  perm:[permType.deleteFile],  moduleType: ['template'] },
        {
            command: "NewMenu", icon: "create", text: "新建", lang: { "zhcn" : "新建" , "zhtw" : "新建" , "en" : "Newly build" , "jp" : "新築" }, entryType: ["teamcreator", "templateroot","teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permType.createFolder],  moduleType: ['template'], children: [
                { command: "doc/folder/create/index", icon: "folder16", text: "新建文件夹", lang:{ "zhcn" : "新建文件夹" , "zhtw" : "新建資料夾" , "en" : "File(s)" , "jp" : "新規フォルダ" }, entryType: ["teamcreator", "teamadmin","teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permType.createFolder], moduleType: ['template'] },
                { command: "doc/folder/createoffice/index", icon: "docx32", text: "Word文档", lang:{ "zhcn" : "Word文档" , "zhtw" : "Word檔案" , "en" : "Folder" , "jp" : "Wordドキュメント" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permType.createFolder], moduleType: ['template'] },
                { command: "doc/folder/createoffice/index", icon: "ppt", text: "PowerPoint文档", lang:{ "zhcn" : "PowerPoint文档" , "zhtw" : "PowerPoint檔案" , "en" : "PPT Document" , "jp" : "PowerPointファイル" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permType.createFolder], moduleType: ['template'] },
                { command: "doc/folder/createoffice/index", icon: "excel", text: "Excel文档", lang:{ "zhcn" : "Excel文档" , "zhtw" : "Excel檔案" , "en" : "Excel Document" , "jp" : "Excelドキュメント" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permType.createFolder], moduleType: ['template'] },
                //{ command: "doc/folder/createoffice/index", icon: "note", text: "新建笔记", entryType: ['publicroot', 'personroot', "folder"], perm: [permType.createFolder], moduleType: ['enterprise', 'person', 'team', 'teamfirsttool'] },
                // modify 2022-04-01 zhengjie 27033【产品】多语言-日语-新建-从模板新建-折行样式优化 jp优化
                { command: "doc/filetemplate/index", icon: "templet", text: "从模板新建", lang: { "zhcn": "从模板新建文档", "zhtw": "從範本新建檔案", "en": "Create from templates" , "jp" : "テンプレートから新規作成" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permType.createFolder], moduleType: ['teamfolder','template'] }]
        },
      /*{ command: "doc/extedit/extedit", icon: "extedit", text: "扩展编辑", entryType: ["file"], perm: [permType.updateFileVersion], moduleType: [ 'template'] },*/
      // modify 2022-03-28 liuchunli #28484【UI】文档模板中的“属性”图标与企业库中的图标不一致
      { command: "doc/folder/edit/index", icon: "attribute",text: "属性", lang: { "zhcn" : "属性" , "zhtw" : "内容" , "en" : "attribute" , "jp" : "属性" }, entryType: ["folder", "virtualfolder", "record", "templateroot"], moduleType: ['template'] },
      { command: "doc/updatefile/index", icon: "update", text: "更新", lang: { "zhcn" : "更新" , "zhtw" : "更新" , "en" : "Update" , "jp" : "更新" }, entryType: ["file"], perm: [permType.updateFileVersion], moduleType: ['template'] },
      {command: "component/permission/index", icon: "authorize", text: "分配权限", lang: { "zhcn" : "分配权限" , "zhtw" : "分配許可權" , "en" : "Allocation permissions" , "jp" : "権限を割り当てる" }, entryType: ['publicroot', 'personroot', "folder", /*"file",*/ "link", "form", "record", "root", "templateroot"], perm:[permType.admin], moduleType: ['template'] },
        { command: "doc/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "download", "jp": "ダウンロード" }, entryType: [/*"folder",*/ "file", "files", "folders", "mix", "shareme"], perm: [permType.download], moduleType: ['template'] },
        { command: "doc/downloadformat/index", icon: "downloadformat", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載", "en": "pdf download", "jp": "ダウンロード" }, entryType: ["file"], perm: [permType.download], moduleType: ['template'] },
     //{ command: "outpublish/outpublishlist/index", icon: "edit", text: "预览", lang: { "zhcn" : "预览" , "zhtw" : "預覽" , "en" : "preview" , "jp" : "プレビュー" }, entryType: ["file"], perm: [], moduleType: ['template'] },
      //{ command: "doc/movecopy/index", icon: "movecopy", text: "复制移动", lang: { "zhcn" : "复制移动" , "zhtw" : "複製移動" , "en" : "Replicating movement" , "jp" : "複製移動" }, entryType: ["folder", "virtualfolder", "file", "files", "folders", "mix", "link", "links"], perm: [permType.deleteFile, permType.download], moduleType: ['template', 'update'] },
      //bug 6398{ command: "system/doc/namecreaterule/index", icon: "edit", text: "生成规则", lang: { "zhcn" : "生成规则" , "zhtw" : "生成規則" , "en" : "Rule of generation" , "jp" : "生成規則" }, entryType: ["file"], moduleType: ['template'] },
      //{ command: "doc/unfavorite/index", icon: "unfavorite", text: "取消收藏", lang: { "zhcn" : "取消收藏" , "zhtw" : "取消收藏" , "en" : "Cancel the collection" , "jp" : "コレクションの取り消し" }, entryType: ["folder", "file", "files", "folders", "mix", "link", "links", "root"], moduleType: ['template'] },

    ]
});
/*
 *whimco创建于2015年6月10日
 *mikahuang更新于2015年6月15日
 描述：菜单配置数据
 command:组件所在的目录的js文件路径，无需后缀
 icon:暂时未使用
 text:菜单或者工具栏显示的文字
 entryType：数组，文件的类型，有file：单文件，folder：单文件夹，virtualfolder：虚拟文件夹，root:根目录，mix：混合类型，files：多文件，folders：多文件夹，form：表单，record：记录
 perm：权限的组合，或关系
 moduleType:支持显示的模块名称，用于标识该组件可显示的模块
 children：子菜单或者子工具栏按钮，二级定义
 */
define('doc/contextmenu/previewMenuCfg',[],function () {
    return [
        { command: "doc/informationbar/comment/quickCreate", icon: "annotation", text: "评论&批注", lang: { "zhcn": "评论&批注", "zhtw": "評論&批註", "en": "Comments", "jp": "コメント" }, entryType: ["file"], perm: [1 << 20], moduleType: ['preview'] },
        //modify chedegen 2023.3.31 去掉扩展编辑
        //{ command: "doc/extedit/extedit", icon: "extedit", text: "扩展编辑", lang: { "zhcn": "扩展编辑", "zhtw": "擴展編輯", "en": "Extended Edit", "jp": "擴展編輯" }, entryType: ["file"], perm: [512], moduleType: ['preview'] },
        { command: "doc/onlineedit/index", icon: "onlineEdit", text: "在线编辑", lang: { "zhcn": "在线编辑", "zhtw": "在線編輯", "en": "Online Editing", "jp": "オンライン編集" }, entryType: ["file"], moduleType: ['preview'] },
        { command: "doc/onlineview/index", icon: "savingupdate", text: "退出编辑", lang: { "zhcn": "退出编辑", "zhtw": "退出編輯", "en": "Exit the editor", "jp": "編集から抜ける" }, entryType: ["file"], moduleType: ['preview'] },
        { command: "doc/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "Download", "jp": "ダウンロード" }, entryType: ["file"], perm: [32], moduleType: ['preview'] },
        { command: "doc/downloadformat/index", icon: "downloadformat", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載", "en": "PDF Download", "ja": "レプリカのダウンロード" }, entryType: ["file",], perm: [32], moduleType: ['preview'] },
        { command: "outpublish/plugins/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "Download", "jp": "ダウンロード" }, entryType: ["file"], perm: [32], moduleType: ['preview'] },
        { command: "doc/localOpen/index", icon: "localopen", text: "本地打开", lang: { "zhcn": "本地打开", "zhtw": "本地打開", "en": "Local open", "jp": "ローカルを開く" }, entryType: ["file"], moduleType: ['preview']},
        { command: "doc/localEdit/index", icon: "localEdit", text: "本地编辑", lang: { "zhcn": "本地编辑", "zhtw": "本地編輯", "en": "Local Edit", "jp": "ローカル編集" }, entryType: ["file"], moduleType: ['preview'] },
        { command: "doc/print/index", icon: "print", text: "打印", lang: { "zhcn": "打印", "zhtw": "打印", "en": "Print", "jp": "印刷" }, entryType: ["file"], perm: [16], moduleType: ['preview'] },
        //{ command: "doc/extedit/extedit", icon: "extedit", text: "编辑", lang: { "zhcn": "编辑", "zhtw": "編輯", "en": "editor", "jp": "編集" }, entryType: ["file"], perm: [512],  moduleType: ['preview'] },
        { command: "doc/favorite/index", icon: "favorite", text: "收藏", lang: { "zhcn" : "收藏" , "zhtw" : "收藏" , "en" : "Favorites" , "jp" : "お気に入り" }, entryType: ["file"], moduleType: ['preview'] },
        { command: "doc/unfavorite/index", icon: "unfavorite.png", text: "取消收藏", lang: { "zhcn": "取消收藏", "zhtw": "取消收藏", "en": "Remove" , "jp" : "コレクションの取り消し" }, entryType: ["file"], moduleType: ['preview'] },
        { command: "doc/publish/index", icon: "public.png", text: "外发", lang: { "zhcn" : "外发" , "zhtw" : "外發" , "en" : "External Share" , "jp" : "外部送信" }, entryType: ["file"], perm: [32768],moduleType: ['preview'] },
        { command: "doc/share/index", icon: "share.png", text: "共享", lang: { "zhcn" : "共享" , "zhtw" : "共亯" , "en" : "Share" , "jp" : "共有" }, entryType: ["file"], perm: [8388608],moduleType: ['preview'] },
        { command: "doc/refreshforsearch/index", icon: "refresh.png", text: "刷新", lang: { "zhcn" : "刷新" , "zhtw" : "重繪" , "en" : "Refresh" , "jp" : "ダウンロード" },entryType:["searchroot"],moduleType: ['enterprise']},
        { command: "doc/refreshforsearch/index", icon: "refresh.png", text: "刷新", lang: { "zhcn" : "刷新" , "zhtw" : "重繪" , "en" : "Refresh" , "jp" : "ダウンロード" },entryType:["searchroot"],moduleType: ['person']},
        { command: "doc/refreshforsearch/index", icon: "refresh.png", text: "刷新", lang: { "zhcn" : "刷新" , "zhtw" : "重繪" , "en" : "Refresh" , "jp" : "ダウンロード" },entryType:["searchroot"],moduleType: ['teamfolder','team']},
        { command: "doc/localLocation/index", icon: "localLocation", text: "本地定位", lang: { "zhcn": "本地定位", "zhtw": "本地定位", "en": "Local Locate", "jp": "ローカル位置" }, entryType: ["file"], moduleType: ['preview'] },
        { command: "doc/previewlog/index", icon: "log", text: "操作日志", lang: { "zhcn": "操作日志", "zhtw": "操作日誌", "en": "Operation log", "jp": "操作ログ" }, entryType: ["file"], moduleType: ['preview'] }
    ];
});

/*
 *lidengbo创建于2018年7月12日
 描述：智能标签/推荐菜单配置数据
 command:组件所在的目录的js文件路径，无需后缀
 icon:暂时未使用
 text:菜单或者工具栏显示的文字
 entryType：数组，文件的类型，有file：单文件，folder：单文件夹，virtualfolder：虚拟文件夹，root:根目录，mix：混合类型，files：多文件，folders：多文件夹，form：表单，record：记录
 perm：权限的组合，或关系
 moduleType:支持显示的模块名称，用于标识该组件可显示的模块
 children：子菜单或者子工具栏按钮，二级定义
 */
define('doc/contextmenu/intagMenuCfg',[],function () {
    return {
        intag: [
            { command: "doc/refresh/index", icon: "refresh.png", text: "刷新", lang: { "zhcn": "刷新", "zhtw": "重繪", "en": "Refresh", "ja": "リフレッシュ" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            //{ command: "doc/intag/addfile/index", icon: "create.png", text: "添加文件", lang: { zhcn: "添加文件", zhtw: "添加檔案", en: "Add a file", jp: "追加ファイル" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            { command: "doc/intag/rename/index", icon: "rename.png", text: "重命名标签", lang: { "zhcn": "重命名标签", "zhtw": "重命名標籤", "en": "rename tag", "ja": "名づけをする" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            //{ command: "doc/intag/edit/index", icon: "edit.png", text: "添加文件", lang: { "zhcn": "添加文件", "zhtw": "添加檔案", "en": "Add a file", "jp": "追加ファイル" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            { command: "doc/intag/removetag/index", icon: "cancel.png", text: "删除标签", lang: { zhcn: "删除标签", zhtw: "删除標籤", en: "delete a tap", ja: "ラベルの削除" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
        ],
        intagfile: [
            { command: "doc/refresh/index", icon: "refresh.png", text: "刷新", lang: { "zhcn": "刷新", "zhtw": "重繪", "en": "Refresh", "ja": "リフレッシュ" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            { command: "doc/intag/removefile/index", icon: "removefile.png", text: "移除文件", lang: { zhcn: "移除文件", zhtw: "移除文件", en: "Remove file", ja: "ファイルを除去する" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
        ],
        intagright: [
            { command: "doc/refresh/index", icon: "refresh.png", text: "刷新", lang: { "zhcn": "刷新", "zhtw": "重繪", "en": "Refresh", "ja": "リフレッシュ" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            { command: "doc/intag/rename/index", icon: "rename.png", text: "重命名标签", lang: { "zhcn": "重命名标签", "zhtw": "重命名標籤", "en": "rename tag", "ja": "名づけをする" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },        
            { command: "doc/intag/edit/index", icon: "edit.png", text: "编辑语料", lang: { "zhcn": "编辑语料", "zhtw": "编辑语料", "en": "Editing language", "ja": "語料を編集する" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
            { command: "doc/intag/removetag/index", icon: "cancel.png", text: "删除标签", lang: { zhcn: "删除标签", zhtw: "删除標籤", en: "delete a tap", ja: "ラベルの削除" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
        ],
        myrecommend: [
            { command: "doc/quick/myrecommendList/exchange/index", icon: "refresh.png", text: "换一换", lang: { "zhcn": "换一换", "zhtw": "換一換", "en": "Exchange", "ja": "換えてみる" }, entryType: ["myrecommend"], moduleType: ['myrecommend'] },
        ],
        intagload:[
            { command: "doc/refresh/index", icon: "refresh.png", text: "刷新", lang: { "zhcn": "刷新", "zhtw": "重繪", "en": "Refresh", "ja": "リフレッシュ" }, entryType: ["file", "files", "intag"], moduleType: ['intag'] },
        ]
    }
});
define('doc/contextmenu/processStrategyCache',[],function () {
    var processStrategys = {};

    function getProcessStrategyByKey(entryId) {
        if (processStrategys[entryId]) {
            return processStrategys[entryId];
        }
        $.ajax({
            data: {
                module: "WebClient",
                fun: "GetProcessStrategyByKey",
                entryId: entryId,
                entryType: "1",
            },
            async: false
        }).done(function (result) {
            processStrategys[entryId] = result.processStrategy || [];
        });
        return processStrategys[entryId];
    }

    function delStrategy(entryId) {
        delete processStrategys[entryId];
    }
    return { get: getProcessStrategyByKey, del: delStrategy };
});
/*
 *wuguangyun创建于2018年9月4日
 描述：智能标签/推荐菜单配置数据
 command:组件所在的目录的js文件路径，无需后缀
 icon:暂时未使用
 text:菜单或者工具栏显示的文字
 entryType：数组，文件的类型，有file：单文件，folder：单文件夹，virtualfolder：虚拟文件夹，root:根目录，mix：混合类型，files：多文件，folders：多文件夹，form：表单，record：记录
 perm：权限的组合，或关系
 moduleType:支持显示的模块名称，用于标识该组件可显示的模块
 children：子菜单或者子工具栏按钮，二级定义
 */
define('doc/contextmenu/processStrategyMenuCfg',[],function () {
    return [
        {
            command: "doc/tactic/processStrategy/processStrategy", icon: "flowoperation", text: "流程操作", lang: { "zhcn": "流程操作", "zhtw": "", "en": "Workflow", "jp": "フロー操作" }, entryType: ["folder", "folders", "files", "file", "mix"], perm: [1], moduleType: ['enterprise'], children: [
                { Id: 332, command: "doc/tactic/processStrategy/process/createProcess", icon: "create", text: "新增流程", lang: { "zhcn": "新增流程", "zhtw": "", "en": "Add New Files", "jp": "新たな流れ" }, entryType: ["folder"], perm: [128], moduleType: ['enterprise'] },
                { Id: 333, command: "doc/tactic/processStrategy/process/updateProcess", icon: "update", text: "更新流程", lang: { "zhcn": "更新流程", "zhtw": "", "en": "Update", "jp": "更新の流れ" }, entryType: ["file"], perm: [512], moduleType: ['enterprise'] },
                { Id: 334, command: "doc/tactic/processStrategy/process/deleteProcess", icon: "delete", text: "删除流程", lang: { "zhcn": "删除流程", "zhtw": "", "en": "Delete Files", "jp": "削除フロー" }, entryType: ["folder", "folders", "files", "file", "mix"], perm: [2048], moduleType: ['enterprise'] },
                //{ Id: 335, command: "doc/tactic/processStrategy/process/downloadProcess", icon: "download", text: "下载流程", lang: { "zhcn": "下载流程", "zhtw": "", "en": "Download process", "jp": "ダウンロードの流れ" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [32], moduleType: ['enterprise'] },
                { Id: 336, command: "doc/tactic/processStrategy/process/attributeProcess", icon: "attribute", text: "权限申请", lang: { "zhcn": "权限申请", "zhtw": "", "en": "Permission Apply", "jp": "権限の申請" }, entryType: ["folder", "file"], perm: [1], moduleType: ['enterprise'] },
                { Id: 337, command: "doc/tactic/processStrategy/process/shareProcess", icon: "share", text: "共享流程", lang: { "zhcn": "共享流程", "zhtw": "", "en": "Sharing", "jp": "共有フロー" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [8388608], moduleType: ['enterprise'] },
                { Id: 338, command: "doc/tactic/processStrategy/process/publicProcess", icon: "public", text: "外发流程", lang: { "zhcn": "外发流程", "zhtw": "", "en": "External Sharing", "jp": "外発の流れ" }, entryType: ["folder", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [32768], moduleType: ['enterprise'] },
                //{ Id: 339, command: "doc/tactic/processStrategy/process/secretlevetProcess",icon:"secretlevet",text:"密级流程",lang: { "zhcn": "密级流程", "zhtw": "", "en": "External development process", "jp": "外発の流れ" }, entryType: ["folder", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], moduleType: ['enterprise'] },
            ]
        }
    ];


});
define('doc/contextmenu/exchangeStrategyCache',[],function () {
    //debugger;
    //文件夹策略缓存
    var enableDirectOuterNetCache = [];
    function enableDirectOuterNet(entryId) {
        //edit by hanshunlong,20220628 RD-932, 删除直接交换外网外发策略后，选择文件仍出现外网外发按钮bug
        //暂时不使用缓存，有bug清除不掉
        var enable = enableDirectOuterNetCache[entryId];
        if (enable) {
            return true;
        } else if (enable == false) {
            return false;
        }
        $.ajax({
            data: {
                module: "FolderExchangeStrategyManager",
                fun: "EnableDirectOuterNetByFolderId",
                folderId: entryId,
                //entryType: "1",
            },
            async: false
        }).done(function (result) {
            enableDirectOuterNetCache[entryId] = result.enable;
        });
        return enableDirectOuterNetCache[entryId];
    }
    function delStrategy(entryId) {
        //debugger;
        delete enableDirectOuterNetCache[entryId];
    }

    return { enableDirectOuterNet: enableDirectOuterNet, del: delStrategy };
});
/*
   *whimco创建于2015年6月10日
   *mikahuang更新于2015年6月15日
   *mikahuang更新于2016年11
   *描述：该模块主要给文档库和快速导航的右键菜单和工具栏提供一个数据源的支持
*/
define('doc/contextmenu/menufilter',["underscore", "jquery", "./menuinfo", "./publishMenuCfg", "./permtype",
    "header/userinfo/constinfo", "./menusort", "./menutemplateconfig", "./previewMenuCfg", "./intagMenuCfg", "./processStrategyCache",
    "./processStrategyMenuCfg", "doc/contextmenu/permtype", "./exchangeStrategyCache"], function (_, $, menuConfig, publishMenuCfg,
        permType, constInfo, menusort, tempConfig, previewMenuCfg, intagMenuCfg, processStrategyCache, processStrategyMenuCfg, permtype, exchangeStrategyCache) {

    function hasproperty(obj, propertyName) {
        //解决if(obj.propertyName)方式，如果propertyName为0返回false的情况
        return propertyName in obj;
    }
    /*
    * whimco创建于2015年5月20日
    * 描述 通过数据返回转换后的数据类型
    * datas:所有选中的文件和文件夹列表
    */
    var getEntryTypes = function (datas) {
        var dataType = [];
        $.each(datas, function (n, obj) {
            if (obj.data && obj.data.shareMenuType == "sharetome") {
                dataType.push("folder");
                dataType.push("folder");
                return false;
            }
            //modify: 2017-12-14 haibin.long 增加搜索结果而未选中任何对象的工具栏显示
            if (obj.data && obj.data.searchroot) {
                dataType.push('searchroot');
                return false;
            }
            //企业内容库
            if (obj.data && hasproperty(obj.data, 'folderType') && obj.data.id == 1) {
                dataType.push("publicroot");
                return false;
            }

            //个人内容库
            if (obj.data && hasproperty(obj.data, 'folderType') && obj.data.id == constInfo.userInfo.topPersonalFolderId) {
                dataType.push("personroot");
                return false;
            }

            //文件模板顶级目录
            if (obj.data && hasproperty(obj.data, 'folderType') && obj.data.id == 5) {
                dataType.push("templateroot");
                return false;
            }

            //folderType为-1是说明是根目录
            if (obj.data && hasproperty(obj.data, 'folderType') && obj.data.folderType == -1) {
                dataType.push("root");
            }

            //如果是文件夹 folderType为1，普通文件夹
            if (obj.data && hasproperty(obj.data, 'folderType') && obj.data.folderType == 1) dataType.push("folder");
            //fileType为2时说明是普通文件
            if (obj.data && obj.data.fileType && obj.data.fileType == 2) dataType.push("file");
            //fileType为32时说明是链接文件
            //增加对链接文件与链接文件夹的区分 Modify: 2017-06-13 haibin.long
            if (obj.data && obj.data.fileType && obj.data.fileType == 32) {
                if (obj.data.shortcut && obj.data.shortcut.entryType == 1) {
                    //文件夹
                    dataType.push("folderlink");
                } else {
                    dataType.push("link");
                }
            }

            //filetype为128是轻文档
            if (obj.data && obj.data.fileType && obj.data.fileType == 128) dataType.push("docflow");

            //我的收藏列表
            //if (obj.data.favorite) dataType.push("favorite");
            if (obj.data && obj.data.favoritetype) {
                dataType.push("favoritetype");
            }
            //if (obj.data.favoritedata) dataType.push("favoritedata");
            if (obj.data && obj.data.favoritelist) {
                dataType.push("favoritelist");
            }
            //我的消息
            if (obj.data && obj.data.mymessage) dataType.push("mymessage");

            //隔离区
            if (obj.data && obj.data.mypartition) dataType.push("mypartition");

            //更新列表
            if (obj.data && obj.data.update) dataType.push("update");

            //我的共享
            if (obj.data && obj.data.myshare) dataType.push("myshare");
            //我的共享刷新
            if (obj.data && obj.data.sharerefesh) dataType.push("sharerefesh");
            //共享给我刷新
            if (obj.data && obj.data.sharemerefesh) dataType.push("sharemerefesh");
            //共享给我刷新
            if (obj.data && obj.data.shareme) dataType.push("shareme");

            //我的订阅
            if (obj.data && obj.data.mysubscribe) dataType.push("mysubscribe");
            //我的订阅-取消订阅
            //      if (obj.data.mysubscribecancel) dataType.push("mysubscribecancel");
            //我的订阅刷新
            //       if (obj.data.subscriberefesh) dataType.push("subscriberefesh");

            //我的外发
            if (obj.data && obj.data.mypublish) dataType.push("mypublish");
            ////我的外发刷新
            //if (obj.data.publishrefesh) dataType.push("publishrefesh");
            ////我的外发-取消外发
            //if (obj.data.mypublishcancel) dataType.push("mypublishcancel");
            ////我的外发-取消所有过期外发
            //if (obj.data.mypublishcancelall) dataType.push("mypublishcancelall");
            ////外发列表返回
            //if (obj.data.outpublishlistback) dataType.push("outpublishlistback");

            //我的锁定
            //if (obj.data.mylock) dataType.push("mylock");
            //我的锁定刷新
            if (obj.data && obj.data.mylock) dataType.push("mylock");

            //最近访问
            if (obj.data && obj.data.myvisit) dataType.push("myvisit");

            //cq.taiqi 2017.04.01 我的标签 按钮
            //if (obj.data.tagrefresh) dataType.push("tagrefresh");
            //if (obj.data.tagfileremove) dataType.push("tagfileremove");
            //if (obj.data.tagadd) dataType.push("tagadd");
            //if (obj.data.tagfileedit) dataType.push("tagfileedit");
            if (obj.data && obj.data.mytag) dataType.push("mytag");
            if (obj.data && obj.data.tagtype) dataType.push("tagtype");
            //团队内容库按钮显示隐藏判断
            if (obj.data && obj.data.parentFolderId == 7) {
                dataType.push('team');
                //团队创建人 
                if ((obj.data.currentTeamUserType && obj.data.currentTeamUserType == 1)) dataType.push("teamcreator");
                //团队管理员
                if (obj.data.currentTeamUserType && obj.data.currentTeamUserType == 2) dataType.push("teamadmin");
                //团队内部人员
                if (obj.data.currentTeamUserType && obj.data.currentTeamUserType == 3) dataType.push("teaminsider");
                //团队外部人员
                if ((obj.data.currentTeamUserType && obj.data.currentTeamUserType == 4) || obj.data.isCurrentTeamUserOutside) dataType.push("teamoutsider");
                //系统内部人员
                if (obj.data.createTeam && obj.data.createTeam == true) dataType.push("sysinsider");
            }

        });
        dataType = _.countBy(dataType);
        var returnType = {};

        //modify: 2017-12-15 haibin.long 搜索
        if (dataType.searchroot) {
            returnType.searchroot = true;
        }

        //我的消息
        if (dataType.mymessage) {
            returnType.mymessage = true;
        }

        //隔离区
        if (dataType.mypartition) {
            returnType.mypartition = true;
        }

        //返回类型为publicroot
        if (dataType.publicroot) {
            returnType.publicroot = true;
        }

        //返回类型为personroot
        if (dataType.personroot) {
            returnType.personroot = true;
        }

        //返回类型为personroot
        if (dataType.templateroot) {
            returnType.templateroot = true;
        }

        //如果file的个数大于1则返回类型为files
        if (dataType.file) {
            if (dataType.file > 1) {
                returnType.files = true;
            } else {
                returnType.file = true;
            }
        }

        //如果folder的个数大于1则返回类型为folders
        if (dataType.folder) {
            if (dataType.folder > 1) {
                returnType.folders = true;
            } else {
                returnType.folder = true;
            }
        }

        //链接文件不能所选，所以只判断是否有link
        if (dataType.link) {
            if (dataType.link > 1) {
                returnType.links = true; //选中是多个引用文件
            } else {
                returnType.link = true;
            }
        }
        //文件夹引用
        if (dataType.folderlink) {
            if (dataType.folderlink > 1) {
                returnType.folderlinks = true; //选中是多个引用文件夹
            } else {
                returnType.folderlink = true;
            }
        }

        //轻文档
        if (dataType.docflow){
            if (dataType.docflow > 1) {
                returnType.docflows = true; //选中多个轻文档
            } else {
                returnType.docflow = true;
            }
        }


        //我的共享 单选时，修改共享、取消共享按钮显示
        if (dataType.myshare) {
            if (datas.length == 1) {
                if (returnType.folder) { //过滤默认文件夹权限
                    delete dataType.folder;
                    delete returnType.folder;
                }
                if (returnType.file) { //过滤默认文件权限
                    delete dataType.file;
                    delete returnType.file;
                }
            }
            returnType.myshare = true;
        }

        ////我的共享刷新按钮显示
        //if (dataType.sharerefesh) {
        //    returnType.sharerefesh = true;
        //}

        //共享给我刷新、下载按钮显示
        if (dataType.shareme) {
            returnType.shareme = true;
        }

        //共享给我刷新按钮显示
        if (dataType.sharemerefesh) {
            returnType.sharemerefesh = true;
        }

        //我的订阅修改按钮显示
        if (dataType.mysubscribe) {
            returnType.mysubscribe = true;
        }

        //我的订阅取消按钮显示
        //  if (dataType.mysubscribecancel) {
        //        //单选与多选的实现
        //       if (dataType.mysubscribecancel > 1) {
        //           returnType.mysubscribecancel = true;
        //          } else {
        //         returnType.mysubscribe = true;
        //          }
        //       }

        //我的订阅刷新按钮显示
        if (dataType.subscriberefesh) {
            returnType.subscriberefesh = true;
        }

        //我的外发刷新按钮显示
        if (dataType.publishrefesh) {
            returnType.publishrefesh = true;
        }
        //我的外发 单选时，修改、取消按钮显示
        if (dataType.mypublish) {
            if (datas.length == 1) {
                if (returnType.folder) { //过滤默认文件夹权限
                    delete dataType.folder;
                    delete returnType.folder;
                }
                if (returnType.file) { //过滤默认文件权限
                    delete dataType.file;
                    delete returnType.file;
                }
            }
            returnType.mypublish = true;
        }
        //锁定刷新
        if (dataType.mylock) {
            returnType.mylock = true;
        }

        //最近访问刷新
        if (dataType.myvisit) {
            returnType.myvisit = true;
        }

        //我的标签
        //if (dataType.tagadd) {
        //    returnType.tagadd = true;
        //}
        //if (dataType.tagfileedit) {
        //    returnType.tagfileedit = true;
        //    returnType.tagadd = true;
        //}
        //if (dataType.tagrefresh) {
        //    returnType.tagrefresh = true;
        //    returnType.tagadd = true;
        //}
        //if (dataType.tagfileremove) {
        //    returnType.tagfileremove = true;
        //    returnType.tagadd = true;
        //}

        if (dataType.mytag) {
            returnType.mytag = true;
        }
        if (dataType.tagtype) {
            returnType.tagtype = true;
        }


        //我的外发刷新按钮显示
        //     if (dataType.mytag) {
        //         returnType.mytag = true;
        //      }

        //我的订阅取消按钮显示
        if (dataType.mypublishcancel) {
            //单选与多选的实现
            if (dataType.mypublishcancel > 1) {
                returnType.mypublishcancel = true;
            } else {
                returnType.mypublish = true;
            }
        }

        //我的订阅取消按钮显示
        if (dataType.mypublishcancelall) {
            returnType.mypublishcancelall = true;
        }

        //根目录不能多选，只判断是否根目录，
        if (dataType.root) {
            returnType.root = true;
        }

        //我的收藏刷新、新建分类
        if (dataType.favoritelist) {
            returnType.favoritelist = true;
        }

        //我的收藏夹重命名、取消收藏分类
        if (dataType.favoritetype) {
            returnType.favoritetype = true;
        }

        //我的收藏移动收藏、取消收藏
        if (dataType.favoritedata) {
            returnType.favoritedata = true;
        }



        if (dataType.update) {
            returnType.update = true;
        }

        //外发
        if (dataType.outpublishlistback) {
            returnType.outpublishlistback = true;
        }


        //团队
        if (dataType.team) {
            returnType.team = true;
        }

        //团队创建人
        if (dataType.teamcreator) {
            returnType.teamcreator = true;
        }

        //团队管理员
        if (dataType.teamadmin) {
            returnType.teamadmin = true;
        }

        //团队内部成员
        if (dataType.teaminsider) {
            returnType.teaminsider = true;
        }

        //团队外部成员
        if (dataType.teamoutsider) {
            returnType.teamoutsider = true;
            delete dataType.folder;
            return returnType;
        }

        //系统内部成员
        if (dataType.sysinsider) {
            returnType.sysinsider = true;
        }

        //如果文件中dataType长度超过1，即多于1种文件类型，则放回mix类型
        if (Object.getOwnPropertyNames(dataType).length > 1) {
            returnType.mix = true;
        }

        return returnType;
    };


    /*
    mikahuang创建于201611
    描述：根据实体类型和文件实体数据获取菜单对象命令
    根据 实体类型集合 在 filterMenuData集合中 获取其效交集部分的数据
    @filterMenuData：菜单对象
    @entryType:当前选择的实体类型
    @entryDatas:当前选择的文件数据对象
    */
    var getMenuCommandsByPerm = function (entryTypes, filterMenuData, entryDatas) {
        var menuCommands = [];
        if (entryTypes.favoritetype && entryTypes.mix) {
            //解决我的收藏中选中文件和收藏分类时没刷新按钮
            if (entryTypes.favoritetype && entryDatas.length > 1) {
                for (var i = 0; i < filterMenuData.length; i++) {
                    if (filterMenuData[i].command == "doc/refresh/index") {
                        menuCommands.push(filterMenuData[i]);
                        return menuCommands;
                    }
                }
            }
            return menuCommands;
        }
        for (entryType in entryTypes) {
            if (menuCommands.length == 0) {
                //gs#这个为0 的if不知道处理了什么，会在下面的menuCommands取交集
                //当包含多个允许的实体类型时会使第一个实体A和第二个AB交集出A而滤掉了B
                //目前处理方式是对A的实体类型取FALSE，而后如果A包含B那么就没问题
                //也许这个问题只出现在A包含B的情况。比如A是刷新，B是增加标签文件。
                var tempCommands = getEnableMenuCommands(filterMenuData, entryType, entryDatas);
                //返回除去所有false值的array副本
                menuCommands = _.compact(tempCommands);
            }
            else {
                var tempCommands = getEnableMenuCommands(filterMenuData, entryType, entryDatas);
                //返回除去所有false值的array副本

                tempCommands = _.compact(tempCommands);
                //取交集菜单类型
                menuCommands = _.intersection(menuCommands, tempCommands);
            }
        }
        //如果选中的同时有，文件夹和“文件或轻文档”，则移除外发共享元数据密级
        if ((entryTypes.folder | entryTypes.folders) && (entryTypes.file || entryTypes.files || entryTypes.docflow || entryTypes.docflows)) {
            for (var i = 0; i < menuCommands.length; i++) {
                if (menuCommands[i]=="doc/publish/index") {
                    menuCommands[i] = "";
                }
                if (menuCommands[i] == "doc/share/index") {
                    menuCommands[i] = "";
                }
                if (menuCommands[i] == "doc/metadata/index") {
                    menuCommands[i] = "";
                }
                if (menuCommands[i] == "doc/informationbar/basicinfo/singlefile/batchmoficationlevel/index") {
                    menuCommands[i] = "";
                }
            }
        }

        return menuCommands;
    };

    /*
    mikahuang创建于201611
    描述：获取可用的操作命令commands
    @filterMenuData：菜单对象
    @entryType:当前选择的实体类型
    @entryDatas:当前选择的文件数据对象
    */
    var getEnableMenuCommands = function (filterMenuData, entryType, entryDatas) {
        var filterMenu = []

        var enableCommands = _.map(filterMenuData, function (menu) {
            if (!menu.entryType) return menu.command;
            //此操作命令是否存在
            //    
            var tempMenuTypes = $.grep(menu.entryType, function (_entryType, n) {
                return _entryType == entryType;
            });


            if (tempMenuTypes.length > 0) {

                //此操作权限过滤
                if (filterByPerm(menu, entryDatas)) {
                    return menu.command;
                }
            } else if(entryType == "searchroot" && menu.commad == "doc/refreshforsearch/index"){ // modify by haiwei.cui/zhengjie 34377点击文件右侧面板中的标签执行搜索，搜索结果页面中，文档操作按钮显示错乱且部分无法点击
                return menu.command;
            } 
            // } else if (menu.command == "doc/goback/index" && entryType == "searchroot" && window.location.hash.indexOf('fromInfobar') > 0) {//从右侧面板搜索标签时，搜索结果页显示返回按钮
           
            // } else if (entryType == "searchroot" && window.location.hash.indexOf('fromInfobar') > 0) {//从右侧面板搜索标签时，搜索结果页显示返回按钮
            //     return menu.command;
            //}
            return "";
        });

        return enableCommands;
    };

    /*
    mikahuang创建于201611
    描述：根据可用菜单command过滤菜单对象
    @filterMenuData：菜单对象
    @enableMenuCommands:菜单command数组
    */
    var filterByEnableMenu = function (filterMenuData, enableMenuCommands) {
        var result = $.grep(filterMenuData, function (menu) {
            //是否包含
            var isContain = _.contains(enableMenuCommands, menu.command);
            return isContain;
        });
        return result;
    };



    /*
    *  whimco创建于2015年5月20日
    *  mikahuang重写于201611
    *  描述：对文件或者文件夹对象的菜单权限进行过滤判断
    *  @menu:单一菜单项目
    *  @datas:选中的文件或者文件夹信息
    */
    var filterByPerm = function (menu, datas) {
        if (datas.length > 0) {
            if (datas[0].shareMenuType == "sharetome") {
                if (menu.command == "doc/delete/index" || menu.command == "doc/goback/index" || menu.command == "doc/refresh/index") {
                    return true;
                } else {
                    return false;
                }
            }
        }
        //无权限限制
        //对引用文件作用于源文件的动作：下载、扩展编辑、更新、锁定、收藏、标签
        var shortcutMenus = ['doc/download/index', 'doc/extedit/extedit', 'doc/updatefile/index', 'doc/favorite/index', 'doc/unfavorite/index', 'doc/informationbar/basicinfo/singlefile/tag/index'];
        //wuguangyun 判断历史版本是否显示
        var perm = false;
        var ProcessAplly = false;
        //if (menu.command == "doc/historyVersion/index") {
        //    $.ajax({
        //        data: {
        //            module: "VersionManager",
        //            fun: "GetFileVersionByPageSize",
        //            fileId: datas[0].data.id,
        //            currentPage: 1,
        //            pageSize: 10

        //        },
        //        async: false
        //    }).done(function (data) {
        //        if (data.fileVerPermission == 0) {
        //            perm = true;
        //        }
        //    })
        //}
        if (menu.command == "doc/tactic/processStrategy/processStrategy") {
            if (datas[0].procType.length == 0) {
                perm = true;
            } else {
                menu.children = _.filter(menu.children, function (d) {
                    return _.contains(datas[0].procType, d.Id)

                })
                var enableCommands = _.map(menu.children, function (menu) {
                    if (menu.command == "doc/tactic/processStrategy/process/attributeProcess") {
                        ProcessAplly = true;
                        return menu;
                    } else {
                        if (filterByPerm(menu, datas)) {
                            return menu;
                        }
                    }
                })

                enableCommands = _.compact(enableCommands);
                menu.children = enableCommands;
                if (menu.children.length == 0) {
                    perm = true;
                }
            }
        }
        if (ProcessAplly) {
            return true;
        }
        if (perm) {
            return false;
        }
        //modify: 2018-04-18 haibin.long 增加复制移动 功能的单独判断
        var movecopyEnable = [];
        if (!menu.perm) return true;
        //循环检查每个文件的权限和菜单需要的权限数组的对应关系
        var downloadtype = true;

        for (var i = 0; i < datas.length; i++) {
            //如果permission没有设置，即放过权限检查,0为没权限,-1为完全控制
            if (!datas[i].data.permission && datas[i].data.permission != 0) continue;

            //modify: 2018-04-15 haibin.long shortcut、源文件 权限兼容 如果shortcut文件，那么进行源文件权限判断
            var tmpData = datas[i].data;
            if (tmpData.shortcut) {
                var tmppermission = tmpData.shortcut.entryPermission;
                var tmpmenu = _.find(shortcutMenus, function (item) {
                    return item == menu.command;
                });
                if (tmpmenu) { //如果menu在shourtcuMens中，则需要进行源文件的地界面判断
                    for (var j = 0; j < menu.perm.length; j++) {
                        if ((menu.perm[j] & tmppermission) == menu.perm[j]) {
                            continue;
                        }
                        return false;
                    }
                }
            }

            var curentModule = require('knockout').$store.state.curentModule();
            if (curentModule && curentModule.hash && curentModule.hash == "quick/myshareme") {
                if (menu.command == "doc/updatefile/index") {
                    if (datas[i].data.sharePerm == (permtype.sharePreview | permtype.sharePrint | permtype.shareDownload | permtype.shareEdit)) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }
            //复制移动具备一个权限即返回true
            //if (menu.command == "doc/movecopy/copy/index"|| menu.command == "doc/movecopy/cut/index") {
            //    var isEnable = false;
            //    for (var j = 0; j < menu.perm.length; j++) {
            //        //如果是文件夹的复制(download权限),有显示权限即可显示
            //        if (datas[i].data.folderType && (menu.perm[j] == permType.download)) {
            //            if ((permType.display & datas[i].data.permission) == permType.display) {
            //                movecopyEnable.push(true); //modify: 2018-04-18 haibin.long 移除代码 return true;
            //                isEnable = true;
            //                break;
            //            }
            //        }
            //        if ((menu.perm[j] & datas[i].data.permission) == menu.perm[j]) {
            //            movecopyEnable.push(true); //modify: 2018-04-18 haibin.long 移除代码 return true;
            //            isEnable = true;
            //            break;
            //        }
            //    }
            //    if (!isEnable) {
            //        //modify: 2018-04-18 haibin.long 移除代码 return false，多选文件时，复制移动 权限控制不准确的Bug #4048
            //        movecopyEnable.push(false);
            //    }
            //}
            //文件夹下载权限有显示权限(临时)
            if (menu.command == "doc/download/index" || menu.command == "doc/movecopy/copy/index") {
                var curentModule = require('knockout').$store.state.curentModule();
                if (datas.length > 1) {
                    for (var j = 0; j < menu.perm.length; j++) {
                        if (datas[i].data.folderType && (menu.perm[j] == permType.download)) {
                            if ((permType.display & datas[i].data.permission) == permType.display) {
                                downloadtype = downloadtype && true;
                            }
                            else {
                                downloadtype = downloadtype && false;
                            }
                            break;
                        }
                        if ((menu.perm[j] & datas[i].data.permission) == menu.perm[j]) {
                            downloadtype = downloadtype && true;
                        } else {
                            downloadtype = downloadtype && false;
                        }
                    }
                    if (menu.perm.length == 0) {
                        downloadtype = false
                    }
                    //共享给我下载权限以共享权限为准
                    if (curentModule && curentModule.hash && curentModule.hash == "quick/myshareme") {
                        if (datas[i].data.sharePerm == (permtype.sharePreview | permtype.sharePrint | permtype.shareDownload) || datas[i].data.sharePerm == (permtype.sharePreview | permtype.shareDownload) || datas[i].data.sharePerm == (permtype.sharePreview | permtype.sharePrint | permtype.shareDownload | permtype.shareEdit)) {
                            return true;
                        } else {
                            return false;
                        }
                    }
                    if (downloadtype == false) {
                        return downloadtype;
                    }
                    if (i == datas.length - 1) {
                        return downloadtype
                    }
                }
                else {
                    //共享给我下载权限以共享权限为准
                    if (curentModule && curentModule.hash && curentModule.hash == "quick/myshareme") {
                        if (datas[i].data.sharePerm == (permtype.sharePreview | permtype.sharePrint | permtype.shareDownload) || datas[i].data.sharePerm == (permtype.sharePreview | permtype.shareDownload) || datas[i].data.sharePerm == (permtype.sharePreview | permtype.sharePrint | permtype.shareDownload | permtype.shareEdit)) {
                            return true;
                        } else {
                            return false;
                        }
                    }
                    for (var j = 0; j < menu.perm.length; j++) {
                        if (datas[i].data.folderType && (menu.perm[j] == permType.download)) {
                            if ((permType.display & datas[i].data.permission) == permType.display) {
                                return true;
                            }
                        }
                        if ((menu.perm[j] & datas[i].data.permission) == menu.perm[j]) {
                            return true;
                        }
                    }
                    return false;
                }
            }
            else {
                for (var j = 0; j < menu.perm.length; j++) {
                    if ((menu.perm[j] & datas[i].data.permission) == menu.perm[j]) {
                        continue;
                    }
                    return false;
                }
            }

        }
        //if (menu.command == "doc/movecopy/index") {
        //    //modify: 2018-04-18 haibin.long #4048 复制移除 的显示，改为选中列表都权限时才显示
        //    var truecount = _.countBy(movecopyEnable, function (item) { return item ? "enable" : "disable"; });
        //    if (truecount.enable != datas.length) {
        //        return false;
        //    }
        //}
        return true;
    };

    /*
    mikahuang创建于201611
    描述：根据模块名称获取可用的菜单对象
    @filterMenuData：菜单对象
    @moduleName:模块字符串
    */
    var filterByModuleName = function (filterMenuData, moduleName) {
        //根据模块名称进行筛选
        var result = $.grep(filterMenuData, function (menu) {
            //是否包含此moduleName
            return _.contains(menu.moduleType, moduleName);
        });
        return result;
    };

    /*
    shaobo.zheng创建于202302
    描述：根据所选文件原库配置，过滤菜单
    @filterMenuData：菜单对象
    @entryDatas:选中数据
    @menuinfoConfig:全部菜单配置
    */
    var filterBySourceModuleSetting = function (filterMenuData, entryDatas, menuinfoConfig) {
        var checkMenus = ['doc/download/index', 'doc/publish/index', 'doc/share/index', 'doc/favorite/index', 'doc/unfavorite/index'];
        var forbiddenMultiMenu = ['doc/publish/index', 'doc/share/index'];// 跨库多选时禁用的按钮，目前是共享和外发
        var modules = [];
        $.each(entryDatas, function (e, entryData) {
            if (modules.indexOf(entryData.module) == -1) {
                modules.push(entryData.module);
            }
        });
        var result = $.grep(filterMenuData, function (menu) {
            if (checkMenus.indexOf(menu.command) == -1) {
                return true;
            }
            else {
                var res = true;
                // 跨库多选时，不能共享和外发
                if (modules.length > 1 && forbiddenMultiMenu.indexOf(menu.command) > -1) {
                    res = false;
                }
                else {
                    // 同库多选时，按原来库中可见按钮菜单二次过滤
                    for (var i = 0; i < modules.length; i++) {
                        var sourceModuleMenus = filterByModuleName(menuinfoConfig, modules[i]);
                        var isexist = _.find(sourceModuleMenus, function (sourceModuleMenu) {
                            return sourceModuleMenu.command == menu.command
                        });
                        if (!isexist) {
                            res = false;
                            break;
                        }
                    }
                }
                return res;
            }
        });
        return result;
    };


    var filterAllfolderModuleSetting = function (filterMenuData, entryDatas) {
        var modules = [];
        $.each(entryDatas, function (e, entryData) {
            if (entryData.path) {
                var PathArry = entryData.path.split("\\");
                //团队库
                if (PathArry[0] == "7") {
                    if (modules.indexOf('doc/metadata/index') == -1) {
                        modules.push("doc/metadata/index");
                    }
                    if (modules.indexOf('doc/search/text/thumbnail/index') == -1) {
                        modules.push("doc/search/text/thumbnail/index");
                    }
                    if (modules.indexOf('component/permission/index') == -1) {
                        modules.push("component/permission/index");
                    }
                    if (modules.indexOf('doc/createreference/index') == -1) {
                        modules.push("doc/createreference/index");
                    }
                    if (modules.indexOf('doc/search/thumbnail/index') == -1) {
                        modules.push("doc/search/thumbnail/index");
                    }
                    if (modules.indexOf('doc/goback/index') == -1) {
                        modules.push("doc/goback/index");
                    }
                }
                //个人库
                if (PathArry[0] == "2") {
                    if (modules.indexOf('doc/metadata/index') == -1) {
                        modules.push("doc/metadata/index");
                    }
                    if (modules.indexOf('doc/search/text/thumbnail/index') == -1) {
                        modules.push("doc/search/text/thumbnail/index");
                    }
                    if (modules.indexOf('component/permission/index') == -1) {
                        modules.push("component/permission/index");
                    }
                    if (modules.indexOf('doc/search/thumbnail/index') == -1) {
                        modules.push("doc/search/thumbnail/index");
                    }
                    if (modules.indexOf('doc/goback/index') == -1) {
                        modules.push("doc/goback/index");
                    }
                }
            }
          
        });
        var result = $.grep(filterMenuData, function (menu) {
            if (modules.indexOf(menu.command)>=0) {
                return false;
            } else {
                return true;
            }
        });
        return result;

    };

    /**
     * 交集函数，返回过滤后的原数组，gs
     * @param {*用于过滤参考的数组} filterArr 
     * @param {*原数组} SourceArr 
     * @param {*映射函数} mapfun 
     */
    var myIntersection = function (filterArr, SourceArr, mapfun) {
        var dic = {};
        var resultArr = [];
        for (var i = 0; i < filterArr.length; i++) {
            dic[mapfun(filterArr[i])] = filterArr[i]
        }
        for (var j = 0; j < SourceArr.length; j++) {
            if (dic[mapfun(SourceArr[j])]) {
                resultArr.push(SourceArr[j]);
            }
        }
        return resultArr;
    }

    /*
    mikahuang创建于201611
    描述：过滤系统业务类型
    @filterMenuData：菜单对象
    @entryType:当前选择的实体类型
    @entryDatas:当前选择的文件数据对象
    */
    var filterByBusiness = function (filterMenuData, entryType, entryDatas, moduleName) {
        var resultFilterMenuData = $.grep(filterMenuData, function (menu) {
            if ((entryType.file || entryType.link) && !entryType.mix) {
                if (menu.command == "doc/versionManagement/publish/index") {
                    if (entryDatas[0].data) {
                        if (entryDatas[0].data.lastVerNumStr.split(".")[1] == 0) {
                            return false;
                        }
                    }
                    else return false;
                }
                /*
                if (menu.command == "doc/lock/index" && entryDatas[0].data.state == "4") return false;
                if (menu.command == "doc/unlock/index" && entryDatas[0].data.state == "0") return false;
                */
            }
            //单个文件或者文件夹、多文件，modify:2017-12-02 haibin.long 增加 单文件引用，单文件夹引用，多文件引用，多文件夹引用 对收藏的判断
            if ((entryType.file || entryType.files || entryType.folder || entryType.folders || entryType.link || entryType.links || entryType.folderlink || entryType.folderlinks || entryType.docflow || entryType.docflows) && !entryType.team) {
                if (menu.command == "doc/onlineedit/index" && entryDatas.length == 1) {
                    if (!entryDatas[0].data.isOnlineEditing || entryDatas[0].data.isOnlineEditing === false)
                        return true;
                    else return false;
                }
                if (menu.command == "doc/onlineview/index" && entryDatas.length == 1) {
                    if (entryDatas[0].data.isOnlineEditing && entryDatas[0].data.isOnlineEditing === true)
                        return true;
                    else return false;
                }
                //收藏,取消收藏过滤,isfavorite=true,为已经被收藏
                //if ((entryType.file || entryType.files || entryType.folder) && !entryType.mix && !entryType.team) {//混合会被自动加入进入
                if (menu.command == "doc/favorite/index") {//只要选中的实体没被全收藏，则显示收藏
                    // var isAllfavorite = true;
                    // $.each(entryDatas, function (i, oo) {
                    //     if (!oo.data.isfavorite)
                    //         isAllfavorite = false;
                    // });
                    // if (isAllfavorite == true)
                    //     return false
                }

                if (menu.command == "doc/unfavorite/index" && entryDatas.length == 1) {
                    // if (entryDatas[0].data.isfavorite)
                    //     return true;
                    // else return false;
                }

                //固定
                if(menu.command == "doc/pushpin/index"){
                    //只要存在非固定数据,则显示固定按钮
                    //是否存在未固定文件
                    var unpushpinfile = _.find(entryDatas, function (item) {
                        return item.data.pushpin == false;
                    })
                    if (unpushpinfile){
                        return true;
                    }
                    else{
                        return false;
                    }
                }

                //取消固定
                if(menu.command == "doc/cancelpushpin/index"){
                    //选中数据都为已固定数据,则显示取消固定按钮
                    //是否存在未固定文件
                    var unpushpinfile = _.find(entryDatas, function (item) {
                        return item.data.pushpin == false;
                    })
                    if (unpushpinfile){
                        return false;
                    }
                    else{
                        return true;
                    }
                }

                if (menu.command == "doc/download/index") {
                    if (entryDatas[0].data.downloadPerm === false)
                        return false;
                    var hasExpired = false;
                    if (entryDatas.length > 0) {
                        $.each(entryDatas, function (n, e) {
                            if (e.data.shareSurplusTimeOld && e.data.shareSurplusTimeOld == 'hasExpired') {
                                hasExpired = true;
                                return false;
                            }
                        });
                    }
                    return !hasExpired;
                    /*
                    //已过期的共享，不允许下载
                    if(entryDatas[0].data.shareSurplusTimeOld&&entryDatas[0].data.shareSurplusTimeOld=='hasExpired'){
                        return false;
                    }
                    */
                }
            }
            if (entryType.file || entryType.files) {
                if (menu.command.indexOf("doc/quick/mytag/tagEdit") == 0) return true;//cq.taiqi选中单个文件,就算没选中标签,也是可以编辑该文件的标签的
                if (menu.command.indexOf("doc/quick/mytag/") == 0 && (!entryDatas[0].tagName)) return false;//gs:如果没有选定标签，pass               
            }
            else if ((entryType.teamcreator && entryType.team) || (entryType.folder && entryType.team)) {
                //团队的置顶与取消置顶
                if (menu.command == 'doc/team/stick/index' && entryDatas[0].data && (entryDatas[0].data.isStick != '0' || entryDatas[0].data.isStick)) { return false; }
                if (menu.command == 'doc/team/unStick/index' && entryDatas[0].data && (entryDatas[0].data.isStick == '0' || !entryDatas[0].data.isStick)) { return false; }
                //判断是否显示修改团队，(管理员、创建者)
                if (menu.command == 'doc/team/edit/index' && entryDatas[0].data && !(entryType.teamadmin || entryType.teamcreator)) { return false; }
            }
            else if (entryType.favoritetype && entryDatas.length > 1) {//gs,包含收藏夹类型的多个实体时移除删除和重命名
                if (menu.command == "doc/quick/myfavorite/deletefavoritetype/index") return false;
                if (menu.command == "doc/rename/index") return false;
            }
            else {
                if (menu.command == "doc/unfavorite/index") return false;

            }
            //如果不是是收藏分类一级目录就不显示“新建分类”按钮
            if ($(".navigation-list").children("li").length != 1) {
                if (menu.command == "doc/quick/myfavorite/newfavoritetype/index") {
                    //   return false;
                };
            } else {
                //如果是一级目录并且是有选中复选框的情况就不显示“新建分类”按钮 备用方案
                //if ($(".enterprise-list.document-list-body table tr") && $(".enterprise-list.document-list-body table tr").hasClass("selected")) {
                //    if (menu.command == "doc/quick/myfavorite/newfavoritetype/index") {
                //        return false;
                //    };
                //}
                //如果是一级目录并且选中文件/文件夹/分类文件夹 就不显示“新建分类”按钮
                if (menu.command == "doc/quick/myfavorite/newfavoritetype/index") {
                    if (entryDatas[0].select && entryDatas[0].select.length != 0) {
                        //    return false;
                    };
                }
            }
            //智能标签需授权才能显示出来
            if (!constInfo.IntelligentTag() && menu.command == "doc/informationbar/basicinfo/singlefile/intelligenttag/index") {
                return false;
            }
            //个人库外发
            if (moduleName != null && moduleName == "person" && menu.command == "doc/publish/index") {
                var IsOutgoing = constInfo.userInfo.IsOutgoing;
                if (IsOutgoing == 0) {
                    return false;
                }
            }
            //个人库预览外发
            if (moduleName != null && moduleName == "preview" && menu.command == "doc/publish/index" && entryDatas[0].path.split("\\")[0] == "2") {
                var IsOutgoing = constInfo.userInfo.IsOutgoing;
                if (IsOutgoing == 0) {
                    return false;
                }
            }
            //个人库预览共享
            if (moduleName != null && moduleName == "preview" && menu.command == "doc/share/index" && entryDatas[0].path.split("\\")[0] == "2") {
                var IsOutgoing = constInfo.userInfo.IsShare;
                if (IsOutgoing == 0) {
                    return false;
                }
            }
            //个人库共享
            if (moduleName != null && moduleName == "person" && menu.command == "doc/share/index") {
                var IsShare = constInfo.userInfo.IsShare;
                if (IsShare == 0) {
                    return false;
                }
            }

            //团队库共享
            if (moduleName != null && moduleName == "team" && menu.command == "doc/share/index") {
                //
                var isShare = true;
                
                for (var i = 0; i < constInfo.userInfo.TeamInfos.length; i++) {
                    var folderPath = entryDatas[0].path == undefined ? entryDatas[0].data.path : entryDatas[0].path;
                    if (folderPath.indexOf("7") == 0) {                        
                        if (folderPath.indexOf(constInfo.userInfo.TeamInfos[i].teamFolderId) > 0) {
                            if (constInfo.userInfo.TeamInfos[i].isShare == 1) {
                                isShare = true;
                            }
                            if (constInfo.userInfo.TeamInfos[i].isShare == 0) {
                                isShare = false;
                            }
                        }
                    }
                }
                if (!isShare) {
                    return false;
                }
            }

            //团队库外发
            if (moduleName != null && moduleName == "team" && menu.command == "doc/publish/index") {
                
                var isOutgoing = true;
                for (var i = 0; i < constInfo.userInfo.TeamInfos.length; i++) {
                    var folderPath = entryDatas[0].path == undefined ? entryDatas[0].data.path : entryDatas[0].path;
                    if (folderPath.indexOf("7") == 0) {
                        if (folderPath.indexOf(constInfo.userInfo.TeamInfos[i].teamFolderId) > 0) {                            
                            if (constInfo.userInfo.TeamInfos[i].isOutgoing == 1) {
                                isOutgoing = true;
                            }
                            if (constInfo.userInfo.TeamInfos[i].isOutgoing == 0) {
                                isOutgoing = false;
                            }

                        }
                    }
                }
                if (!isOutgoing) {
                    return false;
                }
            }

            var IsDataFerry = constInfo.systemInfo.instanceConfigInfo.DataFerry;
            //文件传输
            if (menu.command == 'doc/tactic/processStrategy/process/DataFerryProcess') {
                if (IsDataFerry == false) {
                    return false
                }
                if (entryDatas[0].procType && entryDatas[0].procType.length != 0) {
                    var procType = entryDatas[0].procType.indexOf(340);
                    if (procType == -1) {
                        return false;
                    }
                } else {
                    return false;
                }
            }
            var IsFexFerry = constInfo.systemInfo.instanceConfigInfo.FexFerry;
            //FEX传输
            if (menu.command == 'doc/tactic/processStrategy/process/fexFerryProcess') {
                if (IsFexFerry == false) {
                    return false
                }
                if (entryDatas[0].procType && entryDatas[0].procType.length != 0) {
                    var procType = entryDatas[0].procType.indexOf(341);
                    if (procType == -1) {
                        return false;
                    }
                } else {
                    return false;
                }
            }            
            //流程外网外发
            if (menu.command == 'doc/tactic/processStrategy/process/outerNetPublishProcess') {
                if (entryDatas[0].procType && entryDatas[0].procType.length != 0) {
                    var procType = entryDatas[0].procType.indexOf(342);
                    if (procType == -1) {
                        return false;
                    }
                    //edit by hanshunlong,20221021,缺陷38869 父文件夹设置直接交换外网外发，子文件夹设置外网外发流程，对子文件夹进行操作会出现两个外网外发按钮
                    //流程外网外发和直接外网外发需要根据文件夹就近选择一个
                    if (entryType.file || entryType.files) {
                        var entryId = entryDatas[0].data.parentFolderId;
                        var enableOuterNetPublish = exchangeStrategyCache.enableDirectOuterNet(entryId);
                        if (enableOuterNetPublish) {
                            return false;
                        }
                    }
                } else {
                    return false;
                }
            }

            //直接外网外发按钮判断逻辑,企业库单选或者多选文件，且文件夹设置有直接交换外网外发策略
            if (menu.command == 'doc/tactic/outerNetPublish/index') {
                if (moduleName != null && moduleName == "enterprise") {
                    if (entryType.file || entryType.files) {
                        var entryId = entryDatas[0].data.parentFolderId;
                        var enableOuterNetPublish = exchangeStrategyCache.enableDirectOuterNet(entryId);

                        if (enableOuterNetPublish) {
                            return true;
                        }
                    }
                }
                return false;
            }
            

            // var isShowPublish=constInfo.systemInfo.instanceConfigInfo && constInfo.systemInfo.instanceConfigInfo.SecretEdition;
            // //涉密开启隐藏外发
            // if(isShowPublish && menu.command=='doc/publish/index'){
            //     return false;
            // }
            if (menu.command == "doc/quick/myshare/cancelAllExpiredShare/index" && moduleName == "myshare") {
                if (entryType.myshare) {
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });

        //收藏团队文件，取交集
        if (moduleName === "favoritelist") {

            //在我的收藏中，如果entryDatas中存在，企业、个人、团队等收藏的混合数据，需要做额外的处理 modify: 2018-03-08 haibin.long
            var entrydatasOfType = getFavoritelistMouleName(entryDatas);
            if (!(entrydatasOfType.team.length == 0 && entrydatasOfType.person.length == 0 && entrydatasOfType.enterprise.length == 0)) {
                var favdatas = $.grep(resultFilterMenuData, function (oobj) {//取出收藏夹固有行为
                    return oobj.command.indexOf("doc/quick/myfavorite") === 0;
                });
                for (var key in entrydatasOfType) {
                    var dataOfType = entrydatasOfType[key];
                    if (dataOfType.length == 0) continue;
                    var tmpDatas = filter(dataOfType, key, null);
                    //取交集
                    resultFilterMenuData = myIntersection(tmpDatas, resultFilterMenuData, function (obj) { return obj.command });
                }
                resultFilterMenuData = favdatas.concat(resultFilterMenuData);
            }
            //我的收藏中选中两个收藏文件夹隐藏新建分类
            if (entryDatas.length > 1) {
                resultFilterMenuData = $.grep(resultFilterMenuData, function (menu) {
                    if (menu.command == "doc/quick/myfavorite/newfavoritetype/index") return false;
                    return true
                })

            }
        }
        ////modify: 2017-11-11 haibin.long 在单文件夹选中状态的情况下，需要隐藏 返回上一级 按钮
        //if (entryDatas.length == 1 && entryType.folder && (entryDatas[0].isSelected || entryDatas[0].data.isSelected)) {
        //    resultFilterMenuData = $.grep(resultFilterMenuData, function (menu) {
        //        if (menu.command == 'doc/goback/index') return false;
        //        return true;
        //    });
        //}

        //cq。tianao 共享给我根目录不一致 不执行此代码 
        if (moduleName != 'sharetome') {
            if (!(entryType.searchroot && window.location.hash.indexOf('fromInfobar') > 0)) {
                //modify: 2018-04-13 haibin.long 选中一个文件，且在根目录下面，不显示返回 按钮
                if (entryDatas.length == 1 && (entryType.file || entryType.folder) && (entryDatas[0].isSelected || entryDatas[0].data.isSelected)) {
                    if (constInfo.isRootFolder(entryDatas[0].data.path)) {
                        resultFilterMenuData = $.grep(resultFilterMenuData, function (menu) {
                            if (menu.command == 'doc/goback/index') return false;
                            return true;
                        });
                    }
                }
            }
        }
        //根目录不显示返回按钮
        if (entryDatas[0] && (entryDatas[0].data && entryDatas[0].data.message && (entryDatas[0].data.message == 'myshareme' || entryDatas[0].data.message == 'myshare'))) {
            resultFilterMenuData = $.grep(resultFilterMenuData, function (menu) {
                if (menu.command == 'doc/goback/index') return false;
                return true;
            });
        }



        //查到是否存在锁定文件,并且当前登录人不是文件锁定人,那么部分菜单功能就不可用
        //查到是否为OOS编辑中状态的文件,并且当前登录人不是文件编辑人,那么部分菜单功能就不可用
        $.each(entryDatas, function (index, entryData) {
            if (entryData.data && (entryData.data.state == "4" || entryData.data.state == "512")) {
                resultFilterMenuData = $.grep(resultFilterMenuData, function (menu) {
                    if (entryData.data.currentOperatorId !== constInfo.userInfo.id) {
                        //其他人解锁不可用
                        if (menu.command == "doc/unlock/index") return false;
                    }
                    //删除不可用
                    if (menu.command == "doc/delete/index") return false;
                    //重命名不可用
                    if (menu.command == "doc/rename/index") return false;
                    //更新不可用
                    if (menu.command == "doc/updatefile/index") return false;
                    //属性编辑不可用
                    if (menu.command == "doc/attribute/index") return false;
                    //扩展编辑不可用
                    // if (menu.command == "doc/extedit/extedit") return false;
                    //分配权限不可用
                    if (menu.command == "component/permission/index") return false;
                    //剪切不可用
                    if (menu.command == "doc/movecopy/cut/index") return false;
                    return true;
                });
                return false;
            }
        });
        return resultFilterMenuData;
    };
    //过滤我的共享中过期文件不能下载cq-lr
    var sharemeOverdueFile = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = filterMenuData;
        if (moduleName == "sharetome") {
            for (i = 0; i < entryDatas.length; i++) {
                var thisData = entryDatas[i];
                if (thisData.data && thisData.data.data && thisData.data.data.sharetOverdue) {
                    objMenuData = [];
                    $.each(filterMenuData, function (i, obj) {
                        if (obj.command != "doc/download/index") {
                            objMenuData.push(obj);
                        }
                    })

                }
            }
        }
        return objMenuData;
    };
    /**
     * modify:cq-lr
     * lic控制是否显示通知
     */
    var licControlNotify = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = [];
        var isShowNotify = constInfo.isNotification();
        if (isShowNotify) {
            return filterMenuData;
        } else {
            $.each(filterMenuData, function (i, obj) {
                if (obj.command != "doc/notify/notify") {
                    objMenuData.push(obj);
                }
            });
            return objMenuData;

        }
    }
    var filterAnnotation = function (filterMenuData, entryType, entryDatas, moduleName) {
        if (moduleName) {
            var hiddenAnnotation = false
            for (var i = 0; i < entryDatas.length; i++) {
                var thisData = entryDatas[i];
                if (thisData.hiddenAnnotation) {
                    hiddenAnnotation = true;
                    break;
                }
            }
            if (hiddenAnnotation) {
                var menuDatas = filterMenuData;
                $.each(menuDatas, function (i, obj) {
                    if (obj && obj.command === "doc/informationbar/comment/quickCreate") {
                        filterMenuData.splice(i, 1);
                    }
                })
            }
            return filterMenuData;
        }
    }
    /**
     *modify:cq-lr 2018-9-26
     *锁定状态的时候不显示标签按钮。
     */
    var lockState = function (filterMenuData, entryType, entryDatas, moduleName) {

        var objMenuData = filterMenuData;
        if (moduleName) {
            var isLock = false;//是否有锁定状态的文件
            var isCurrentLocker = true;//是否全是当前锁定人
            for (var i = 0; i < entryDatas.length; i++) {
                var thisData = entryDatas[i];
                if (thisData.data && [4, 512].indexOf(thisData.data.state) >= 0) {
                    isLock = true;
                    if (thisData.data.editorId != constInfo.userInfo.id) {
                        isCurrentLocker = false;
                    }
                }
            }
            var _objMenuData = [];
            $.each(filterMenuData, function (i, obj) {
                var filterMeuns = ['doc/lock/index', 'doc/unlock/index'];
                // 非 我的锁定去除锁的显示
                if (moduleName != 'mylock') {
                    if (filterMeuns.indexOf(obj.command) < 0) {
                        if (isLock && "doc/updatefile/index" == obj.command) {
                        } else {
                            _objMenuData.push(obj);
                        }
                    }
                } else {
                    if ("doc/updatefile/index" != obj.command) {
                        _objMenuData.push(obj);
                    }
                }
            });
            filterMenuData = objMenuData = _objMenuData;
            return objMenuData;
        }
    }
    /**
     * 未解密状态时候过滤了解锁按钮保留锁定
     */
    var isWithDecryption = function (filterMenuData, entryType, entryDatas, moduleName) {

        var objMenuData = filterMenuData;
        //缺陷-feiya.zhi-20220219-自动化测试的时候，模板文件上传后，控制台报错data.state undefined
        if (entryDatas.length == 1 && entryDatas[0].data && entryDatas[0].data.state == 128) {
            objMenuData = [];
            $.each(filterMenuData, function (i, obj) {
                if (obj.command != "doc/unlock/index") {
                    objMenuData.push(obj);
                }
            });
        }
        return objMenuData;
    }
    /**
     * modify:cq-lr 2018-10-30
     * 当状态为元数据维护中得时候，只允许有元数据按钮
     */
    var metadataMaintenanceState = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = filterMenuData;
        if (moduleName) {
            for (var i = 0; i < entryDatas.length; i++) {
                var thisData = entryDatas[i]
                if (thisData.data && thisData.data.state == 1024) {
                    objMenuData = [];
                    $.each(filterMenuData, function (i, obj) {
                        if (obj.command == "doc/goback/index" || obj.command == "doc/refresh/index"
                            || obj.command == "doc/delete/index" || obj.command == "doc/metadata/index"
                            || obj.command == "doc/recyclebin/restore/index" || obj.command == "doc/recyclebin/deleteUltimate/index"
                            || obj.command == "doc/recyclebin/deleteAll/index") {
                            objMenuData.push(obj);
                        }
                    });
                }
            }
            return objMenuData;
        }
    }
    /***
     * modify:cq-lr 2018-11-20
     * 过滤粘贴按钮
     */
    var isShowPaste = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = filterMenuData;
        var iscopy = $.cookie('copy');
        var iscut = $.cookie('cut');
        if (iscopy != "true" && iscut != "true") {
            objMenuData = [];
            $.each(filterMenuData, function (i, obj) {
                if (obj.command != "doc/movecopy/paste/index") {
                    objMenuData.push(obj)
                }
            });

        }
        return objMenuData;
    }
    /**
     * 
     *搜索无权限文件时 去除多余按钮
     */
    var isPermission = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = filterMenuData;

        for (var i = 0; i < entryDatas.length; i++) {
            if (entryDatas[i] && entryDatas[i].permission == '0') {
                objMenuData = [];
                $.each(filterMenuData, function (i, obj) {
                    if (obj.command == "doc/goback/index" || obj.command == "doc/refresh/index" || obj.command == "doc/tactic/processStrategy/processStrategy") {
                        objMenuData.push(obj)
                    }
                });
                return objMenuData;
            }


        }
        return objMenuData;
    }

    //zhifeiya-20220322-添加返回按钮
    var getQueryVariable = function getQueryVariable(variable) {
        var query = window.location.hash.substring(1);
        var vars = query.split("&");
        for (var i = 0; i < vars.length; i++) {
            var pair = vars[i].split("=");
            if (pair[0] == variable) { return pair[1]; }
        }
        return (0);
    }
    /**
     * 
     * 共享给我移除删除菜单按钮
     */
    var issharetome = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = filterMenuData;
        if (moduleName == 'sharetome') {
            objMenuData = [];
            //zhifeiya-20220322-添加返回按钮
            var shareId = getQueryVariable("shareId");
            $.each(filterMenuData, function (i, obj) {
                if ((shareId - 0) > 0) {
                    if (obj.command != "doc/delete/index") {
                        objMenuData.push(obj);
                    }
                } else {
                    if (obj.command != "doc/delete/index" && obj.command != "doc/goback/index") {
                        objMenuData.push(obj);
                    }
                }

            })
        }
        return objMenuData;
    };
    /**
     * modify:cq-lr 2018-11-30
     * 更具相应的权限过滤复制剪切按钮
     */
    var isShowCopyCut = function (filterMenuData, entryType, entryDatas, moduleName) {
        var objMenuData = filterMenuData;
        var isShowCut = false;
        var isShowCopy = false;
        $.each(filterMenuData, function (i, obj) {
            if (obj.command == "doc/delete/index" || entryType.folderlink || entryType.link || entryType.links || entryType.folderlinks) {
                isShowCut = true;
            }
            if (obj.command == "doc/download/index" || entryType.folderlink || entryType.link || entryType.links || entryType.folderlinks) {
                isShowCopy = true;
            }
        });
        //如果没有删除权限过滤剪切按钮
        if (!isShowCut) {
            objMenuData = [];
            $.each(filterMenuData, function (i, obj) {
                if (obj.command != "doc/movecopy/cut/index") {
                    objMenuData.push(obj)
                }
            });
            filterMenuData = objMenuData;
        }
        if (!isShowCopy) {
            objMenuData = [];
            $.each(filterMenuData, function (i, obj) {
                if (obj.command != "doc/movecopy/copy/index") {
                    objMenuData.push(obj)
                }
            });
        }

        return objMenuData;
    };
    var filterByVDLocalindex = function (filterMenuData, entryType, entryDatas, moduleName) {
        //文件模板-->预览的工具栏
        if (moduleName == "preview" && entryDatas[0]) {
            var filterMenuDatas = new Array();
            for (i in filterMenuData) {

                if (window.location.href.indexOf('sharecode') > -1 || window.location.href.indexOf('code') > -1) {
                    if (filterMenuData[i].command != "doc/localOpen/index" && filterMenuData[i].command != "doc/localLocation/index") {
                        filterMenuDatas.push(filterMenuData[i]);
                    }
                } else {
                    filterMenuDatas.push(filterMenuData[i]);
                }

            }
            filterMenuData = filterMenuDatas;
        }
        return filterMenuData;
    }
    /*
    描述：过滤外发加密按钮
    @filterMenuData：菜单对象
    @entryType:当前选择的实体类型
    @entryDatas:当前选择的文件数据对象
    */
    var isEnableSecPublish = function (filterMenuData, entryType, entryDatas, moduleName) {
        //var InstanceConfig = constInfo.getSysConfigInfo();
        //if (InstanceConfig && InstanceConfig.encrypted == "") {
        var filterMenuDatas = new Array();
        for (i in filterMenuData) {
            if (filterMenuData[i].command != "doc/secpublish/index") {
                filterMenuDatas.push(filterMenuData[i]);
            }
        }
        filterMenuData = filterMenuDatas;
        //}
        return filterMenuData;
    }

    //modify zhaobaosheng 2022-02-25   [缺陷编号:26779] 外发文件预览界面存在收藏、以文搜文、评论批注、右侧面板等按钮
    var isEnablequickCreate = function (filterMenuData, entryType, entryDatas, moduleName) {
        //var InstanceConfig = constInfo.getSysConfigInfo();
        //if (InstanceConfig && InstanceConfig.encrypted == "") {
        var filterMenuDatas = new Array();
        for (i in filterMenuData) {
            if (filterMenuData[i].command != "doc/informationbar/comment/quickCreate") {
                filterMenuDatas.push(filterMenuData[i]);
            }
        }
        filterMenuData = filterMenuDatas;
        //}
        return filterMenuData;
    }

    //modify: 2022-03-01 ruochen.mei 缺陷26876 搜索结果页面，点击刷新后，出现其他操作按钮（缩略图模式下）
    var isEnableSearch = function (filterMenuData, entryType, entryDatas, moduleName) {
        var filterMenuDatas = new Array();
        for (i in filterMenuData) {
            if (filterMenuData[i].command == "doc/refreshforsearch/index" || filterMenuData[i].command == "doc/refresh/index") {
                filterMenuDatas.push(filterMenuData[i]);
            }
        }
        filterMenuData = filterMenuDatas;
        return filterMenuData;
    }
    /*
    描述：
    1.过滤个人库在开启杀毒功能下的按钮过滤
    2.过滤文件夹的下载
    @filterMenuData：菜单对象
    @entryType:当前选择的实体类型
    @entryDatas:当前选择的文件数据对象
    */
    var isEnableSecurityScan = function (filterMenuData, entryType, entryDatas, moduleName) {
        var _enableSecurityScan = constInfo.getSysConfigInfo().enableSecurityScan;
        var _securityScanFileLock = constInfo.getSysConfigInfo().securityScanFileLock;
        if (_enableSecurityScan && _securityScanFileLock) {
            var _filterActionCommads = [];

            var filterAction = function (commads, filterMenu) {
                var filterMenuDatas = new Array();
                for (i in filterMenu) {
                    if (commads.indexOf(filterMenu[i].command) < 0) {
                        filterMenuDatas.push(filterMenu[i]);
                    }
                }
                filterMenu = filterMenuDatas;
                return filterMenuDatas;
            }
            if (moduleName === 'person' && entryType.file) {
                if (!(
                    entryDatas.length &&
                    entryDatas[0].data &&
                    entryDatas[0].data.FileSecurityscan == -1)) {
                    return filterMenuData;
                }
                _filterActionCommads = ['doc/updatefile/index'];
                filterMenuData = filterAction(_filterActionCommads, filterMenuData);
            }
            if (entryType.folder) {
                _filterActionCommads = ["doc/download/index"]
                filterMenuData = filterAction(_filterActionCommads, filterMenuData);
            }
        }
        return filterMenuData;
    }

    /*
    mikahuang创建于201611
    描述：过滤其他类型
    @filterMenuData：菜单对象
    @entryType:当前选择的实体类型
    @entryDatas:当前选择的文件数据对象
    */
    var filterByOther = function (filterMenuData, entryType, entryDatas, moduleName) {
        //tianao 团队预览支持共享和外发
        // if (moduleName == "preview" && entryDatas[0] && entryDatas[0].path && entryDatas[0].path.split("\\")[0] == "7") {//过滤预览界面来自团队内容库的的外发按钮。
        //     var tmpArray = [];
        //     $.each(filterMenuData, function (index, data) {
        //         if (data.command != "doc/publish/index" && data.command != "doc/share/index") {
        //             tmpArray.push(data);
        //         }
        //     })
        //     filterMenuData = tmpArray;
        // }


        //个人库隐藏‘操作日志’按钮
        if (moduleName == "person") {
            var filterMenuDatas = new Array();
            for (i in filterMenuData) {
                if (filterMenuData[i].command != "doc/docoptlog/index") {
                    filterMenuDatas.push(filterMenuData[i]);
                }
            }
            filterMenuData = filterMenuDatas;
        }
        //多选共享文件时隐藏修改共享
        if (moduleName == "myshare" && entryDatas.length > 1) {
            var filterMenuDatas = new Array();
            for (i in filterMenuData) {
                if (filterMenuData[i].command != "doc/share/index") {
                    filterMenuDatas.push(filterMenuData[i]);
                }
            }
            filterMenuData = filterMenuDatas;
        }
        //文件模板-->预览的工具栏
        if (moduleName == "preview" && entryDatas[0] && entryDatas[0].mould == "Template") {
            var filterMenuDatas = new Array();
            for (i in filterMenuData) {
                //添加工具栏菜单
                if (filterMenuData[i].icon === "download") {
                    filterMenuDatas.push(filterMenuData[i]);
                }
            }
            filterMenuData = filterMenuDatas;
        }
        //判断文件模版 文件是否内置文件
        //文件模板内置文件夹与其中文件菜单过滤
        var i;
        if (moduleName == "template") {
            var res = entryDatas[0].path;
            if (!res) {
                res = entryDatas[0].data && entryDatas[0].data.namePath ? entryDatas[0].data.namePath : "";
            }
            var resarr = res.split("\\");
            var Isbool = false;
            //判段选中的是文件还是文件夹
            if (resarr.length >= 2 && !entryDatas[0].parentFolderId) {
                Isbool = (resarr[0] == "5" && resarr[1] == "13");
            } else {
                Isbool = resarr[0] == "13";
            }
            if (Isbool) {
                var filterMenuDatas = new Array();
                for (i in filterMenuData) {
                    //添加工具栏菜单
                    if (filterMenuData[i].icon === "refresh" || filterMenuData[i].icon === "update" || filterMenuData[i].icon === "download") {
                        filterMenuDatas.push(filterMenuData[i]);
                    }
                }
                filterMenuData = filterMenuDatas;
            }
        }
        //文件模板内置文件夹右键菜单与工具栏菜单过滤
        if (entryDatas[0].name == "SystemTemplate" && moduleName == "template") {
            var filterMenuDatas = new Array();
            for (i in filterMenuData) {
                if (!(filterMenuData[i].icon == "delete" || filterMenuData[i].icon == "folder16" || filterMenuData[i].icon == "edit" || filterMenuData[i].icon == "authorize")) {
                    filterMenuDatas.push(filterMenuData[i]);
                }

            }
            filterMenuData = filterMenuDatas;
        }
        //对外发权限的过滤
        /*
         * item的属性值说明
         * isPublishFolder: false,//是否是文件夹外发
         * isPublishFile: false,//是否是文件外发
         * isSingleFile: false,//是否是单文件
         * isMultiFile: false,//是否是多文件
         * isSingleFolder: false,//是否是单文件夹
         * isMultiFolder: false,//是否是多文件夹
         * isFileAndFolder: false,//是否是既包含文件又包含文件夹
         * isInPublishRoot: false,//是否是在外发根目录
         * isInSubFolder: false//是否是在子文件夹(仅文件夹外发存在)
         * canRefresh:false,//能否进行刷新操作
         */
        if (moduleName === "outpublish") {
            //按钮集合
            var btns = {
                back: 'back',
                download: 'download',
                downloadformat: 'downloadformat',
                upload: 'upload',
                // extedit: 'extedit',
                extedit: "",
                log: 'log',
                refresh: 'refresh',
                update: "update"
            };
            var item = entryDatas[0].operationData;
            var perm = entryDatas[0].permissionData;
            var list = null;
            //文件外发的操作过滤
            if (item.isPublishFile) {
                if (item.isSingleFile) {
                    if (item.isDetelefile) {
                        list = [btns.extedit, btns.log];
                    } else {
                        // modify: 2023-01-09 mengfc #41832 开启加解密隐藏副本下载按钮
                        var confInfo = constInfo.getSysConfigInfo();
                        if (confInfo.encryptpolicy == 1) {
                            list = [btns.download, btns.update, btns.extedit, btns.log];
                        } else {

                            list = [btns.download, btns.update, btns.extedit, btns.log, btns.downloadformat];
                        }
                    }

                } else if (item.isMultiFile && !item.isDetelefile) {
                    list = [btns.download, btns.log];
                } else {
                    //都不是，则是未选择文件状态
                    list = [btns.log];
                }
                if (item.canRefresh) {
                    list.unshift(btns.refresh);
                }
            }
            //文件夹外发的操作过滤
            //full buttons => back,refresh,download,upload,update,extedit,log
            if (item.isPublishFolder) {
                if (item.isSingleFile) {
                    list = [btns.download, btns.update, btns.extedit, btns.log];
                } else if (item.isMultiFile) {
                    list = [btns.download, btns.log];
                } else if (item.isSingleFolder) {
                    if (item.isSelectAllFolde) {
                        list = [btns.download, btns.upload, btns.log];
                    } else {
                        list = [btns.upload, btns.log];
                    }
                } else if (item.isMultiFolder) {
                    list = [btns.download, btns.log];
                } else if (item.isFileAndFolder) {
                    list = [btns.download, btns.log];
                }


                if (item.canRefresh) {
                    list.unshift(btns.refresh);
                }

                if (item.isInSubFolder) {
                    list.unshift(btns.back);
                }
            }
            //用于过滤掉还未开发好的功能按钮 ，目前有 扩展编辑 2018年4月24日 hao.huang@macrowing.com
            list = _.reject(list, function (item) {
                return item === "";
            });

            //编辑 包含 扩展编辑和更新
            if (!perm.canEdit) {
                list = _.reject(list, function (item) {
                    return item === btns.extedit || item === btns.update;
                });
            }
            if (!perm.canUpload) {
                list = _.reject(list, function (item) {
                    return item === btns.upload;
                });
            }
            //不能下载的情况
            //1.无权限下载
            //2.下载次数足够，但不是选中了单个文件

            //需求调整#6230 选中多个文件、允许批量下载，后台验算次数上限
            //if (!perm.canDownload ||
            //    (perm.canDownload && perm.downloadTime > 0 && !item.isSingleFile)) {
            if (!perm.canDownload) {
                list = _.reject(list, function (item) {
                    return item === btns.download;
                });
            }
            //modify 20220801 by zhenyu.wang 副本下载
           
            if (!perm.canDownloadformat) {
                list = _.reject(list, function (item) {
                    return item === btns.downloadformat;
                });
            }
            filterMenuData = _.filter(filterMenuData, function (menu) {
                return _.contains(list, menu.id);
            });
        }
        //过滤源文件已删除的引用连接 haibin.long 2017-08-02
        var item = _.find(entryDatas, function (item) {
            return item.data && item.data.shortcut && item.data.shortcut.isDelete ? true : false;
        });
        if (item) {
            var reserveCommon = ['doc/delete/index', 'doc/refresh/index'];
            filterMenuData = _.filter(filterMenuData, function (menu) {
                var commond = menu.command.toLowerCase();
                return _.contains(reserveCommon, commond);
            });
        }
        //过滤引用文件的源文件删除功能结束

        //预览模块权限的过滤 modify: 2017-11-29 haibin.long
        if (moduleName == 'preview') {

            //var getQueryString = function (name) {
            //    var reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)");
            //    var r = window.location.search.substr(1).match(reg);
            //    if (r != null) return unescape(r[2]); return null;
            //};
            //  var vdlang = getQueryString("vdlang");
            var publishDownloadStr = 'outpublish/plugins/download/index';
            var docDownloadStr = 'doc/download/index';
            var printloadStr = 'doc/print/index';
            var entry = entryDatas[0];
            var filterItems = [];
            // 缺陷 27588  显示取消收藏按钮重复
            filterItems.push('doc/unfavorite/index');
            //wei1.zhou 20220318 缺陷28128 （UI）预览界面收藏文件后取消收藏，右侧多出收藏按钮
            filterItems.push('doc/favorite/index');
            //在共享、外发进入的预览界面，工具只可能 显示下载。其他按钮都隐藏 modify: 2018-02-07 haibin.long
            var addNormClearItem = function () {
                filterItems.push('doc/publish/index');
                filterItems.push('doc/share/index');
                //cq.wuyongwen修改于2017-12-22，共享无权限时没有收藏项
                filterItems.push('doc/favorite/index');
                filterItems.push('doc/unfavorite/index');
                //filterItems.push('doc/print/index');
            }

            if (entry.type == 'publish') { //外发就不要在有外发按钮了撒
                addNormClearItem();
                filterItems.push('doc/onlineedit/index');
                filterItems.push('doc/onlineview/index');
                filterItems.push(docDownloadStr);

                //修复预览没有下载按钮的bug
                if (!entry.canDown) {
                    filterItems.push(publishDownloadStr);
                    if (entry.canDownformat) {
                        filterMenuData.push({ command: "outpublish/plugins/downloadformat/index", icon: "downloadformat.png", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載", "en": "pdf download", "jp": "ダウンロード" }, entryType: ["file"], perm: [32], moduleType: ['preview'] });
                    }
                }
                
                //屏蔽掉扩展编辑 hao.huang@macrowing.com #4482
                // if (!entry.canEdit) {
                filterItems.push('doc/extedit/extedit');
                // }
            }
            else if (entry.type == 'share') { //共享时不要共享按钮，好麻烦
                addNormClearItem();
                // filterItems.push(docDownloadStr);

                for (var i = 0; i < filterMenuData.length; i++) {
                    if (filterMenuData[i].command == "outpublish/plugins/download/index") {
                        filterItems.push(docDownloadStr);
                    }
                }


                //修复预览没有下载按钮的bug

                filterItems.push('doc/favorite/index');

                if (!entry.canDown) {
                    filterItems.push(publishDownloadStr);
                }
                if (!entry.canPrint) {
                    filterItems.push('doc/print/index');
                }
                //if (!entry.canEdit) {
                filterItems.push('doc/extedit/extedit'); //共享不要扩展编辑
                //}
                //modify: 2018-01-05 haibin.long 如果被共享者对文件无下载权限，但共享者赋了下载权限。则应该有权限 ps：
                if ((entry.canEdit || entry.canDown) && !_.find(filterMenuData, function (item) { return item.command.toLowerCase() == publishDownloadStr })) {
                    var downloadItem = _.find(previewMenuCfg, function (item) { return item.command.toLowerCase() == publishDownloadStr });
                    var printItem = _.find(previewMenuCfg, function (item) { return item.command.toLowerCase() == printloadStr });
                    filterMenuData.push(printItem);
                    filterMenuData.push(downloadItem);
                }
                if (!entry.canEdit && !entry.canDown && entry.canPrint && !_.find(filterMenuData, function (item) { return item.command.toLowerCase() == printloadStr })) {
                    var printItem = _.find(previewMenuCfg, function (item) { return item.command.toLowerCase() == printloadStr });
                    filterMenuData.push(printItem);
                }
            } else {
                filterItems.push(publishDownloadStr);
            }
            //不是在线编辑预览器不需要在线编辑按钮
            if (entry.unneedOnlineEditing === undefined || (entry.unneedOnlineEditing && entry.unneedOnlineEditing === true)) {
                filterItems.push('doc/onlineedit/index');
                //filterItems.push('doc/onlineview/index');
            }
            //视频,音频不需要打印按钮
            if (entry.unneedPrint) {
                filterItems.push('doc/print/index');
            }
            if (entry.unneedDownload) {
                filterItems.push('doc/download/index');
                filterItems.push('outpublish/plugins/download/index');
            }
            //附件预览不需要收藏共享外发
            if (entry.unneedFavorite) {
                filterItems.push('doc/favorite/index');
                filterItems.push('doc/unfavorite/index');
            }
            if (entry.unneedPublish) {
                filterItems.push('doc/publish/index');
            }
            if (entry.unneedShare) {
                filterItems.push('doc/share/index');
            }
            if (entry.unneedExtedit) {
                filterItems.push('doc/extedit/extedit');
            }
            if (entry.unneedLocalOpen) {
                filterItems.push('doc/localopen/index');
            }
            if (entry.unneedLocalLocation) {
                filterItems.push('doc/locallocation/index');
            }
            if (!entry.canExtEdit) {//格式验证
                filterItems.push('doc/extedit/extedit');
            }
            if (("\\" + entry.path + "\\").indexOf("\\6\\") > -1)//流程文件预览不能扩展编辑
            {
                filterItems.push('doc/extedit/extedit');
            }

            try {
                var isWindows = function () {
                    return /windows|win32/i.test(navigator.userAgent);
                }();
                if (!isWindows) {
                    filterItems.push('doc/extedit/extedit');  //只有windows 显示扩展编辑按钮
                }
            } catch (e) {
                console.log(e)
            }

            //锁定状态 ，非锁定人不显示扩展编辑
            // if (entry.state == "4" && (entry.currentOperatorId !== constInfo.userInfo.id)) {
            //     filterItems.push('doc/extedit/extedit');
            // }


            //预览日志按钮： GMP开关控制& 共享 外发不需要
            if (entry.unneedOplog || entry.type == 'share' || entry.type == 'publish') {
                filterItems.push('doc/previewlog/index');
            }

            //按配置控制是否显示
            var previewBtnSetting = constInfo.systemInfo.instanceConfigInfo.PreviewBtnSetting
            if (previewBtnSetting) {
                for (var item in previewBtnSetting) {
                    if (previewBtnSetting[item].IsShow == false) {
                        filterItems.push(previewBtnSetting[item].SettingKey.toLowerCase());
                        if (previewBtnSetting[item].SettingKey == "doc/download/index") {
                            filterItems.push('outpublish/plugins/download/index');
                        }
                    }
                }
            }


            var index = filterMenuData.indexOf("doc/download/index");
          filterMenuData = _.filter(filterMenuData, function (menu) {
            if (!menu)
              return false;
                var commond = menu.command.toLowerCase();
                return !_.contains(filterItems, commond);
                //if (vdlang) {
                //    if (commond != "doc/download/index" && commond != "outpublish/plugins/download/index" && commond != "doc/print/index") {
                //        return !_.contains(filterItems, commond);
                //    }
                //} else {
                //    return !_.contains(filterItems, commond);
                //}

            });

            //排序
            for (var item = 0; item < filterMenuData.length; item++) {
                var findSetting = _.filter(previewBtnSetting, function (menu) { return menu.SettingKey.toLowerCase() == filterMenuData[item].command.toLowerCase() });
                if (findSetting && findSetting.length > 0) {
                    filterMenuData[item].sort = findSetting[0].SettingValue;
                }
                else {
                    if (filterMenuData[item].command.toLowerCase() == "doc/onlineview/index") {
                        var findSetting = _.filter(previewBtnSetting, function (menu) { return menu.SettingKey.toLowerCase() == "doc/onlineedit/index" });
                        if (findSetting && findSetting.length > 0) {
                            filterMenuData[item].sort = findSetting[0].SettingValue;
                        }
                    }
                    else {
                        filterMenuData[item].sort = item * 100;
                    }
                }
            }

            filterMenuData = _.sortBy(filterMenuData, function (item) { return item.sort });

            //排序值相同，以2级菜单显示，
            var alreadyProcess = [];
            var tempMenu = [];
            for (var item = 0; item < filterMenuData.length; item++) {
                var command = filterMenuData[item].command.toLowerCase();
                if (_.contains(alreadyProcess, command)) {
                    continue;
                }
                alreadyProcess.push(command);
                if (false && (command == "doc/onlineedit/index" || command == "doc/onlineview/index")) {//PI13520 
                    tempMenu.push(filterMenuData[item]);
                }
                else {
                    var findMenu = _.filter(filterMenuData, function (menu) { return menu.sort == filterMenuData[item].sort && menu.command.toLowerCase() != command });
                    if (findMenu && findMenu.length > 0) {
                        filterMenuData[item].children = findMenu;
                        $.each(findMenu, function (i, v) {
                            alreadyProcess.push(v.command.toLowerCase());
                        })
                    }
                    tempMenu.push(filterMenuData[item])
                }
            }
            filterMenuData = tempMenu;

        }

        //预览模块权限过滤结束
        if (entryDatas && entryDatas.length > 0) {
            if (entryDatas[0].module != "recyclebinlist") {
                //如果文件状态是正在上传，则只显示删除#6640 modify: 2018-01-26 haibin.long
                var datas = _.find(entryDatas, function (item) {
                    return item.data && item.data.state == 2; //state为2表示文件正在上传
                });
                if (datas) {
                    var reserveCommon = ['doc/delete/index'];
                    filterMenuData = _.filter(filterMenuData, function (menu) {
                        var command = menu.command.toLowerCase();
                        return _.contains(reserveCommon, command);
                    });
                }
                //正在上传状态判断结束
                //var datasec = _.find(entryDatas, function (item) {
                //    return item.data && item.data.state == 128; //state为2表示文件正在上传
                //    });
                //if (datasec) {
                //    var reserveCommon = ['doc/delete/index'];
                //    filterMenuData = _.filter(filterMenuData, function (menu) {
                //        var command = menu.command.toLowerCase();
                //        return _.contains(reserveCommon, command);
                //    });
                //}
                //var datasecfail = _.find(entryDatas, function (item) {
                //    return item.data && item.data.state == 256; //state为2表示文件正在上传
                //});
                //if (datasecfail) {
                //    var reserveCommon = ['doc/delete/index'];
                //    filterMenuData = _.filter(filterMenuData, function (menu) {
                //        var command = menu.command.toLowerCase();
                //        return _.contains(reserveCommon, command);
                //    });
                //}
            }
        }
        if (moduleName === 'sharetome') {
            //在共享给我的子级界面，去掉刷新按钮
            if (entryDatas.length > 0 && _.first(entryDatas).isInSub) {
                filterMenuData = _.filter(filterMenuData, function (item) {
                    return item.command !== 'doc/refresh/index';
                })
            }
        }
        // modify:2022-01-25 liuchunli    默认收藏夹 过滤删除,重命名按钮
        if (moduleName === 'favoritelist') {
            if (entryDatas[0].data.default == 1) {
                var reserveCommon = ['doc/quick/myfavorite/deletefavoritetype/index', 'doc/rename/index'];
                filterMenuData = _.filter(filterMenuData, function (menu) {
                    var commond = menu.command.toLowerCase();
                    return !_.contains(reserveCommon, commond);
                });
            }
        }
        // 收藏优化去除取消收藏按钮
        filterMenuData = _.filter(filterMenuData,function(item){
            return  item.command !== 'doc/unfavorite/index';
        })
        return filterMenuData;
    };


    /*
    mikahuang创建于201611
    描述：根据菜单Command类型获取菜单对象
    @filterMenuData：菜单对象
    @command:菜单ITEM的命令字符串
    */
    var filterByCommand = function (filterMenuData, command) {
        return $.grep(filterMenuData, function (itemMenu) {
            //取出此菜单
            if (itemMenu.command == command) {
                return true;
            }
            else {
                return false;
            }
        });
    };

    /**
     * 以企业库、个人库、团队库做为分类，对数据集合进行分解
     * @param {*} entryDatas 
     */
    function getFavoritelistMouleName(entryDatas) {
        var result = {
            team: [],
            person: [],
            enterprise: []
        };

        for (var irow = 0; irow < entryDatas.length; ++irow) {
            var tmp = entryDatas[irow];
            if (tmp.data && tmp.data.path) {
                var rootPath = tmp.data.path.split("\\")[0];
                if (rootPath == "7") {
                    result.team.push(tmp);
                }
                else if (rootPath == "2") {
                    result.person.push(tmp);
                }
                else if (rootPath == "1") {
                    result.enterprise.push(tmp);
                }
            }
        }
        return result;
    }



    /*
    whimco创建于2015年5月20日
    mikahuang重写于201611
    描述：用于在菜单或者工具栏中，接收过滤过权限、模块、文件类型等等的最终菜单数据
    @entryDatas：当前传递过来的文件或者文件夹数组
    @module:当前操作的模块
    @command:菜单ITEM的命令字符串
    */
    var filter = function (entryDatas, moduleName, command) {
        var IsAllFolder = false;
        var hash = window.location.hash;
        if (hash.indexOf('search') >= 0 && hash.indexOf('newrange') >= 0) {
            var UrlArry = decodeURIComponent(hash).split("&")
            var TargetStr = "";
            for (var i = 0; i < UrlArry.length; i++) {
                if (UrlArry[i].indexOf('newrange') >= 0) {
                    TargetStr = UrlArry[i];
                }
            }
            if (TargetStr.length > 0) {
                var TargetArry = TargetStr.split("=");
                var SearchType = TargetArry[1];
                if (SearchType.toLowerCase() == "allfolder") {
                    moduleName = "enterprise";
                    IsAllFolder = true;
                }
            }
        }
     
        //智能标签
        if (moduleName == "intag") {
            return intagMenuCfg.intag;
        }
        if (moduleName == "myrecommend") {
            return intagMenuCfg.myrecommend;
        }
        if (moduleName == "intagFiles") {
            return intagMenuCfg.intagfile;
        }
        if (moduleName == "intagRightMenu") {
            var menu = $.extend(true, [], intagMenuCfg.intagright);
            return menu;
        }
        if (moduleName == "intagload") {
            return intagMenuCfg.intagload;
        }
        if (moduleName == "docflowList") {
            var docflowTop = entryDatas && entryDatas[0].docflowTop;
            var topArr = docflowTop==0 ? { id: 'docflow-2', command: "doc/quick/mydocflow/funcbtns/top", icon: "goback.png", text: "置顶", lang: { "zhcn": "置顶", "zhtw": "置頂", "en": "Bring To Front", "jp": "前面に移動" }, entryType: ["docflow"], moduleType: ['docflow-top'] } : { id: 'docflow-2', command: "doc/quick/mydocflow/funcbtns/untop", icon: "goback.png", text: "取消置顶", lang: { "zhcn": "取消置顶", "zhtw": "取消置頂", "en": "Unstick", "jp": "スタックキャンセル" }, entryType: ["docflow"], moduleType: ['docflow-untop'] };

            var base = [{ id: 'docflow-1', command: "doc/quick/mydocflow/funcbtns/open", icon: "goback.png", text: "新窗口打开", lang: { "zhcn": "新窗口打开", "zhtw": "新窗口打開", "en": "New window opens", "jp": "新しいウィンドウを開く" }, entryType: ["docflow"], moduleType: ['docflow-open']},
            { id: 'docflow-3', command: "doc/quick/mydocflow/funcbtns/del", icon: "goback.png", text: "移除", lang: { "zhcn": "移除", "zhtw": "移除", "en": "Remove", "jp": "リムーブ" }, entryType: ["docflow"], moduleType: ['docflow-del'] }];
            base.splice(1, 0 , topArr);

            //modify: 2018-01-03 haibin.long 多语言转换
            var lang = constInfo.getLang();
            var langMap = {
                "zh-cn": "zhcn",
                "zh-tw": "zhtw",
                "en": "en",
                "ja": "jp"
            };

            var langKey = langMap[lang];
            _.each(base, function (item) {
                if (item.lang) {
                    item.text = item.lang[langKey] || item.text;
                }
            });
            return base
        }
        //合并模板菜单对象
        var menuinfoConfig = menuConfig.concat(tempConfig);
        menuinfoConfig = menuinfoConfig.concat(publishMenuCfg);
        //合并预览
        menuinfoConfig = menuinfoConfig.concat(previewMenuCfg);
        if (moduleName == "enterprise") {
            //合流程操作
            var processStrategymenu = $.extend(true, [], processStrategyMenuCfg);
            menuinfoConfig = menuinfoConfig.concat(processStrategymenu);
        }
        //最终菜单对象
        var resultMenuData = menuinfoConfig;

        //实体类型
        var entryTypes = {};
        //可用菜单项
        var enableMenuCommands = [];
        //有command时根据command先过滤下菜单对象
        if (command) {
            resultMenuData = filterByCommand(resultMenuData, command);
        }


        //根据传入的实体文件数据获取实体类型
        entryTypes = getEntryTypes(entryDatas);
        //根据模块名称获取可用的菜单对象,先过滤模块提高效率
        resultMenuData = filterByModuleName(resultMenuData, moduleName);
        // modify by shaobo.zheng 2023.02.07 QC19483后台外发去除未控制到最近菜单最近里的外发
        // "最近"模块，根据文件所在库设置，过滤操作按钮
        if (moduleName == "myvisit") {
            resultMenuData = filterBySourceModuleSetting(resultMenuData, entryDatas, menuinfoConfig)
        }

        //未启用文件夹元数据时toolbar去掉元数据菜单
        if (entryTypes.folder || entryTypes.folders) {
            var InstanceConfig = constInfo.getSysConfigInfo();
            if (InstanceConfig && !InstanceConfig.enableFolderMetaData) {
                for (var i = 0; i < resultMenuData.length; i++) {
                    if (resultMenuData[i].command == "doc/metadata/batchadd/index") {
                        resultMenuData.splice(i, 1);
                        break;
                    }
                }
            }
        }

        //判断是否扫描中且非自己创建
        if (constInfo.systemInfo.instanceConfigInfo.UDCAntiVirusEnableDownload) {
            for (var i = 0; i < entryDatas.length; i++) {
                if (entryDatas[i].isScanning && !entryDatas[i].isCurEditUser) {
                    return resultMenuData.filter(obj => obj.command == "doc/goback/index" || obj.command == "doc/refresh/index").sort((a, b) => a.sort - b.sort);
                }
            }
        }

        //wuguangyun 流程策略
        if (moduleName == "enterprise") {
            var entryId = 0;
            if (entryDatas.length > 1) {
                entryId = entryDatas[0].data.parentFolderId;
            } else {
                if (entryTypes.file == true) {
                    entryId = entryDatas[0].data.parentFolderId;
                } else {
                    entryId = entryDatas[0].data.id
                }
            }
            var procType = [];
            //获取选中文件夹数量
            var folderCount = 0;
            for (var i = 0; i < entryDatas.length; i++) {
                if (entryDatas[i].isFolder) {
                    folderCount++;
                }
            }
            var process = processStrategyCache.get(entryId);
            for (var i = 0; i < process.length; i++) {
                switch (process[i].procType) {
                    case 1:
                        if (entryDatas.length == 1 && entryTypes.folder == true) {
                            procType.push(332);
                        }
                        break;
                    case 2:
                        if (entryDatas.length == 1 && entryTypes.file == true) {
                            procType.push(333);
                        }
                        break;
                    case 3:
                        procType.push(334);
                        break;
                    case 4:
                        if (entryTypes.mix != true && folderCount < 2) {
                            procType.push(336);
                        }
                        break;
                    case 5:
                        if (entryTypes.mix != true && entryTypes.folders != true) {
                            procType.push(337);
                        }
                        break;
                    case 6:
                        if (entryTypes.mix != true && entryTypes.folders != true) {
                            procType.push(338);
                        }
                        break;
                    case 7:
                        if (entryTypes.mix != true && entryTypes.folders != true) {
                            procType.push(339);
                        }
                        break;
                    case 8:
                        if (entryTypes.mix != true && entryTypes.folders != true) {
                            //edit by hanshunlong,20220708,缺陷34018，fex交换策略中添加网闸摆渡/数据摆渡流程策略后，选择文件只会显示网闸摆渡按钮
                            //if (procType.indexOf(341) == -1) {
                            //    procType.push(340);
                            //}
                            //else {
                            //    //todo
                            //    //procType.push(340);
                            //}
                            procType.push(340);
                        }
                        break;
                    case 9:
                        if (entryTypes.mix != true && entryTypes.folders != true) {
                            procType.push(341);
                        }
                        break;
                    case 10://流程外网外发
                        if (entryTypes.mix != true && entryTypes.folders != true) {
                            procType.push(342);
                        }
                        break;
                }
            }
            entryDatas[0]["procType"] = procType;

        }
        //modify: 2018-01-02 菜单排序
        var _menusort = menusort;
        var sortByModule = _menusort[moduleName]; //获取此模块的排序列表
        if (sortByModule) {
            _.map(entryTypes, function (val, key) {
                var sortByModleAndType = sortByModule[key];
                if (sortByModleAndType) {
                    //fill in sort
                    _.map(sortByModleAndType, function (item) {
                        var menuIndex = _.findIndex(resultMenuData, function (m) { return m.Id == item.menuId; });
                        if (menuIndex > -1) {
                            resultMenuData[menuIndex].sort = item.sort;
                        }
                    });
                }
            });
            resultMenuData = _.sortBy(resultMenuData, "sort"); //执行排序
        }
        //根据实体数据和类型获取可用的菜单类型
        enableMenuCommands = getMenuCommandsByPerm(entryTypes, resultMenuData, entryDatas);
        //我的收藏中同时选中收藏分类文件夹和文件的情况没刷新按钮
        if (entryTypes.favoritetype && enableMenuCommands.length == 1) {
            resultMenuData = enableMenuCommands;
        } else {
            //根据可用菜单类型获取可用的菜单对象
            resultMenuData = filterByEnableMenu(resultMenuData, enableMenuCommands);
            //根据系统业务过滤菜单对象
            resultMenuData = filterByBusiness(resultMenuData, entryTypes, entryDatas, moduleName);
            //其他过滤
            resultMenuData = filterByOther(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = sharemeOverdueFile(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = licControlNotify(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = lockState(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = filterAnnotation(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = isShowPaste(resultMenuData, entryTypes, entryDatas, moduleName);
            //resultMenuData=isShowCopyCut(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = metadataMaintenanceState(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = filterByVDLocalindex(resultMenuData, entryTypes, entryDatas, moduleName);
            resultMenuData = isPermission(resultMenuData, entryTypes, entryDatas, moduleName);
            //未解密状态的时候去除解锁按钮
            resultMenuData = isWithDecryption(resultMenuData, entryTypes, entryDatas, moduleName);

            //共享给我移除删除菜单按钮
            resultMenuData = issharetome(resultMenuData, entryTypes, entryDatas, moduleName);
        }

        //未启用加密引擎时过滤加密外发按钮  //518先屏蔽加密外发按钮
        resultMenuData = isEnableSecPublish(resultMenuData, entryTypes, entryDatas, moduleName);

        ////缺陷 27077 2022年3月2日 刷新按钮应该在第一个
        //if(resultMenuData){
        //    resultMenuData.sort(function(a,b){
        //        if(a.command ==="doc/refresh/index"){return -1;}
        //        return 0;
        //    });
        //}

        //modify zhaobaosheng 2022-02-25   [缺陷编号:26779] 外发文件预览界面存在收藏、以文搜文、评论批注、右侧面板等按钮
        var ispublish = ($.getQueryString("ispublish")) && (window.location.pathname.indexOf('preview') > -1);
        if (ispublish) {
            resultMenuData = isEnablequickCreate(resultMenuData, entryTypes, entryDatas, moduleName);
        }

        //modify: 2018-01-03 haibin.long 多语言转换
        var lang = constInfo.getLang();
        var langMap = {
            "zh-cn": "zhcn",
            "zh-tw": "zhtw",
            "en": "en",
            "ja": "jp"
        };
        //企业库中：个人库目录创建的引用文件去掉锁定菜单
        if (entryDatas.length == 1 && entryDatas[0].data && entryDatas[0].data.type && entryDatas[0].data.type == 32) {
            if (entryDatas[0].data.shortcut && entryDatas[0].data.shortcut.entryPath.split('\\')[0] == "2") {
                for (var i = 0; i < resultMenuData.length; i++) {
                    if (resultMenuData[i].command == "doc/lock/index") {
                        resultMenuData.splice(i, 1);
                    }
                };
            }
        }

        if (moduleName == "teamcreate") {
            var newMenuData = [];
            $.each(resultMenuData, function (i, val) {
                // 判断是否新建团队菜单 & 判断是否有新建团队权限
                if (val.command != "doc/team/create/index" || constInfo.ShowCreateTeam()) {
                    newMenuData.push(val);
                }
            })
            resultMenuData = newMenuData;
        }
        var curentModule = require('knockout').$store.state.curentModule();
        if (curentModule && curentModule.hash == "quick/myfavorite") {
            var isDelete = false;
            $.each(entryDatas, function (i, val) {
                if (val.data.isDeleted) {
                    isDelete = true;
                }
            })
            var tmep = [];
            if (isDelete) {
                $.each(resultMenuData, function (i, val) {
                    if (val.command == "doc/refresh/index" || val.command == "doc/quick/myfavorite/deletefavorite/index") {
                        tmep.push(val);
                    }
                })
                resultMenuData = tmep;
            }
        }
        //最近访问, 过滤已删除删除, 无权限数据, 不显示操作按钮
        if (curentModule && curentModule.hash == "quick/myvisit") {
            var isDelete = false;
            $.each(entryDatas, function (i, val) {
                if (val.data.isDeleted || val.data.permission == 1) {
                    isDelete = true;
                }
            })
            var tmep = [];
            if (isDelete) {
                $.each(resultMenuData, function (i, val) {
                    if (val.command == "doc/refresh/index" || val.command == "doc/cancelpushpin/index") {
                        tmep.push(val);
                    }
                })
                resultMenuData = tmep;
            }
        }
        var langKey = langMap[lang];
        _.each(resultMenuData, function (item) {
            if (item.lang) {
                item.text = item.lang[langKey] || item.text;
                if (item.children) {
                    _.each(item.children, function (c) {
                        c.text = c.lang[langKey] || item.text;
                    });
                }
            }
        });
        //modify: 2022-01-25 jiangyang.li 修复缺陷[26221]非图片文件隐藏“以图搜图”按钮。
        var licInfo = constInfo.getLicInfo();
        // 测试代码
        // var index = resultMenuData.findIndex(v => v.icon == 'searchfortext.png');
        // if (index !== -1) {
        //     resultMenuData[index].command = 'doc/search/text/thumbnail/index';
        // };
        for (var i = 0; i < resultMenuData.length; i++) {
            //modify: 2022-02-24 jiangyang.li 修复缺陷[26746]，多语言状态，选择文件，有以图搜图，以文搜文按钮显示。
            //modify: 2022-03-01 jiangyang.li 修复缺陷[26856],仅企业库搜索面板显示搜图icon。
            if (resultMenuData[i].command == 'doc/search/thumbnail/index') {
                var typeList = ['.bmp', '.dib', '.jpeg', '.jpg', '.jpe', '.jp2', '.png', '.webp', '.pbm', '.pgm', '.ppm', '.pxm', '.pnm', '.pfm', '.sr', '.ras', '.tiff', '.tif', '.exr', '.hdr', '.pic', '.gif'];
                var data = entryDatas[0];
                if (data && data.name) {
                    // modify: 2022-03-04 jiangyang.li 新增：修复用户修改文件名造成文件判断错误的情况
                    var list = data.name.split('.');
                    var fileName = '.' + list[list.length - 1];
                    // modify: 2022-03-19 jiangyang.li 缺陷[28031] 大写的文件后缀名没有图文搜索按钮。
                    var imgType = typeList.some(v => fileName.toUpperCase().includes(v.toUpperCase()));
                    //modify: 2022-02-22 jiangyang.li 无权限隐藏搜图。
                    if (!imgType || !licInfo.EnabledModules.includes('SearchImage')) {
                        resultMenuData.splice(i, 1);
                        i--;
                    }
                }
                //modify: 2022-02-15 jiangyang.li 非文本文件隐藏“以文搜文”按钮。
                //modify: 2022-02-24 jiangyang.li 修复缺陷[26746]，多语言状态，选择文件，有以图搜图，以文搜文按钮显示。
            } else if (resultMenuData[i].command == 'doc/search/text/thumbnail/index') {
                //添加.mdoc格式，轻文档需要兼容‘已文搜文’
                var textList = ['.mdoc','.doc', '.docx', '.wps', '.rtf', '.xlsx', '.xls', '.et', '.ppt', '.pptx', '.pot', '.pps', '.dps', '.pdf', '.txt', '.log', '.xml', '.htm', '.html', '.csv', '.one', '.config', '.msg', '.eml'];
                var data = entryDatas[0];
                if (data && data.name) {
                    // modify: 2022-03-04 jiangyang.li 新增：修复用户修改文件名造成文件判断错误的情况
                    var list = data.name.split('.');
                    var fileName = '.' + list[list.length - 1];
                    // modify: 2022-03-19 jiangyang.li 缺陷[28031] 大写的文件后缀名没有图文搜索按钮。
                    var textType = textList.some(v => fileName.toUpperCase().includes(v.toUpperCase()));
                    //modify: 2022-02-22 jiangyang.li 无权限隐藏搜文。
                    if (!textType || !licInfo.EnabledModules.includes('SearchText')) {
                        resultMenuData.splice(i, 1);
                        i--;
                    };
                }
            } else if (resultMenuData[i].command == 'doc/downloadformat/index') {
                var data = entryDatas[0];
                if (data && data.name) {
                    var officeList = ['doc', 'docx', 'ppt', 'pptx'];
                    var docType = data.name.substring(data.name.lastIndexOf('.') + 1);
                    if (!officeList.some(v => docType.toUpperCase().includes(v.toUpperCase()))) {
                        resultMenuData.splice(i, 1);
                        i--;
                    }
                }

                //if (entryDatas[0].data) {
                //    var reserveCommon = ['doc/downloadformat/index'];
                //    var docType = entryDatas[0].name.substring(entryDatas[0].name.lastIndexOf('.') + 1);
                //    if (!officeList.some(v => docType.toUpperCase().includes(v.toUpperCase()))) {
                //        filterMenuData = _.filter(filterMenuData, function (menu) {
                //            var commond = menu.command.toLowerCase();
                //            return !_.contains(reserveCommon, commond);
                //        });
                //    }

                //}
            }
        }
        if (IsAllFolder) {
            resultMenuData = filterAllfolderModuleSetting(resultMenuData, entryDatas)

        }
        return resultMenuData;
    };

    return { filter: filter };
}
);

// const { toJSON } = require("knockout");

/*
  whimco创建于2015年5月18日
  描述:企业内容库右键菜单生成模块
*/
define('doc/contextmenu/index',["jquery", "komapping", "./menufilter","doc/tactic/processStrategy/processStrategy",lang(),"header/userinfo/constinfo",], function ($, komapping, menuList,process,lang,constinfo) {
    
    var ko = require("knockout");
    var app = require("durandal/app");
    //当前需要显示的经过过滤的菜单列表数据
    var currentMenuList = ko.observableArray(); 
    //当前菜单对象
    var currentElement;
    //菜单点击事件
    var onMenuClick;
    //回调方法的上下文环境
    var callbackScope;
    //回调消息  
    var callbackMessage;
    //模块传递过来的待处理的数据
    var currentDatas = [];
    //发起菜单的模块名称
    var moduleName;   
    // currentElement的副本对象
    var cloneCurrentElement;
    //右键菜单显示记时Id
    var rightMenuShowId=0;
    var folder=ko.observable();

    var menuActive = null;
    var menuActivecommand = '';

    var rightMenuMouseout = null;
    /**
     * 获取当前团队文件夹权限
     */
    var textList = ['.doc', '.docx', '.wps', '.rtf', '.xlsx', '.xls', '.et', '.ppt', '.pptx', '.pot', '.pps', '.dps', '.pdf', '.txt', '.log', '.xml', '.htm', '.html', '.csv', '.one', '.config', '.msg', '.eml'];
    var screenCommand={
        mainCommandList:['doc/download/index','doc/rename/index','doc/movecopy/cut/index','doc/movecopy/copy/index','doc/movecopy/paste/index','component/upload/index','doc/updatefile/index'],
        secondaryCommandList:['doc/quick/myfavorite/movefavorite/index',"doc/share/index","doc/publish/index",'doc/informationbar/basicinfo/singlefile/tag/index','doc/favorite/index','doc/quick/myfavorite/deletefavorite/index','doc/unfavorite/index','doc/search/thumbnail/index','doc/search/text/thumbnail/index','doc/tactic/processStrategy/processStrategy'],
        moreList:[''],
        sharORpubilc:["doc/share/index","doc/publish/index"]
    }
    
    
    var getTeamFolderInfo=function(id)
    {
       $.ajax({
           data:{
            module: "WebClient",
            fun: "GetTeamFolderInfoByTeamId",
            TeamId: id,
           },
           async: false
       }).done(function(data){
           folder(data.folderInfo);
       })
    }
    var rightModelPanel = function(data,module) {
            if (module == "enterprise" || module == "team" || module == "person"){
                if (data.length == 1 && (data[0].isFolder == false || data[0].data.fileType == 2)){
                    var entityType = 'singlefile';
                    var type = 'singlefile';
                    var filetype = data[0].data.fileType;
                    var curLanger = $.getLan();
                    var select = data;
                    var optiontype = '';
                    var path = module;
                    if (module == "enterprise"){
                        optiontype = "1001";
                    }else if (module == "team"){
                        optiontype = "7001";
                    }else if (module == "person"){
                        optiontype = "3001";
                    }
                    var showModuels = constinfo.getTypeRightList(optiontype, type, filetype, path, curLanger, entityType, select);
                    return showModuels;
                }
        }
    }
    // 以文搜文优化（缺陷34605）优化
    var keyJumpMethod = function(data){
        $('.right-menu > .menu').hide();
        // modify: 2022-09-02 cuizhichao #36853以图搜图结果页面中，选中文件，点击操作按钮、右键菜单的搜图，提示 文件内容未进索引；
        if (data.toolparam2017 && data.toolparam2017 == '以图搜图' || data.toolparam2017 == 'SearchForPictures' || data.toolparam2017 == '図で図を捜す.') {
            return true;
        }else if (data && data.name) {
            var list = data.name.split('.');
            var fileName = '.' + list[list.length - 1];
            var textType = textList.some(v => fileName.toUpperCase().includes(v.toUpperCase()));
            if( window.location.href.includes('search?') && data.data.content == ""){
                // modify: cuizhichao 2022-09-08 #37119 未被索引的文件以文搜文后，不应该显示出搜索内容；(右键菜单、toolbar需同步修改)
                app.trigger("fileNotIndexed",true)
                app.trigger("message:show", { type: 'error', message: lang.fileNotIndexed });
                return false
            }
            
            if(textType && data.data.IndexState != 2 && !window.location.href.includes('search?')){
                app.trigger("fileNotIndexed",true)
                app.trigger("message:show", { type: 'error', message: lang.fileNotIndexed });
                return false
            }else{
                return true
            }
        }else{
            return true
        }
    }

    return { 
        lang:lang,
        uploadFileNumTip: ko.observable(lang.uploadFileNumTip),
        bindingComplete: function (element) {
            currentElement = element;
            cloneCurrentElement = $('.right-menu > .menu').clone(true);
            var $elem = $(cloneCurrentElement); 
        },

        /**
         * 创建右键菜单
         * modify:2018-01-19 添加的此方法
         */
        buildMenu: function(){
            var $elem = $(cloneCurrentElement); 
            $elem.perfectScrollbar('update').perfectScrollbar();
        },

        destroyMenu:function(){
            if(!cloneCurrentElement) return;
            var $elem = $(cloneCurrentElement);
            var state = $.data($elem[0], 'menu');
            if(state){
                $elem.menu('destroy');
            }
        },

        /*
         whimco创建于2015年5月18日
         描述：菜单项的点击回调事件
         data:当前菜单项的数据对象，见menuconfig中单个菜单项
        */
        onMenuClick: function (data, event) {
            event.stopImmediatePropagation();
            if (data.command == 'uploadFile' || data.command == 'uploadFiles') {
                if (data.command == 'uploadFile'){//文件
                    // let selectFile_btn = $(".easy-upload-class .dnd-upload-button .filePicker:first >div:last label")
                    // selectFile_btn.click();
                    // app.trigger("upload:showUploadModal",)
                    app.trigger("component:show", {
                        moduleId: data.command,
                        datas: currentDatas,
                        message: callbackMessage,
                        moduleName: moduleName
                    });
                    app.trigger("upload:showUploadModal",1)
                } else if (data.command == 'uploadFiles') {//文件夹
                    app.trigger("component:show", {
                        moduleId: data.command,
                        datas: currentDatas,
                        message: callbackMessage,
                        moduleName: moduleName
                    });
                    app.trigger("upload:showUploadModal",2)
                    // let selectFolder_btn = $(".easy-upload-class .dnd-upload-button .filePicker:last >div:last input")
                    // selectFolder_btn.click();
                    // app.trigger("upload:showUploadModal",)
                }
                return
            }
            // modify: #33868 cuizhichao 选择文件拖拽上传，上传成功后，传输面板缩小后，选择子文件夹进行复制到目标文件夹下，传输面板框显示有误
            if (data.command == 'doc/movecopy/copy/index') {
                app.trigger("upload:changeUploadModal",true)
            }
            if (data.command == 'doc/search/thumbnail/index' || data.command == 'doc/search/text/thumbnail/index') {
                //34468普通搜索结果页面，执行以文搜文后，左上角的搜索内容无法关闭
                // if (window.location.hash.includes("type=searchForPictures") || window.location.hash.includes("type=searchByText")) {
                // }else{
                //     $.cookie('clearSearchToggleHash',window.location.href);
                // }
                // 搜图、搜文后面包屑关闭操作优化（此处右键菜单，对应toolbar菜单同样需要修改）
                // if(!keyJumpMethod(currentDatas && currentDatas[0])) return false
                if (!window.location.href.includes('search?')) {
                    $.cookie('clearSearchToggleHash',window.location.href);
                }
                if (data.command == 'doc/search/text/thumbnail/index') {
                    app.trigger("navigation:changeSearchHide", {searchText:currentDatas[0].name,isEnterprise:false  });
                }
            }
            if (currentDatas.length) {
                currentDatas[0].toolparam2017 = data.text;
            }
            if (!data.inline) {
                $('.right-menu > .menu').hide();
                if (data.command == "doc/folder/createoffice/index" || data.command == "doc/filetemplate/index") {
                    if (!process.getProcessStrategy(currentDatas, "create")) {
                        app.trigger("component:show", {
                            moduleId: data.command,
                            datas: currentDatas,
                            message: callbackMessage,
                            moduleName: moduleName
                        });
                    }
                }
                else if(data.command == "doc/uploadAttach/index"){
                    app.trigger("upload:showUploadModal",1,true,currentDatas[0].data.id)
                    app.trigger("component:show", {
                        moduleId: 'component/upload/index',
                        datas: { masterFile: komapping.fromJS(currentDatas[0].data) },
                        moduleName: "",
                        message: "informationbarattachment:reload"
                    });
                    // app.trigger("component:show", {
                    //     moduleId: 'component/upload/index',
                    //     datas: { masterFile: komapping.fromJS(currentDatas[0].data)},
                    //     moduleName: "",
                    //     message: "informationbarattachment:reload"
                    // });
                }
                // modify: 2022-01-20 jiangyang.li 添加搜图入口
                //modify: 2022-02-23 jiangyang.li 解决搜文点击无效的问题，因为没进判断。
                else if (data.command == "doc/search/thumbnail/index" || data.command == 'doc/search/text/thumbnail/index') {
                     // 根据url参数判断当前列表是主页面列表还是结果页列表
                     //modify: 2022-01-25 jiangyang.li 跳转搜图页面找不到id跳转失败 判断匹配的方式修改，因为id是动态获取。
                    if(!keyJumpMethod(currentDatas && currentDatas[0])) return false
                    var type =  window.location.href.includes('/search');
                    if (type) {
                        // modify: 2022-02-23 jiangyang.li 新增截图搜图功能, 添加改变截图背景图片回调。
                        if (currentDatas && currentDatas[0]) {
                            app.trigger("navigation:searchImgCallBack",{mark: (currentDatas[0] || {}).id || ''});
                        };
                        onMenuClick('结果页搜图' + data.command);
                    } else {
                        //modify: 2022-02-24 jiangyang.li 修复缺陷[26746]，多语言状态，选择文件，有以图搜图，以文搜文按钮显示。
                        onMenuClick('主页搜图' +  data.command);
                    }
                    // modify: 2022-03-01 jiangyang.li 新增：搜文隐藏列表截图。
                    if (data.command == "doc/search/thumbnail/index") {
                        app.trigger("navigation:searchImgHide",{type: true});
                    }
                }else{
                    app.trigger("component:show", {
                        moduleId: data.command,
                        datas: currentDatas,
                        message: callbackMessage,
                        moduleName: moduleName
                    });
                }
            } else {

                if (!onMenuClick) return;

                if (!callbackScope) {
                    onMenuClick();
                } else {
                    onMenuClick.call(callbackScope, data.command);
                }
            }

            $(cloneCurrentElement).menu('destroy'); 
        },
        onHidden: function(data,event){
            $('.right-menu > .menu').hide();
        },
        /*
            whimco创建于2015年5月18日
            描述：有子菜单的菜单鼠标进入时显示其子菜单
        */
        onSubmenuMouseover: function (data, event) {
            clearTimeout(menuActive)
            menuActivecommand = data.command
            var childMenuDemo = $(event.currentTarget);
            var browserHeight = window.innerHeight;//获取浏览器内部高度
            var rightMuneHeight = $(childMenuDemo.children()[1]).height();//获取右键菜单高度
            var topMenuOffet = $(event.currentTarget).offset().top;
            var paddingHeight = $(childMenuDemo.children()[1]).children().length * 32;  // css3动画获取最终高度
            if(rightMuneHeight <  paddingHeight ){
                rightMuneHeight = paddingHeight
            }
            var MenuOffetTop =  browserHeight - (rightMuneHeight + topMenuOffet);
            var top = browserHeight > top + rightMuneHeight ? top : (browserHeight - rightMuneHeight-23);
            if (data.command === "doc/tactic/processStrategy/processStrategy" && data.children.length > 1) {
                // $($(event.currentTarget).children()[1])
                    // .css("left", $(event.currentTarget).offset().left + $(event.currentTarget).width())
                    // .css("top", $(event.currentTarget).offset().top - ((data.children.length - 1) * 26))
                    // .show();
                    if(browserHeight - (rightMuneHeight + topMenuOffet) < 0){
                        $($(event.currentTarget).children()[1])
                        .css("left", $(event.currentTarget).offset().left + $(event.currentTarget).width())
                        .css("top", ($(event.currentTarget).offset().top - ~MenuOffetTop - 25))
                        .show();
                    }else{
                        $($(event.currentTarget).children()[1])
                        .css("left", $(event.currentTarget).offset().left + $(event.currentTarget).width())
                        .css("top", ($(event.currentTarget).offset().top))
                        .show();
                    }
            } else {
                if(browserHeight - (rightMuneHeight + topMenuOffet) < 0){
                    $($(event.currentTarget).children()[1])
                    .css("left", $(event.currentTarget).offset().left + $(event.currentTarget).width())
                    .css("top", ($(event.currentTarget).offset().top - ~MenuOffetTop - 25))
                    .show();
                }else{
                    $($(event.currentTarget).children()[1])
                    .css("left", $(event.currentTarget).offset().left + $(event.currentTarget).width())
                    .css("top", ($(event.currentTarget).offset().top))
                    .show();
                }
                
            }
            $(event.currentTarget).siblings().children(".menu-item-children-menu").hide()
           

        },
        onMouseOverMenu: function () {
            //清除记时
            clearTimeout(rightMenuShowId);
        },
        /*
            whimco创建于2015年5月18日
            描述：有子菜单的菜单鼠标离开时隐藏其子菜单
        */
        onSubmenuMouseout: function (data,event) {
            clearTimeout(menuActive)
            menuActive = setTimeout(function(){
                $($(event.currentTarget).children()[1]).hide();
            },200)
        },
        /*
            whimco创建于2015年5月18日
            描述：模块通过该方法显示菜单
            left:菜单显示的left值
            top: 菜单显示的top值
            datas:选中数据的数组，里面可能包含文件夹或者文件或者组合模式，或者其他类型，例如收藏之类，这个需要业务组件根据业务进行判断
            callback：发起操作的回调函数，可以为null
            scope：回调函数的context对象，可以为null
            message:列表发出菜单操作后的，操作之后的回调消息，一般为刷新列表
            module：指定是谁发起的菜单请求,因为不同的模块可能有不同的菜单值
        */
        showMenu: function (left, top, datas, callback, scope, message, module) {
            currentDatas = ko.toJS(datas);
            
            
            var rightModel = rightModelPanel(currentDatas,module);
            //点击团队左侧列表的情况
            if(datas[0]&&datas[0].isTimeList)
            {
                getTeamFolderInfo(currentDatas[0].folderId);
                currentDatas[0].currentTeamUserType= folder().currentTeamUserType;
            }
            //点击团队列表的情况
            if(datas.isTimeList)
            {
                getTeamFolderInfo(currentDatas[0].data.folderId);
                currentDatas[0].data.currentTeamUserType = folder().currentTeamUserType;
                //by jiali.yao 20220913 缺陷37292：团队库成员仅为下载权限时，右键菜单和上方操作按钮不一致
                currentDatas[0].data["permission"] = folder().permission;
            }
            onMenuClick = callback;
            moduleName = module;
            callbackMessage = message;
            callbackScope = scope; 

            var filtedData = menuList.filter(currentDatas, module);
           
            filtedData = filtedData.filter(item => (item.command !== "doc/refresh/index" && item.command !== 'doc/goback/index' && item.command !== "doc/docoptlog/index"))
           
            if ($.isArray(filtedData) && filtedData.length === 0) { return; }//没有可显示项就不显示
            // modify 2022-09-05 liuchunli #36922系统管理-配置管理-右侧面板中关闭对应功能按钮显示后，前台文档库的功能按钮也要跟随隐藏 
            if (rightModel) {
                var isInfo = rightModel.some(item => item.name == "historyFile");
                if (!isInfo) {
                    filtedData = filtedData.filter(item => (item.command !== "doc/historyVersion/index"))
                }
            }
            var filtedDataLen = filtedData.length <= 7;
            if(filtedDataLen){
                newFiltedData = filtedData
            }else{
                var newFiltedData = [];            
                var newMenu = [];
                var mainList = [],lastCommandString = '';
                var secondaryList = [], secondaryCommandString ='';
                var sharList = [];
                var moreList = [{
                    Id: 9999,
                    command: "NewMore",
                    entryType: ['folder', 'publicroot'],
                    icon: "create.png",
                    isShowLine: false,
                    lang: {zhcn: '更多'},
                    moduleType: ['enterprise', 'enterprise'],
                    perm: ['64'],
                    sort: 3,
                    text: lang.more,
                    children:[]
                }];

                var shareMore = [];
                var isAddShar = true;
                $.each(filtedData, function(index,item){
                    item.isShowLine = false;
                    if("NewMenu".includes(item.command)){
                        item.isShowLine = true
                        newMenu.push(item)
                    }else
                    if(screenCommand.mainCommandList.includes(item.command)){
                        lastCommandString = item.command;
                        mainList.push(item)
                    }else 
                    if(screenCommand.secondaryCommandList.includes(item.command)){
                        if(item.command == 'doc/publish/index' || item.command == 'doc/share/index'){
                            shareMore.push(item);
                            item.children = []
                            if(isAddShar){
                                isAddShar = false;
                                secondaryCommandString = item.command;
                                var newItem = JSON.parse(JSON.stringify(item));
                                newItem.text = lang.share;
                                // console.log(item)
                                secondaryCommandString = newItem.command
                                secondaryList.push(newItem)
                            }
                        }else{
                            secondaryCommandString = item.command;
                            secondaryList.push(item)
                        }
                    }
                    else {
                        var confInfo = constInfo.getSysConfigInfo();
                        if (item.command === 'doc/downloadformat/index') {
                            if (confInfo.encryptpolicy != 1) {
                                moreList[0].children.push(item)
                            }
                        } else {
                            moreList[0].children.push(item)
                        }
                        
                    }
                })

                newFiltedData = newMenu.concat(mainList,secondaryList,moreList,newFiltedData)


               
            }
            $.each(newFiltedData, function(index,item){
                if(!filtedDataLen){
                    if(item.command == 'doc/publish/index' || item.command == 'doc/share/index'){
                        item.children = shareMore
                    }

                    if(item.command == lastCommandString){
                        item.isShowLine = true
                    }

                    if(item.command == secondaryCommandString ){
                        item.isShowLine = true
                    }
                }else{
                    item.isShowLine = false
                }
                
            })
            // var a = JSON.parse(JSON.stringify(filtedData));
            // var b = JSON.parse(JSON.stringify(newFiltedData));
           
            currentMenuList.removeAll();
            currentMenuList(newFiltedData);
            // PD-2030（UI）文件（夹）动态 cuizhichao 过滤掉企业、团队库右键菜单操作日志按钮
            var curentModule = require('knockout').$store.state.curentModule();
            // modify: 2022-06-23 cuizhichao 32176	【文档模板】后台管理文档模板，选择文件夹右键没有弹出右键操作，且控制台报错
            if (curentModule) {
                if (curentModule.hash.indexOf('enterprise') > -1 && curentModule.hash.indexOf('team') > -1 ) {
                    var newCurrentMenuList = currentMenuList().filter(item => (item.command !== "doc/docoptlog/index"))
                    currentMenuList.removeAll();
                    currentMenuList(newCurrentMenuList);
                }
                // modify: 2023-01-09 mengfc #41832 开启加解密隐藏副本下载按钮
                var confInfo = constInfo.getSysConfigInfo();
                if (confInfo.encryptpolicy == 1) {
                    var newCurrentMenuList = currentMenuList().filter(item => (item.command !== 'doc/downloadformat/index'))
                    currentMenuList.removeAll();
                    currentMenuList(newCurrentMenuList);
                }
            }

            this.buildMenu();
            var browserHeight = window.innerHeight ;//获取浏览器内部高度
            var rightMuneHeight = $('.right-menu > .menu').height();//获取右键菜单高度
            var menuPaddingHeight = $('.right-menu > .menu').children(".menu-item").length * 32;
            var menuLineHeight = $('.right-menu > .menu').children(".menu-item-bottom-line").length * 17;
            if(rightMuneHeight < (menuPaddingHeight + menuLineHeight)){
                rightMuneHeight = menuPaddingHeight + menuLineHeight
            }
            var top = browserHeight - 20 > top + rightMuneHeight ? top : (browserHeight - rightMuneHeight - 25);
            $('.right-menu > .menu').css({
                left: left,
                top: top,
                display: 'block'
            });
            $(document).one("click", function(){
                $('.right-menu > .menu>.menu-item').addClass("menuHide-item")
                $('.right-menu > .menu').hide(100);
            });
        
            //记时隐藏防止一直不消失
            rightMenuShowId=setTimeout(function(){
                $('.right-menu > .menu').hide();
            },1500)
        },
        pngConversionSvg:constinfo.pngConversionSvg,
        getTime: function(){
            var time = new Date();
            return time.getSeconds() + ":" + time.getMilliseconds();
        },
        currentMenuList: currentMenuList
    };
});
/*
*  mikahuang创建于2015年5月
*  描述:工具栏创建,事件响应
*/
define('doc/toolbar/index',["jquery", "durandal/app", 'plugins/router', "durandal/events", "knockout", "komapping", "../contextmenu/menufilter", "doc/contextmenu/permtype", lang(), "doc/tactic/processStrategy/processStrategy", "doc/contextmenu/index", "header/userinfo/constinfo"],
    function (jquery, app, router, Event, ko, komapping, menuFilter, permtype, lang, process, contextmenu, constinfo) {
        
        var index = 0;
        var opendIndex = 0;
        var isLocalOpen = true;//是否是第一次
        var isSuccess = false;//webSocket是否成功
        //企业库中webSocket是否断开
        var isLocalBnt = false;
        var threeButMenu = null;
        var uploadFlag = 0;
        var textList = ['.doc', '.docx', '.wps', '.rtf', '.xlsx', '.xls', '.et', '.ppt', '.pptx', '.pot', '.pps', '.dps', '.pdf', '.txt', '.log', '.xml', '.htm', '.html', '.csv', '.one', '.config', '.msg', '.eml'];
        /**
         * 工具栏的核心方法和处理
         */
        function ToolbarCore(view) {
            this.view = view;
            //更多按钮
            this.moreBtn = { command: "more", icon: "more", text: lang.more, width: 66, isActive: ko.observable(false), cls: 'more' };
            //一些菜单相关变量
            this.filterMenuData = null;
            //需要留出来的空间距离
            this.distance = 0;
        }
        function SelectUploadType(type) {
            if (type == 'uploadFile'){//文件
                uploadFlag = 1
            }else if (type == 'uploadFiles') {//文件夹
                uploadFlag = 2
            }
            let login_btn = document.querySelector(".bread_v6_news_tag_names")
            login_btn.click();

        }
        var corePt = ToolbarCore.prototype;
        /*
        * mikahuang创建于2015年5月
        *过滤菜单项目
        * @param {array} currentDatas - 当前实体数据对象列表
        * @param {string} moduleName - 当前所在的模块名称
        * @return undefined
        */
        corePt.filterMenu = function (currentDatas, moduleName) {
            var that = this;
            that.filterMenuData = menuFilter.filter(currentDatas, moduleName);
            $.each(that.filterMenuData, function (index, obj) {
                obj.isActive = ko.observable(false);
                if (obj.command == 'doc/recyclebin/search/index') {
                    obj.recyclebinsearch = ko.observable("");
                }
            });
        }
        // modefi: 2022-04-25 cuizhicaho 26236【产品未提供设计】【UI】页面缩放状态下，三库回收站搜索按钮点击无反应  -- 开始
        function getDeleteFlag(nowUrls) {
            // console.log('nowUrls',nowUrls);
            if (nowUrls.indexOf("/team/")>=0) {
                if (nowUrls.indexOf("recycle=-1")>=0) {
                    return '-857'
                }
            }else{
                // 存储包含回收站的页面路由
                let urlsData = [/enterprise/,/person/]
                let storageUrlTag = null
                urlsData.map(k=>{
                    if (nowUrls.indexOf(k) >= 0) {
                        storageUrlTag = k
                    }
                })
                let cd = nowUrls.split(`${storageUrlTag}`)
                return cd[1]
            }
        }
        /*
        * 创建于2022年8月
        * 是否索引成功
        * @param {data} currentDatas - 当前实体数据对象
        * @return false | true
        * 以文搜文优化（缺陷34605）优化
        */

       corePt.keyJumpMethod = function(data){
            $('.right-menu > .menu').hide();
            // modify: 2022-09-02 cuizhichao #36853以图搜图结果页面中，选中文件，点击操作按钮、右键菜单的搜图，提示 文件内容未进索引；
            if (data.toolparam2017 && data.toolparam2017 == '以图搜图') {
                return true;
            }else  if (data && data.name) {
                var list = data.name.split('.');
                var fileName = '.' + list[list.length - 1];
                var textType = textList.some(v => fileName.toUpperCase().includes(v.toUpperCase()));
                if( window.location.href.includes('search?') && data.data.content == ""){
                    // modify: cuizhichao 2022-09-08 #37119 未被索引的文件以文搜文后，不应该显示出搜索内容；(右键菜单、toolbar需同步修改)
                    app.trigger("fileNotIndexed",true)
                    app.trigger("message:show", { type: 'error', message: lang.fileNotIndexed });
                    return false
                }
                
                if(textType && data.data.IndexState != 2 && !window.location.href.includes('search?')){
                    app.trigger("fileNotIndexed",true)
                    app.trigger("message:show", { type: 'error', message: lang.fileNotIndexed });
                    return false
                }else{
                    return true
                }
            }else{
                return true
            }
        }
        // modify 2022-09-05 liuchunli #36922系统管理-配置管理-右侧面板中关闭对应功能按钮显示后，前台文档库的功能按钮也要跟随隐藏 
        corePt.rightModelPanel = function() {
            if (this.view){
                if (this.view.moduleName == "enterprise" || this.view.moduleName == "team" || this.view.moduleName == "person"){
                    if (this.view.currentDatas.length == 1 && (this.view.currentDatas[0].isFolder == false || this.view.currentDatas[0].data.fileType == 2)){
                        var entityType = 'singlefile';
                        var type = 'singlefile';
                        var filetype = this.view.currentDatas[0].data.fileType;
                        var curLanger = $.getLan();
                        var select = this.view.currentDatas;
                        var optiontype = '';
                        var path = this.view.moduleName;
                        if (this.view.moduleName == "enterprise"){
                            optiontype = "1001";
                        }else if (this.view.moduleName == "team"){
                            optiontype = "7001";
                        }else if (this.view.moduleName == "person"){
                            optiontype = "3001";
                        }
                        var showModuels = constinfo.getTypeRightList(optiontype, type, filetype, path, curLanger, entityType, select);
                        return showModuels;
                    }

                }
            }

        }
        function resizeSizeAll(Flag) {
            if (Flag == '-857') {
                jquery(".container-docsetbar").css({
                    width: '100px',
                })
            }else  if (Flag == "-1") {
                jquery(".container-docsetbar").css({
                    width: '50px',
                })
            }else{
                jquery(".container-docsetbar").css({
                    width: '250px',
                })
            }
        }

        jquery(window).bind('hashchange', function() {
            let Flag = getDeleteFlag(decodeURI(window.location.href))
            if (Flag == "-1") {
                // modify: 2022-04-26 cuizhichao 29737回收站打开右侧面板，界面缩放页面至遮挡部分搜索框，F5刷新页面，操作按钮显示...，hover下只显示刷新按钮，点击刷新按钮，或者回收站导航栏，按钮恢复正常
                if ( Flag == '-857') {
                    jquery(".docsetbar-btn-list .docsetbar-btn").eq(3).css({
                        display:'none'
                    })
                    jquery(".docsetbar-btn-list .docsetbar-btn").eq(2).css({
                        display:'none'
                    })
                }
                jquery(".container-docsetbar").css({
                    width: '50px',
                })
                return
            }else{
                jquery(".container-docsetbar").css({
                    width: '250px',
                })
            }
        });   
        // modefi: 2022-04-25 cuizhicaho 26236【产品未提供设计】【UI】页面缩放状态下，三库回收站搜索按钮点击无反应  -- 结束
        corePt.buildToolbar = function (currentDatas, moduleName) {  
            // console.log('buildToolbar');
            this.filterMenu(currentDatas, moduleName);
            var that = this;
            var buttonList = this.view.buttonList;
            
            if (!this.filterMenuData || !this.filterMenuData.length) return;//37219【搜索结果页】选中无权限的文件刷新后，按钮消失

            var jqToolbar = this.view.jqToolbar;
            // 解决报错
            if (moduleName == 'preview'  && jqToolbar) {//预览页面按钮数量可配置,先设置大点，后面计算后会修改
                jqToolbar.parent().width(2000);
            }

            var maxWidth = jqToolbar && jqToolbar.parent().width();
            if (maxWidth <= 0) return; //如果宽度为0，肯定有一个Bug，但没找到原因 modify: 2018-01-31
            // (jqToolbar.parent().find(".right-panel").length > 0 ? jqToolbar.parent().find(".right-panel").width() : this.distance)- 
            var rightWidth = 250
            if(jqToolbar.parent().find(".PreviewModelView").length > 0){
                rightWidth = jqToolbar.parent().find(".PreviewModelView").length > 0 ? jqToolbar.parent().find(".PreviewModelView").width() : this.distance
            }
            var toolbarMaxWidth = maxWidth- (jqToolbar.parent().find(".docsetbar").length > 0 ? jqToolbar.parent().find(".docsetbar").width() : this.distance) - this.moreBtn.width - (jqToolbar.parent().find(".right-panel").length > 0 ? jqToolbar.parent().find(".right-panel").width() : this.distance)-rightWidth;
            var toolbarCurrWidth = 0;
            var hideBtnList = [];
            buttonList.removeAll();
            toolbarMaxWidth = window.parseInt(toolbarMaxWidth * 1);
            if ($(document).width() > 1400) {
                toolbarMaxWidth = window.parseInt(toolbarMaxWidth * 0.8);
            }
            // modify: 2022-04-26 cuizhichao 29737回收站打开右侧面板，界面缩放页面至遮挡部分搜索框，F5刷新页面，操作按钮显示...，hover下只显示刷新按钮，点击刷新按钮，或者回收站导航栏，按钮恢复正常
            let Flag = getDeleteFlag( decodeURI(window.location.href))
            resizeSizeAll(Flag)
            if (Flag == "-1" || Flag == '-857') {
                if ( Flag == '-857') {
                    jquery(".docsetbar-btn-list .docsetbar-btn").eq(3).css({
                        display:'none'
                    })
                    jquery(".docsetbar-btn-list .docsetbar-btn").eq(2).css({
                        display:'none'
                    })
                }
                toolbarMaxWidth = jquery(".center-panel").width() 
            }else{
                toolbarMaxWidth=toolbarMaxWidth
            }
            var filterMenuDataTemp = [];
            var filterMenuProcess;
            var previewBtnCount = 0;
            
            //modify:ldb pi:1407流程操作菜单
            for (var i = 0; i < this.filterMenuData.length; i++) {
                if (this.filterMenuData[i].command === 'doc/onlineview/index') {
                    app.trigger("attraction:hide", {
                        message: 'preview',
                        select: that.view.currentDatas[0],
                        module: that.filterMenuData[i].command
                    });
                }
                //超出最大长度时,添加进更多菜单
                if (this.filterMenuData[i].command == "doc/tactic/processStrategy/processStrategy") {
                    filterMenuProcess = this.filterMenuData[i];
                    //modify: liyangyang 2022-03-07 [缺陷26231] 工具栏按钮自适应问题
                    //如果有流程操作按钮,最大宽度-100
                    toolbarMaxWidth -= 100;
                } else {
                    //流程外发去掉修改按钮
                    if (moduleName == "mypublish") {
                        if (currentDatas != undefined && currentDatas.length > 0) {
                            //Publish_State = 1  流程外发
                            if (currentDatas[0].data.Publish_State == 1 && this.filterMenuData[i].command == "doc/publish/index") {
                                continue;
                            }
                            //edit by hanshunlong,20220707，缺陷33998，外网外发在我的外发中可以进行修改（与产品确认不能进行修改）
                            if (currentDatas[0].data.Publish_State == '2' && this.filterMenuData[i].command == "doc/publish/index") {
                                continue;
                            }
                        }
                        // modify 2022-04-01 zhengjie 28178【ui】我的共享、我的外发界面在无内容时，建议取消：【取消所有过期】按钮
                        if(currentDatas != undefined && currentDatas[0].data.count == 0){
                            if(this.filterMenuData[i].command == 'doc/quick/mypublish/cancelallpublish/index'){
                                continue;
                            }
                        }
                    }
                    // modify 2022-04-01 zhengjie 28178【ui】我的共享、我的外发界面在无内容时，建议取消：【取消所有过期】按钮
                    if(moduleName == "myshare"){
                        // modify 2022-04-26 cuizhichoa 还原代码冲突误删的代码
                        if (currentDatas != undefined && (currentDatas[0].myShareRecordCount == undefined || currentDatas[0].myShareRecordCount == 0)){
                            if(this.filterMenuData[i].command == 'doc/quick/myshare/cancelAllExpiredShare/index'){
                                continue;
                            }
                        }
                    }

                    if(moduleName == "myshare"){
                        if(window.location.hash.indexOf('shareType=2') > -1){
                            if(this.filterMenuData[i].command == 'doc/quick/myshare/cancelAllExpiredShare/index'){
                                continue;
                            }
                        }
                        
                    }
                    // if(moduleName == "myshare"){
                    //     if (currentDatas != undefined && (currentDatas[0].message == undefined || currentDatas[0].message == "sharerecord")){
                    //         if(this.filterMenuData[i].command == 'doc/quick/myshare/cancelAllExpiredShare/index'){
                    //             continue;
                    //         }
                    //     }
                    // }

                    //modify 2022-04-20 zhengjie 29072【我的消息】没有数据建议去掉“全部标为已读 ”（产品已确认）
                    if(moduleName == "mymessage"){
                        if (currentDatas != undefined && currentDatas[0].data.count == 0) {
                            if(this.filterMenuData[i].command == 'doc/quick/mymessage/read'){
                                continue;
                            }
                        }
                    }
                    
                    //过滤企业库操作日志按钮
                    if(moduleName == "enterprise"){
                        if (currentDatas != undefined) {
                            if(this.filterMenuData[i].command == 'doc/docoptlog/index'){
                                continue;
                            }
                        }
                    }
                    // modify: 2023-01-09 mengfc #41832 开启加解密隐藏副本下载按钮
                    var confInfo = constInfo.getSysConfigInfo();
                    if (confInfo.encryptpolicy == 1) {
                        /*buttonList = buttonList.remo(x => x.command = 'doc/downloadformat/index')*/
                        if (currentDatas != undefined) {
                            if (this.filterMenuData[i].command == 'doc/downloadformat/index') {
                                continue;
                            }
                        }
                    }
                    filterMenuDataTemp.push(this.filterMenuData[i]);
                }
            };
            // modify: 2022-09-13 cuizhichao #37298 团队库成员下载权限时，收藏文件/文件夹后，显示出操作日志按钮；
            // modify: 2022-09-13 cuizhichao #37299 企业库文档位编辑权限时，收藏文件后，显示出收缩按钮，实际并没有按钮被收缩；收藏文件夹显示出操作日志按钮；(#37298修改后此缺陷同时解决)
            filterMenuDataTemp.map((item,index)=>{
                if (item.command == 'doc/docoptlog/index') {
                    filterMenuDataTemp.splice(index,1)
                }
            })
            if (this.filterMenuData[0] && this.filterMenuData[0].command == "doc/quick/myrecommendList/exchange/index") {
                var lang = constInfo.getLang();
                if (lang == "en") {
                    this.filterMenuData[0].text = this.filterMenuData[0].lang.en;
                }
                else if (lang == "ja") {
                    this.filterMenuData[0].text = this.filterMenuData[0].lang.ja;
                }
                else if (lang == "zh-tw") {
                    this.filterMenuData[0].text = this.filterMenuData[0].lang.zhtw;
                }
            }
            function LaodToolbar($this) {
                var  rightModelPanel = $this.rightModelPanel();
                if (rightModelPanel) {
                    var isInfo = rightModelPanel.some(item => item.name == "historyFile");
                    if (!isInfo) {
                        filterMenuDataTemp = filterMenuDataTemp.filter(item => (item.command !== "doc/historyVersion/index"))
                    }
                }
                if(moduleName == 'team'){
                    filterMenuDataTemp = filterMenuDataTemp.filter(item => (item.command !== "doc/docoptlog/index"))
                }
                var licInfo = constInfo.getLicInfo();

                $.each(filterMenuDataTemp, function (index, obj) {
                    if (obj.command == "NewMenu" || obj.command == "doc/folder/create/index") {
                        var newMenu = ko.observableArray([
                            { command: "doc/folder/create/index", icon: "folder16", text: "新建文件夹", lang: { "zhcn": "新建文件夹", "zhtw": "新建資料夾", "en": "New Folder", "ja": "新規フォルダ" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['template'] },
                            { command: "doc/folder/createoffice/index", icon: "docx32", text: "Word文档", lang: { "zhcn": "Word文档", "zhtw": "Word檔案", "en": "Word Document", "ja": "Wordドキュメント" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['template'] },
                            { command: "doc/folder/createoffice/index", icon: "ppt", text: "PowerPoint文档", lang: { "zhcn": "PowerPoint文档", "zhtw": "PowerPoint檔案", "en": "PPT Document", "ja": "PowerPointファイル" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [], moduleType: ['template'] },
                            { command: "doc/folder/createoffice/index", icon: "excel", text: "Excel文档", lang: { "zhcn": "Excel文档", "zhtw": "Excel檔案", "en": "Excel Document", "ja": "Excelドキュメント" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['template'] },
                            // modify 2022-04-01 zhengjie 27033【产品】多语言-日语-新建-从模板新建-折行样式优化 jp优化
                            { command: "doc/filetemplate/index", icon: "templet", text: "从模板新建", lang: { "zhcn": "从模板新建文档", "zhtw": "從範本新建檔案", "en": "New from templates", "ja": "テンプレートから新規作成" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['teamfolder', 'template'] }
                        ]);
                        //判断lic
                        if (licInfo.EnabledModules.includes('DocFlow')){
                            //添加轻文档创建入口
                            //团队库个人库暂无轻文档入口 enterprise, moduleName == 'person' || moduleName == 'team'
                            //企业库,团队库,个人库工具栏有创建,其它无,比如 系统管理文档模板
                            if (moduleName == 'enterprise' || moduleName == 'person' || moduleName == 'teamfolder'|| moduleName == 'team'){
                                newMenu.splice(1, 0, { command: "doc/folder/createdocflow/index", icon: "mdoc-1", text: "在线文档", lang: { "zhcn": "在线文档", "zhtw": "在线檔案", "en": "mWord Document", "ja": "mWordドキュメント" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['template'] })
                            }
                        }
                        $.each(newMenu(), function (i, o) {
                            var lang = constInfo.getLang();
                            switch (lang) {
                                case "en":
                                    o.text = o.lang.en;
                                    break;
                                case "ja":
                                    o.text = o.lang.ja;
                                    break;
                                case "zh-tw":
                                    o.text = o.lang.zhtw;
                                    break;
                            }
                        })
                        obj.children = newMenu;
                    }
                    if (window.location.href.indexOf('outpublish.html?') <= -1) {
                        if (obj.command == "component/upload/index") {
                            var newMenu = ko.observableArray([{ command: "uploadFile", icon: "", text: "上传文件", lang: { "zhcn": "上传文件", "zhtw": "上傳文件", "en": "File(s)", "ja": "新規フォルダ" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['template'] },
                                { command: "uploadFiles", icon: "", text: "上传文件夹", lang: { "zhcn": "上传文件夹", "zhtw": "上傳文件夾", "en": "Folder", "ja": "Wordドキュメント" }, entryType: ["teamcreator", "teamadmin", "teaminsider", "sysinsider", 'publicroot', 'personroot', "mix"], perm: [permtype.createFolder], moduleType: ['template'] },
                            ]);
                            $.each(newMenu(), function (i, o) {
                                var lang = constInfo.getLang();
                                switch (lang) {
                                    case "en":
                                        o.text = o.lang.en;
                                        break;
                                    case "ja":
                                        o.text = o.lang.ja;
                                        break;
                                    case "zh-tw":
                                        o.text = o.lang.zhtw;
                                        break;
                                }
                            })
                            obj.children = newMenu;
                        }
                    }
                    if ($.getQueryString("ispublish") == "true" && obj.command == "doc/downloadformat/index") {
                        obj.command = "outpublish/plugins/downloadformat/index"
                    }
                    // modify 20220801 by zhenyu.wang 副本下载
                    //var officeList = ['doc', 'docx', 'ppt', 'pptx'];
                    //if (obj.command == "doc/download/index" || obj.command == "outpublish/plugins/download/index" || obj.command == "outpublish/plugins/downloadformat/index") {
                    //    if ($this.view.currentDatas) {
                    //        if ($this.view.currentDatas.length == 1 && $this.view.currentDatas[0].isFolder != true && window.location.href.indexOf('preview.html?') > -1) {

                    //            var docType = $this.view.currentDatas[0].name.substring($this.view.currentDatas[0].name.lastIndexOf('.') + 1);
                    //            var textType = officeList.some(v => docType.toUpperCase().includes(v.toUpperCase()));
                    //            if (textType) {

                    //                var newMenu = ko.observableArray([{ command: "doc/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "Download", "ja": "ダウンロード" }, entryType: ["folder", "file", "files", "folders", "mix", "shareme"], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'template', 'preview'] },
                    //                    { command: "doc/downloadformat/index", icon: "downloadformat", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載", "en": "PDF Download", "ja": "レプリカのダウンロード" }, entryType: ["file",], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'preview'] },
                    //                ]);
                    //                if (obj.command == "outpublish/plugins/download/index") {
                    //                    newMenu = ko.observableArray([{ command: "outpublish/plugins/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "Download", "ja": "ダウンロード" }, entryType: ["folder", "file", "files", "folders", "mix", "shareme"], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'template', 'preview'] },
                    //                        { command: "outpublish/plugins/downloadformat/index", icon: "downloadformat", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載", "en": "PDF Download", "ja": "レプリカのダウンロード" }, entryType: ["file",], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'preview'] },]);
                    //                }
                    //                if (obj.command == "outpublish/plugins/downloadformat/index") {
                    //                    newMenu = ko.observableArray([{ command: "outpublish/plugins/downloadformat/index", icon: "downloadformat", text: "副本下载", lang: { "zhcn": "副本下载", "zhtw": "副本下載", "en": "PDF Download", "ja": "レプリカのダウンロード" }, entryType: ["file",], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'preview'] },]);
                    //                    obj.icon = "download";
                    //                } 
                    //                var confInfo = constInfo.getSysConfigInfo();
                    //                if (confInfo.encryptpolicy == 1) {
                    //                    newMenu = ko.observableArray([{ command: "doc/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "Download", "ja": "ダウンロード" }, entryType: ["folder", "file", "files", "folders", "mix", "shareme"], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'template', 'preview'] },
                    //                    ]);
                    //                    if (obj.command == "outpublish/plugins/download/index") {
                    //                        newMenu = ko.observableArray([{ command: "outpublish/plugins/download/index", icon: "download", text: "下载", lang: { "zhcn": "下载", "zhtw": "下載", "en": "Download", "ja": "ダウンロード" }, entryType: ["folder", "file", "files", "folders", "mix", "shareme"], perm: [permtype.download], moduleType: ['enterprise', 'team', 'person', 'template', 'preview'] }]);
                    //                    }
                    //                }
                    //                $.each(newMenu(), function (i, o) {
                    //                    var lang = constInfo.getLang();
                    //                    switch (lang) {
                    //                        case "en":
                    //                            o.text = o.lang.en;
                    //                            break;
                    //                        case "ja":
                    //                            o.text = o.lang.ja;
                    //                            break;
                    //                        case "zh-tw":
                    //                            o.text = o.lang.zhtw;
                    //                            break;
                    //                    }
                    //                })
                    //                obj.children = newMenu;
                    //                obj.command = "";

                    //            } 

                                
                    //        } 
                    //    }
                        
                    //}
                    //if (obj.command == "doc/downloadformat/index" || obj.command == "outpublish/plugins/downloadformat/index") {
                    //    if ($this.view.currentDatas && $this.view.currentDatas.length > 0) {
                    //        var docType = $this.view.currentDatas[0].name.substring($this.view.currentDatas[0].name.lastIndexOf('.') + 1);
                    //        var textType = officeList.some(v => docType.toUpperCase().includes(v.toUpperCase()));
                    //        if (!textType) {
                    //            return;
                    //        }
                    //    }
                    //}
                    if (obj.command == 'doc/onlineview/index') {
                        var saveTimeText = { "zh-cn": "保存于", "zh-tw": "保存於", "en": "Save in", "ja": "保存＃ホゾン＃" };
                        var lang = constInfo.getLang();

                        obj.saveTime = currentDatas[0].modifyTimeV;
                        obj.saveText = saveTimeText[lang];
                    }
                    if (obj.moduleType && obj.moduleType[0] == "intag") {
                        var lang = constInfo.getLang();
                        if (lang == "en") {
                            obj.text = obj.lang.en;
                        } else if (lang == "ja") {
                            obj.text = obj.lang.ja;
                        } else if (lang == "zh-tw") {
                            obj.text = obj.lang.zh - tw;
                        }

                    }



                    if (obj) {
                        if (moduleName == 'preview') {//预览页面按钮数量可配置
                            var previewMenuNum = constInfo.systemInfo.instanceConfigInfo.PreviewMenuNum;
                            if (previewBtnCount < previewMenuNum) {
                                buttonList.push(obj);
                                previewBtnCount++;
                            }
                            else {
                                hideBtnList.push(obj);
                            }
                        }
                        else {
                            //超出最大长度时,添加进更多菜单
                            if (toolbarMaxWidth < toolbarCurrWidth) {
                                //if (obj.command == "doc/tactic/processStrategy/processStrategy") {
                                //    for (var i = 0; i < obj.children.length; i++) {
                                //        hideBtnList.push(obj.children[i]);
                                //    }
                                //} else {
                                //    hideBtnList.push(obj);
                                //}
                                // modify: 2022-04-26 cuizhichao 29737回收站打开右侧面板，界面缩放页面至遮挡部分搜索框，F5刷新页面，操作按钮显示...，hover下只显示刷新按钮，点击刷新按钮，或者回收站导航栏，按钮恢复正常
                                hideBtnList.push(obj);
                            }
                            else {
                                if (obj.moduleType.indexOf("sharetome") != -1 && obj.command.indexOf("doc/delete/index") != -1) { return; }
                                buttonList.push(obj);
                                toolbarCurrWidth = jqToolbar.width();
                                //超出最大长度时,添加进更多菜单
                                if (toolbarMaxWidth < toolbarCurrWidth) {
                                    //if (obj.command == "doc/tactic/processStrategy/processStrategy") {
                                    //    var children = buttonList.pop().children;
                                    //    for (var i = 0; i < children.length; i++) {
                                    //        hideBtnList.push(children[i]);
                                    //    }
                                    //} else {
                                    //    hideBtnList.push(buttonList.pop());
                                    //}
                                    // modify: 2022-04-26 cuizhichao 29737回收站打开右侧面板，界面缩放页面至遮挡部分搜索框，F5刷新页面，操作按钮显示...，hover下只显示刷新按钮，点击刷新按钮，或者回收站导航栏，按钮恢复正常
                                    hideBtnList.push(buttonList.pop());
                                }
                            }
                        }
                    }
                });

                // console.log(filterMenuDataTemp);
                //判断工具栏按钮是否包含创建轻文档,控制左侧导航创建轻文档按钮显示隐藏
                var createFileMenu = _.find(filterMenuDataTemp, function (item) {
                    return item.command == 'NewMenu';
                })

                // 判断轻文档不展示
                if (createFileMenu && window.location.hash!='#doc/quick/mydocflow'){
                    //显示左侧导航,轻文档创建按钮
                    app.trigger('docflowMenu:show', {showMenu : true});
                }
                else{
                    //隐藏左侧导航,轻文档创建按钮
                    app.trigger('docflowMenu:show', {showMenu : false});
                }

                //把不显示的按钮加入更多按钮下面
                if (hideBtnList.length > 0) {
                    //更多菜单里面存在2级的，平铺。
                    if (moduleName == 'preview') {
                        var temphideBtnList = [];
                        for (var i = 0; i < hideBtnList.length; i++) {
                            var itemBtn = hideBtnList[i];
                            if (itemBtn.children && itemBtn.children.length > 0) {
                                var tempChildren = itemBtn.children;
                                delete itemBtn.children;
                                temphideBtnList.push(itemBtn);
                                $.each(tempChildren, function (i, v) {
                                    temphideBtnList.push(v);
                                })
                            }
                            else {
                                temphideBtnList.push(itemBtn);
                            }
                        }
                        hideBtnList = temphideBtnList;
                    }

                    for (var i = 0; i < hideBtnList.length; i++) {
                        if (hideBtnList[i].icon.toLowerCase() == "cancel.png") {
                            hideBtnList[i].icon = "";
                        }
                        if (!hideBtnList[i].icon && (hideBtnList[i].command == "doc/quick/myshare/cancelshare/index" || hideBtnList[i].command == "doc/quick/mytag/removetag/index")) {
                            hideBtnList[i].icon = "cancel.png";
                        }
                    }
                    // PD-2030（UI）文件（夹）动态 cuizhichao 过滤掉企业、团队库更多按钮中的操作日志按钮
                    //modify:2022-06-22 by jiali.yao 缺陷32739 收藏中，勾选文件/文件夹，“更多”中过滤掉“操作日志”按钮
                    if (moduleName == 'enterprise' || moduleName == 'team'|| moduleName=='favoritelist') {
                        var newhideBtnList = hideBtnList.filter(item => (item.command !== "doc/docoptlog/index"))
                        hideBtnList = newhideBtnList
                    }
                    //将流程操作菜单放到一级菜单最后边
                    if (filterMenuProcess) {
                        filterMenuProcess.children = ko.observableArray(filterMenuProcess.children);
                        buttonList.push(filterMenuProcess);
                    }
                    $this.moreBtn.children = ko.observableArray(hideBtnList);
                    buttonList.push($this.moreBtn);
                } else {
                    //将流程操作菜单放到一级菜单最后边
                    if (filterMenuProcess) {
                        filterMenuProcess.children = ko.observableArray(filterMenuProcess.children);
                        buttonList.push(filterMenuProcess);
                    }
                }
                if (moduleName == 'preview') {
                    var previewBtnWidth = jqToolbar.width() + 2;
                    jqToolbar.parent().width(previewBtnWidth);
                    setTimeout(function () { $(window).resize() }, 100);
                }
                // //在企业库中添加本地打开、本地定位按钮
                // if(moduleName=="enterprise" && currentDatas.length==1 && (currentDatas && currentDatas[0].data.fileType==2)){
                //     function addLocalBnt(){
                //         //往更多按钮下面追加本地定位、打开按钮
                //         if(buttonList()[buttonList().length-1].command=="more"){
                //             var sttr= { command: "doc/localOpen/index", 
                //                 icon: "localOpen.png", 
                //                 Id: 501, 
                //                 text: "本地打开",
                //                 lang:{zhcn: "本地打开", zhtw: "本地打開", en: "Local open", jp: "ローカルを開く"},
                //                 moduleType:["enterprise"],
                //                 sort:20,
                //                 text:"本地打开",
                //                 entryType:["file"],
                //                 isActive:ko.observable(false)};
                //             var sttr1={ command: "doc/localLocation/index", 
                //                         icon: "localLocation.png", 
                //                         Id: 502, 
                //                         text: "本地定位",
                //                         lang:{zhcn: "本地定位", zhtw: "本地定位", en: "Local location", jp: "ローカル位置"},
                //                         moduleType:["enterprise"],
                //                         sort:21,
                //                         entryType:["file"],
                //                         text:"本地定位",
                //                         isActive:ko.observable(false)};
                //             buttonList()[buttonList().length-1].children.push(sttr);
                //             buttonList()[buttonList().length-1].children.push(sttr1);
                //         }
                //     }
                //     if(isLocalBnt){
                //         addLocalBnt();
                //     }else{
                //         var ws = new WebSocket("ws://127.0.0.1:54321");
                //         ws.onopen = function()
                //         {
                //             isLocalBnt=true;
                //             addLocalBnt();
                //         };
                //         // 断开 web socket 连接成功触发事件
                //         ws.onclose = function () {
                //             isLocalBnt=false;
                //         };
                //     }
                // }
                
            }
            //tian.ao 判断本机装有vd并且vd是登入状态才显示本地定位、打开两按钮 
            if (moduleName == 'preview') {
                //判断是否有下载权限才显示按钮
                if (isLocalOpen && (currentDatas && currentDatas[0].type == "share" || (currentDatas && currentDatas[0].permission & permtype.download) == permtype.download)) {
                    var isIE = (!!window.ActiveXObject || "ActiveXObject" in window);
                    var userAgent = navigator.userAgent;
                    var isFF = userAgent.indexOf("Firefox") > -1;
                    //附件预览按钮只有下载和打印
                    if ($.getQueryString("attachment") == "true") {
                        var temp = [];
                        $.each(filterMenuDataTemp, function (i, v) {
                            if (v.command == "doc/download/index" || v.command == "doc/print/index") {
                                temp.push(v);
                            }
                        })
                        filterMenuDataTemp = temp;
                        //易写过滤
                        if (constInfo.getLicInfo().ProductType == "InWriter") {
                            var temp = [];
                            $.each(filterMenuDataTemp, function (i, v) {
                                if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                                    return true;
                                }
                                if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                                    //过滤本地编辑
                                    //过滤本地定位
                                    //过滤本地打开
                                    //过滤扩展编辑
                                    temp.push(v);
                                }
                                filterMenuDataTemp = temp;
                            })
                        }
                        LaodToolbar(this);

                    }
                    else if ((isIE || isFF) && location.protocol.toLocaleLowerCase() == "https:") {
                        //var curentModule = require('knockout').$store.state.curentModule();

                        //删除本地定位、打开按钮
                        //filterMenuDataTemp.splice(filterMenuDataTemp.length - 2, 2);
                        var attr = [];
                        $.each(filterMenuDataTemp, function (i, r) {
                            if (r.command != "doc/localOpen/index" && r.command != "doc/localLocation/index" && r.command != "doc/localEdit/index") {
                                attr.push(r);
                            }
                        });
                        filterMenuDataTemp = attr;
                        //易写过滤
                        if (constInfo.getLicInfo().ProductType == "InWriter") {
                            var temp = [];
                            $.each(filterMenuDataTemp, function (i, v) {
                                if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                                    return true;
                                }
                                if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                                    //过滤本地编辑
                                    //过滤本地定位
                                    //过滤本地打开
                                    //过滤扩展编辑
                                    temp.push(v);
                                }
                                filterMenuDataTemp = temp;
                            })
                        }
                        LaodToolbar(this);
                    }
                    else {
                        // 初始化一个 WebSocket 对象
                        var ws = new WebSocket("ws://127.0.0.1:54321");
                        var $this = this;
                        if (!(window.location.href.toLowerCase().indexOf('sharecode') > -1) && !(window.location.href.toLowerCase().indexOf('code') > -1)) {
                            ws.onopen = function () {
                                isLocalOpen = false;
                                isSuccess = true;
                                //会触发会回调多次此函数 但只需执行一次此逻辑
                                opendIndex++;
                                if (opendIndex == 1) {
                                    //if (filterMenuDataTemp.length >= 3) {
                                    //    var temp = filterMenuDataTemp[1];
                                    //    filterMenuDataTemp[1] = filterMenuDataTemp[filterMenuDataTemp.length - 2];
                                    //    filterMenuDataTemp[filterMenuDataTemp.length - 2] = temp;
                                    //}
                                    //易写过滤
                                    if (constInfo.getLicInfo().ProductType == "InWriter") {
                                        var temp = [];
                                        $.each(filterMenuDataTemp, function (i, v) {
                                            if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                                                return true;
                                            }
                                            if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                                                //过滤本地编辑
                                                //过滤本地定位
                                                //过滤本地打开
                                                //过滤扩展编辑
                                                temp.push(v);
                                            }
                                            filterMenuDataTemp = temp;
                                        })
                                    }

                                    LaodToolbar($this);
                                }
                            };

                            ws.onerror = function (evt) {
                                isLocalOpen = false;
                                //ie下会隔段时间自动断开连接 
                                if (isSuccess) {
                                    return;
                                }
                                //如果连接不上会触发会回调多次此函数 但只需执行一次此逻辑
                                index++;
                                if (index == 1) {
                                    //删除本地定位、打开按钮
                                    //filterMenuDataTemp.splice(filterMenuDataTemp.length - 2, 2);
                                    var attr = [];
                                    $.each(filterMenuDataTemp, function (i, r) {
                                        if (r.command != "doc/localOpen/index" && r.command != "doc/localLocation/index" && r.command != "doc/localEdit/index") {
                                            attr.push(r);
                                        }
                                    })
                                    filterMenuDataTemp = attr;
                                    //易写过滤
                                    if (constInfo.getLicInfo().ProductType == "InWriter") {
                                        var temp = [];
                                        $.each(filterMenuDataTemp, function (i, v) {
                                            if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                                                return true;
                                            }
                                            if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                                                //过滤本地编辑
                                                //过滤本地定位
                                                //过滤本地打开
                                                //过滤扩展编辑
                                                temp.push(v);
                                            }
                                            filterMenuDataTemp = temp;
                                        })
                                    }
                                    LaodToolbar($this);
                                }
                            };
                        } else {
                            var attr = [];
                            $.each(filterMenuDataTemp, function (i, r) {
                                if (r.command != "doc/localOpen/index" && r.command != "doc/localLocation/index" && r.command != "doc/localEdit/index") {
                                    attr.push(r);
                                }
                            })
                            filterMenuDataTemp = attr;
                            //if (window.location.href.toLowerCase().indexOf('sharecode') > -1 && filterMenuDataTemp.length >= 2 && filterMenuDataTemp[0].command == "doc/print/index") {
                            //    var temp = filterMenuDataTemp[1];
                            //    filterMenuDataTemp[1] = filterMenuDataTemp[filterMenuDataTemp.length - 2];
                            //    filterMenuDataTemp[filterMenuDataTemp.length - 2] = temp;
                            //}
                            //易写过滤
                            if (constInfo.getLicInfo().ProductType == "InWriter") {
                                var temp = [];
                                $.each(filterMenuDataTemp, function (i, v) {
                                    if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                                        return true;
                                    }
                                    if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                                        //过滤本地编辑
                                        //过滤本地定位
                                        //过滤本地打开
                                        //过滤扩展编辑
                                        temp.push(v);
                                    }
                                    filterMenuDataTemp = temp;
                                })
                            }
                            LaodToolbar(this);
                        }
                    }
                } else {
                    if (!((currentDatas && currentDatas[0].permission & permtype.preview) == permtype.preview)) {
                        if (filterMenuDataTemp[0].command == "doc/favorite/index" || filterMenuDataTemp[0].command == "doc/unfavorite/index") {
                            filterMenuDataTemp.shift(0, 1)
                        }
                    }
                    if (!isSuccess) {
                        //删除本地定位、打开按钮
                        //filterMenuDataTemp.splice(filterMenuDataTemp.length-2,2);
                        var attr = [];
                        $.each(filterMenuDataTemp, function (i, r) {
                            if (r.command != "doc/localOpen/index" && r.command != "doc/localLocation/index" && r.command != "doc/localEdit/index") {
                                attr.push(r);
                            }
                        })
                        filterMenuDataTemp = attr;
                    } else {
                        //if (filterMenuDataTemp.length >= 3) {
                        //    var temp = filterMenuDataTemp[1];
                        //    filterMenuDataTemp[1] = filterMenuDataTemp[filterMenuDataTemp.length - 2];
                        //    filterMenuDataTemp[filterMenuDataTemp.length - 2] = temp;
                        //}
                    }
                    //易写过滤
                    if (constInfo.getLicInfo().ProductType == "InWriter") {
                        var temp = [];
                        $.each(filterMenuDataTemp, function (i, v) {
                            if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                                return true;
                            }
                            if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                                //过滤本地编辑
                                //过滤本地定位
                                //过滤本地打开
                                //过滤扩展编辑
                                temp.push(v);
                            }
                            filterMenuDataTemp = temp;
                        })
                    }
                    LaodToolbar(this);
                }
            } else {
                //易写过滤
                if (constInfo.getLicInfo().ProductType == "InWriter") {
                    var temp = [];
                    $.each(filterMenuDataTemp, function (i, v) {
                        if (constInfo.getLicInfo().InWriterProductModel == "standard" && v.command == "doc/informationbar/comment/quickCreate") {
                            return true;
                        }
                        if (v.command != "doc/localEdit/index" && v.command != "doc/localLocation/index" && v.command != "doc/localOpen/index" && v.command != 'doc/extedit/extedit') {

                            //过滤本地编辑
                            //过滤本地定位
                            //过滤本地打开
                            //过滤扩展编辑
                            temp.push(v);
                        }
                        filterMenuDataTemp = temp;
                    })
                }
                LaodToolbar(this);
            }
        }

        /**
         * 工具栏组件对象
         */
        function Toolbar(name) {
            /**
             * 工具栏应用位置名称，用于resize事件中的判断
             */
            this.name = name;
            /**
             * 工具栏按钮列表
             */
            this.buttonList = ko.observableArray();
            this.core = new ToolbarCore(this);
            /**
             * toolbar的jquery对象
             */
            this.jqToolbar = null;

            this.currentDatas = [];
            this.message = null;
            this.moduleName = null;
            this.maskHandlerId;
            this.isLoadSearch = false;
            //modify 20220322 by xuying 缺陷28180 文件预览界面右上角按钮提示对应文字
            this.isShowTip = window.location.href.indexOf('preview.html') >= 0 ? ko.observable(true) : ko.observable(false);

            this.viewUrl = 'doc/toolbar/index.html';

        }
        var toolbarPt = Toolbar.prototype;
        /**
       对more按钮显示内容进行重载（预览界面需要）
       */
        toolbarPt.reNameForMore = function () {
            this.core.moreBtn.text = lang.moreOpt;
        }
        toolbarPt.activate = function () {
        }
        toolbarPt.compositionComplete = function (child, parent, settings) {
            this.jqToolbar = $(child);
            
        }
        toolbarPt.detached = function () {
            // console.log('toolbar detached');
        }
        toolbarPt.onSubmenuMouseover = function (data, event) {
            if ($(event.currentTarget).children("div").get(0)) {
                var left = $(event.currentTarget).offset().left + $(event.currentTarget).width();
                var top = $(event.currentTarget).offset().top;
                threeButMenu = $(event.currentTarget);
                $($(event.currentTarget).children("div").get(0)).show();
            }
        }
        toolbarPt.onSubmentMouseout = function (data, event) {
            if ($(event.currentTarget).children("div").get(0)) {
                $($(event.currentTarget).children("div").get(0)).hide();
            }
        }
        /**
         * 刷新工具栏按钮
         * @param {*} cur 
         */
        toolbarPt.refreshData = function (cur) {
            if (!cur) return;
            // modify: #33133 2022-07-07 cuizhichao 后台上传文件时，无法选中新建文件夹
            // modify: #34378 2022-07-18 cuizhichao [UI] 选中文件（文件夹）执行搜索后，搜索结果页面中的操作按钮不消失；
            if (cur.select.localFolderSelect && $("#upload_icon").hasClass('uploading'))  return;
            var that = this;
            that.currentDatas = cur.select;
            that.message = cur.message;
            that.moduleName = cur.module;

            //modify: 2017-12-15 haibin.long 如果是搜索界面强制刷新，需要加一个标记
            //modify by 20220311 haiwei.cui 缺陷26876，搜索结果页面，点击刷新后，出现其他操作按钮（缩略图模式下）
            var hash = window.location.hash;
            if (hash.indexOf('search') >= 0 && that.currentDatas.length == 1 && that.currentDatas[0].folderType == 1) {
                var currFolderId = hash.split('/')[2];
                if (that.currentDatas[0].id == currFolderId) {
                    that.currentDatas[0].data.searchroot = true;
                    app.trigger('search:get-toolbar-selected-items', cur);
                }                
            }
            //构建toolbar菜单
            that.core.buildToolbar(that.currentDatas, that.moduleName);
            that.reload();
        }

        /**
         * 重新计算尺寸
         */
        toolbarPt.reLoadSize = function () {
            //modify: 2018-01-24 haibin.long 增加name属性，用于判断resize事件发起者是否为当前toolbar所属对象，防止 系统管理 中进行resize一把，doc就出问题
            if (this.name) {
                var regEx = new RegExp(this.name, 'gi');
                var moduleId = router.activeInstruction().config.moduleId;
                if (regEx.test(moduleId)) {
                    this.core.buildToolbar(this.currentDatas, this.moduleName);
                }
            } else {
                this.core.buildToolbar(this.currentDatas, this.moduleName);
            }
        }

        /*
        * mikahuang创建于2015年5月
        *鼠标点击菜单项时触发的事件
        * @data 记录上下对象
        * @event jquery包装的event对象
        * @return undefined
        */
        toolbarPt.doCommand = function ($data, event) {
            event.stopPropagation();
            contextmenu.onHidden();
            var that = this;
            if ($data.command == "uploadFile" || $data.command == "uploadFiles") {
                SelectUploadType($data.command)
                return
            }else if ($data.command == "component/upload/index") {
                    if (window.location.href.indexOf('outpublish.html?') > -1) {
                        app.trigger("upload:showUploadModal",1)
                        app.trigger("component:show", {
                            moduleId: $data.command,
                            datas: this.currentDatas,
                            message: this.message,
                            moduleName: this.moduleName
                        });
                    }else{
                        if (uploadFlag == 1) {
                            // modify : cuizhichao 2022-06-22 32595 【流程策略】设置新增流程后，点击上传会弹出选择文件弹窗和新增流程页面
                            app.trigger("component:show", {
                                moduleId: $data.command,
                                datas: this.currentDatas,
                                message: this.message,
                                moduleName: this.moduleName
                            });
                            app.trigger("upload:showUploadModal",uploadFlag)
                            uploadFlag = 0
                        }else if (uploadFlag == 2) {
                            // modify : cuizhichao 2022-06-22 32595 【流程策略】设置新增流程后，点击上传会弹出选择文件弹窗和新增流程页面
                            app.trigger("component:show", {
                                moduleId: $data.command,
                                datas: this.currentDatas,
                                message: this.message,
                                moduleName: this.moduleName
                            });
                            app.trigger("upload:showUploadModal",uploadFlag)
                            uploadFlag = 0
                        }
                    }
                return;
            }
            // modify: #33868 cuizhichao 选择文件拖拽上传，上传成功后，传输面板缩小后，选择子文件夹进行复制到目标文件夹下，传输面板框显示有误
            if ($data.command == 'doc/movecopy/paste/index') {
                app.trigger("upload:changeUploadModal",true)
            }

            if ($data.command == 'doc/search/thumbnail/index' || $data.command == 'doc/search/text/thumbnail/index') {
                // if (window.location.hash.includes("type=searchForPictures") || window.location.hash.includes("type=searchByText")) {
                // }else{
                //     $.cookie('clearSearchToggleHash',window.location.href);
                // }
                // 搜图、搜文后面包屑关闭操作优化（此处toolbar菜单，对应右键菜单同样需要修改）
                // if(!that.core.keyJumpMethod(that.currentDatas[0])) return 
                if (!window.location.href.includes('search?')) {
                    $.cookie('clearSearchToggleHash',window.location.href);
                }
                if ($data.command == 'doc/search/text/thumbnail/index') {
                    app.trigger("navigation:changeSearchHide", {searchText:that.currentDatas[0].name,isEnterprise:false  });
                }
            }
            //modify: 2022-02-23 jiangyang.li 解决搜文点击无效的问题，因为没进判断。
            if ($data.command == 'doc/search/thumbnail/index' || $data.command == 'doc/search/text/thumbnail/index') {
                // modify: 2022-09-02 cuizhichao #36853以图搜图结果页面中，选中文件，点击操作按钮、右键菜单的搜图，提示 文件内容未进索引；
                if ($data.command == 'doc/search/thumbnail/index') {
                    that.currentDatas[0].toolparam2017 = '以图搜图';
                }
                if(!that.core.keyJumpMethod(that.currentDatas[0])) return 
                // 根据url参数判断当前列表是主页面列表还是结果页列表
                //modify: 2022-01-25 jiangyang.li 跳转搜图页面找不到id跳转失败 判断匹配的方式修改，因为id是动态获取。
                var type = window.location.href.includes('/search');
                //modify: 2022-02-15 jiangyang.li 修复缺陷[26220] 点击一下搜图调用多次后端的搜图接口。
                // 获取cookie判断是列表还是缩略图
                // if (type) {
                //     // modify: 2022-02-25 jiangyang.li 新增：修复问题不是搜图或搜文模式下进入结果页，点击按钮搜图或搜文应该进入搜图或搜文模式。
                //     if (document.cookie.includes('searchThumb=true')) {
                //         app.trigger("Core:SearchResultThumbnail",{searchType: $data.command});
                //     } else {
                //         app.trigger("Core:searchList",{searchType: $data.command});
                //     }
                // } else {
                //     //modify: 2022-02-24 jiangyang.li 修复缺陷[26746]，多语言状态，选择文件，有以图搜图，以文搜文按钮显示。
                //     if (document.cookie.includes('enterpriseThumb=true')) {
                //         app.trigger("Thumbnail:abc", {searchType: $data.command});
                //     } else {
                //         app.trigger("List:abc", {searchType: $data.command});
                //     }
                // }
                // modify: 2022-03-02 jiangyang.li 新增：修复请求发送多次和路由被缓存的时候点击按钮无效。
                var url = 'doc/enterprise/1/search?searchkey=&type=' + ($data.command.includes('doc/search/thumbnail/index') ? 'searchForPictures' : 'searchByText') + '&fileid=' + that.currentDatas[0].data.id;
                router.navigate(url + ('&val=' + new Date().getTime()), { trigger: true, replace: true });
                // modify: 2022-03-01 jiangyang.li 新增：搜文不显示列表截图。
                if ($data.command == "doc/search/thumbnail/index") {
                    app.trigger("navigation:searchImgHide", { type: true });
                    // modify: 2022-03-15 jiangyang.li 缺陷[26226] 搜图结果页默认略缩图模式
                    document.cookie = "searchThumb=true";
                }
                return;
            }

            //modify: 2018-01-22 haibin.long 
            if ($data.command === 'more' || $data.command === 'NewMenu'){
                return;
            }

            //防止快速点击弹出多个窗口,但是 返回按钮需要及时响应
            if (this.maskHandlerId && $data.command !== "doc/goback/index" && $data.command !== "outpublish/plugins/back/index" && $data.command !== "more") {
                return;
            }
            //zhouwei 2017-02-20 添加一个参数 office新建
            if (this.currentDatas.length) {
                this.currentDatas[0].toolparam2017 = $data.text;
                //currentDatas[0].name = currentDatas[0].name;
            }
            this.maskHandlerId = 1;

            //流程
            var currentAction = "";
            if ($data.command === "doc/tactic/processStrategy/processStrategy") {

                if ($data.icon == "create.png") {
                    this.currentDatas.action = "create";
                }
                if ($data.icon == "update.png") {
                    this.currentDatas.action = "update";
                }
                if ($data.icon == "delete.png") {
                    this.currentDatas.action = "delete";
                }
                if ($data.icon == "attribute.png") {
                    this.currentDatas.action = "attribute";
                }

            }
            var curentModule = require('knockout').$store.state.curentModule();
            if ($data.command == "doc/folder/createoffice/index" || $data.command == "doc/filetemplate/index" || $data.command == "doc/folder/createdocflow/index") {
                if (curentModule && curentModule.module === "favoritelist") {
                    var data = process.IsProcessStrategy(this.currentDatas, 1);
                    if (data.IsProcessStrategy) {
                        app.trigger("message:show", { type: 'error', message: lang.ProcessEnabled });

                    } else {
                        app.trigger("component:show", {
                            moduleId: $data.command,
                            datas: this.currentDatas,
                            message: this.message,
                            moduleName: this.moduleName
                        });
                    }
                }
                else if (curentModule && curentModule.module === "enterprise") {
                    if (!process.getProcessStrategy(this.currentDatas, "create")) {
                        app.trigger("component:show", {
                            moduleId: $data.command,
                            datas: this.currentDatas,
                            message: this.message,
                            moduleName: this.moduleName
                        });
                    }
                } else {
                    app.trigger("component:show", {
                        moduleId: $data.command,
                        datas: this.currentDatas,
                        message: this.message,
                        moduleName: this.moduleName
                    });
                }

            } else {
                //附件上传
                if ($data.command == "doc/uploadAttach/index") {
                    app.trigger("upload:showUploadModal",1,true,this.currentDatas[0].data.id)
                    app.trigger("component:show", {
                        moduleId: 'component/upload/index',
                        datas: { masterFile: komapping.fromJS(this.currentDatas[0].data) },
                        moduleName: "",
                        message: "informationbarattachment:reload"
                    });
                } else if ($data.command == "doc/recyclebin/search/index") {
                    app.trigger('recyclebin:list');
                }
                else {
                    // //点击菜单项时触发,弹出具体窗体
                    // app.trigger("component:show", {
                    //     moduleId: $data.command,
                    //     datas: this.currentDatas,
                    //     message: this.message,
                    //     moduleName: this.moduleName
                    // });
                    if(this.moduleName == "myshare"){
                        var currentDatas = this.currentDatas;
                        currentDatas[0].module = 'sharetome';
                        app.trigger("component:show", {
                            moduleId: $data.command,
                            datas: currentDatas,
                            message: this.message,
                            moduleName: 'sharetome'
                        });
                    }else{
                        app.trigger("component:show", {
                            moduleId: $data.command,
                            datas: this.currentDatas,
                            message: this.message,
                            moduleName: this.moduleName
                        });
                    }
                    
                }

            }
            setTimeout(function () {
                that.maskHandlerId = 0;
            }, 1000);
        }
        /*
       * mikahuang创建于2015年5月
       *鼠标移入菜单项事件
       * @$data 记录上下对象
       * @event jquery包装的event对象
       * @return undefined
       */
        toolbarPt.submenuMouseover = function ($data, event) {
            //移入菜单项
            // modify: 2022-09-01 cuizhichao #36740 选中文件点击历史版本后，操作按钮收缩图标高亮显示；
            // $data.isActive(true);
            if ((typeof $data.children) == 'function' && $data.children().length === 0) { return; }
            $($(event.currentTarget).children()[1]).show();
            if ($(event.currentTarget).find(".threeChildren")) {
                var left = $(event.currentTarget).offset().left + $($(event.currentTarget).children()[1]).width();
                var top = $(event.currentTarget).offset().top;
                if (threeButMenu) {
                    top = threeButMenu.offset().top;
                }
                // modify: 20230104 hushuchen 41793【多语言】进入共享给我文件预览界面，下载下拉框显示被遮挡                
                if(window.location.href.indexOf("preview.html") > -1){
                    var winW = window.innerWidth;
                    var MenuLeft = winW - $(event.currentTarget).offset().left;
                    if(MenuLeft < 190){
                        $($(event.currentTarget).children()[1]).css("left", winW - 190)
                    }
                }                
                $(event.currentTarget).find(".threeChildren").css("left", left).css("top", top);
            }
        }
        /*
        * mikahuang创建于2015年5月
        *鼠标移出菜单项事件
        * @$data 记录上下对象
        * @event jquery包装的event对象
        * @return undefined
        */
        toolbarPt.submenuMouseout = function ($data, event) {
            //移出菜单项
            // $data.isActive(false);
            $($(event.currentTarget).children()[1]).hide();
        }

        toolbarPt.lang = lang;
        toolbarPt.reload = function () {
            // modify: 2022-08-24 cuizhichao #34584 执行以图搜图/搜文后，搜索结果页面选中文件后，操作按钮收缩图标高亮显示；
            // modify 2022-08-18 liuchunli #36162【UI】预览多个图片类型的文件，切换相册顺序后，预览页面右侧的“更多”按钮图标显示不正确
            if(window.location.href.indexOf('/preview.html') == -1){
                if ($('.toolbar-btn-list .more > button').hasClass("active")) {
                    $('.toolbar-btn-list .more > button').removeClass("active")
                }
            }
            app.trigger("component:show", {
                moduleId: undefined,
                datas: this.currentDatas,
                message: this.message,
                moduleName: this.moduleName
            });
        }

        toolbarPt.pngConversionSvg = constinfo.pngConversionSvg;
        toolbarPt.isSvg = constinfo.isSvg;
        toolbarPt.lang = lang;
        return {
            /**
             * 创建一个toolbar实例，并绑定事件通知和监听等相关方法
             */
            createToolbar: function (name) {
                var toolbar = new Toolbar(name);
                Event.includeIn(toolbar);
                return toolbar;
            }
        };
    });
define('doc/informationbar/formatdatabytype',[
    'require', 'jquery', "header/userinfo/constinfo", lang()
], function(require, $, constinfo, lang) {
    //通过给定的数据类型，对数据进行格式化和包装

    /**
     * 文件夹对象格式化
     */
    function FormatDataSingleFolder(){

    }
    FormatDataSingleFolder.prototype.format = function(data){
        var createTime="";
        var modifyTime="";
        var childFileCount=0;
        var childFolderCount=0;                                                                                               
        var remark="";
        if(window._currentFolder){
            //解决点击空白处时间没有的问题
            createTime=window._currentFolder.createTime;
            modifyTime=window._currentFolder.modifyTime;
            childFileCount=window._currentFolder.childFileCount;
            childFolderCount=window._currentFolder.childFolderCount;
            remark=window._currentFolder.remark;
        }
        data.createTimeV = $.dateFormat(data.createTime||createTime);
        data.modifyTimeV = $.dateFormat(data.modifyTime||modifyTime);
        data.conclude = $.format(lang.conclude, (data.childFileCount || childFileCount), (data.childFolderCount || childFolderCount) );
        data.showBtnCalculate = true;
        data.showProcessBar = false;
        data.showUsedSize = false;
        data.folderUsedSize = '';//初始化值,缺陷26332 右侧面板文件大小偶发显示-1,2022年2月18日 liyangyang
        data.secName = ''; //密级信息
        data.remark= data.remark||remark;
        $.each(constInfo.sercurityLevel(), function (index, sec) {
            //modify:2017-12-08 haibin.long 返回的文件夹实例没有securityLevel属性，但有securityLevelId，改之。原值为：securityLevel
            //什么鬼，FolderById接口返回的是securityLevel。
            if ((data.securityLevelId || data.securityLevel) == sec.SecLevelId) {
                data.secName = sec.SecLevelName;
            }
        })
        return data;
    }

    /**
     * 单文件格式化
     */
    function FormatDataSingleFile(){

    }
        FormatDataSingleFile.prototype.format = function (data) {
            data.createTimeV = data.createTime;
            data.modifyTimeV = data.modifyTime;
            if (typeof data.createTime == "string" && data.createTime.toLowerCase().indexOf('date') > -1) {
                data.createTimeV = $.dateFormat(data.createTime);
            }
            if (typeof data.modifyTime == "string" && data.modifyTime.toLowerCase().indexOf('date') > -1) {
                data.modifyTimeV = $.dateFormat(data.modifyTime);
            }
        if (data.state == 0) {
            data.stateDesc = lang.norm;
            data.showStatePic = false;
            data.LockTip = "";
        }
        else if (data.state == 2) {
            data.stateDesc = lang.uploading;
            data.showStatePic = true;
            data.LockTip = "";
        }
        else if(data.state==1024)
        {
            data.stateDesc = lang.metadataMaintenance;
            data.showStatePic = false;
            data.LockTip = ""
        }
        else {
            if (data.state == 4 || data.state == 512) {
                data.stateDesc = lang.lock;
                data.showStatePic = true;
                if (data.state == 512) {
                    data.stateDesc = lang.editing;
                }
                //modify: 2017-12-07 haibin.long 增加对预览界面中信息面板的兼容
                //1. 在引用文件类型时，archiveTime时间不对
                if ( (data.currentOperator || data.editorName ) && (data.archiveTime || data.checkOutTime) ) {
                    data.LockTip = (data.currentOperator || data.editorName ) + lang.lockIn + (data.archiveTime ? constInfo.getDateInterval(data.archiveTime) : constInfo.getDateInterval(data.checkOutTime));
                }else{
                    //存在currentOperator没有值的情况
                    data.LockTip = "";
                }
            }
            else {
                data.LockTip = "";
            }
        }
        //data.size = (data.size && constInfo.formatSize(data.size)) || "";
        var reg = /^[0-9]+.?[0-9]*$/;
        if (reg.test(data.size)) {
            data.size = constInfo.formatSize(data.size) || "";
        }
        else {
            data.size = data.size|| "";
        }
        // 杀毒状态
        data.FileSecurityscan = data.FileSecurityscan || 0;
        if (data.FileSecurityscan > 0) {
            var engines = constinfo.getSysConfigInfo().securityScanEngineList;
            var descArr=[];
            if(engines){
                for (var index = 0; index < engines.length; index++) {
                    var element = engines[index];
                    if( (data.FileSecurityscan & element.Engine )>0){
                        descArr.push(element.Name);
                    }
                }
                data.fileSecurityscanDesc = lang.securityScan2;
            }
        }
        switch (data.FileSecurityscan) {
            case 0:
                data.fileSecurityscanDesc = lang.securityScan0;
                break;
            case -1:
                data.fileSecurityscanDesc = lang.securityScan1;
                break;
            case -10:
                data.fileSecurityscanDesc = lang.securityScan3;
                break;
            case -2:
                data.fileSecurityscanDesc = lang.securityScan4;
                break;
        }
        return data;
    }

    /**
     * 混合类型
     */
    function FormatDataComposed(){

    }
    FormatDataComposed.prototype.format = function(data){
        var folder = 0;
        var file = 0;
        var other = 0;
        // $.each(data, function (n, obj) {
        //     if (obj.data.folderType) {
        //         folder++;
        //     }
        //     else if (obj.data.fileType && obj.data.fileType == 2) {
        //         file++;

        //     } else {
        //         other++;
        //     }
        // });
        //modify 2018年2月27日10:54:04  cq.lr  
        $.each(data, function (n, obj) {
            if (obj.data.type==1||obj.data.fvType==1||obj.data.folderType==1) {
                folder++;
            }
            else if (obj.data.type == 2||obj.data.fileType==2) {
                file++;

            } else {
                other++;
            }
        });
        var allCount = folder + file + other;
        var all = $.format(lang.chooseNum, allCount);
        return { folder: folder, file: file, other: other, all: all };
    }

    /** 
     * 格式化处理提供工厂
    */
    function FormatDataProviders(){
        this.providers = {
            "singleFolder": new FormatDataSingleFolder(),
            "singleFile": new FormatDataSingleFile(),
            "composed": new FormatDataComposed()
        };
    }
    var frDataProviders = FormatDataProviders.prototype;
    frDataProviders.formate = function(data){

        //modify: 2017-12-05 haibin.long 优化兼容data是需要直接格式化的数据对象
        var _data, type = 1;
        var tempData = null;
        if(data.select && data.select.length == 1){
            _data = data.select[0];
          //  _data.data = _data.data.data ? _data.data.data : _data.data; //以前代码有，不能删除
        }else if(data.select){
            //复合类型
            _data = data.select;
            type = 2;
        }else{
            _data = data;
            //_data.data = data;
            tempData = data;
        }

        var formatdataHandler;

        if (type == 1) {
            if (_data.data) {
                tempData = _data.data;
            }
            if (tempData && tempData.data) {
                if (tempData.data != null) {
                    tempData = tempData.data;
                }                
            }
            //文件夹
            if(tempData.folderType!==null&
                tempData.folderType!==undefined&&
                tempData.folderType>=-1&&
                tempData.folderType<=1){
                    tempData.teamTotalCount=data.teamTotalCount;//teamcount cq.taiqi 2018.01.22
                    formatdataHandler = this.providers.singleFolder
            }
            //文件
            //128轻文档
            if (tempData.fileType!=undefined && (tempData.fileType == 2 || tempData.fileType == 128)) {
                formatdataHandler = this.providers.singleFile;
            }
            //引用
            if (tempData.fileType!=undefined && tempData.fileType == 32) {
                formatdataHandler = this.providers.singleFile;
            }    
            //轻文档
            // if (tempData.fileType!=undefined && tempData.fileType == 128) {
            //     formatdataHandler = this.providers.singleDocflow;
            // } 
            
        } else {
            formatdataHandler = this.providers.composed;
            tempData = _data;
            formatdataHandler = this.providers.composed;
        }

        if(formatdataHandler){
            return formatdataHandler.format(tempData);
        }else{
            return tempData;
        }
    }

    return FormatDataProviders;
});
/* created by cq.taiqi 2017.12.02
* 信息面板显示项的配置
* 功能：定义选中项对象 和右侧面板显示的对应关系
*/
define('doc/informationbar/service',["knockout", "header/userinfo/constinfo", './formatdatabytype'],function (ko, constinfo ,FormatDataProviders) {
    /**
     * 右侧信息面板服务对象
     */
    function InfomationBarService(){
        this.formatdataProvider = new FormatDataProviders();
        this.blockPaths= {
            teamInfo:"team/index",
            matedataInfo: "meta/index",
            flowInfo: "workfolw/index",
            historyFile:"historyfile/index",
            attachFile: "relativefile/index",
            accessory: "attachment/index",
            commentsAndAnnotation: "comment/index",
            teamLog: "teamlog/index",
            teamnotice: "teamnotice/index",
            basicInfo: "basicinfo/{0}/index",
        }
    }
    var sPt = InfomationBarService.prototype;

    // /**
    //  * 根据模块Id，得到对应的模块文件路径
    //  * @param {*} blockName 
    //  */
    // sPt.getBlockPath = function(blockName){
    //     var path = this.blockPaths[blockName];
    //     return 'doc/informationbar/' + ( path || ( blockName + "/index" ) );
    // }

    /**
     * 根据模块Id，得到对应的模块文件路径
     * @param {*} baseInfo 
     */
    sPt.getBlockPath = function(baseInfo){
        var basename = baseInfo.name
        var path = this.blockPaths[basename];
        path = 'doc/informationbar/' + ( path || ( basename + "/index" ) );
        var data = constinfo.getAllRightSidePanelList(baseInfo.id,path);
        if(data){
            return data;
        }
        return path;
    }

    /**
     * 根据文件夹Id获取目录路径
     * @param {*} folderId 
     */
    sPt.getNamePath = function(folderId){
        var promise = $.ajax({
            async: false,
            data: {
                module: "WebClient",
                fun: "GetFolderPath",
                folderId:folderId
            }
        });
        return promise;
    }

    /**
     * 获取右侧信息面板所有的可显示的模块
     * @param {*} code 
     */
    sPt.getAllInfoBar = function(code){
        var def = new $.Deferred();
        $.ajax({data:{
                module: "SystemManager",
                fun: "GetAllInfoBar",
                token:getTokenByCondition(),
                code:code
            }
        }).done(function(result){ 
            var barList = result.InfoBarList.sort(function (a, b) {return a.sort>b.sort?-1:1});
            var barItems = [];
            var curLanger=$.getLan();
            for(var i=0;i<barList.length;i++)
            {
                if(curLanger&&curLanger=="ja")
                {
                    barList[i].title=barList[i].jp;
                }
                else if(curLanger&&curLanger=="zh-tw")
                {
                    barList[i].title=barList[i].zhtw;
                }
                else if(curLanger&&curLanger=="en")
                {
                    barList[i].title=barList[i].en;
                }
            }
           
            for(var i = 0; i < barList.length; i++){
                barItems.push({
                    "id": barList[i].id,
                    "name": barList[i].name.slice(0),
                    "show": barList[i].show,
                    "sort": barList[i].sort,
                    "title": barList[i].title.slice(0),
                    "iscustom": barList[i].iscustom,
                    "classname": barList[i].classname.slice(0),
                    "isActive": ko.observable(false)
                });
            }
            barItems = barItems.sort(function(a, b) {
                return a.sort < b.sort ? -1 : 1;
            });
            def.resolve(barItems);

        }).fail(function(error){
            def.reject(error);
        });

        return def.promise();
    }
    //获取选中文件夹策略
    sPt.GetTypeRightSidePanelList = function(type){
        $.ajax({async:false,data:{
                module:"WebClient",
                fun:"GetTypeRightSidePanelList",
                type:type
            }
        }).done(function(result){
            //排序
            //var barList = result.list.sort(function (a, b) {return a.sort>b.sort?-1:1});
            var barList = result.list;
            return barList;
        })
    }
    /**
     * 计算空间大小
     */
    sPt.calculateSpaceSize = function(itemInfo){ 

        var def = new $.Deferred();

        //当前选中文件夹id
        var currentSelectedFld = 0;
        //path
        var currentPath = "";
        var arrparm = constinfo.getCurrentPlace();
        if (itemInfo.fileType!=1) {
            def.reject();
            return;
        }
        var result;
        var calculateFolderId = "GetFolderSizeChildCount";

        if (itemInfo.name == lang.recyleBin) {
            calculateFolderId = "GetDeleteFolderSizeChildCount";
            currentPath = arrparm[0]; currentSelectedFld = arrparm[1];
        } else {
            currentSelectedFld = itemInfo.id;
            if (itemInfo.deletedBy ) {//回收站 文件夹
                calculateFolderId = "GetDeleteFolderSizeChildCount";
            }
        }

        $.ajax({
            data: {
                module: "WebClient",
                fun: calculateFolderId,
                folderId: currentSelectedFld,
                path: currentPath,
                token:getTokenByCondition()
            }
        }).done(function (data) {
            if (!data || data.folderSize === undefined) {
                return;
            }
            result = {
                folderSize: data.folderSize,
                childFolderCurr: data.childFolderCurr,
                childFileCurr: data.childFileCurr
            }
            def.resolve(result);
        });

        return def.promise();
    }

    /**
     *  sailer创建于2015年5月
     * 时间格式化    与const的时间格式化方法不一样，不能通用
     * @data 时间
     * @return 格式化后的时间
     */
    sPt.formatDateTime = function(date){
        var seperator1 = "-";
        var seperator2 = ":";
        var month = date.getMonth() + 1;
        var strDate = date.getDate();
        if (month >= 1 && month <= 9) {
            month = "0" + month;
        }
        if (strDate >= 0 && strDate <= 9) {
            strDate = "0" + strDate;
        }
        var currentdate = date.getFullYear() + seperator1 + month + seperator1 + strDate
                + " " + date.getHours() + seperator2 + date.getMinutes()
                + seperator2 + date.getSeconds();
        return currentdate;
    }

    /**
     * 数据格式化
     * @param {*} data 
     */
    sPt.formateData = function(data){
        var newData = this.formatdataProvider.formate(data);
        return newData;
    }

    /**
     * 根据实体对象，获取实体的信息面板类型(普通)
     * @param {*} data 
     */
    sPt.getEntityTypeNorm = function(data){
        var result = {
            type: null,
            ishide: null
        };

        //团队顶级文件夹Id
        var teamTopFolderId = constInfo.getTeamTopFolderId();
        if (data.select.length == 1) {
            //if (data.select[0].data&&data.select[0].data.data) { data.select[0].data = data.select[0].data.data; }
            //采用变量替换解决上面代码的问题，gs
            var tempData = null;
            if (data.select[0] && data.select[0].data) {
                tempData = data.select[0].data;
            }
            if (data.select[0] && tempData&& tempData.data) {
                
                //tempData = tempData.data;//为什么要加这个东西？cq.taiqi 2018.01.22
            }
            //当前数据类型为文件夹
            // if (tempData && tempData.folderType && tempData.folderType == 1) {
            if (tempData && tempData.folderType!==null&
                    tempData.folderType!==undefined&&
                    tempData.folderType>=-1&&
                    tempData.folderType<=1){

                result.type = "singlefolder";
                //新增团队信息版面判断
                if (tempData.parentFolderId == teamTopFolderId) {
                    //如果当前文件夹为团队顶级文件夹、团队文件夹
                    if (tempData.id != teamTopFolderId) {
                        result.type = "team";
                    }
                    else{
                        result.type = "topteam";
                    }
                } 
                //快速导航屏蔽右侧信息
                if (tempData.fromquick && tempData.fromquick == true) {
                    result.ishide = 1;
                    return result;
                }                
                else if (tempData.parentFolderId == teamTopFolderId) {
                    //如果当前文件夹为团队顶级文件夹则屏蔽右侧信息、团队文件夹显示团队信息
                    if (tempData.id != teamTopFolderId) {
                        result.type = "team";
                    }
                    else {
                        result.type = "topteam";
                    }
                } 
                return result;
            }
             
            //文件
            if (tempData && tempData.fileType) {

                //引用
                if (tempData.fileType == 32) {
                    //modify: 2017-12-05 haibin.long 如果是引用文件，是需要区分下是文件，还是文件夹。
                    if (tempData && tempData.shortcut&&tempData.shortcut.entryType == 1){
                        result.type = "singlefolder";
                    }else{
                        result.type = "singlefile";
                    } 
                    return result;
                }
                //轻文档
                // if (tempData.fileType == 128) {
                //     result.type = "singledocflow";
                //     return result;
                // }

                result.type = "singlefile";
                if (tempData.fromquick && tempData.fromquick == true) {
                    result.ishide = 1; 
                    return result;
                } 
                return result;
            }

            if(tempData && tempData&& (tempData.fvId||tempData.fvId===0)){
                result.type = "singlecoll";
                result.ishide = 1;
                return result;
            }
            if(tempData && tempData&&tempData.module=="sharefilelist"){
                result.type = "singleshare";
                result.ishide = false;
                return result;
            }
            //当前数据类型为文件夹
            if (tempData && tempData&& tempData.folderType && tempData.folderType == -1) {
                //新增团队信息版面判断
                if (tempData.parentFolderId == teamTopFolderId) {
                    //如果当前文件夹为团队顶级文件夹、团队文件夹
                    if (tempData.id != teamTopFolderId) {
                        result.type = "team";
                    }
                    else{
                        result.type = "topteam";
                    }
                } else if (data.module=="team") {
                    result.type = "team";
                }
                //isActive(changeSelected(1));
                if (tempData.parentFolderId == teamTopFolderId) {
                    //如果当前文件夹为团队顶级文件夹则屏蔽右侧信息、团队文件夹显示团队信息
                    if (tempData.id == teamTopFolderId) {
                        result.type = "topteam";
                    } else {
                        result.type = "team";
                    }
                } else if (data.module == "team") {
                    result.type = "singlefolder";
                }
                //回收站
                if (data.module == "recyclebinlist") {
                    result.type = "singlefolder";
                } 
                return result;
            }
            if(tempData&&tempData.folderType!=undefined){
                result.type = "singlefolder"; 
                return result;
            }
        } else {
            //团队顶级文件夹
            if (data.select.length > 1) {
                if (tempData && tempData.parentFolderId == teamTopFolderId) {
                    result.type = "topteam";
                } else {
                    for(var i = 1; i < data.select.length; i++){
                        //第一个选中类型为文件
                        if(data.select[0].data.fileType){
                            //判断i个选中类型是否为文件
                            if(!data.select[i].data.fileType){
                                //混合文件
                                result.type = "composed";
                                break;
                            }else{
                                //多文件
                                result.type = "multiplefiles";
                            }
                        }else{
                            if(data.select[i].data.fileType){
                                //混合文件
                                result.type = "composed";
                                break;
                            }else{
                                //多文件夹
                                result.type = "multiplefolders";
                            }
                        }
                    }
                }
            } else {
                if(data.module=="mylock")
                {
                    result.type = "mylockinfo";
                } else if (data.select.data && data.select.data.data.id == teamTopFolderId) {
                    result.type = "topteam";
                } else {
                    for(var i = 1; i < data.select.length; i++){
                        //第一个选中类型为文件
                        if(data.select[0].data.fileType){
                            //判断i个选中类型是否为文件
                            if(!data.select[i].data.fileType){
                                //混合文件
                                result.type = "composed";
                                break;
                            }else{
                                //多文件
                                result.type = "multiplefiles";
                            }
                        }else{
                            if(data.select[i].data.fileType){
                                //混合文件
                                result.type = "composed";
                                break;
                            }else{
                                //多文件夹
                                result.type = "multiplefolders";
                            }
                        }
                    }
                }
            }
        }
        return result;
    };
       //我的锁定对应面板
    var getlockinfobar = function () {
        return {
            name: 'mylockinfo',
            infobarlist: ['basicInfo']
        }
    }

    /**
     * 根据实体对象，获取实体的信息面板类型
     * @param {*} message 
     */
    sPt.getEntityType = function(message, activationData){
        var selectData;
        var result;
        if(message.select && message.select[0] && message.select[0].data){
            selectData = message.select[0].data;
            if (message.module == 'sharetome' && selectData.isHasFiles) {
                result = {
                    type: "singleshare", //需要加载的类型
                    ishide: false //是否隐藏所有
                }
            } else if (selectData.fvId != null && selectData.fvType == 1) {
                result = {
                    type: "singlecoll",
                    ishide: false
                }
            } else if (selectData.mylock) {
                result = {
                    type: null,
                    ishide: false
                };
            }
        }
        //外发结果页，外发预览页，信息面板应该保持一致
        if(activationData.isInOutpublish() || activationData.isInPublishPreview()){
            result = {
                //type: "publish",
                type:'composed',
                ishide: false
            }
            if(message.select){
                if(message.select.length==1){
                    var data=message.select[0].data;
                    if(data.folderType){//文件夹
                        result.type='publishfolder';
                    }
                    if(data.fileType){
                        result.type='publishfile';
                    }
                    //result.type="publish";
                }
            }
        }
        if(!result){
            result = this.getEntityTypeNorm(message);
        }

        return result;
    }

    /**
     * 对格式化后的数据，进行附加属性的包装
     */
    sPt.wrapData = function(data, messager){
        if(data){
            //个人库，选中一个团队，文件夹时
            //modify: 2018-01-24 haibin.long 团队列表界面，单击空白处data会为空                    
            data.useCountCalculate = constInfo.useCountCalculateFn(data.maxFolderSize, data.size); //文件夹的大小统计方法
            data.useRateCalculate=constInfo.useRateCalculate(data.maxFolderSize, data.size); //使用率
            //团队右侧信息处理，用户信息、团队Id
            if (typeof (messager.select.teamUser) != "undefined") {
                data.teamId = messager.select[0].data.teamId;
                data.teamUser = messager.select.teamUser;
               
            } else if (typeof (messager.select[0]) != "undefined" && typeof (messager.select[0].teamUser) != "undefined") {
                //团队右侧信息处理，团队Id
                if (messager.select[0].data.teamId) {
                    data.teamId = messager.select[0].data.teamId;
                }
                //团队右侧信息处理，用户信息、团队Id
                if (messager.select[0].teamUser) {
                    data.teamUser = messager.select[0].teamUser;
                    data.teamId = messager.select[0].teamUser[0].TeamId;
                }
            }

            if(data.fileType !== undefined){ //文件
                data["testtype"] = 2;
            }
            if(data.folderType !== undefined){ //文件夹
                data["testtype"] = 1;
            }
            if (data.id == 1) {
                data["showQcCode"] = false;
             } else if (data.folderType && data.folderType == -1) {
                data["showQcCode"] = false;
             }
             else if (data.path != undefined && this.filterPath(data.path.replace(/\\/g, '_'))) {
                data["showQcCode"] = false;
             } else if( data != undefined ){
               data["showQcCode"] = true;
             }
        }
    }
    /**
     * 过滤顶级
     * @param {*} path 
     */
    sPt.filterPath = function(path){
        if (path==undefined) {
            return false;
        }
        //path = path.replace(/\\/g, "_");
        if (path.indexOf("2_")>-1) {
            var sp = path.split("_");
            if (sp.length==2) {
                return true;
            }
        }
        return false;
    }
    return InfomationBarService;
});
/* created by wwx 2021.04.19
* 信息面板显示项的配置
* 功能：pd-133批注评论
*/
define('doc/informationbar/gmpAnnotation',["knockout", "header/userinfo/constinfo", lang()], function (ko, constinfo, lang) {

    var gmp = new GMPAnnotation();
    /**
     * gmp批注评论
     */
    function GMPAnnotation() {
        this.annotations = {};
        this.hiddenAnnotation = false;
        this.annotationInvokeError = false;
        this.commentInvokeError = false;
        this.annotationInvokeErrorCode = 200;
        this.commentInvokeErrorCode = 200;

    }
    var sPt = GMPAnnotation.prototype;

    sPt.isGMP = function () {
        //return window.location.search.toLowerCase().indexOf('incident') > 0 && constInfo.systemInfo.instanceConfigInfo.EnableGMP;
        return  constInfo.systemInfo.instanceConfigInfo.EnableGMP;
    }
    sPt.isInvokeError = function (method) {

        if (this.isGMP()) {
            if (method === 'annotation') {
                if (this.annotationInvokeError) {
                    $.messager.alert(lang.error, lang.invokeErrMsg, 'error');
                    return true;
                }
            }
            else if (method === 'comment') {
                if (this.commentInvokeError) {
                    $.messager.alert(lang.error, lang.invokeErrMsg, 'error');
                    return true;
                }
            }
        }
        return false;
    }
    if (gmp.isGMP()) {
        //有参数时
        if (window.location.search.toLowerCase().indexOf('incident') > 0) {
        var fileVerId = $.getQueryString('fileVerId');
        var fileId = $.getQueryString('fileId');
        var incident = $.getQueryString('incident'); 
        var userId = constInfo.userInfo.guid;
        fileId = fileId ? fileId : 0;
        fileVerId = fileVerId ? fileVerId : 0;
        var annUrl = "";
        if (constInfo.systemInfo.instanceConfigInfo && constInfo.systemInfo.instanceConfigInfo.GMPAnnotationPermissionAPI) {
            annUrl = constInfo.systemInfo.instanceConfigInfo.GMPAnnotationPermissionAPI;
        }
        if (!annUrl) {
            gmp.hiddenAnnotation = false;
            gmp.annotationInvokeError = true;
        }
        $.ajax({
            type: "get",
            async: false,
            url: annUrl, // + "?FileId=" + fileId + "&FileVerId=" + fileVerId + "&UserId=" + userId + "&Incident=" + incident,
            data: { FileId: fileId, FileVerId: fileVerId, UserId: userId, Incident: incident }
        }).then(function (result) {
            if (result.code === 200) {
                gmp.hiddenAnnotation = !result.data.annotate;
                gmp.annotations = result.data.annotations;
                gmp.annotationInvokeError = false;
                gmp.annotationInvokeErrorCode = 200;
            }
            else {
                gmp.hiddenAnnotation = false;
                gmp.annotationInvokeError = true;
                gmp.annotationInvokeErrorCode = result.code;
            }
        }, function (e) {
            gmp.hiddenAnnotation = false;
            gmp.annotationInvokeError = true;           
        });
        }

        else {//无参数时
            gmp.hiddenAnnotation = true;
            gmp.annotations = [];
            gmp.annotationInvokeError = false;
            gmp.annotationInvokeErrorCode = 200;
        }
    }


    //批注编辑按钮隐藏及评论发布框隐藏
    sPt.getGMPCommentHidden = function (annotationId) {
        that = this;
        var data = {
            hiddenComment: false,
            comments: [],
        }
        if (this.isGMP()) {
            //GMP有参数时
            if (window.location.search.toLowerCase().indexOf('incident') > 0) {

                var fileVerId = $.getQueryString('fileVerId');
                var fileId = $.getQueryString('fileId');
                var incident = $.getQueryString('incident');
                var userId = constInfo.userInfo.guid;
                fileId = fileId ? fileId : 0;
                fileVerId = fileVerId ? fileVerId : 0;
                var comUrl = "";
                if (constInfo.systemInfo.instanceConfigInfo && constInfo.systemInfo.instanceConfigInfo.GMPCommentPermissionAPI) {
                    comUrl = constInfo.systemInfo.instanceConfigInfo.GMPCommentPermissionAPI;
                }
                if (!comUrl) {
                    that.commentInvokeError = true;
                    data.hiddenComment = false;
                }
                $.ajax({
                    type: "get",
                    async: false,
                    url: comUrl, //"/gmp/comment/permission?FileId=" + fileId + "&FileVerId=" + fileVerId + "&UserId=" + userId + "&Incident=" + incident + "&AnnotationId=" + annotationId,
                    data: { FileId: fileId, FileVerId: fileVerId, UserId: userId, Incident: incident, AnnotationId: annotationId }
                }).then(function (result) {
                    console.log(result);
                    if (result.code === 200) {
                        data.hiddenComment = !result.data.comment;
                        data.comments = result.data.comments;
                        that.commentInvokeError = false;
                        that.commentInvokeErrorCode = 200;
                    }
                    else {
                        that.commentInvokeError = true;
                        that.commentInvokeErrorCode = result.code;
                        data.hiddenComment = false;
                    }
                }, function (e) {
                    data.hiddenComment = false;
                    that.commentInvokeError = false;
                    // console.log(e);
                });
            } else {//GMP无参数时
                data.hiddenComment = true;
                data.comments = [];
                that.commentInvokeError = false;
                that.commentInvokeErrorCode = 200;
            }

        }

        return data;
    }

    return gmp;
});
define('header/userinfo/fileAndFolderCache',["header/userinfo/constinfo", "jquery"], function (constInfo) {
    var calcDocPermissions = {};
     
    function rTrim(str, flag) {
        return str.replace(new RegExp((flag || " ") + "$"), "");
    }
    function delperm() {
        var timestamp = Date.parse(new Date());
        for (var key in calcDocPermissions) {
            var value = calcDocPermissions[key];
            var tim = value.split("|")[1];
            var t = timestamp - parseInt(tim)
            if (t>60000) {
                delete  calcDocPermissions[key]
            }
        }

    }
    setInterval(function () {
        delperm();
    }, 60000)
    function GetCalcDocPermission(entryIds) {
        var docIdTypes = entryIds.split("|");
        DocPermissions = {};
        if (docIdTypes.length == 1 && calcDocPermissions[docIdTypes[0]]) {
            DocPermissions[docIdTypes[0]] = calcDocPermissions[docIdTypes[0]];
            return DocPermissions;
        }
        var ids = "";
        for (var i = 0; i < docIdTypes.length; i++) {
            var id = docIdTypes[i];
            if (id == "") {
                continue;
            }
            if (calcDocPermissions[id] != undefined && calcDocPermissions[id]!=null) {
                DocPermissions[id] = calcDocPermissions[id];
            } else {
                ids += id + "|";
            }
        }
        if (ids == "" || ids=="|") {
            return DocPermissions;
        }
        $.ajax({
            data: {
                module: "PermissionManager",
                fun: "CalcDocPermissions",
                docIds: ids,
                token: constInfo.userInfo.token
            },
            async: false
        }).done(function (result) {
            var res = JSON.parse(result.Result);
            var docIds = ids.split("|");
            for (var i = 0; i < docIds.length; i++) {
                var Id = docIds[i];
                if (Id != "") {
                    var timestamp = Date.parse(new Date());
                    var perm = res.permission[Id] || 0;
                    perm = perm+"|" + timestamp
                    calcDocPermissions[Id] = perm
                    
                }
            }
        });
       
        return calcDocPermissions;
    }
    function delDocPerm(entryId) {
        delete calcDocPermissions[entryId];
    }
    function delDocPermission() {
        calcDocPermissions = {};
    }
    return { get: GetCalcDocPermission, del: delDocPermission };
});
/*
 *   sailer创建于2015-5
 *   信息面板模块
 *   chenjiantao修改于2016-09
 *   gaosong 2017年6月27日 10:48:47 与后端配置对接数据库
 *   huang.hao edit at 2017年11月21日
 *
 *   modify by taiqi.chen
 *   增加配置文件moduleconfig 用于配置选中项 和需要显示的面板的对应关系
 *   如果你新增了一个配置项,需要设置对应哪些实体类型时进行显示,请将其配置到moduleconfig的相关项里面去
*/
define('doc/informationbar/index',['require', 'knockout', 'komapping', 'scrollbar', "durandal/app", "underscore", "jquery", lang(),
    "header/userinfo/constinfo", "./service", "./gmpAnnotation", "doc/contextmenu/permtype","header/userinfo/fileAndFolderCache"],
    function (require, ko, komapping, scrollbar, app, _, $, lang,
        constinfo, Service, gmpAnnotation,permtype,fileAndFolderCache) {
        // 数据接受
        // 显示模块选择和加载
        /** 信息面板的折叠 */
        function InfoBarCollapsing(view) {
            this.view = view;
            //信息面板默认宽度
            this.defaultPanelWidth = 50; 
        }
        var rightPanelState = ko.observable(false);//记录右侧面板展开的状态
        var rightPanelContentStates = ko.observable(false);
        var icPt = InfoBarCollapsing.prototype;
        icPt.init = function () {
            var that = this;
            //注册事件处理信息面板的折叠情况
            app.on("informationbar:toggleinformationBar").then(function (obj, callback) {
                //cq.tianao 如果存在isRecall说明是用户手动操作 存入当前右侧面板状态到cookie 
                if (obj && obj.isRecall) {
                    var curentModule = require('knockout').$store.state.curentModule();
                    if (curentModule.module == 'enterprise') {
                        $.cookie("enterpriseRightPanelType", !rightPanelState());
                    } else if (curentModule.module == 'team') {
                        $.cookie("enterpriseRightPanelType", !rightPanelState());
                    } else if (curentModule.module == 'person') {
                        $.cookie("enterpriseRightPanelType", !rightPanelState());
                    }
                }
                if (obj && obj.type == 0) {
                    that.toggleRightPanel(false, function (isShow) {
                        callback && callback(isShow);
                    });
                    return;
                }

                if (obj && obj.type == 1) {
                    that.toggleRightPanel(true, function (isShow) {
                        callback && callback(isShow);
                    });
                    return;
                }
                if (obj && obj.type == 3) {
                    //修正视图的显示
                    activeView("doc/informationbar/teaminfo/index");
                }
                if (obj && obj.type == 4) {
                    that.view.clearAllSelected()
                    // that.view.BaseItemValues  应该是老的面板数据
                    var data = that.view.BaseItemValues.filter(function (item) {
                        return item.name === 'historyFile';
                    });
                    that.view.infoWidth(370);

                    // 新增改变新的面板数据列表
                    $.each(that.view.InfobarItems(), function (i, v) {
                        if (v.name == data[0].name) {
                            v.isActive(true)
                        } else {
                            v.isActive(false)
                        }
                    })
                    data[0]["entityType"] = "singlefile";
                    data[0].isActive(true);
                    that.view.currentModule = data[0];
                    that.view.loadCurrentModule()

                    that.toggleRightPanel(true, function (isShow) {
                        callback && callback(isShow);
                    });
                    return;
                }
                if (obj && obj.type == 5) {
                    that.view.activeView(null)
                    that.view.activeView("doc/informationbar/comment/index");
                    that.view.activationData.command = 'locate';
                    that.view.activationData.annotation = obj.annotation;
                }
                if (obj && obj.type == 6) {
                    var data = that.view.BaseItemValues.filter(function (item) {
                        return item.name === 'commentsAndAnnotation';
                    });
                    // 新增改变新的面板数据列表
                    $.each(that.view.InfobarItems(), function (i, v) {
                        if (v.name == data[0].name) {
                            v.isActive(true)
                        } else {
                            v.isActive(false)
                        }
                    })

                    that.view.activeView(null)
                    that.view.activeView("doc/informationbar/comment/index");
                    that.view.activationData.command = 'quickCreate';
                    that.view.activationData.capture = obj.capture;
                }

                that.toggleRightPanel(undefined, function (isShow) {
                    callback && callback(isShow);
                });
                $(".document-list-body").perfectScrollbar('update');
            });
            app.on("informationbar:toRightButtonStart").then(function (obj) {
                var start = obj.start;
                if (start == 0) {
                    $(".v6-attraction").hide();
                    $(".v6-attraction-open").hide();
                } else {
                    if ($(".informationbar-wrap").width() !== 0) {
                        $(".v6-attraction").hide();
                        $(".v6-attraction-open").hide();
                    } else {
                        $(".v6-attraction").show();
                        $(".v6-attraction-open").hide();
                    }
                }
            })
            app.on("setRightPanelWidth").then(function (obj) {
                if (obj && obj.type == 'preview') {
                    // that.view.infoWidth(50);
                    // that.setSize(50);
                    // app.trigger('centerPanel:width');
                    app.trigger('openPreviewRightPanel',that.view.infoWidth());
                }
            }),
            app.on("changeRightPanel").then(function (obj) {
                if (obj && obj.type == 'changeRightPanel') {
                    if ($.cookie("enterpriseRightPanelType") == 'true' || $.cookie("enterpriseRightPanelType") == 'true' || $.cookie("enterpriseRightPanelType") == 'true') { 
                        that.setSize(that.view.infoWidth());
                        app.trigger('centerPanel:width');
                    }
                }
            }),
            
            // 预览模式窗口打开关闭需要重设右侧面板状态宽度
            app.on("PreviewModelViewPanel").then(function (obj) {
                if (obj && obj.type == 'PreviewModelViewPanel') {
                    if ($.cookie("enterpriseRightPanelType") == 'true' || $.cookie("enterpriseRightPanelType") == 'true' || $.cookie("enterpriseRightPanelType") == 'true') { 
                        that.setSize(that.view.infoWidth());
                        app.trigger('centerPanel:width');
                    }
                }
            }),
            
            //重新绑定size操作 
            this.view.jRightPanel.resizable({
                disabled:true,
                onResize: _.throttle(function () {
                    that.setSize($(this).width());
                }, 250),
                onStopResize: function () {
                    //小于100是自动折叠
                    if (that.view.jRightPanel.width() < 50) {
                        that.setSize(0);
                    }
                    that.view.infoWidth(that.view.jRightPanel.width());
                },
                edge: 5,
                maxWidth: 500,  //最大宽度500
                minWidth: 99,
                handles: "w"
            });
            $(window).on("resize.docpanel", function () {
                //cq.tianao 用cookie记忆三个内容库的右侧面板
                var curentModule = require('knockout').$store.state.curentModule();
                var type = 0;
                if (curentModule) {
                    if (curentModule.module == 'enterprise') {
                        var enterpriseRightPanelType = eval($.cookie('enterpriseRightPanelType'));
                        if (enterpriseRightPanelType != null) {
                            type = enterpriseRightPanelType ? 1 : 0;
                        }
                    } else if (curentModule.module == 'team') {
                        var enterpriseRightPanelType = eval($.cookie('enterpriseRightPanelType'));
                        if (enterpriseRightPanelType != null) {
                            type = enterpriseRightPanelType ? 1 : 0;
                        }
                    } else if (curentModule.module == 'person') {
                        var enterpriseRightPanelType = eval($.cookie('enterpriseRightPanelType'));
                        if (enterpriseRightPanelType != null) {
                            type = enterpriseRightPanelType ? 1 : 0;
                        }
                    }
                    //tianyi.wang-202203021150-缺陷编号27084-我的共享/共享给我，刷新网页后出现磁吸
                    else if (curentModule.hash == 'quick/myshareme') {
                        $('.v6-attraction').hide();
                        $(".v6-attraction-open").hide();
                    } else if (curentModule.hash == 'quick/myshare') {
                        $('.v6-attraction').hide();
                        $(".v6-attraction-open").hide();
                    }   
                    else if (curentModule.hash && curentModule.hash == "quick/mytag" && $(".icon-toright-selected").length > 0) {
                        type = 1;
                    }
                    else {
                        type = 0;
                    }
                } else {
                    type = 0;
                }
                //ldb PI:341 共享给我中隐藏和显示左侧面板时对右侧面板不影响  pi9037
                var rightPanelWidth = $(".right-panel").width();
                //modify 2022-03-15 yangyang.li [缺陷27783] 页面缩放110%, 右侧面板点击无反应
                //原因: right-panel添加了border-left 1px,导致 this.view.jRightPanel.width() 获取的值为0.005681999999999965或1.005682(不知道为啥),所以判断>0时会一直关闭
                if (curentModule && curentModule.hash && (curentModule.hash == "quick/myshareme" || curentModule.hash == "quick/myfavorite") && rightPanelWidth > 50) {
                    type = 1;
                }
                //调用右侧面板 宽度赋值
                type == 1 ? that.setSize(that.view.infoWidth()) : that.setSize(0);



            });
        }
        icPt.toggleRightPanel = function (type, callback) {
            var cixiStatus = constInfo.getSysConfigInfo().MagneticAttractionSwitch;
            if (type == false) {
                this.setSize(0);
                callback && callback(false);
                return;
            }
            if (type == true) {
                if (this.view.infoWidth() == 0) {
                    this.setSize(this.defaultPanelWidth);
                    this.view.infoWidth(this.defaultPanelWidth);
                }
                else {
                    this.setSize(this.view.infoWidth());
                }
                callback && callback(true);
                return;
            }
            //modify 2022-03-15 yangyang.li [缺陷27783] 页面缩放110%, 右侧面板点击无反应
            //原因: right-panel添加了border-left 1px,导致 this.view.jRightPanel.width() 获取的值为0.005681999999999965或1.005682(不知道为啥),所以判断>0时会一直关闭
            if (this.view.jRightPanel.width() < 50) {
                //折叠
                if (this.view.infoWidth() == 0) {
                    this.setSize(this.defaultPanelWidth);
                    //panelWidth为拖拽为0时,恢复默认值
                    this.view.infoWidth(this.defaultPanelWidth);
                }
                else {
                    this.setSize(this.view.infoWidth());
                    this.view.panelAttraction.hide();
                }
                callback && callback(true);
                if (this.view.activeView()&&this.view.activeView().__moduleId__ == "doc/informationbar/basicinfo/singlefile/index") {
                    app.trigger("infobartaglist:reload", { id: this.view.initData().id(), force: true })
                }
            }
            else {
                //modify by haiwei.cui 20220419 缺陷29245，（企业库/个人库）根目录中打开右侧面板再关闭出现磁吸
                var that = this;
                var isShowCixi = true;
                if (that.view && that.view.initData && that.view.initData.sourceData && that.view.initData.sourceData) {
                    if (that.view.initData.sourceData.id && that.view.initData.sourceData.parentFolderId) {
                        if (that.view.initData.sourceData.id == 1) {
                            isShowCixi = false;
                        }
                        else if (that.view.initData.sourceData.parentFolderId == 2) {
                            isShowCixi = false;
                        }
                    }
                    else if (that.view.initData.sourceData.folder && that.view.initData.sourceData.file) {
                        if (that.view.initData.sourceData.folder + that.view.initData.sourceData.file > 1) {
                            isShowCixi = false;
                        }
                    }
                }
                //关闭
                this.setSize(0);
                callback && callback(false);
                //modify by haiwei.cui 缺陷28228，在团队库选中一个团队后打开右侧面板再关闭，出现磁吸
                if (window.location.hash != "#doc/team/7" && isShowCixi) {
                    if (cixiStatus) {
                        //modify by haiwei.cui 右侧面板关闭时磁吸恢复缓存状态
                        var IspanelAttractionWidth = $.cookie('paramWidth');
                        if (IspanelAttractionWidth != null) {
                            if (IspanelAttractionWidth == "true") {
                                this.view.panelAttraction.show();
                                $(".v6-attraction").css('width', '50px');
                                $(".v6-attraction-open").hide();
                            } else if (IspanelAttractionWidth == "false") {
                                this.view.panelAttraction.show();
                                $(".v6-attraction").css('width', '0px');
                                $(".v6-attraction-open").show();
                            }
                        } else {
                            $.cookie('paramWidth', true);
                            this.view.panelAttraction.show();
                            $(".v6-attraction").css('width', '50px');
                            $(".v6-attraction-open").hide();
                        }
                    }
                    else {
                        $(".v6-attraction").css({ 'width': '0px' });
                        this.view.panelAttraction.hide();
                        $(".v6-attraction-open").hide();
                    }                    

                }
            }
            //modify 2022-04-14 liuchunli  #28474 外发结果界面，点击磁吸，磁吸消失   
            if(window.location.pathname=="/outpublish.html"){
                $(".v6-attraction").css({ 'width': '0px' });
                this.view.panelAttraction.hide();
                $(".v6-attraction-open").hide();
            }
        }

        icPt.setSize = function (width) {
            //通过右边信息面板的宽度来判断按钮是否设置为选中状态
            //modify by 20220302 haiwei.cui 获取磁吸开关不再每次访问接口，设置时重载
            this.view.panelAttractionIsOpen(constinfo.getSysConfigInfo().MagneticAttractionSwitch)
            if (width == 0) {
                //右边信息面板存在则按钮为选中状态
                $(".icon-toright").removeClass("icon-toright-selected").addClass("icon-toright");

                this.view.infobarStatus(false)
                app.trigger("hidehotword", false);


            } else {

                //右边信息面板存在则按钮为选中状态
                $(".icon-toright").removeClass("icon-torigh").addClass("icon-toright-selected");
                this.view.infobarStatus(true);
                app.trigger("hidehotword", true);
                this.view.panelAttraction.hide();
            };
            // if (width <= 50) {
            //     this.view.clearAllSelected()
            // }
            // 外发详情右侧面板位置之前一直不对，需要特殊处理
            var outpublishRightLeft = 0;
            if (window.location.href.indexOf("/outpublish") > -1) {
                outpublishRightLeft = 20;
                $(".navbar-list-r .right-panel-close").hide()
            }else{
                outpublishRightLeft = 0;
                $(".navbar-list-r .right-panel-close").show()
            }
            // 相关模块互斥
            //modify 2022-12-28 cuizhichao  #41469 展开业务工作区，左侧按钮出现跳动，导致会同时出现业务工作区的“收缩”、“展开”按钮
            // icPt.setSetBarRightIconToggle(width)
            this.view.jRightPanel.width(width).css("left", $(window).width() - this.view.jLeftPanel.outerWidth() - width - $('.leftNavList').width() - outpublishRightLeft);
            this.view.jRightPanel.prev().css("right", this.view.jRightPanel.width());



            //刷新右侧滚动条
            app.trigger("singlefolder:index", { type: true });
            // 重新渲染顶部操作按钮
            app.trigger("doc:rightPanelSizeChange");
            // 预览页
            if (window.location.pathname.indexOf('preview') > -1) {
                var mainInformationbar = $("#mainInformationbar").width();
                if (mainInformationbar == 0) {
                    $(".v6-preview-left-panel").removeClass("active")
                    this.view.panelAttraction.show();
                } else {
                    $(".v6-preview-left-panel").addClass("active")
                    this.view.panelAttraction.hide();
                }
            }
            app.trigger("docsetbar:setNavtionRightBar", width);//modify 20230227 43211【UI】选中文件并展开右侧面板，右侧面板的“展开/收缩”按钮会同时显示
        }
        icPt.setAttraction = function (paramWidth) {
            if (paramWidth == 0) {
                $(".v6-attraction").css('width', '50px');
                $(".v6-attraction-open").hide();
            } else {
                $(".v6-attraction").css('width', '0px');
                $(".v6-attraction-open").show()
            }
        }
        icPt.setSetBarRightIconToggle = function (width) {
            return
            var curentModules = require('knockout').$store.state.curentModule();
            if (curentModules) {
                // 三库
                if (curentModules.module == 'enterprise' || curentModules.module == 'team' || curentModules.module == 'person') {
                    if (width > 40) {
                        setTimeout(() => {
                            $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide() 
                        }, 100);
                    }else{
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                    }
                }
                // 标签
                if (curentModules.hash == "quick/mytag" &&
                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-removefile").parent() &&
                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-removefile").parent().css("display") == 'block'
                ) {
                    if (width > 40) {
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                    }else{
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                    }
                }
                // 最近
                if (curentModules.hash == "quick/myvisit") {
                    setTimeout(() => {  
                        if(
                            $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-pushpin").parent() &&
                                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-pushpin").parent().css("display") == 'block' ||
                                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-unpushpin").parent() &&
                                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-unpushpin").parent().css("display") == 'block' 
                            ){
                                if (width > 40) {
                                    $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                                }else{
                                    $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                                    setTimeout(() => {
                                        if (
                                            $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-pushpin").parent().css("display") != 'block' &&
                                            $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-unpushpin").parent().css("display") != 'block'  &&
                                            window.location.hash.indexOf("/myvisit") > -1
                                        ) {
                                            $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                                        }
                                    }, 100);
                                }
                            }else {
                        }                     
                    }, 0);
                }
                // 收藏
                if (curentModules.hash == "quick/myfavorite" &&
                    $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-refresh").parent().css("display") == 'block' &&
                    $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-create").parent().css("display") != 'block' 
                ){
                    if (width > 40) {
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                    }else{
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                        setTimeout(() => {
                            if (
                                // $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-refresh").parent().css("display") == 'block' &&
                                // $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-create").parent().css("display") == 'block' &&
                                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-historyversion").parent().css("display") != 'block' 
                            ) {
                                $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                            }
                        }, 200);
                    }
                }
                // 共享给我
                if (curentModules.hash == 'quick/myshareme' 
                ){
                    if (width > 40) {
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                    }else{
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                        setTimeout(() => {
                            if (
                                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-cancelexp").parent().css("display") == 'block'
                            ) {
                                $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                            }
                        }, 0);
                    }
                }
                // 搜索
                if (curentModules.hash.indexOf('/search') > -1) {
                    if (width > 40) {
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide() 
                    }else{
                        $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                        setTimeout(() => {
                            if (
                                $(".toolbar .toolbar-btn-list .toolbar-btn-item .icon-goback").parent().css("display") != 'block'
                            ) {
                                $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                            }
                        }, 0);
                    }
                }
            }
        }


        //面板位置记录
        /**
         * 给数据体添加属性stateDesc
         * @param {Object} data
         */
        function addPropStateDesc(data) {
            var state = lang["DocState" + data.state];
            var curentModule = require('knockout').$store.state.curentModule();
            // 缺陷42371 回收站的文件状态为 -  ，应该显示为  ‘已删除’  ge.lin 20230110  正常状态&扫描中 文件状态才显示 -
            if (data.state === 0 && data.fileSecurityscanDesc == lang.securityScan1) {
                data.stateDesc = "-";
            } else {
                data.stateDesc = state;
            }
            if(data.state == 1024 && curentModule.id =='myvisit'){
                data.stateDesc = state
            }
        }

        //信息面板对象
        function InformationBar() {
            this.activationData = {
                isInRecycle: ko.observable(false),//回收站
                isInOutpublish: ko.observable(false), //是否在外发结果页，或者外发结果页的预览页
                isInPreview: ko.observable(false), //是否在预览页
                isInPublishPreview: ko.observable(false)//是否在外发预览页
            };
            //视图
            this.activeView = ko.observable();
            //选中情况
            this.isActive = ko.observable();
            //允许展示的项（文件，文件夹各有区别）
            this.InfobarItems = ko.observableArray();
            //对象类型 file为文件 folder 为文件夹
            this.objType = ko.observable();
            this.initData = ko.observable();
            this.infoWidth = ko.observable(50);
            //url配置
            this.isurlconfig = ko.observable(false);
            //url配置地址
            this.urladdress = ko.observable();
            this.BaseItemValues;

            //this.hiddenAnnotation = ko.observable(false);
            //this.delAnnotateHidden = ko.observable(false);

            //通过GMP开关判断是否隐藏评论批注按钮
            this.hiddenAnnotation = ko.observable(false);
            //预览页面过滤项
            this.PreviewPageFilterItemValues = [];

            this.infobarStatus = rightPanelState;
            /**
             * 附加的优先权限：如共享，外发等（对接预览界面）
             */
            this.permission = {
                /**
                 * 下载权限
                 */
                canDown: ko.observable(true),
                //permVal: ko.observable(-1)
            }
            this.jRightPanel;
            this.jLeftPanel;

            // 磁吸面板
            this.panelAttraction;
            this.panelAttractionIsOpen = ko.observable(true)

            //当前选中项
            this.currentModule;

            //模块实体缓存
            this.modulesCache = [];

            //服务对象创建
            this.service = new Service();
            //调整大小对象
            this.infobarCollapsing = new InfoBarCollapsing(this);
            this.rightPanelContentStates = ko.observable(false);
        }
        var inforBarPt = InformationBar.prototype;
        /**
         * 获取需要加载的列表
         * @param {*} modules
         */
        inforBarPt.getShowModules = function (modules, entityType, module, type) {
            //过滤出可显示的列表
            var tmp = [], modules = modules.infobarlist;
            var newModules = [];
            // //暂时隐藏关联、附件功能#5918 lr
            // for(var i=0;i<modules.length;i++)
            // {
            //   if(modules[i]!="attachFile"&&modules[i]!="accessory")
            //   {
            //     newModules.push(modules[i])
            //   }
            // }
            // modules=newModules;

            for (var irow = 0; irow < this.BaseItemValues.length; irow++) {
                var baseItem = this.BaseItemValues[irow];
                if (this.PreviewPageFilterItemValues.indexOf(baseItem.name) > -1) {
                    continue;
                }
                for (var icol = 0; icol < modules.length; ++icol) {
                    if (modules[icol].toLowerCase() == baseItem.name.toLowerCase() && baseItem.show) {
                        var m = $.extend({}, baseItem);
                        m.entityType = entityType;
                        //个人库/团队库：右侧面板去掉元数据菜单
                        if (module == "person" || module == "team") {
                            if (baseItem.name != "matedataInfo") {
                                tmp.push(m);
                            }
                        } else {
                            tmp.push(m);
                        }
                    }
                }
            }
            if (tmp.length == 0) {
                return tmp;
            }
            //设置选中项
            var findSelectedModule = false;
            if (this.currentModule) {
                for (var irow = 0; irow < tmp.length; ++irow) {
                    if (this.currentModule.name.toLowerCase() == tmp[irow].name.toLowerCase()) {
                        tmp[irow].isActive(true);
                        this.currentModule = tmp[irow];
                        findSelectedModule = true;
                    }
                }
            }
            if (!findSelectedModule) {
                tmp[0].isActive(true);
                this.currentModule = tmp[0];
            }
            return tmp;
        }

        /**
         * 清空所有的选中项
         */
        inforBarPt.clearAllSelected = function () {
            var items = this.InfobarItems();
            for (var irow = 0; irow < items.length; ++irow) {
                items[irow].isActive(false);
            }
        }
        /**
         * 获取模块文件的路径
         * @param {*} baseInfo
         */
        inforBarPt.getModulePath = function (baseInfo) {
            var path = this.service.getBlockPath(baseInfo);
            //如果是singlefile需要特殊处理
            if (baseInfo.name.toLowerCase() == 'basicinfo') {
                if (baseInfo.entityType == 'publishfolder') {
                    baseInfo.entityType = 'singlefolder';
                }
                else if (baseInfo.entityType == 'publishfile') {
                    baseInfo.entityType = 'singlefile';
                }
                if (baseInfo.entityType == "multiplefiles" || baseInfo.entityType == "multiplefolders") {
                    baseInfo.entityType = "composed";
                }
                path = $.format(path, baseInfo.entityType);
            }
            return path;
        }
        inforBarPt.activate = function (first, second) {
            var enterpriseRightPanelType = $.cookie("enterpriseRightPanelType");
            // 浏览器存在cookie设置浏览器cookie，否则设置接口设置状态
            if (enterpriseRightPanelType == 'false') {
                $.cookie("enterpriseRightPanelType", false);
            }else if (enterpriseRightPanelType == 'true') {
                $.cookie("enterpriseRightPanelType", true);
            }else{
                $.cookie("enterpriseRightPanelType", constinfo.getSysConfigInfo().MagneticAttractionGpc);
            }
        },
        inforBarPt.compositionComplete = function (child, parent, settings) {
            var that = this;
            //判断是否是在外发结果页和在外发的预览页
            if (($(parent).parent().data("tag") && $(parent).parent().data("tag") === "outpublish") || $.getQueryString(window.decodeURIComponent(window.location.href), "isPublish") === "true") {
                this.activationData.isInOutpublish(true);
            }
            //判断是否是在预览页
            if ($(parent).parent().data("tag") && $(parent).parent().data("tag") === "preview") {
                this.activationData.isInPreview(true);
            }
            //判断是否是在外发预览页
            if ($(parent).parent().data("tag") && $(parent).parent().data("tag") === "preview" && $.getQueryString('code')) {
                this.activationData.isInPublishPreview(true);
            }
            this.jRightPanel = $(".right-panel", $(child).parent().parent()).eq(0);
            this.jLeftPanel = $(".left-panel", this.jRightPanel.parent().parent().parent()).eq(0);
            if (that.BaseItemValues == undefined) {
                this.service.getAllInfoBar($.getQueryString("code")).then(function (items) {
                    //所有对象
                    that.BaseItemValues = items;
                });                
            }
            // 磁吸面板
            this.panelAttraction = $(".v6-attraction");



            
            //modify 2022-06-02 tianyi.wang 缺陷30918【GMP】开启GMP之后，磁吸面板和右侧面板仍然显示批注按钮
            var result = gmpAnnotation.getGMPCommentHidden();
            if (result.hiddenComment === undefined) {
                that.hiddenAnnotation(true);
            } else {
                
                that.hiddenAnnotation(result.hiddenComment);
            } 
            app.on('upload:init').then(function (messager) {
                // console.log('333333',$(".PreviewModelView-panel .PreviewModelView").width());
                
                if (messager.module == "update"
                    // || messager.module == "favoritelist"
                    || messager.module == "myshare"
                    || messager.module == "mypublish"
                    // || messager.module == "mylock"
                    || messager.module == "mysubscribe"
                    || messager.module == "template"
                ) {
                    //隐藏所有面板
                    that.activeView(null);
                    return;
                }
                if (messager.module == "recyclebinlist") {
                    that.activationData.isInRecycle(true);
                } else {
                    that.activationData.isInRecycle(false);
                }
                if (messager.module && messager.module == "null") {
                    //空信息
                    that.activeView(null);
                    return;
                }
                var optiontype;
                var entityType = that.service.getEntityType(messager, that.activationData);
                var data = that.formateData(messager);
                var modulepath = messager.module;
                var type;
                //团队目录 团队顶级  回收站
                if (data != null && data.path && modulepath != "teamcreate" && modulepath != "teamfolder" && modulepath != "recyclebinlist") {
                    var path = data.path.split('\\')
                    if (path.length > 0) {
                        if (path[0] == "1") {
                            type = path[0];
                            modulepath = "enterprise";
                        } else if (path[0] == "7") {
                            type = path[0];
                            modulepath = "team";
                        } else if (path[0] == "2") {
                            modulepath = "person";
                        }
                    }
                }

                //if (modulepath === "teamcreate") {
                //    $('.v6-attraction').hide();
                //}
                
                //引用文件跟随源文件
                if (data != null && data.name) {
                    var linkname = data.name.split(".");
                    if (linkname.length > 0 && linkname[linkname.length - 1] == "link") {
                        if (data.shortcut && data.shortcut.entryPath) {
                            var linkpath = data.shortcut.entryPath.split("\\");
                            if (linkpath.length > 0) {
                                if (linkpath[0] == "1") {
                                    type = linkpath[0];
                                    modulepath = "enterprise";
                                } else if (linkpath[0] == "7") {
                                    type = linkpath[0];
                                    modulepath = "team";
                                } else if (linkpath[0] == "2") {
                                    type = linkpath[0];
                                    modulepath = "person";
                                }
                            }
                        }
                    }
                }
                if (modulepath) {
                    //企业库    
                    if (modulepath == "enterprise" || modulepath == "preview" || modulepath=="myvisit") {
                        switch (entityType.type) {
                            //企业库单文件
                            case "singlefile":
                                optiontype = "1001";
                                break;
                            //企业库多文件
                            case "multiplefiles":
                                optiontype = "1002";
                                break;
                            //企业库文件夹
                            case "singlefolder":
                                optiontype = "1003";
                                break;
                            //企业库多文件夹
                            case "multiplefolders":
                                $('.v6-attraction').hide() && $(".v6-attraction-open").hide();
                                optiontype = "1004";
                                break;
                            //企业库混合文件
                            case "composed":
                                optiontype = "1005";
                                break;
                            //企业库外发
                            case "publishfile":
                                optiontype = "1008";
                                break;
                            case "publishfolder":
                                optiontype = "1008";
                                break;
                        }
                    } else if (modulepath == "person") {
                        switch (entityType.type) {
                            //个人库单文件
                            case "singlefile":
                                optiontype = "3001";
                                break;
                            //个人库多文件
                            case "multiplefiles":
                                optiontype = "3002";
                                break;
                            //个人库文件夹
                            case "singlefolder":
                                optiontype = "3003";
                                break;
                            //个人库多文件夹
                            case "multiplefolders":
                                optiontype = "3004";
                                break;
                            //个人库混合文件
                            case "composed":
                                optiontype = "3005";
                                break;
                            //个人库外发
                            case "publishfile":
                                optiontype = "3008";
                                break;
                            case "publishfolder":
                                optiontype = "3008";
                                break;
                        }
                    } else if (modulepath == "team") {
                        switch (entityType.type) {
                            //团队库单文件
                            case "singlefile":
                                //modify: 2022-03-04 shiyan.li 缺陷[27263]（ui）团队库选择文件夹打开右侧面板，通过磁吸打开右侧面板，进入文件夹，磁吸消失
                                !rightPanelState() && $('.v6-attraction').show() && $(".v6-attraction-open").hide();
                                optiontype = "7001";
                                break;
                            //团队库多文件
                            case "multiplefiles":
                                optiontype = "7002";
                                break;
                            //团队库文件夹
                            case "singlefolder":
                                !rightPanelState() && $('.v6-attraction').show() && $(".v6-attraction-open").hide();
                                optiontype = "7003";
                                break;
                            //团队库多文件夹
                            case "multiplefolders":
                                optiontype = "7004";
                                break;
                            //团队库混合文件
                            case "composed":
                                optiontype = "7005";
                                break;
                            //团队库外发
                            case "publishfile":
                                optiontype = "7008";
                                break;
                            case "publishfolder":
                                optiontype = "7008";
                                break;
                            case "team":
                                optiontype = "7007";
                                break;
                        }
                    } else if (modulepath == "recyclebinlist") {
                        //回收站
                        if (messager && messager.select.length > 0 && messager.select[0].path) {
                            //判断选中的第一个文件源属于企业库还是团队库
                            var recyclepath = messager.select[0].path.split("\\");
                            if (recyclepath[0] == "1") {
                                optiontype = "1009";
                            } else if (recyclepath[0] == "7") {
                                optiontype = "7009";
                            } else if (recyclepath[0] == "2") {
                                optiontype = "3009";
                            }
                        } else if (window.location.hash) {
                            var hash = window.location.hash.split("/")
                            if (hash.length > 2) {
                                if (hash[1] == "enterprise") {
                                    optiontype = "1009";
                                } else if (hash[1] == "team") {
                                    optiontype = "7009";
                                } else if (hash[1] == "person") {
                                    optiontype = "3009";
                                }
                            }
                        }
                    } else if (modulepath == "mylock") {
                        switch (entityType.type) {
                            //我的锁定
                            case "mylockinfo":
                                optiontype = "8005" //锁定首页
                                break;
                            case "singlefile":
                                optiontype = "1001";
                                break;
                            //多文件
                            case "multiplefiles":
                                optiontype = "1002";
                                break;
                            //文件夹
                            case "singlefolder":
                                optiontype = "1003";
                                break;
                            //多文件夹
                            case "multiplefolders":
                                optiontype = "1004";
                                break;
                            //混合文件
                            case "composed":
                                optiontype = "1005";
                                break;
                        }
                    } else if (modulepath == "favoritelist") {
                        //我的收藏
                        switch (entityType.type) {
                            case "singlefile":
                                optiontype = "1001";
                                break;
                            //多文件
                            case "multiplefiles":
                                optiontype = "1002";
                                break;
                            //文件夹
                            case "singlefolder":
                                optiontype = "1003";
                                break;
                            //多文件夹
                            case "multiplefolders":
                                optiontype = "1004";
                                break;
                            //混合文件
                            case "composed":
                                optiontype = "1005";
                                break;
                        }
                    } else if (modulepath == "teamcreate") {
                        //团队顶级 创建团队页面
                        optiontype = "7006";
                    } else if (modulepath == "teamfolder") {
                        //选中团队
                        optiontype = "7007";
                    } else if (modulepath == "mytag") {
                        //我的标签
                        switch (entityType.type) {
                            //单文件
                            case "singlefile":
                                optiontype = "1001";
                                break;
                            //多文件
                            case "multiplefiles":
                                optiontype = "1002";
                                break;
                            //文件夹
                            case "singlefolder":
                                optiontype = "1003";
                                break;
                            //多文件夹
                            case "multiplefolders":
                                optiontype = "1004";
                                break;
                            //混合文件
                            case "composed":
                                optiontype = "1005";
                                break;
                        }
                    } else if (modulepath == "sharetome") {
                        switch (entityType.type) {
                            //单文件
                            case "singlefile":
                                optiontype = "1001";
                                break;
                            //多文件
                            case "multiplefiles":
                                optiontype = "1002";
                                break;
                            //文件夹
                            case "singlefolder":
                                optiontype = "1003";
                                break;
                            //多文件夹
                            case "multiplefolders":
                                optiontype = "1004";
                                break;
                            //混合文件
                            case "composed":
                                optiontype = "1005";
                                break;
                        }
                    } else if (modulepath == "outpublish") {
                        //我的外发文件不选中显示信息
                        switch (entityType.type) {
                            case "composed":
                                optiontype = "1008";
                                break;
                        }
                    }
                }

                //把干掉的外发附加的数据重新附加一遍
                if (messager.select && messager.select.length === 1 && messager.select[0].permissionData) {
                    data.permissionData = messager.select[0].permissionData;
                    data.operationData = messager.select[0].operationData;
                }
                //添加属性stateDesc
                addPropStateDesc(data);
                if (data) {
                    if (typeof (data.name) != "undefined") {
                        data.shortName = data.name;
                    }
                    //复选框选中文件会没有type属性  手动加上
                    if (data.type == undefined) {
                        data.type = data.testtype;
                    }
                    that.initData.sourceData = data;
                    //modify by 20220304 zhengjie v6解决多选文件报错问题
                    that.initData.fromJS(data); //note: 20127-10-09 haibin.long 这样的fromJS应该是做的对象合并，如果上一次是文件夹对象，本次是文件对象，合并有会有fileType和folerType共存的现象，对吗？
                    //modify: 2018-01-31 haibin.long 处理shortcut的影响，如果原文件没有shortcut且initData上存在shortcut，移除
                    if (!data.shortcut && that.initData().shortcut) {
                        delete that.initData().shortcut;
                    }
                }
                //缺陷28309 -zhifeiya/haiwei.cui-20220324（KB上无磁吸）我的收藏点击文件，右侧面板磁吸图标会出来后再隐藏
                // 磁吸模块式显示控制
                var IsOpenType = that.panelAttractionIsOpen() ? 1 : 0;
                if ((window.location.hash == "#doc/team/7" && messager.module == "teamfolder") || messager.module == "mylock" || messager.module == "recyclebinlist" || messager.module == "myfavorite" || messager.select.length == 0) {
                    IsOpenType = 0;
                }
                //modify by haiwei.cui 20220419 个人库根目录不显示磁吸
                if (messager.module == "person" && data.parentFolderId == 2) {
                    IsOpenType = 0;
                }

                var type = entityType.type;
                var filetype = data.type;
                //轻文档
                // if (data.fileType == 128){
                //     filetype = data.fileType;
                // }
                //当前路径
                var path = messager.module;
                var curLanger = $.getLan();
                var entityType = entityType.type;
                var select = messager.select;
                //缺陷28309 -zhifeiya/haiwei.cui-20220324磁吸逻辑调整
                var showModuels = constinfo.getTypeRightList(optiontype, type, filetype, path, curLanger, entityType, select);
                var showModuelsCount;
                if (IsOpenType == 1) {
                    showModuelsCount = constinfo.getTypeRightListCount(optiontype, type, filetype, path, curLanger, entityType, select);
                }
                // modify 2022-04-20 liuchunli #28894 搜索结果页面附件的右侧面板附件为空时，提示语位置错误并且缺少添加按钮
                if (select.length == 1) {
                    if (select[0].isAttachment && select[0].isAttachment == true) {
                        showModuels = _.filter(showModuels, function (item) {
                            return item.name !== "accessory";
                        })
                    }
                }    
                //modify by hanshunlong,2022-04-27,[缺陷29759]，企业内容库根目录右侧面板存在元数据入口
                //如果选择的是企业根目录，过滤掉元数据
                //modify by shangmeng.cai,20230104,[QC19190],隐藏文件夹6 根节点过滤掉元数据
                if (select && select.length == 1 && (select[0].id == 1 || select[0].id == 6)) {
                    showModuels = _.filter(showModuels, function (item) {
                        return item.name !== "matedataInfo";
                    })
                }

                //如果是轻文档,只显示基本信息和动态
                //if (data.fileType == 128 && select && select.length == 1){
                //    showModuels = _.filter(showModuels, function (item) {
                //        return item.name == "basicInfo" || item.name == "dynamic";
                //    })
                //}

                // modify 2022-08-17 liuchunli #35191【产品确认】共享给用户编辑权限，进入共享给我中，选择文件进行添加附件
                if (select && select.length == 1 && select[0].module && select[0].module == "sharerecoredlist") {
                    showModuels = _.filter(showModuels, function (item) {
                        return item.name == "basicInfo" || item.name == "historyFile";
                    })
                }
                //判断缓存存入的磁吸开关
                ////缺陷28309 -zhifeiya/haiwei.cui-20220324磁吸逻辑调整
                if (IsOpenType == 1 && showModuels && showModuels.length > 0) {
                    if (!showModuelsCount || JSON.stringify(showModuelsCount) == '{}') {
                        //modify: 2022-03-04 shiyan.li 缺陷[27158]（ui）三库的根目录和多选文件（夹）时显示磁吸面板（产品文档要求都不显示磁吸）
                        $(".v6-attraction").css({ 'width': '0' });
                        $(".v6-attraction-open").hide();
                    }
                    else if (rightPanelState()) { //modify by haiwei.cui 20220419 缺陷29209，页面缩放110%时磁吸不显示问题
                        $(".v6-attraction").css({ 'width': '0' });
                        $(".v6-attraction-open").hide();
                    }
                    else {
                        var IspanelAttractionWidth = $.cookie('paramWidth');
                        if (IspanelAttractionWidth != null) {
                            if (IspanelAttractionWidth == "true") {
                                $(".v6-attraction").show();
                                $(".v6-attraction").css('width', '50px');
                                $(".v6-attraction-open").hide();
                                var curentModules = require('knockout').$store.state.curentModule();
                                if (curentModules && curentModules.hash != 'quick/mydocflow') {
                                    if ($.cookie("enterpriseRightPanelType") == 'true' && curentModules && curentModules.hash) {
                                        that.infoWidth(50);
                                        that.infobarCollapsing.setSize(50);
                                        app.trigger('centerPanel:width');
                                        if (curentModules.hash == "quick/mytag" || curentModules.hash == "quick/myvisit"|| curentModules.hash == "quick/myfavorite"|| curentModules.hash == 'quick/myshareme' 
                                        || curentModules.hash.indexOf('/search') > -1
                                        ) {
                                            // if (that.infoWidth() > 40) {
                                            //     $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                                            // }else{
                                            //     $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                                            // }
                                        }
                                    }
                                }
                            } else if (IspanelAttractionWidth == "false") {
                                $(".v6-attraction").css('width', '0px');
                                $(".v6-attraction-open").show();
                                var curentModules = require('knockout').$store.state.curentModule();
                                if (curentModules && curentModules.hash != 'quick/mydocflow') {
                                    if ($.cookie("enterpriseRightPanelType") == 'true' && curentModules && curentModules.hash) {
                                        that.infoWidth(50);
                                        that.infobarCollapsing.setSize(50);
                                        app.trigger('centerPanel:width');
                                        if (curentModules.hash == "quick/mytag" || curentModules.hash == "quick/myvisit"|| curentModules.hash == "quick/myfavorite"|| curentModules.hash == 'quick/myshareme' 
                                        || curentModules.hash.indexOf('/search') > -1
                                        ) {
                                            // if (that.infoWidth() > 40) {
                                            //     $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                                            // }else{
                                            //     $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                                            // }
                                        }
                                    }
                                }
                            }
                        } else {
                            $.cookie('paramWidth', true);
                            $(".v6-attraction").show();
                            $(".v6-attraction").css('width', '50px');
                            $(".v6-attraction-open").hide();
                            var curentModules = require('knockout').$store.state.curentModule();
                            if (curentModules && curentModules.hash != 'quick/mydocflow') {
                                if ($.cookie("enterpriseRightPanelType") == 'true' && curentModules && curentModules.hash) {
                                    that.infoWidth(50);
                                    that.infobarCollapsing.setSize(50);
                                    app.trigger('centerPanel:width');
                                    if (curentModules.hash == "quick/mytag" || curentModules.hash == "quick/myvisit"|| curentModules.hash == "quick/myfavorite"|| curentModules.hash == 'quick/myshareme' 
                                    || curentModules.hash.indexOf('/search') > -1
                                    ) {
                                        if (that.infoWidth() > 40) {
                                            $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().hide()
                                        }else{
                                            $(".docsetbar .docsetbar-btn-list .docsetbar-btn .right-panels").parent().show()
                                        }
                                    }
                                }
                                    
                            }
                        }
                        //磁吸显示
                        for (var i = 0; i < showModuels.length; i++) {
                            switch (showModuels[i].name) {
                                // 元数据信息
                                case "basicInfo":
                                    showModuels[i].attractionCount = -1
                                    break;

                                // 元数据信息
                                case "matedataInfo":
                                    showModuels[i].attractionCount = showModuelsCount.data.metaCount > 99 ? '99+' : showModuelsCount.data.metaCount
                                    break;

                                // 历史文件 
                                case "historyFile":
                                    showModuels[i].attractionCount = showModuelsCount.data.historyVerCount > 99 ? '99+' : showModuelsCount.data.historyVerCount
                                    break;

                                // 关联文件
                                case "attachFile":
                                    showModuels[i].attractionCount = showModuelsCount.data.relatedCount > 99 ? '99+' : showModuelsCount.data.relatedCount
                                    break;

                                // 附件
                                case "accessory":
                                    showModuels[i].attractionCount = showModuelsCount.data.attachCount > 99 ? '99+' : showModuelsCount.data.attachCount
                                    break;

                                // 动态
                                // case "dynamic":
                                //     showModuels[i].attractionCount = 981216 // 接口还没返回暂时先写死放在这等接口处理放开下行注释
                                //     // showModuels[i].attractionCount = showModuelsCount.data.dynamic > 99 ? '99+' : showModuelsCount.data.dynamic
                                //     break;
                                // 相关文件
                                case "relatedFile":
                                    // 相关文件为附件和关联文件合一，手动计算下两者总量
                                    let relatedFileNum = showModuelsCount.data.relatedCount + showModuelsCount.data.attachCount;
                                    showModuels[i].attractionCount = relatedFileNum > 99 ? '99+' : relatedFileNum
                                    break;

                                // 团队日志
                                case "teamLog":
                                    showModuels[i].attractionCount = -1
                                    break;

                                // 团队公告
                                case "teamnotice":
                                    showModuels[i].attractionCount = -1
                                    break;

                                // 评论数量
                                case "commentsAndAnnotation":
                                    showModuels[i].attractionCount = showModuelsCount.data.annotationsCommentCount > 99 ? '99+' : showModuelsCount.data.annotationsCommentCount

                            }
                        }
                    }
                }
                else {
                    $(".v6-attraction").css({ 'width': '0' });
                    $(".v6-attraction-open").hide();
                }
                ////缺陷28309 -zhifeiya/haiwei.cui-20220324磁吸逻辑调整（和右侧面板业务分开处理）
                //根据类型获取右侧面板
                if (path == "myfavorite") {
                    showModuels = []
                }
                if (showModuels && showModuels.length > 0) {
                    var showModuelsItemCount = constinfo.getTypeRightListCount(optiontype, type, filetype, path, curLanger, entityType, select);
                    if (JSON.stringify(showModuelsCount) != '{}') {
                    for (var i = 0; i < showModuels.length; i++) {
                        switch (showModuels[i].name) {
                            // 元数据信息
                            case "basicInfo":
                                showModuels[i].attractionCount = -1
                                break;

                            // 元数据信息
                            case "matedataInfo":
                                showModuels[i].attractionCount = showModuelsItemCount.data.metaCount > 99 ? '99+' : showModuelsItemCount.data.metaCount
                                break;

                            // 历史文件 
                            case "historyFile":
                                showModuels[i].attractionCount = showModuelsItemCount.data.historyVerCount > 99 ? '99+' : showModuelsItemCount.data.historyVerCount
                                break;

                            // 关联文件
                            case "attachFile":
                                showModuels[i].attractionCount = showModuelsItemCount.data.relatedCount > 99 ? '99+' : showModuelsItemCount.data.relatedCount
                                break;

                            // 附件
                            case "accessory":
                                showModuels[i].attractionCount = showModuelsItemCount.data.attachCount > 99 ? '99+' : showModuelsItemCount.data.attachCount
                                break;
                            // 相关文件
                            case "relatedFile":
                                // 相关文件为附件和关联文件合一，手动计算下两者总量
                                let relatedFileNum = showModuelsItemCount.data.relatedCount + showModuelsItemCount.data.attachCount;
                                showModuels[i].attractionCount = relatedFileNum > 99 ? '99+' : relatedFileNum
                                break;

                            // 团队日志
                            case "teamLog":
                                showModuels[i].attractionCount = -1
                                break;

                            // 团队公告
                            case "teamnotice":
                                showModuels[i].attractionCount = -1
                                break;

                            // 评论数量
                            case "commentsAndAnnotation":
                                showModuels[i].attractionCount = showModuelsItemCount.data.annotationsCommentCount > 99 ? '99+' : showModuelsItemCount.data.annotationsCommentCount

                        }
                    }
                    }
                }
                if (showModuels && showModuels.length > 0) {

                    if (path == 'teamfolder' && showModuels.findIndex(function (item) { return item.classname == 'teammemberlist' }) == -1) { // 团队库信息右侧面板 前端 额外添加一个按钮
                        var extraButton = {
                            "name": "teammemberlist",
                            "show": true,
                            "title": lang.memberInformation,
                            "classname": "teammemberlist",
                            "isActive": ko.observable(false),
                            "entityType": "team",
                            "ContentAddress": null,
                            "ContentBaseUrl": null,
                            "ContentType": 2,
                            "ParameterSet": null,
                            "attractionCount": -1
                        };
                        showModuels.splice(1, 0, extraButton);
                    };

                    
                    //modify zhaobaosheng 2022-02-25   [缺陷编号:26779] 用户1共享文件给不在该团队的用户2，用户2在共享给我中可以对附件进行查看、下载等操作
                    var isshare = ($.getQueryString("myshare") || $.getQueryString("shareperm") != '' || $.getQueryString("shareCode") != '') &&  (window.location.pathname.indexOf('preview') > -1) ;
                    if (messager.module == "sharetome" && entityType == "singlefile") {
                        isshare = true;
                    }

                    var shareshowModuels = new Array();
                    // $.each(showModuels, function (index, item) {
                    //     if (isshare) {
                    //         //modify 2022-03-16 yangyang.li 缺陷27742 【UI】打开共享给我右侧面板后再打开企业库和最近访问的右侧面板会缺少关联文件tab页，反之共享给我右侧面板多出关联文件tab页
                    //         item.name != "accessory" && item.name != "attachFile" && shareshowModuels.push(item);
                    //     }
                    //     else if (path == "favoritelist" && item.name != "accessory"){
                    //         shareshowModuels.push(item)
                    //     }
                    //     else {
                    //         shareshowModuels.push(item);
                    //     }
                    // });
                    //modify 2022-03-16 yangyang.li 缺陷27742 【UI】打开共享给我右侧面板后再打开企业库和最近访问的右侧面板会缺少关联文件tab页，反之共享给我右侧面板多出关联文件tab页
                    for (var i = 0; i < showModuels.length; i++){
                        var item = showModuels[i];
                        if (isshare) {
                            if (item.name == "accessory" || item.name == "attachFile") {
                                continue;
                            }
                        }
                        else if (path == "favoritelist") {
                            if (item.name == "attachFile") {
                                continue;
                            }
                        }
                        else if (path == "mylock") {
                            if (item.name == "attachFile") {
                                continue;
                            }
                        }
                        else if (path == "mytag") {
                            if (item.name == "attachFile") {
                                continue;
                            }
                        }
                        //modify 2022-06-02 tianyi.wang 缺陷30918【GMP】开启GMP之后，磁吸面板和右侧面板仍然显示批注按钮
                        else if (path == "preview" && that.hiddenAnnotation()) {
                            if (item.name == "commentsAndAnnotation") {
                                continue;
                            }
                        }
                        //modify 20220704 zhengjie 32733文件（夹）动态，只有编辑及以上权限时，才能看到动态
                        if (item.name == "dynamic") {
                            if(!messager.select || messager.select.length != 1){
                                continue;
                            }
                            else {
                                var permissionValue = messager.select[0].data.permission;
                                if(permissionValue == -2){
                                    var doctype = 1;
                                    if(messager.select[0].isFolder==false || messager.select[0].fileType==2){
                                        doctype = 2;
                                    }
                                    var docId = doctype + ":" + messager.select[0].data.id;
                                    var resultPerm = fileAndFolderCache.get(docId);
                                    var perm = resultPerm[docId].split("|")[0];
                                    permissionValue = parseInt(perm);
                                }
                                if((permissionValue & permtype.viewLog) != permtype.viewLog){
                                    continue;
                                }
                            }
                        }
                        shareshowModuels.push(item);
                    }

                    app.trigger("infobartagAttractionlist:reload", { data: shareshowModuels })

                    this.currentModule = shareshowModuels[0];
                    // console.log('showModuels',showModuels);
                    // RD-884 右侧面板优化 -- 磁吸上下拖动位置处理
                    var bodyHeight = $('body').height()
                    var cixiHeight = $("#v6-attraction-circle").height()
                    if ($("#v6-attraction-circle").css('top')) {
                        if (bodyHeight - cixiHeight == $("#v6-attraction-circle").css('top').split("px")[0] || $("#v6-attraction-circle").css('top').split("px")[0] > bodyHeight - cixiHeight) {
                            $("#v6-attraction-circle").css({
                                top:bodyHeight-cixiHeight + 'px'
                            })
                        }
                    }
                    // modify by lxf 切换保持右侧展示模块不变
                    let currentItem = that.InfobarItems().filter(el => {
                        return el.isActive();
                    })
                    let repeat_index = 0;
                    currentItem.length && shareshowModuels.forEach((el, i) => {
                        if (currentItem[0].name == el.name) {
                            repeat_index = i
                        }
                    });
                    //  2022-12-28  #38324 【需产品设计】附件在搜索结果页面中，右侧面板应只展示基本信息
                    var newAttachmentList = [];//前端处理附件不显示右侧面板其他信息
                    if (that.initData() && that.initData().isAttachment && that.initData().isAttachment()) {
                        shareshowModuels.map(item=>{
                            if (item.name == "basicInfo") newAttachmentList.push(item)
                        })
                        that.InfobarItems(newAttachmentList);
                    }else{
                        that.InfobarItems(shareshowModuels);
                    }
                    that.loadCurrentModules(shareshowModuels[repeat_index]);
                    setTimeout(() => {
                        if (that.infoWidth()<=50) {
                            that.clearAllSelected()
                        }
                    }, 0);
                    // modify: cuizhichao 2023-01-03 #41952 文件/文件夹切换时，业务工作区无法及时切换为对应内容
                    that.InfobarItems().forEach(item=>{
                        if (item.attractionCount >= 0 && $(`.rightPanelBar .testClass${item.classname}`)) {
                            $(`.rightPanelBar .testClass${item.classname}`).text(item.attractionCount)
                        }
                    })
                }
                // }
            });
            this.infobarCollapsing.init();
        }
        //modify by cq.taiqi 2017.9.1 用数字序号匹配有问题，直接用data去匹配  
        inforBarPt.onInfobarClick = function (data, event) {
            var that = this;

            // 传递消息给轻文档编辑器，重置编辑器宽度
            var docflowIfarm = document.getElementById('DocflowIfarm')
            if (docflowIfarm) {
                docflowIfarm.contentWindow.postMessage(
                {
                    type: 'ECM-DOCFLOW-RESIZE-EDITOR-SIZE',
                    data: {
                    paddingRight: $(event.currentTarget).hasClass("selected") ? 50 : 370
                    },
                },
                '*'
                )
            }
            

            //缺陷27785-zhifeiya-右侧面板多次点击元数据按钮，元数据tab页出现错误（修改方式是：禁止重复恶意点击）
            if (event && event.currentTarget) {
                var currentClass = $(event.currentTarget).hasClass("selected")
                if (currentClass) {


                    // console.log('点击了当前已经展开来的面面板，这时候需要取消显示');
                    // if (this.rightPanelContentStates()) {
                    //     this.rightPanelContentStates(false)
                    // }
                    // if(!this.rightPanelContentStates()){
                    //     // $(".right-content").hide()
                    // }
                    // this.currentModule.isActive(false);
                    // 重新设置大小
                    // if ($(".right-content").css("display") == 'none') {
                    //     //这个时候要重新设置宽度了 宽度应该设置为右侧工作区的宽度 50
                    //     that.infoWidth(50)
                    //     this.infobarCollapsing.setSize(50)
                    // }
                    this.clearAllSelected();
                    that.infoWidth(50)
                    this.infobarCollapsing.setSize(50)
                    if (window.location.href.indexOf("/preview.html") > -1) {
                        app.trigger("setRightPanelWidth:childrenclose", { type: 'previewChildren'});
                    }
                    app.trigger('centerPanel:width');
                    return false;
                }else{
                    // this.rightPanelContentStates(true)
                    // if(this.rightPanelContentStates()){
                    //     // $(".right-content").show()
                    // }
                }
                // if ($(".right-content").css("display") == 'block') {
                    //这个时候要重新设置宽度了 右侧工作区的宽度加上右侧面板的宽度 
                    // that.infoWidth(370)
                    // this.infobarCollapsing.setSize(370)
                    $(".informationbarcolor").css({"border-left":"1px solid #eee"})
                // }
            }
            this.clearAllSelected();
            this.currentModule = data;
            this.currentModule.isActive(true);
            this.loadCurrentModule();
            var curentModule = require('knockout').$store.state.curentModule();
            if (window.location.pathname.indexOf('preview') > -1) {
                if ($(".informationbar-wrap").is(":hidden")) {
                    if ($("#mainInformationbar").is(":hidden")) {
                        $("#mainInformationbar").show();
                    }
                    $("#mainInformationbar").css({ "width": "300px", "margin-left": "-300px" });
                    $("#mainComponent").css({ "margin-right": "300px" });
                    $(".informationbar-wrap").show();
                    $(".switch-item").hide().eq(0).show();
                    $(window).resize();
                }

            }
            // if (!curentModule) {

            //     this.panelAttraction.hide()

            // } else {
            //     this.infobarCollapsing.setSize(this.infoWidth())
            // }
            that.infoWidth(370)
            this.infobarCollapsing.setSize(370)
            if (window.location.href.indexOf("/preview.html") > -1) {
                app.trigger("setRightPanelWidth:children", { type: 'previewChildren'});
            }
            //modify: 缺陷26150 打开右侧面板，会遮蔽1.翻页按钮及页面数显示2.缩略图状态下文件/文件夹被隐藏 by mengfangchao 2022.02.25-start
            // modify by shaobo.zheng 2022.04.18 缺陷29116点击右侧面板，点击任何右侧面板菜单栏，页面分页器居中显示
            //var pageSection = document.getElementById('pageSection');
            //if (pageSection != null && parseInt(pageSection.style.right.replace('px'))<320) {
            //    pageSection.style.right = parseInt(pageSection.style.right.replace('px')) + 320 + 'px';
            //}
            app.trigger('centerPanel:width');
            //modify: 缺陷26150 打开右侧面板，会遮蔽1.翻页按钮及页面数显示2.缩略图状态下文件/文件夹被隐藏 by mengfangchao 2022.02.25-end

        }
        inforBarPt.onInfobarClickOpen = function (data, event) {
            // var panelAttractionWidth = this.panelAttraction.width();
            // //磁吸开关存入缓存
            // if (panelAttractionWidth == 0) {
            //     $.cookie('paramWidth', true);
            // } else {
            //     $.cookie('paramWidth', false);
            // }
            // this.infobarCollapsing.setAttraction(panelAttractionWidth)
        }
        inforBarPt.toggleRightPanelBar = function (data, event) {
            var that = this;
            // if ($(".right-content").css("display") == 'block') {
            //     this.clearAllSelected();
            //     that.infoWidth(50)
            //     $(".right-content").hide()
            // }
            var $closeRightPanel = $(".docsetbar-btn-list .right-panels").parent();
            if ($closeRightPanel) $closeRightPanel.click();
        },
        inforBarPt.formateData = function (messager) {
            if (!$.isArray(messager.select)) {
                return this.service.formateData(messager); //shortcut的格式优化
            }
            var data = this.service.formateData(messager);
            this.service.wrapData(data, messager);
            return data;
        }
        /*
        * sailer创建于2015年5月
        * 大小计算
        * @size 大小
        */
        inforBarPt.clculateSize = function (size) {
            return constInfo.formatSize(size);
        }
        /**
         * 加载模块，存在则使用缓存，并调用数据刷新接口（如果存在此接口）
         */
        inforBarPt.loadCurrentModule = function () {
            if (this.currentModule.entityType == 'publish') {
                this.currentModule.entityType = 'singlefolder';
            }
            var that = this;
            // modify: 2022-04-22 cuizhichao 27546【专项优化】【UI】切换标签页时，右侧面板上方的标题（文件/文件夹名称）会闪烁 
            let oldDom = $('.informationbar-box > div').eq(0).clone(true)
            $('.right-content').append(oldDom);
            $('.right-content > div:last').css({
                position: 'absolute',
                top:'0',left:'0',right:'0',bottom:'0',
                background: 'rgba(255,255,255,1)',
                width:'100%',zIndex: 5,
            })
            //当类型为url配置的时候
            if (this.currentModule && this.currentModule.ContentType == 1) {
                that.getAddress();
            } else {
                this.isurlconfig(false)
                path = this.getModulePath(this.currentModule);
                var funExecute = function (module, isExecute) {
                    if (module.refreshData && isExecute) { //执行数据刷新方法
                        module.refreshData(that);
                    }
                    that.activeView(module);
                    // modify: 2022-04-22 cuizhichao 27546【专项优化】【UI】切换标签页时，右侧面板上方的标题（文件/文件夹名称）会闪烁 
                    // modify: 2022-04-24 cuizhichao  27546【专项优化】【UI】切换标签页时，右侧面板上方的标题（文件/文件夹名称）会闪烁 (优化时间避免初次头部闪烁)
                    setTimeout(() => {
                        $('.right-content > div:last').remove()
                    }, 260);
                };
                if (that.modulesCache[path]) {
                    funExecute(that.modulesCache[path], true);
                    return;
                }
                require([path], function (module) {
                    funExecute(module);
                    that.modulesCache[path] = module;
                });
            }
        }

        inforBarPt.loadCurrentModules = function (data) {
            if (data && this.infoWidth() > 50) {
                if (this.infoWidth() > 50) {
                    if (window.location.hash.indexOf("team") > -1) {
                        if ($.cookie("enterpriseRightPanelType") == 'true') {
                            this.infobarCollapsing.setSize(370)
                        }
                    }else if (window.location.hash.indexOf("person") > -1) {
                        if ($.cookie("enterpriseRightPanelType") == 'true') {
                            this.infobarCollapsing.setSize(370)
                        }
                    }else if (window.location.hash.indexOf("enterprise") > -1) {
                        if ($.cookie("enterpriseRightPanelType") == 'true') {
                            this.infobarCollapsing.setSize(370)
                        }
                    }
                }
                $.each(this.InfobarItems(), function (i, v) {
                    if (data.id == v.id) {
                        v.isActive(true);
                    } else {
                        v.isActive(false);
                    }
                })
            }
            if (data.entityType == 'publish') {
                data.entityType = 'singlefolder';
            }
            var that = this;
            if (data.ContentType == 1) {
                that.getAddress(data);
            } else {
                this.isurlconfig(false)
                path = this.getModulePath(data);
                var funExecute = function (module, isExecute) {
                    if (module.refreshData && isExecute) {
                        module.refreshData(that);
                    }
                    if (that.activeView() && that.activeView().__moduleId__ == module.__moduleId__ && that.activeView().__moduleId__ == "doc/informationbar/team/index") {
                        return;
                    }
                    that.activeView(module);
                };
                if (that.modulesCache[path]) {
                    funExecute(that.modulesCache[path], true);
                    return;
                }
                require([path], function (module) {
                    funExecute(module);
                    //modify 2022-03-18 yangyang.li 缺陷27895 进入团队打开右侧面板,再返回团队库首页,右侧面板不刷新
                    //问题原因,双击进入团队时, 执行了两次单击事件,一次双击事件,两次单击事件第一次数据是目标团队库,第二次是首页团队库7,两次几乎同时执行造成此处脏数据,topteam的path,赋值了team/index的module
                    if (that.modulesCache[path]){
                        return;
                    }
                    that.modulesCache[path] = module;
                });
            }
        }

        inforBarPt.getAddress = function (datas) {
            this.isurlconfig(true);
            //转换数据
            var data = "";
            if (this.currentModule && this.currentModule.ParameterSet) {
                data = JSON.parse(this.currentModule.ParameterSet);
            } else {
                data = datas.ParameterSet;
            }

            if (!data.custom) {
                data = JSON.parse(data);
            }
            var parment = "?";
            var lang = "";
            //获取参数
            for (var i = 0; i < data.custom.rows.length; i++) {
                var custom = "";
                if (data.custom.rows[i].value == "{fileId}") {
                    data.custom.rows[i].value = this.initData().id();
                } else if (data.custom.rows[i].value == "{lastVerId}") {
                    data.custom.rows[i].value = this.initData().lastVerId();
                } else if (data.custom.rows[i].value == "{token}") {
                    data.custom.rows[i].value = constinfo.userInfo.token;
                }
                if (i != data.custom.rows.length - 1) {
                    custom = data.custom.rows[i].field + "=" + data.custom.rows[i].value + "&"
                } else {
                    custom = data.custom.rows[i].field + "=" + data.custom.rows[i].value;
                }
                parment = parment + custom;
            }
            //判断当前使用多语言
            if (constinfo.getLang() == "zh-cn") {
                lang = "&" + data.parameterfield + "=" + data.langZhcn;
            } else if (constinfo.getLang() == "en") {
                lang = "&" + data.parameterfield + "=" + data.langEn;
            } else if (constinfo.getLang() == "ja") {
                lang = "&" + data.parameterfield + "=" + data.langJa;
            } else {
                lang = "&" + data.parameterfield + "=" + data.langZhtw;
            }
            //拼接地址
            var address = this.currentModule.ContentBaseUrl + this.currentModule.ContentAddress + parment + lang;
            this.urladdress(address);
        }
        return new InformationBar();
    });

/*
@author huang.hao@edoc2.com
@time 2017年11月25日
@desc 结合easyui tooltip和qrcode形成的模块,
      每个发出tip的锚点必须有data-make-code="false",data-qrcode-content="xxx"两个属性
      data-make-code,表示是否在第一次调用onShow的时候根据data-qrcode-content（如果没有，则为传入的config.text的值）的值生成过了二维码（避免重复生成二维码）
      data-qrcode-content="xxx"则用于在选择器选择了多个节点（如用类选择器）的时候，在onSHow的时候，会根据此属性值生成二维码
      锚点格式示例：<span data-make-code="false" data-qrcode-content="xxx">anchor</span>
      config.getNewContent和data-qrcode-content="xxx的作用一样,不过是通过js赋值，属性那个则适用于ko对dom属性赋值
      实现方式如
      config.getNewContent = function(){
         return "nihao";
      }
*/
define('common/edoc2v5-qrcode',['qrcode', 'jquery',lang(),'knockout', "komapping","header/userinfo/constinfo"], function (QRCode, $,lang,ko, komapping,constInfo) {
    //实现hover显示二维码的效果
    /*用法如下
        makeHoverQrcode({
			selector: '#anchor1',
			text: 'ni hao',
			width: 200,
			height: 200,
			position:'top',
            hideDelay:200,
            getNewContent:function(){},
            onShow:function(){}
		});
    */
    var app = require('durandal/app')
    function makeHoverQrcode(config) {
        if (!config) throw Error(lang.PleasePassObj);
        var selector = null;
        if (!config.selector) {
            throw Error(lang.PleasePassJquery)
        } else {
            selector = config.selector;
        }
        var text = config.text ? config.text : "hello world";
        var position = config.position ? config.position : 'right';
        var hideDelay = config.hideDelay ? config.hideDelay : 200;
        var width = config.width ? config.width : 120;
        var height = config.height ? config.height : 120;
        var bgColor = config.bgColor ? config.bgColor : "#ffffff";
        var color = config.color ? config.color : "#000000";
        var deltaX = config.deltaX ? config.deltaX :0;
        var deltaY = config.deltaY ? config.deltaY :0;
        var qrcodeConfig = {
            width: width,
            height: height,
            colorDark: color,
            colorLight: bgColor,
            correctLevel: QRCode.CorrectLevel.H
        };
        var eleStr = '<div style="width:' + width + 'px;height:' + height + 'px;"></div>';
        //用于企业团队文件件内链功能
        if(config&&config.showLink){
            var webLink=config.getNewContent();
            eleStr="<div id='codeImg' class='codeImg fileUploadModalDoc' style='width:"+ 188 +"px;height:${height}px;'>" +
                "<span id='copyQR' class='copyQR form-button_v6' type='button'>"+lang.down+"</span>"+
                "<div style='position: absolute;padding-top:6px;padding-bottom:10px;'>" +
                    "<input type='text' value="+ webLink +" id='copy-text'  class='copy-text' style='opacity: 0;padding-left:1px;margin-right:0px;border-radius: 4px;border: 1px solid #d0d4d9;max-width: 120px;height:22px;font-size:12px;' />" +
                    "<span class='copyLink copy_QR_v6' type='button'>"+' '+lang.copy+"</span>"+
                "</div>"+
            "</div>";

           $("body").off("click",".codeImg .copyQR");//解决下载多次的问题
             $("body").on("click",".codeImg .copyQR", function(){
                var img =  $(this).closest(".codeImg").find("img").attr("src");
                constInfo.downloadFileImg('二维码.png', img);
                //alert(src);
                app.trigger("postMessageCloseRightPanel:show", {closeRightPanel:true});
             });
        }
        if(config&&config.isPublishManger||config.isMyPublish)
        {
            eleStr = '<div class="codeImg" style="width:' + width + 'px;height:' + 150 + 'px;"><div><span class="copyQR form-button_v6" type="button" style="position: absolute;margin-top: 120px;margin-left: 0px;">'+lang.down+'</span></div></div>';
             $("body").off("click",".codeImg .copyQR");//解决下载多次的问题
             $("body").on("click",".codeImg .copyQR", function(){
                var img =  $(this).closest(".codeImg").find("img").attr("src");
                constInfo.downloadFileImg('二维码.png', img);
                //alert(src);
             });
            
        }

        if(config && config.isRightPanel){ // 右侧面板 下载按钮特殊处理
            eleStr = '<div class="codeImg fileUploadModalDoc" style="width:' + width + 'px;height:' + 180 + 'px;"><div><span class="copyQR form-button_v6 " type="button" style="position: absolute;margin-top: 111px;margin-left: 0px;">'+lang.down+'</span></div></div>';
            $("body").off("click", ".codeImg .copyQR");
            $("body").on("click", ".codeImg .copyQR", function () {//modify:2021-01-23 caishangmeng v6 bug 绑定标签的类样式名错误
                var img = $(this).closest(".codeImg").find("img").attr("src");
                constInfo.downloadFileImg('二维码.png', img);
                //alert(src);
                app.trigger("postMessageCloseRightPanel:show", {closeRightPanel:true});
             });
        }
        if (config && config.isOutpublish || config.isShowDownBut || config.isMyPublish)
        {
            //26766【UI】外发页面的二维码下载按钮与二维码重叠
            // modify 2022-04-19 liuchunli #29167（UI）图片和其他类型文件在预览界面，二维码悬浮状态下大小不一致（KB也有此问题）
            if (config.isMyPublish){  //外发列表二维码样式
                 eleStr = '<div class="codeImg" style="width:' + width + 'px;height:' + 150 + 'px;"><div><span class="copyQR form-button_v6" type="button" style="position: absolute;margin-top: 120px;margin-left: 0px;">' + lang.down + '</span></div></div>';
                 $("body").off("click", ".codeImg .copyQR");//解决下载多次的问题
                 $("body").on("click", ".codeImg .copyQR", function () {//modify:2021-01-23 caishangmeng v6 bug 绑定标签的类样式名错误
                     var img = $(this).closest(".codeImg").find("img").attr("src");
                     constInfo.downloadFileImg('二维码.png', img);
                     //alert(src);
                 });
            } else if (config.isOutpublish) { //文件预览二维码样式
                //缺陷-26087-fei.chen-20220219-预览界面-预览二维码无下载按钮
                // 2022-03-18 cuizhichao 28123（UI）图片文件预览界面二维码下载按钮错位
                // eleStr = '<div class="codeImg" style="width:' + width + 'px;height:' + 180 + 'px;"><div><span class="copyQR form-button_v6" type="button" style="position: absolute;margin-top: 110px;margin-left: 0px;">' + lang.down + '</span></div></div>';
                // modify 2022-04-19 liuchunli #29167（UI）图片和其他类型文件在预览界面，二维码悬浮状态下大小不一致（KB也有此问题）
                if (width == '150') {
                    eleStr = '<div class="codeImg" style="width:' + width + 'px;height:' + 180 + 'px;"><div><span class="copyQR form-button_v6" type="button" style="position: absolute;margin-top: 158px;margin-left: 23px;">' + lang.down + '</span></div></div>';
                }else{
                    eleStr = '<div class="codeImg" style="width:' + width + 'px;height:' + 150 + 'px;"><div><span class="copyQR form-button_v6" type="button" style="position: absolute;margin-top: 120px;margin-left: 0px;">' + lang.down + '</span></div></div>';
                }
                $("body").off("click", ".codeImg .copyQR");//解决下载多次的问题
                $("body").on("click", ".codeImg .copyQR", function () {//modify:2021-01-23 caishangmeng v6 bug 绑定标签的类样式名错误
                    var img = $(this).closest(".codeImg").find("img").attr("src");
                    constInfo.downloadFileImg('二维码.png', img);
                    //alert(src);
                });
            } else {   //其他二维码样式
                 eleStr = '<div class="codeImg" style="width:' + width + 'px;height:' + 180 + 'px;"><div><span class="copyQR form-button_v6" type="button" style="position: absolute;margin-top: 155px;margin-left: 25px;">'+lang.down+'</span></div></div>';
                 $("body").off("click",".codeFileImg .copyQR");//解决下载多次的问题
                 $("body").on("click",".codeFileImg .copyQR", function(){
                     var img =  $(this).closest(".codeFileImg").find("img").attr("src");
                     constInfo.downloadFileImg('二维码.png', img);
                     //alert(src);
                 });
             }
            
           
        }
        if (config && config.isFilePublish)
        {
            // eleStr = '<div class="codeFileImg" style="width:' + width + 'px;height:' + 150 + 'px;"><div><input id="copyQR" class="copyQR form-button" type="button" value='+lang.down+' style="position: absolute;margin-top: 155px;margin-left: 25px;display:none;"/ ></div></div>';
             eleStr = '<div class="codeFileImg" style="width:' + width + 'px;height:' + height + 'px;"><div><input id="copyQR" class="copyQR form-button" type="button" value='+lang.down+' style="position: absolute;margin-top: 85px;margin-left: 53px;display:none;"/ ></div></div>';//v6解决外发二维码背景问题,文件外发下载二维码按钮样式优化
             $("body").off("click",".codeFileImg .copyQR");//解决下载多次的问题
             $("body").on("click",".codeFileImg .copyQR", function(){
                var img =  $(this).closest(".codeFileImg").find("img").attr("src");
                constInfo.downloadFileImg('二维码.png', img);
                //alert(src);
             });
           
        }
       
        $(selector).tooltip({
            'position': position,
            content: eleStr,
            hideDelay: hideDelay,
            deltaX:deltaX,
            deltaY:deltaY,
            onShow: function (e) {
                var $this = $(this);
                if (!$this.data('make-code')) {//如果值为undefined和false的时候，生成qrcode
                    $this.data('make-code', true);
                    var anchor = $(eleStr).get(0);
                    var qrcode = new QRCode(anchor, qrcodeConfig);
                    $this.data("anchor", anchor);
                    $this.data("qrcode", qrcode);
                }
                if ($this.data('qrcode-content')) {
                    text = $this.data('qrcode-content');
                } else if (config.getNewContent) {
                    text = config.getNewContent();
                }
                $this.data("qrcode").makeCode(text);
                $this.tooltip('update', $this.data("anchor"));
                $this.tooltip('tip').css({
                    'padding': config.isOutpublish ? '14px':'15px',
                    'border-radius': '5px'
                });
                if(config.onShow){
                    config.onShow.apply(this,arguments);
                }
                if(config.onCopy){
                    $this.url=text;
                    config.onCopy.apply($this,arguments);
                }
            },
        });
    }
    return {
        makeHoverQrcode: makeHoverQrcode,
    }
});
define('doc/informationbar/basicinfo/service',['jquery', 'knockout','../'+lang()], function ($, ko,lang) {
    function Core() {

    }
    var pt = Core.prototype;
    /**
    * 获取文件的基本信息
    * @param {number} - 文件Id
    */
    pt.getFileInfo = function (fileId) {
        return $.ajax({
            data: {
                module: "WebClient",
                fun: "GetFileById",
                fileId:fileId
            }
        });
    }
    /**
    * 获取文件夹的基本信息
    * @param {number} - 文件夹Id
    */
    pt.getFolderInfo = function (folderId) {
        var code = $.getQueryString("code");
        return $.ajax({
            data:{
                module: "WebClient",
                fun: "GetFolderId",
                folderId:folderId,
                token:getTokenByCondition(),
                code: code.length !== 0 ? code : ''
            }
        });
    }

    /**
     * 对快捷方式的文件进行包装
     * @param {object} shortCutFileInfo 需要包装的对象
     * @param {function} filter 对文件进行格式化的方法
     */
    pt.decorateFileByShortCut = function(shortCutFileInfo, filter){
        this.getFileInfo(shortCutFileInfo.shortcut.entryId()).done(function(result){
            if(result.result == 0){
                var fileInfo = filter(result.file);
                shortCutFileInfo.lastVerNumStr(fileInfo.lastVerNumStr);
                shortCutFileInfo.stateDesc(fileInfo.stateDesc);
                shortCutFileInfo.securityLevelName(fileInfo.securityLevelName);
            }      
        });
    }

    /**
     * 对快捷方式的文件夹进行包装
     * @param {*} shortCutFolderInfo 需要包装的对象 
     * @param {*} filter 对文件夹进行格式化的方法
     */
    pt.decorateFolderByShortCut = function(shortCutFolderInfo, filter){
        this.getFolderInfo(shortCutFolderInfo.shortcut.entryId()).done(function(result){
            if(result.result == 0 ){
                var folderInfo = filter(result.folder);
                shortCutFolderInfo.secName(folderInfo.secName);
                var folderDetails=$.format(lang.conclude, (folderInfo.childFileCount || 0), (folderInfo.childFolderCount || 0) )
                shortCutFolderInfo.conclude(folderDetails);
            }
        });
    }

    pt.ajaxGetCollInfoById=function(id){
       return $.ajax({
            data: {
                module: "WebClient",
                fun: "GetFavoritesByParentId",
                mnParentId: id,
                mnPermission: 0,
                docViewId: 0,
                argsXml: "<GetListArgs><PageSize>100000</PageSize><PageNum>1</PageNum></GetListArgs>"
            },
        })
    }
    

    return Core;
});
/*
    author:whimco
    create:2015-5-4
    desc:文档左侧列表导航目录
    mikahuang修改于20150518
    mikahuang修改于20161230 路由调整
    gaosong:创建二级导航视图2017年6月14日 17:07:35
    @editor:hao.huang@macrowing.com 优化doc界面的二级路由
*/
define('doc/index',["jquery",'durandal/app', 'plugins/router', 'knockout', "underscore", "header/userinfo/constinfo",
    "durandal/system", lang(), "doc/toolbar/index", "doc/contextmenu/index"],
    function ($,app, router, ko, _, constInfo,
        system, lang, Toolbar, contextmenu, utilsFun) {
        var navigationModel;
        var activeView = ko.observable();
        var defaultPanelWidth =  200;;
        var panelWidth = ko.observable(defaultPanelWidth);
        var panelOldWidth = ko.observable(200);
        var panelDocflowOldWidth = ko.observable(320);
        var navigationView = ko.observable("doc/navigation/index");
        var toolbarView = Toolbar.createToolbar('doc/');
        var docSetBarView = ko.observable("doc/docSetBar/index");
        // infoPreview
        var infoPreview =  ko.observable("doc/infoPreview/index");
        //信息面板
        var informationbarView = ko.observable("doc/informationbar/index");
        var PreviewModelView = ko.observable("doc/PreviewModel/index");
        //0,底部导航；1,左侧导航
        var curSelectNavType = ko.observable(1);
        
        var showDofclowEmpty = ko.observable(false);

        //0，图视图；2，图文视图   
        //if ($.cookie('sidebarType') && $.cookie('sidebarType').length < 0) {
        //    curSelectNavView(true);
        //} else {
        //    if ($.cookie('sidebarType') == 'mini') {
        //        curSelectNavView(false);
        //    } else if ($.cookie('sidebarType') == 'full') {
        //        curSelectNavView(true);
        //    }
        //}
        //cq.tianao 用cookie展开 关闭
        var sidebarType = $.cookie('sidebarType');//取出cookie值
        var curSelectNavView = ko.observable((sidebarType && sidebarType.length < 0) ? true : (sidebarType == 'mini' ? false : true))
        //左导航轻文档创建菜单
        var showDocflowMenu = ko.observable(false);
        var newTagFlag = ko.observable(false)//判断是否是标签页面，是否需要设置内容和按钮组的边距(PD-2017 cuizhichao （UI）我的标签优化)
        var isDocflowFlag = ko.observable(false)//判断是否是轻文档
        var newTagHeight = ko.observable()

        // 轻文档未读数据数量
        var docflowNowReadNum = ko.observable(0)
        var showDocflowNowReadNum = ko.observable(false)

        //默认选择项
        var defaultSelectItem = null;
        var childRouter = null;
        var jLeftPanel;
        var data = "";
        var isOpenLeftBar = ko.observable();
        var app = app;
        var rightPanelHeight = ko.observable(); // 右侧面板高度
        //modify: 2018-01-09 haibin.long 增加对工具栏更新事件的拦截
        function InterceptEvent() {
        }
        var pt = InterceptEvent.prototype;
        var treePutawayTop = ko.observable();
        var treePutawayLeft = ko.observable(340);
        var isMouseoverPutaway = ko.observable("");
        // var hideTreePutaway = ko.observable("block");//这个控制有时候不生效
        var showClosePanelBtn = ko.observable(true); //modify 2022-03-21 yangyang.li 缺陷28172 左侧导航来回切换时,左侧面板收缩按钮bug,拖拽分割线bug
        var isTreePutawayPutAway = ko.observable(false); // true 允许最大化 false 允许最小化
        var isTreeHideClass = ko.observable(true);
        var isRightPanlResize = ko.observable(false);   // modify:  20220426  shuchuen.hu #29241 点击左侧目录树的线（不是箭头），跳出目录（KB也有此问题）

        var isOnLeftTree = ko.observable(constInfo.systemInfo.instanceConfigInfo.DirectoryTreeSwitchSwitch);//记录左侧目录树收起展开状态
        var isOnLeftMainNavigation = ko.observable(constInfo.systemInfo.instanceConfigInfo.LeftNavSwitchVisiableSwitch);//记录左导航收起展开状态
        var isMyalbum = ko.observable(false);  // 是否为专辑页面

        var sharebutShow = ko.observable(false); // v6.1
        var sharcurentModule = ko.observable("");  // v6.1
        var docflowIframeUrl = ko.observable("");
        var sharebutText = ko.observable(lang.myshare);
        var mySharemeList = ko.observable();
        app.on('shareTabtext:lang').then(function(message) {
            if(message.module == "myshare"){
                sharebutText(lang.myshareme)
            }

            if(message.module == "myshareme"){
                sharebutText(lang.myshare)
            }
        })
        app.on('centerPanel:width').then(function () {
            var totalWidth = document.body.clientWidth;
            var leftNavListWidth = $(".leftNavList").width();
            var leftPanelWidth = $(".left-panel").width();
            var rightPanelWidth = $(".right-panel").width();
            var PreviewModelViewPanelWidth = $(".PreviewModelView").width();
            // console.log(leftNavListWidth,rightPanelWidth, leftPanelWidth)
            var newwidth = totalWidth - leftNavListWidth - leftPanelWidth - rightPanelWidth - 40 - PreviewModelViewPanelWidth; //外层 .docmain-panel div 有left\right 20px，减去
            var newtitle = newwidth + 'px';
            if (location.hash.indexOf("team") > -1) {
                //团队库
                if ($('#Thumbnail')) {
                    $('#Thumbnail').css('width', newtitle);
                }
            }
            else {
                // 企业库和个人库
                if ($('#thumbnail')) {
                    $('#thumbnail').css('width', newtitle);
                }
            }
            //  modify：shuchen.hu 2022-0419  #28915 外发结果界面中，分页框显示不能跟随右侧面板打开关闭进行自适应
            if ($(".doclist-panel") && location.href.indexOf("outpublish") == -1) {
                $('.doclist-panel').css('width', newtitle);
            }
            if ($('.doc-view-footer')) {
                $('.doc-view-footer').css('width', newtitle);
            }
        });

        app.on('doclistPanel:reset').then(function () {
            if ($(".doclist-panel")) {
                $('.doclist-panel').css('width', "auto");
            }
        });        
        app.on("doc:toggleLeftPanelPerbar").then(function (obj) {
            var teamListPer = $('.left-content');
            teamListPer.scrollLeft(0).scrollTop(0).perfectScrollbar('destroy').perfectScrollbar();
            //   perfectScrollbar('update');
        });
        app.on("doc:rightPanelSizeChange").then(function () {
            if (toolbarView.moduleName) {
                toolbarView.reLoadSize();
            }
        });
        // 动态计算内容去距离顶部的距离(PD-2017 cuizhichao （UI）我的标签优化)
        app.on("newGetTagListSetSize:reloaddata").then(function (data) {
            var height = 0
            if (data.height == 0 ) {
                height = 0
            }else if (data.height>100) {
                height = 105
            }else{
                height = data.height + 5
            }
            newTagHeight(height)
        });
        // modify： 2022-07-26 cuizhichao 月度计划左侧目录树点击自适应文件夹宽度
        app.on('setLeftPanel:width').then(function (width) {
            if (width != 99999) {
                panelWidth(width);
                panelOldWidth(width)
            }
        });
        app.on('setLeftPanel:docflowWidth').then(function (notDocflow) {
            if (notDocflow) {
                panelOldWidth() && panelWidth(panelOldWidth())
                return
            }
            if (panelDocflowOldWidth() > 320) {
                panelWidth(panelDocflowOldWidth());
                return
            }
            panelOldWidth(panelWidth())
            panelDocflowOldWidth(320)
            panelWidth(320);
        });
        pt.intercept = function () {
            this.bindComputed();
            //捕获工具栏更新的事件响应
            app.on('toolbar:update').then(function (msg) {
                ko.$store.dispatch("setCurrent", msg.select, msg.module, msg.message);
            });
            //捕获窗口变化
            $(window).on('resize.docpanel', function () {
                if (rightPanelHeight() != (document.body.offsetHeight - 105)) {
                    rightPanelHeight(document.body.offsetHeight - 105);
                };
                if (toolbarView.moduleName) {
                    toolbarView.reLoadSize();
                }
            });
        }
        /**
         * 绑定$store.current的改变，用于通知toolbar的更新
         */
        pt.bindComputed = function () {
            //监听数据的变化
            ko.computed(function () {
                var cur = ko.$store.state.currentData();
                var isLoaded = ko.$store.state.toolbarComplationCompleted(); //表示工具栏加载完成
                if (cur && isLoaded) {
                    toolbarView.refreshData(cur);
                }
            });
        }

        //------------------------------------------------------------------
        //隐藏左侧区域
        app.on("doc:toggleLeftPanelHide").then(function (obj) {
            if (obj.hide) {
                setSize(0);
            }
        });
        //   解决 无左侧二级面板&没有数据--隐藏左侧面板
        var setSize = function (width) {
            //modify 2022-03-21 yangyang.li 缺陷28172 左侧导航来回切换时,左侧面板收缩按钮bug,拖拽分割线bug
            //$('.left-panel').hide();方式处理分割线拖拽有问题,所以去掉用resizable的disable属性
            // $('.left-panel').show();
            // isRightPanlResize(false)
            // modify:  20220426  shuchuen.hu #29241 点击左侧目录树的线（不是箭头），跳出目录（KB也有此问题）
            if(width === 0){
                isRightPanlResize(false)
            }else{
                isRightPanlResize(true)
            }
            // console.log(isRightPanlResize(),width)
            var curentModule = require('knockout').$store.state.curentModule();
            if (curentModule && curentModule.module) {
                //cq.tianao 判断三库  在其加载对应的左侧面板状态
                if (curentModule.module == 'enterprise') {
                    let enterpriseLeft = $.cookie('enterpriseLeftPanel') === 'true' ? true : false;
                    if (enterpriseLeft != null && enterpriseLeft) {
                        width = 0;
                    }
                } else if (curentModule.module == 'team') {
                    let teamLeft = $.cookie('teamLeftPanel') === 'true' ? true : false;
                    if (teamLeft != null && teamLeft) {
                        width = 0;
                    }
                    showClosePanelBtn(false);
                } else if (curentModule.module == 'person') {
                    let personLeft = $.cookie('personLeftPanel') === 'true' ? true : false;
                    if (personLeft != null && personLeft) {
                        width = 0;
                    }
                }
                if (constInfo.userInfo.LeftSideTreeNoShow) {
                    showClosePanelBtn(false);
                    app.trigger('docsetbar:hide_to_right');
                    width = 0;
                }
            }
            //解决共享给我，我的标签无数据F5刷新左侧空白（临时解决，待优化）
            
            // if (curentModule && curentModule.id == "myshareme" && window.edoc2MyShareMe) {
            //     width = 0;
            //     showClosePanelBtn(false);
                
            // }
            //modify 2022-03-16 yangyang.li 缺陷27949 最近访问拖动左侧收缩导航条能够展开
            if (curentModule && curentModule.id == "myvisit") {
                width = 0;
                showClosePanelBtn(false);
            }
             //ruochen.mei 2022-03-10 缺陷27610，共享给我、我的标签、我的收藏鼠标移至收缩按钮的线条上，无效果  
            // if (curentModule && curentModule.id == "myshareme" && width > 0) {
            //     hideTreePutaway("block");
            // }
            //只要是我的推荐就不存在左侧面板
            if (curentModule && curentModule.id == "myrecommend") {
                width = 0;
                //wwx 20220302 缺陷26494：【UI】我的标签，共享给我，智能推荐页面没有数据时，点击F5刷新，左侧出现“展开/收缩”，且出现两种不同的现象（详细见图）
                showClosePanelBtn(false);
            }
            // 轻文档隐藏左侧面板
            if (curentModule && curentModule.id == "mydocflow") {
                width = 0;
                setTimeout(() => {
                    showClosePanelBtn(false);
                }, 50)    
            }
            //隔离区不显示左侧容器
            if (curentModule && curentModule.id == "mypartition") {
                width = 0;
                showClosePanelBtn(false);
            }
            // 我的标签
            if (curentModule && curentModule.hash == "quick/mytag") {
                // $(".v6_tree_putaway").css({
                //     display: 'block',
                // })
                // $(".left-panel").css({
                //     display: 'block',
                // })
                // app.trigger('newGetTagList:reload', null);
                width = 0;
                showClosePanelBtn(false);
            }else{
                // $(".v6_tree_putaway").css({
                //     display: 'block',
                // })
                // $(".left-panel").css({
                //     display: 'block',
                // })
            }
            if (curentModule && curentModule.hash == "quick/mytag" && window.edoc2MyTags) {
                width = 0;
                //wwx 20220302 缺陷26494：【UI】我的标签，共享给我，智能推荐页面没有数据时，点击F5刷新，左侧出现“展开/收缩”，且出现两种不同的现象（详细见图）
                // hideTreePutaway("none");
                //ruochen.mei 2022-03-10 缺陷27610，共享给我、我的标签、我的收藏鼠标移至收缩按钮的线条上，无效果  
                showClosePanelBtn(false);
                
            }
            if (curentModule && curentModule.id == "mylock") {
                width = 0;
                showClosePanelBtn(false);
                
            }
            if (curentModule && curentModule.hash == "quick/mypublish") {
                width = 0;
                showClosePanelBtn(false);
            }
            // 我的共享
            // if (curentModule && curentModule.hash == "quick/myshare") {
            //     width = 0;
            //     showClosePanelBtn(false);
               
                
            // }
            // 共享给我

            // if (curentModule && curentModule.hash == "quick/myshareme" && (window.location.hash.indexOf('shareType=1') > -1)) {
            //     width = 0;
            //     showClosePanelBtn(false);
            //     isRightPanlResize(false)
            // }
            // 我的消息
            if (curentModule && curentModule.hash == "quick/mymessage") {
                width = 0;
                showClosePanelBtn(false);
            }
            //通过左边面板的宽度判断折叠按钮样式是否设置为选中样式
            if (width == 0) {
                //左边信息面板不存在则按钮为未选中状态
                $(".bar-left").removeClass("icon-toleft-selected").addClass("icon-toleft");
                isOpenLeftBar(false);
            } else {
                //左边信息面板存在则按钮为选中状态
                $(".bar-left").removeClass("icon-toleft").addClass("icon-toleft-selected");
                isOpenLeftBar(true);
            };
            // modify： #34590 2022-07-20 cuizhichao 列表页预览，切换文件夹，不需要显示左侧目录树；
            if ($.cookie("PreviewModel") == 'isOff' && $(".PreviewModelView").width() > 0) {
                width = 0;
                showClosePanelBtn(false);
            }
            if (!jLeftPanel) jLeftPanel = $(".left-panel").eq(0);
            if (!jLeftPanel) return;
            jLeftPanel.width(width);
            //jLeftPanel.next().css("left", jLeftPanel.width());
            if (constInfo.userInfo.LeftSideTreeNoShow) {
                jLeftPanel.css("boder", 0);
            }
            if (constInfo.userInfo.LeftNavigationMenuNoShow) {
                jLeftPanel.css("left", 0);
                jLeftPanel.next().css("margin-left", width);
            } else {
                jLeftPanel.next().css("left", width);
            }
            $(window).trigger("resize.docpanel");
        };
        //FC:条件过滤参数,为真则不能关闭面板。
        //type:true展开false关闭
        var toggleLeftPanel = function (type, FC) {
            var curentModule = require('knockout').$store.state.curentModule();
            if (FC) type = true;
            //wwx 20220302 缺陷26963（ui）智能推荐页面，两次点击后出现展开 / 收缩箭头
            if (type == false
                || (curentModule && (curentModule.id == "mypartition" || curentModule.id == "myrecommend"))
                // || (curentModule && curentModule.hash === "team" || curentModule.hash === "team/7")
            ) {
                showClosePanelBtn(false);
                setSize(0);
                return;
            }

            // 判断 团队库首页不展示左侧二级目录
            // var folderId = "";
            // if (curentModule.id === undefined) {
            //     folderId = currentFolderPosition.folderId.toString()
            // } else {
            //     folderId = curentModule.id;
            // }
            // if ((folderId === '7' ) && curentModule.module ==='team' ) {
            // 42577 搜索结果页面中，文件创建/更新人下拉面板显示异常
            if (curentModule && curentModule.module === 'team') {//团队库不展示左侧二级目录
                showClosePanelBtn(false);
                setSize(0);
                return;
            }

            // 27111关闭目录树，点击文件夹，默认打开了目录树 zy
            if (isTreePutawayPutAway()) return;

            if (type == true) {
                if (panelWidth() == 0) {
                    setSize(defaultPanelWidth);
                    panelWidth(defaultPanelWidth);
                    isTreePutawayPutAway(true);
                }
                else {
                    setSize(panelWidth());
                    isTreePutawayPutAway(false);
                }
                treePutawayLeft(panelWidth() + curSelectWidth()); // 计算收缩箭头距左距离
                
                if (window.location.hash.indexOf('search?Panelcontrol') > -1) {
                    showClosePanelBtn(false);
                } else if ((curentModule && curentModule.hash == "quick/mypublish")  || constInfo.userInfo.LeftSideTreeNoShow){
                    showClosePanelBtn(false);
                }
                else {
                    // modify： #34590 2022-07-20 cuizhichao 列表页预览，切换文件夹，不需要显示左侧目录树；
                    
                    if ($.cookie("PreviewModel") == 'isOff' && $(".PreviewModelView").width() > 0 ) {
                        showClosePanelBtn(false);
                    }else{
                        showClosePanelBtn(true);
                    }
                }
                // 42576 搜索结果页面中，点击筛选条件后，再次点击其他区域时，该面板不会收缩
                if(curentModule && curentModule.hash == 'quick/mytag'){
                    showClosePanelBtn(false);
                }

                return;
            }
            if (jLeftPanel.width() > 0) {
                setSize(0);
            }
            else {
                if (panelWidth() == 0) {
                    setSize(defaultPanelWidth);
                    panelWidth(defaultPanelWidth);
                }
                else {
                    setSize(panelWidth());
                }
            }
        };
        /**
         * 同步加载路由数据，当前导航视图
         * gs for 2017年8月19日 11:20:15
         * @param {*成功时的回调} success
         * @param｛*指定是否更新路由｝
         */
        function loadViewData(success, update) {
            var code = $.getQueryString("code");
            $.ajax({//从数据库中重新获取
                data: {
                    module: "SystemManager",
                    fun: "GetShowSubNav",
                    navType: 1,
                    code: code.length !== 0 ? code : ''
                },
                async: false
            }).done(function (result) {
                if (result.url && result.errorCode == "ErrorCode4") {
                    window.top.location.href = result.url + "?returnUrl=" + window.encodeURIComponent(window.location.href);
                    return;
                }
                data = result.nav;
                var curlang = constInfo.getLang() ? constInfo.getLang() : "zh-cn";
                for (var i = 0; i < data.length; i++) {
                    if (curlang == "en") {
                        data[i].title = data[i].en;
                    }
                    else if (curlang == "ja") {
                        data[i].title = data[i].ja;
                    }
                    else if (curlang == "zh-tw") {
                        data[i].title = data[i].zhtw;
                    }
                }
                var temp_new_data = {};
                $.each(data, function (i, obj) {
                    temp_new_data[obj.id] = { nav: obj.nav, sort: obj.sort, title: obj.title, activity: obj.activity }
                });//转换为字典以加速后面的迭代
                update === false && window.navdata && (data = window.navdata);//备份导航列表的数据以便在不重建路由的情况下动态切换导航类型
                (update == true || update == null || !window.navdata) && (window.navdata = data);
                for (var i = 0; i < data.length; i++) {
                    //更新show状态和sort状态
                    data[i].nav = temp_new_data[data[i].id].nav;
                    data[i].sort = temp_new_data[data[i].id].sort;

                    if (data[i].route.indexOf("[") > -1 && data[i].route.lastIndexOf("]") > -1) {//去""号

                        data[i].route = evil(data[i].route);

                    }
                    if (data[i].activity) {
                        defaultSelectItem = data[i];//记录数据库上设置的缺省选择项
                    }
                    data[i].activity = ko.observable(data[i].activity);
                }
                navigationModel = data.sort(function (obj1, obj2) {
                    return obj1.sort > obj2.sort ? 1 : -1
                });
                if (success) success();
            });
            //移除底部导航类型，gs at 2017年12月29日
            //$.ajax({
            //    data: {
            //        module: "SystemManager",
            //        fun: "GetSubNavViewConfig"
            //    },
            //    async: false
            //}).done(function (result) {
            //    curSelectNavType(result.SubNvaViewConfig.NvaType);
            //    curSelectNavView(result.SubNvaViewConfig.NvaView);
            //});

        }
        function evil(fn) {
            var Fn = Function;  //一个变量指向Function，防止有些前端编译工具报错
            return new Fn('return ' + fn)();
        }
        /**
         * 设置导航当前选中项
         *  gs for 2017年8月19日 11:20:38
         * @param {*应该选择的项} navigationModelItem 
         */
        function SetCurActItem(navigationModelItem) {
            if (!navigationModelItem) for (var i = 0; i < navigationModel.length; i++) navigationModel[i].activity(false);
            else if (navigationModelItem.id != null) {
                for (var i = 0; i < navigationModel.length; i++) navigationModel[i].activity(false);
                navigationModelItem.activity(true);
            }
        }
        /**
         * map路由数据到路由
         * gs for 2017年8月19日 11:21:42
         */
        function maprouter() {
            childRouter = router.createChildRouter().makeRelative({ moduleId: 'doc', fromParent: true }).map(navigationModel).buildNavigationModel();
            return childRouter
        }
        //modify 2022-03-15 liuchunli #27740【UI】团队库文档列表页面，右侧面板展开时，遮挡文件显示（缩略图模式时
        // 监听右侧面板大小变化
        //app.on('rightpanel:width').then(function(width) {
        //    if ($('.doclist-panel')) {
        //        var thumbnailwidth = $('.docmain-panel').width();
        //        var newwidth = parseInt(thumbnailwidth) - width;
        //        var newtitle = newwidth + 'px';
        //        $('.doclist-panel').css('width', newtitle);
        //    }
        //});

        // 刷新轻文档未读数据
        app.on('docflow:resetNotReadNumber').then(function () {
            getUnReadCount()
        });
        function getUnReadCount() {
            var token = $.cookie("token");
            if (!token) {
                token = getTokenByCondition();
            }
            $.ajax({
                data: {
                    module: "DocFlowManager",
                    fun: "GetUnReadCount",
                    token: token
                }
            }).done(function (res) {
                if (res.result == 0) {
                    showDocflowNowReadNum(!!res.allCount)
                    var num = res.allCount;
                    if (res.allCount > 99) {
                        num = '99+'
                    }
                    docflowNowReadNum(num)
                }
            })
        }
        getUnReadCount()

        // 轻文档为空时的处理
        app.on('myDocflowTree:unfoundListData').then(function(message) {
            if(message.empty) {
                showDofclowEmpty(true);
                docflowIframeUrl('');
                app.trigger('doc:leftpanelclose');
                return
            }

            showClosePanelBtn(true);
            if (showDofclowEmpty()) {
                showDofclowEmpty(false);
                
                $.cookie('sidebarTypeTree', 'isOn');
                isTreePutawayPutAway(false);
                setSize(panelWidth());
                treePutawayLeft(panelWidth() + curSelectWidth());
                app.trigger("navigation:init", {
                    message: "leftPanelResize"
                });
                app.trigger('centerPanel:width');
            }
        })

        /**
         * 发布二级导航ui改变事件
         * gs for 2017年8月19日 11:24:44
         */
        app.on("uichange:subnav").then(function () {
            loadViewData(function () {
                //childRouter = router.createChildRouter().makeRelative({ moduleId: 'doc', fromParent: true }).map(navigationModel);
                childRouter = router.createChildRouter().map(navigationModel);
                var list = $.grep(childRouter.routes, function (obj) {
                    return obj.nav;
                });
                var IntelligentCommend = constInfo.IntelligentCommend();
                var attr = [];
                $.each(list, function (e, i) {
                    if (IntelligentCommend || i.moduleId != 'doc/quick/myrecommend/index') {
                        attr.push(i);
                    }
                })
                _retuenobj.items(attr);
                jLeftPanel = $(".left-panel").eq(0);
                if (list.length) {
                    jLeftPanel.find(".navbar-list>a").css({ width: ((100 / list.length) + "%") });
                }
            }, false);
        });
        app.on('doc:navigateTo', function (url) {
            if (childRouter) {
                childRouter.navigate(url);
            }
        });
        /** 
            * PD-2011 预览模式 
            * 预览模式 左侧树关闭
        * */ 
        app.on('doc:leftpanelclose', function () {
            setSize(0);
            showClosePanelBtn(false);
            app.trigger('centerPanel:width');
        });
        //预览模式 左侧树打开
        app.on('doc:leftpanelopen', function () {
            if(window.location.hash.indexOf('team')>-1 
            || window.location.hash.indexOf('doc/quick/myvisit')>-1){
                setSize(0);
            }else{
                var sidebarTypeTree = $.cookie('sidebarTypeTree', sidebarTypeTree);
                if (sidebarTypeTree == 'isOff') {
                    treePutawayLeft(curSelectWidth());
                } else {
                    setSize(panelWidth());
                    treePutawayLeft(panelWidth()+curSelectWidth());
                }
                // 缺陷#31621 shuchen.hu 20220702 【v6.0.0.6新增】点击菜单栏详情页左上角的字样 页面出现目录树的展开/收起按钮
                // 缺陷#31606 shuchen.hu 20220630【V6.0.0.6hotfix新增】进入共享给我中，点击面包屑的共享给我，出现目录收缩条，切换进入我的共享，点击面包屑的我的共享，出现目录收缩条
                var curentModule = require('knockout').$store.state.curentModule();
                if(curentModule && curentModule.id == 'mytag'  || curentModule && curentModule.id == 'mypublish' ){
                    showClosePanelBtn(false);
                }else{
                    if(constInfo.userInfo.LeftSideTreeNoShow){
                        showClosePanelBtn(false);
                    }else{
                        // 如果是轻文档页面并且当前无数据，那么不展示按钮
                        var nowShow = isDocflowFlag() && showDofclowEmpty();
                        if (nowShow) {
                            showClosePanelBtn(false);
                        }
                        showClosePanelBtn(true);
                    }
                }
            }
            app.trigger('centerPanel:width');
        });

        //v6.1共享、我的共享
        app.on("doc:shareView").then(function (obj) {
            sharcurentModule(obj)
            sharebutShow(true);
        });

        loadViewData();
        ////个人内容库设置禁用 ,不需要从组织管理里面设置了，可以用个人库策略去控制 by tanyo
        //if (!constInfo.userInfo.enablePersonFolder) {
        //    navigationModel.splice(3, 1);
        //}
        if (constInfo.userInfo.isOutsider) {
            //navigationModel.splice(1, 1);
            navigationModel = [
                { route: 'team(/:id)(/:id)', moduleId: 'team/index', title: lang.team, hash: '#doc/team', nav: true, className: "nav-team" }
            ];
        }
        maprouter();

        function curSelectWidth() {
            if (curSelectNavView() == 1) {
                
                return 140;
            } else {
                return 50;
            }
        };

        var _retuenobj;
        var lastNavigateId;
        var lastNav;
        return _retuenobj = {
            treePutawayTop: treePutawayTop,
            treePutawayLeft: treePutawayLeft,
            isMouseoverPutaway: isMouseoverPutaway,
            infoPreview: infoPreview,
            sharebutText: sharebutText,
            mySharemeList: mySharemeList,
            // hideTreePutaway: hideTreePutaway,//有时候不生效,而且也不再用$(".v6_tree_putaway").hide(); 太乱了,用下边visible属性
            showClosePanelBtn: showClosePanelBtn,//modify 2022-03-21 yangyang.li 缺陷28172 左侧导航来回切换时,左侧面板收缩按钮bug,拖拽分割线bug
            isTreePutawayPutAway: isTreePutawayPutAway,
            lang:lang,
            v6LeftPanelMouseover: function () { isMouseoverPutaway("mouseover") },
            v6LeftPanelMouseout: function () { isMouseoverPutaway("") },
            v6LeftPanelClick: function () {
                // console.log($(".left-panel").width(),panelWidth())
                //modify 2022-04-13 zhengjie 29155页面缩放90%或110%，收起目录树后无法展开，F5刷新后由“<”变成“>”，此时无论页面缩放如何点击任意路径，左侧树都会收起且无法展开
                // console.log(panelWidth())
                if ($(".left-panel").width() > 100 ) {
                    $.cookie('sidebarTypeTree', 'isOff');// modify 2022-03-25 zhengjie v6左侧目录树配置开关
                    isTreePutawayPutAway(true);
                    // panelWidth(0);
                    setSize(0);
                    treePutawayLeft(curSelectWidth());
                } else {
                    $.cookie('sidebarTypeTree', 'isOn');// modify 2022-03-25 zhengjie v6左侧目录树配置开关
                    isTreePutawayPutAway(false);
                    // panelWidth(200);
                    setSize(panelWidth());
                    treePutawayLeft(panelWidth() + curSelectWidth());
                    showClosePanelBtn(true); //modify 2022-03-14 liuchunli #27610
                };
                app.trigger("navigation:init", {
                    message: "leftPanelResize"
                });
                app.trigger('centerPanel:width');
            },
            openDocFlow: function () {
                $('.toolbar-btn-menu .bread_v6_news_doc_flow').click();
            },
            addNewDocflow: function() {
                app.trigger('docflow:addNewDocflow')
            },
            openLfetPanel: function () {
                curSelectNavView(!curSelectNavView());
                treePutawayLeft(jLeftPanel.width() + curSelectWidth());//优化左侧主导航收缩引起左侧二级目录树收缩按钮偏移
                //true 展开  false 关闭
                if (curSelectNavView()) {                    
                    $.cookie('sidebarType', 'full');
                } else {
                    $.cookie('sidebarType', 'mini');
                }
                if (!curSelectNavView() && curSelectNavView()!=1) {
                    $(".leftNavList .navtipTitle a").tooltip({ showDelay: curSelectNavView() ? 1000 : 100, hideDelay: 50, position: 'right', });
                    curSelectNavView.subscribe(function (value) {

                        $(".leftNavList .navtipTitle a").tooltip({ showDelay: value ? 2000 : 100, hideDelay: 50, position: 'right', });

                    });
                }
                //展开没有触发resize事件 导致页面布局出现问题  modify:cq.tianao 2018.6.13
                $(window).trigger('resize.docpanel');
                app.trigger('centerPanel:width');
            },
            canActivate: function () {
                var _router = router;
                var prevFolderIdPosition = ko.$store.state.currentFolderPosition;
                if (!prevFolderIdPosition.folderId) { //为空，表示第一次，直接返回true，执行后面的逻辑
                    return true;
                }
                var hash = window.location.hash;
                hash = hash.substr(0, hash.indexOf('?') > 0 ? hash.indexOf('?') : hash.length);
                //处理系统管理->文档的跳转，路由与实际组件不对应的Bug
                if (hash == '#doc') {
                    var tmpRoute = ko.$store.state.curentModule();
                    if (tmpRoute) {
                        // ldb pi:1282 工作流及自定义导航中点击“文档”自动定位到企业库根目录
                        if (!tmpRoute.hash) {
                            tmpRoute.hash = "enterprise/1";
                            tmpRoute.id = "1";
                            tmpRoute.module = "enterprise";
                        }
                        var timeIndex = window.setTimeout(function () {
                            window.clearTimeout(timeIndex);
                            //解决跳转缺失搜索条件
                            var routeReg = /^enterprise\/[\d]+\/search$/i;
                            if (routeReg.test(tmpRoute.hash) || tmpRoute.hash == "enterprise/1") {
                                var url = '#doc/' + tmpRoute.hash.replace(/search\/?/, "") + "?key=" + new Date().getTime();
                                router.navigate(url);
                            } else {
                                router.navigate('#doc/' + tmpRoute.hash);
                            }
                        });
                        return true;
                    }
                }

                var params = hash.split('/');
                var position = params[1], folderId = params[2];
                if (position == 'quick') {
                    //快速导航
                    prevFolderIdPosition.position = position;
                    ko.$store.dispatch('setCurrentFolderPosition', prevFolderIdPosition);
                    return true;
                }
                if (position != prevFolderIdPosition.position) {
                    //位置不同，需要执行后的逻辑 
                    //setTimeout(function(){
                    router.navigate(params[0] + '/' + params[1] + '/' + (folderId || prevFolderIdPosition.folderId), false);
                    //},0); 
                    return true;
                } else if (!folderId && prevFolderIdPosition.folderId) {
                    //位置相同，且本次folderId为空，且上一次的folderId不为空，不执行后面的逻辑
                    router.navigate(params[0] + '/' + params[1] + '/' + prevFolderIdPosition.folderId, false);
                    return true;
                } else {
                    return true;
                }

            },
            activate: function (path) {
                // 重置列表宽度
                // if (path && path == '/quick/mydocflow') {
                //     defaultPanelWidth(320)
                // } else {
                //     defaultPanelWidth(200)
                // }
                // panelWidth(defaultPanelWidth());

                if(path && path == '/quick/myshare'){
                    // router.navigate('doc/quick/myshareme?userId=&shareId=&type=&shareType=1')
                    window.location.href = window.location.origin + window.location.pathname + '#doc/quick/myshareme?userId=&shareId=&type=&shareType=1'
                }

                // 左侧目录树点击事件优化(app/doc/index.js中的setSize方法很慢) s1
                // toggleLeftPanel(false);  
                // modify 2022-03-25 zhengjie v6左侧主导航配置开关
                var currSideBarType = $.cookie('sidebarType');
                var curentModule = require('knockout').$store.state.curentModule();
                // 左侧目录树点击事件优化(app/doc/index.js中的setSize方法很慢) s1-e3
                // if(curentModule && curentModule.id != 'myshareme'){
                //     toggleLeftPanel(false);
                // }
                if(!currSideBarType) {
                    currSideBarType = isOnLeftMainNavigation() ? "full":"mini";
                }
                $.cookie('sidebarType', currSideBarType);
                curSelectNavView(currSideBarType == 'full' ? 1: 0)
                // modify 2022-03-25 zhengjie v6左侧目录树配置开关
                var currSideBarTypeTree = $.cookie('sidebarTypeTree');
                if(!currSideBarTypeTree) {
                    currSideBarTypeTree = isOnLeftTree() ? 'isOn' : 'isOff';//on 开启 off关闭
                }
                $.cookie('sidebarTypeTree', currSideBarTypeTree);
                if (currSideBarTypeTree == 'isOff') {
                    isTreePutawayPutAway(true);
                    // panelWidth(0);
                    setSize(0);
                    treePutawayLeft(curSelectWidth());
                } else {
                    isTreePutawayPutAway(false);
                    // panelWidth(200);
                    // 左侧目录树点击事件优化(app/doc/index.js中的setSize方法很慢) s1
                    // setSize(panelWidth());
                    // modify #34476 20220809 shuchen.hu 享页面点击面包屑中的用户名称，目录树按钮会消失
                    treePutawayLeft(panelWidth() + curSelectWidth());
                    // 缺陷31603  shuchen.hu-20220630 【V6.0.0.6hotfix新增】共享给我中无数据时，点击共享给我，再次点击共享给我，出现目录树收缩条
                    // 缺陷33727  shuchen.hu-20220704  进入共享，点击文档，界面出现目录收缩条
                    if(curentModule && curentModule.id == 'mytag' || curentModule && curentModule.id == 'mypublish' || curentModule && curentModule.id == 'myvisit'){
                        showClosePanelBtn(false);
                    }else{
                        showClosePanelBtn(true); //modify 2022-03-14 liuchunli #27610
                    }
                    // 缺陷31603  shuchen.hu-20220630 【V6.0.0.6hotfix新增】共享给我中无数据时，点击共享给我，再次点击共享给我，出现目录树收缩条
                    // if(curentModule && curentModule.id == 'myshareme' && (window.location.hash.indexOf('shareType') > -1)){
                    //     showClosePanelBtn(false);
                    // }
                };

                // 发送右侧面板大小变化消息
                if ($(".right-panel")) {
                    app.trigger('centerPanel:width');
                };
                if (path && typeof path === "string") {
                    // 缺陷27591 收藏页面切换之后无法展开
                    var expand = ['enterprise', 'person', 'myfavorite'];//如果是企业内容库和个人内容库，默认展开
                    // 缺陷29150 shuchen.hu-20220418 -- 主页进入我的共享中，再次点击后台共享管理后，返回前台界面，我的共享中出现左侧目录树的收缩条;
                    var collapse = ['mypublish', 'mysubscribe', 'mylock', 'mypartition'];//如果是我的外发，没有展开的
                    // 27111关闭目录树，点击文件夹，默认打开了目录树 zy 增加了&& !isTreePutawayPutAway() 这个判断 已关闭就不展开了
                    if (_.find(expand, function (item) { return path.indexOf(item) > -1 }) && !isTreePutawayPutAway()) {
                        toggleLeftPanel(true);
                        isTreePutawayPutAway(false);//缺陷26219-tianyi.wang-20220219(UI)关闭左侧树，通过文件路径跳转任意文件夹下，左侧树方向展开按钮异常
                        // 修复共享给我点击隐藏
                    } else if (_.find(collapse, function (item) { return path.indexOf(item) > -1 }) && path != "/quick/myshareme") {
                        toggleLeftPanel(false);
                    };

                    // 27111关闭目录树，点击文件夹，默认打开了目录树 zy 关闭状态时需要有展开的按钮
                    if (_.find(expand, function (item) { return path.indexOf(item) > -1 }) && isTreePutawayPutAway()) {
                        showClosePanelBtn(true);
                    }
                    // 团队库搜索优化左侧二级目录树展示问题
                    if(path.indexOf('team')>-1){
                        // panelWidth(0);
                        setSize(0);
                        treePutawayLeft(curSelectWidth());
                    }
                };
                // 是否是轻文档
                // isDocflowFlag(path == "/quick/mydocflow")
                if (path == "/quick/mydocflow") {
                    isDocflowFlag(true)
                } else {
                    setTimeout(function() {
                        isDocflowFlag(false)
                    }, 100);
                }

                // 删除其他页面中标签页面向面包屑添加的dom元素(PD-2017 cuizhichao （UI）我的标签优化)
                if (path == "/quick/mytag") {
                    newTagFlag(true)
                    app.trigger("mytag:setcurrentPage")
                }else{
                    newTagFlag(false)
                    $('.center-panel > .crumbs-nav > .v6-1-news-tag-list').remove()
                    $('.center-panel .crumbs-nav .navigation-list .v6-tag-title').remove()
                }
                // v6.1
                // #33727  进入共享，点击文档，界面出现目录收缩条
                if(curentModule && curentModule.id == 'myshareme' ){
                    sharebutShow(true)
                }else{
                    sharebutShow(false)
                }
                //  shuchen.hu  20220711 #34085 从共享页面切换至企业库/个人库后，目录树不显示展开收起按钮
                // if( curentModule && curentModule.id == 'myshareme' && mySharemeList() == 0){
                    // showClosePanelBtn(false);
                // }
            },
            panelWidth: panelWidth,
            binding: function () {
                var intercept = new InterceptEvent();
                intercept.intercept();
                if (constInfo.userInfo.LeftNavigationMenuNoShow) {
                    $("#langLeft").hide();
                }
                return { cacheViews: true };
            },
            attached: function (child, parent, settings) {
            },
            compositionComplete: function (child, parent, settings) {
                // var curentModule = require('knockout').$store.state.curentModule();
                // if (curentModule && curentModule.hash == "quick/mytag") {
                //     // app.trigger('newGetTagList:reload', null);
                // }
                // treePutawayTop(((document.body.offsetHeight - 60) / 2) - 18); // 计算收缩箭头距上距离
                treePutawayTop(10);

                // 添加轻文档切换监听
                app.on('docflowPreflow:change').then(function (message) {
                    if (message && message.notSelected && !message.fileId) {
                        docflowIframeUrl('');
                        return
                    }
                    var fileId = message.fileId;
                    var url = "";
                    var permParam = JSON.parse(message.docflowPermParam)
                    url = window.location.origin + '/preview.html?fileid=' + fileId + '&module=edit_right';
                    if (permParam && permParam.shareid) {
                        url = url + "&sharecode=" + permParam.shareid
                    }
                    if (message.notSelected) {
                        url = url + '&notSelected=true'
                    }
                    // var url =  'http://172.16.3.189:30008/docflow/word/?id=3a0744ce-72c7-924b-bd05-d1f8d42357c2&fileid=43' + '&module=ecm';
                    docflowIframeUrl(url)
                })

                //判断展示在线文档创建入口 企业库 团队库（非根目录） 个人库展示
                //lic
                var licInfo = constInfo.getLicInfo();
                if (licInfo.EnabledModules.includes('DocFlow')){
                    //监听轻文档创建按钮显示隐藏
                    app.on("docflowMenu:show").then(function(message){
                        // console.log(message);
                        if (message.showMenu == true){
                            showDocflowMenu(true);
                        }
                        else{
                            showDocflowMenu(false);
                        }
                    });
                }

                // treePutawayTop(((document.body.offsetHeight - 60) / 2) - 18); // 计算收缩箭头距上距离
                // console.log(panelWidth())
                // 左侧目录树收起后F5刷新，会自动展开
                if($.cookie("sidebarTypeTree") == "isOff"){
                    treePutawayLeft(0 + curSelectWidth());
                }else{
                    treePutawayLeft(panelWidth() + curSelectWidth()); // 计算收缩箭头距左距离
                }
                // 
                // 发送右侧面板大小变化消息
                $(".right-panel", child).eq(0).resizable({
                    onStopResize: function () {
                        app.trigger('centerPanel:width');
                    },
                    onResize: _.throttle(function () {
                        app.trigger('centerPanel:width');
                    }, 100),
                });
                // 窗口大小变化
                $(window).resize(function () {
                    app.trigger('centerPanel:width');
                });

                if (constInfo.userInfo.isGaValidata) {
                    if (constInfo.systemInfo.loginPageUrl !== undefined && constInfo.systemInfo.loginPageUrl !== '') {
                        window.top.location.href = constInfo.systemInfo.loginPageUrl + "?returnUrl=" + window.encodeURIComponent(location.href);
                        return;
                    }
                    window.top.location = "/api/auth/login";
                    return;
                }
                //erms产品直接访问非消息快速导航时自动跳转的我的消息
                //var productType = constInfo.getLicInfo().ProductType.toLowerCase();
                //if (productType =="erms") {
                //    var urlHash = window.location.hash;
                //    if (urlHash.indexOf("#doc/enterprise") == 0
                //        || urlHash.indexOf("#doc/team") == 0
                //        || urlHash.indexOf("#doc/person") == 0
                //        || urlHash.indexOf("#doc/quick/myshareme") == 0
                //        || urlHash.indexOf("#doc/quick/mytag") == 0
                //        || urlHash.indexOf("#doc/quick/myfavorite") == 0
                //        || urlHash.indexOf("#doc/quick/mylock") == 0
                //        || urlHash.indexOf("#doc/quick/mypublish") == 0
                //        || urlHash.indexOf("#doc/quick/myshare") == 0) {
                //        router.navigate("doc/quick/mymessage");
                //    }
                //}
                var $parent = ko.dataFor(parent);
                var self = this;
                //modify: 2018-01-13 haibin.long 发出toolbar构建完成的事件
                ko.$store.dispatch('toolbarcomplation');
                /////////开始处理页面布局相关/////////////////////////////////

                jLeftPanel = $(".left-panel", child).eq(0);
                //底部导航长度计算
                navshowlength = $.grep(navigationModel, function (obj) { return obj.nav; }).length;
                if (navshowlength) {
                    jLeftPanel.find(".navbar-list>a").css({ width: ((100 / navshowlength) + "%") });
                }
                //当没有左侧面板时,禁止拖拽
                // modify:  20220426  shuchuen.hu #29241 点击左侧目录树的线（不是箭头），跳出目录（KB也有此问题）
                showClosePanelBtn.subscribe(function(newValue){
                    if (newValue == true){
//                         jLeftPanel.resizable('enable');
                    }
                    else{
//                         jLeftPanel.resizable('disable');
                    }
                });
                if(!isRightPanlResize()){
                    jLeftPanel.resizable('disable');
                    // $(".right-panel", child).eq(0).resizable('disable');
                }else{
                    jLeftPanel.resizable('enable');
                    // $(".right-panel", child).eq(0).resizable('enable');
                }
                isRightPanlResize.subscribe(function(newValue){
                   
                    if(!newValue){
                        jLeftPanel.resizable('disable');
                        // $(".right-panel", child).eq(0).resizable('disable');
                    }else{
                        jLeftPanel.resizable('enable');
                        // $(".right-panel", child).eq(0).resizable('enable');
                    }
                })
                //还原上次拖放到宽度
                // 
                // 左侧目录树点击事件优化(app/doc/index.js中的setSize方法很慢) s1-e5
                // if($.cookie('sidebarTypeTree') == "isOn"){
                //     setSize(panelWidth());
                // }else{
                //     setSize(0);
                // }
                //重新绑定size操作
                jLeftPanel.resizable({
                    onStartResize: function () {
                        showClosePanelBtn(false);
                        isTreeHideClass(false);
                        if (window.location.href.indexOf("mydocflow") > -1 && $("#doc-flow-modal")) {
                            $("#doc-flow-modal").css({display:'block'})
                            $(".putaway_mouseover").css({display:'block'})
                            isMouseoverPutaway('mouseover');
                        }
                    },
                    onResize: _.throttle(function () {
                        setSize($(this).width());
                        // panelWidth($(this).width())
                        app.trigger('centerPanel:width');
                        if (window.location.hash.indexOf('quick/mydocflow') > -1) {
                            app.trigger('setSearchListWidth:width');
                        }
                    }, 100),
                    // onResize: function () { // 不用节流函数否则按钮效果不好 但是有点卡顿...
                    //     setSize($(this).width());
                    //     treePutawayLeft($(this).width() + 140);
                    // },
                    onStopResize: function () {
                        if (window.location.href.indexOf("mydocflow") > -1 && $("#doc-flow-modal")) {
                            $("#doc-flow-modal").css({display:'none'})
                            isMouseoverPutaway('');
                        }
                        isTreeHideClass(true);
                        //小于100是自动折叠
                        if (jLeftPanel.width() < 100) {
                            setSize(0);
                            isTreePutawayPutAway(true);
                        } else {
                            isTreePutawayPutAway(false);
                            panelWidth(jLeftPanel.width());//modify 2022-04-14 29214左侧目录树收起后点击目录树（不要点箭头），再次点击展开变为<且无法展开
                        };
                        // console.log(jLeftPanel.width())
                        // panelWidth(jLeftPanel.width());
                        if (window.location.hash.indexOf('search?Panelcontrol') > -1) {
                            showClosePanelBtn(false);
                        } else {
                            showClosePanelBtn(true);
                        }

                        treePutawayLeft(jLeftPanel.width() + curSelectWidth());
                        if (window.location.href.indexOf("mydocflow") > -1) {
                            panelDocflowOldWidth(jLeftPanel.width())
                        } else {
                            panelOldWidth(jLeftPanel.width())
                        }
                        app.trigger("navigation:init", { message: "leftPanelResize" });
                       
                        //modify 2022-03-16 yangyang.li 缺陷27949 最近访问拖动左侧收缩导航条能够展开
                        var curentModule = ko.$store.state.curentModule();
                        if (curentModule && curentModule.id == "myvisit") {
                            jLeftPanel.width(0);
                            showClosePanelBtn(false);
                        }
                        app.trigger('centerPanel:width');
                    },
                    edge: 10,
                    maxWidth: 500,  //最大宽度500
                    minWidth: 99,
                    handles: "e"
                });
                //附加tip提示控件
                if (!curSelectNavView() && curSelectNavView()!=1) {
                    $(".leftNavList .navtipTitle a").tooltip({ showDelay: curSelectNavView() ? 1000 : 100, hideDelay: 50, position: 'right', });
                    curSelectNavView.subscribe(function (value) {

                        $(".leftNavList .navtipTitle a").tooltip({ showDelay: value ? 2000 : 100, hideDelay: 50, position: 'right', });

                    });
                }
                $(".leftNavList a").addClass("navtip");
                //加载导航视图的滚动条
                var Content = $(".left-content");
                //   Content.css({ "position": "relative" });
                Content.perfectScrollbar('update').perfectScrollbar();
                app.on("doc:toggleLeftPanel").then(function (obj) {
                    if (obj && obj.type === 10) {
                        if (obj.isSelected) {
                            obj.type = 0;
                        } else {
                            obj.type = 1;
                        }
                    }
                    if (obj && obj.type === 0) {

                        if (curSelectNavType() == 0 && obj.module == "message") {//特殊处理底部导航时head message点击
                            toggleLeftPanel(true);
                        }
                        else {
                            toggleLeftPanel(false);
                            $(".document-list-body").perfectScrollbar('update');
                        }
                        return;
                    }
                    if (obj && obj.type === 1) {
                        if (obj && obj.myTags === 0) {
                            toggleLeftPanel(false);
                        }
                        // else if (obj && obj.myShareme === 0){
                        //     //共享给我没有数据时关闭左侧面板
                        //     // mySharemeList(obj.myShareme)
                        //     // toggleLeftPanel(false);
                        // }
                        else {
                            toggleLeftPanel(true);
                            // 团队库左侧导航隐藏
                            if(window.location.hash.indexOf("mytag")>-1)
                            {
                                // 34201 【UI】收缩企业库/个人库的左侧目录树，再切换到标签页面，“展示/收缩”箭头仍然存在  shuchen.hu  2022-0713
                                // showClosePanelBtn(true);
                            }else if (window.location.hash.indexOf("myshareme")>-1 && window.location.hash.indexOf("shareType=2")>-1){
                                showClosePanelBtn(true);
                            }
                            else if(window.location.hash.indexOf("team")>-1){
                                showClosePanelBtn(false);
                            }
                            
                        }
                        return;
                    }
                    toggleLeftPanel();
                });
                ///////////////结束处理页面布局相关////////////////////////////
                //第一次进入，并且没有激活的路由项
                if ((window.location.hash == "#doc" || window.location.hash == "") && childRouter.activeItem() == null) {
                    if (constInfo.userInfo.isOutsider) {
                        router.navigate("doc/team");
                    }
                    else if (!constInfo.userInfo.enablePersonFolder && !defaultSelectItem) {
                        router.navigate("doc/enterprise");
                    }
                    else {
                        if (defaultSelectItem)
                            router.navigate(defaultSelectItem.hash);
                        else
                            router.navigate("doc/person");
                    }
                }
                //确定是否第一次加载树
                var oobj;
                for (var p = 0; p < childRouter.routes.length; p++) {//这东西在路由转换的时候可以直接注入
                    //hsc 20220302 缺陷41897：【UI】41897 刷新页面之后，目录树收起，但按钮仍然显示
                    if (childRouter.routes[p].isActive()  || window.location.href.indexOf(childRouter.routes[p].hash) > -1)
                        oobj = childRouter.routes[p]
                }
                if (oobj) {
                    //wwx 20220302 缺陷26494：【UI】我的标签，共享给我，智能推荐页面没有数据时，点击F5刷新，左侧出现“展开/收缩”，且出现两种不同的现象（详细见图）
                    var nonLeftPanel = (oobj.hash === '#doc/quick/mytag' && window.edoc2MyTags) || oobj.hash === '#doc/quick/myrecommend';
                    toggleLeftPanel(oobj.loadTree && !nonLeftPanel, curSelectNavType() == 0);
                    setTimeout(function () {//解决第一次加载在服务器上有关闭失效的情况
                        toggleLeftPanel(oobj.loadTree && !nonLeftPanel, curSelectNavType() == 0);
                    }, 100);

               
                }
                //modify by 20220310 haiwei.cui 个人库隐藏容量显示
                //订阅个人空间容量显示
                //app.on("doc:toggleDisplayCapacity").then(function (isShow) {
                //    if (isShow) {
                //        $(".left-capacity").eq(0).show();
                //        $(".left-content").eq(0).css({ "bottom": "80px" });
                //    }
                //    else {
                //        $(".left-capacity").eq(0).hide();
                //        $(".left-content").eq(0).css({ "bottom": "50px" });
                //    }
                //});
                this.activeView.subscribe(function (viewUrl) {

                    if (typeof viewUrl === "string" && viewUrl.toLowerCase().indexOf('/search') >= 0) {
                        //搜索
                        self.isHidden(true);
                    } else {
                        self.isHidden(false);
                    }
                    //modify by 20220310 haiwei.cui 个人库隐藏容量显示
                    //var modueId = system.getModuleId(viewUrl);
                    //var IsObj = typeof viewUrl === "object";
                    //if (IsObj && modueId === "component/docView/DocView" && viewUrl.vm.viewName() === "person") {
                    //    app.trigger("doc:toggleDisplayCapacity", true);
                    //}
                    //else {
                    //    app.trigger("doc:toggleDisplayCapacity", false)
                    //}
                });

                // // 鼠标划入tree树时保持收缩按钮选中态
                // $(".enterprise-tree").mousemove(function(e){ 
                //     if(!isMouseoverPutaway()){
                //         isMouseoverPutaway('mouseover');
                //     };
                // });
                // 鼠标离开时tree树范围时改变样式 不够完美


                $(document).mousemove(function (e) {
                    var isMousemove = false;
                    if ($(e.target).attr("class") == "window-mask" || $(".window-mask").is(':visible')) {
                        isMousemove = true;
                    }
                    var curentModule = require('knockout').$store.state.curentModule();
                    // 缺陷31606 shuchen.hu 20220630【V6.0.0.6hotfix新增】进入共享给我中，点击面包屑的共享给我，出现目录收缩条，切换进入我的共享，点击面包屑的我的共享，出现目录收缩条
    
                    // if(curentModule.id == 'myshareme' || constInfo.userInfo.LeftSideTreeNoShow || $.cookie("PreviewModel") == 'isOff' && $(".PreviewModelView").width() > 0){
                    //     return
                    // }
                    // 原来的值 e.clientX > (treePutawayLeft() - 8) && e.clientX < (treePutawayLeft() + 20
                    if (e.clientX > (treePutawayLeft() - 8) && e.clientX <= (treePutawayLeft() + 1) && curentModule.module != 'team' && curentModule.module != 'quick' && !isMousemove) {
                        if (!isMouseoverPutaway() && isRightPanlResize()) {//判断开启左侧面板时候才显示hover,否则个人库左侧面板关闭,再切换到共享给我(无数据时),分割线会显示hover样式
                            isMouseoverPutaway('mouseover');
                        };
                    } else if (e.clientX > (treePutawayLeft() - 8) && e.clientX <= (treePutawayLeft()) && curentModule.module != 'team' && curentModule.id == 'myshareme' && !isMousemove) {
                          //ruochen.mei 2022-03-10 缺陷27610，共享给我、我的标签、我的收藏鼠标移至收缩按钮的线条上，无效果
                        if (!isMouseoverPutaway() && isRightPanlResize()) {
                            isMouseoverPutaway('mouseover');
                        };
                    } else if (e.clientX > (treePutawayLeft() - 8) && e.clientX <= (treePutawayLeft()) && curentModule.module != 'team' && curentModule.id == 'myfavorite' && !isMousemove) {
                        //ruochen.mei 2022-03-10 缺陷27610，共享给我、我的标签、我的收藏鼠标移至收缩按钮的线条上，无效果
                        if (!isMouseoverPutaway() && isRightPanlResize()) {
                            isMouseoverPutaway('mouseover');
                        };
                    } else if (e.clientX > (treePutawayLeft() - 8) && e.clientX <= (treePutawayLeft()) && curentModule.module != 'team' && curentModule.id == 'mytag' && !isMousemove) {
                        //ruochen.mei 2022-03-10 缺陷27610，共享给我、我的标签、我的收藏鼠标移至收缩按钮的线条上，无效果
                        if (!isMouseoverPutaway() && isRightPanlResize()) {
                            isMouseoverPutaway('');
                        };
                    }
                    if ((e.clientX < (treePutawayLeft() - 8) ||
                        e.clientX > (treePutawayLeft() ) ||
                        e.clientY < 60) && isTreeHideClass()
                    ) {
                        if (isMouseoverPutaway()) {
                            isMouseoverPutaway("");
                        };
                    };
                }) 
                if(toolbarView.moduleName == 'myalbum'){
                    navigationView("doc/navigationtab/index");
                    isMyalbum(true)
                }else{
                    navigationView("doc/navigation/index");
                    isMyalbum(false)
                }
            },
            router: childRouter,
            //  // modify 2022-05-13 liuchunli PD-2013 左导航优化
            OneCategory: ko.observableArray($.grep(childRouter.routes, function (obj) {
                return obj.groupId == '0' && obj.nav;
            })),
            TwoCategory: ko.observableArray($.grep(childRouter.routes, function (obj) {
                return obj.groupId == '1' && obj.nav;
            })),
            ThreeCategory: ko.observableArray($.grep(childRouter.routes, function (obj) {
                var IntelligentCommend = constInfo.IntelligentCommend();
                var isShowPublish = constInfo.systemInfo.instanceConfigInfo && constInfo.systemInfo.instanceConfigInfo.SecretEdition;
                //lic控制是否显示智能推荐
                if (!IntelligentCommend && obj.moduleId == 'doc/quick/myrecommend/index') {
                    return "";
                }
                // return obj.groupId == '2' && obj.nav;
                // 左侧主导航消息过滤
                return obj.groupId == '2' && obj.nav && obj.moduleId !== "doc/quick/mymessage/index";
            })),
            items: ko.observableArray($.grep(childRouter.routes, function (obj) {
                var IntelligentCommend = constInfo.IntelligentCommend();
                var isShowPublish = constInfo.systemInfo.instanceConfigInfo && constInfo.systemInfo.instanceConfigInfo.SecretEdition;
                //lic控制是否显示智能推荐
                if (!IntelligentCommend && obj.moduleId == 'doc/quick/myrecommend/index') {
                    return "";
                }
                return obj.nav;
            })),
            selectLang: function (data) {
                var curlang = getCurLangName();
                curlang = curlang.split('-').join("");
                if (curlang == "jp") curlang = "ja";
                return data[curlang];
            },
            activeView: activeView,
            navigationView: navigationView,
            toolbarView: toolbarView,
            docSetBarView: docSetBarView,
            informationbarView: informationbarView,
            PreviewModelView: PreviewModelView,
            isHidden: ko.observable(false),
            app: app,
            submenuMouseover: function (data, event) {
                if (window.location.hash.indexOf("/mydocflow") > -1) {
                    setTimeout(function() {
                        if ($("#applicationHost .right-menu .menu")) {
                            $("#applicationHost .right-menu .menu").css({display:'none'})
                        }
                    }, 200);
                }
            },
            onclickLeftNavItem: function (data, event) {
                // 刷新轻文档未读消息
                app.trigger('docflow:resetNotReadNumber')

                //modify 20220624 by zhengjie 32954共享给我无数据时，进入我的共享中，再次点击左侧目录树-共享，面包屑我的共享变为共享给我，列表数据最下方出现暂无共享图标
                //modify: #34109 2022-07-15 cuizhichao 左侧导航双击隔离区后，出现左侧目录树收缩按钮；
                if (window.location.hash.indexOf("quick/myshareme") > -1 && data.moduleId == "doc/quick/myshareme/index" || window.location.hash.indexOf("quick/mypartition") > -1 && data.moduleId == "doc/quick/mypartition/index") {
                    return
                }
                if (data.moduleId == "doc/quick/myvisit/index") {
                    width = 0;
                    showClosePanelBtn(false);
                }
                // 进入轻文挡模块默认隐藏掉右侧面板
                if (data.moduleId == "doc/quick/mydocflow/index") {
                    if ($(".docsetbar .docsetbar-btn-list .right-panels") && $(".docsetbar .docsetbar-btn-list .right-panels").hasClass("color_toright_v6")) {
                        var $closeRightPanel = $(".docsetbar .docsetbar-btn-list .right-panels").parent();
                        $closeRightPanel.click();
                    }
                } else  {
                    app.trigger('setLeftPanel:docflowWidth', true)
                }
                event.stopPropagation();
                //隐藏右键菜单
                contextmenu.onHidden();
                var self = this;
                //modify 2022-03-17 yangyang.li 缺陷28020 共享给我面包屑卡顿
                //modify 2022-03-17 yangyang.li 造成共享给我多次请求,注释之后测试也没有影响
                // if (data.moduleId == "doc/quick/myshareme/index") {
                //     $.ajax({
                //         data: {
                //             module: "DocShareManager",
                //             fun: "GetAllShareEntry",
                //             pageNum: 1,
                //             pageSize: 10,
                //             async: false
                //         }
                //     }).done(function (data) {
                //         if (data.listShare && data.listShare.length == 0) {
                //             toggleLeftPanel(false, false);
                //         }
                //         else if (data.listShare && data.listShare.length > 0) {
                //             window.edoc2MyShareMe = false;//cq.tiano modify 解决共享给我第一次加载左侧面板没展开
                //             toggleLeftPanel(true, true);
                //             // self.app.trigger("sharemeTree:reload");
                //         }
                //     });
                // }
                //刷新导航视图的滚动条
                setTimeout(function () {
                    //$(".left-content").perfectScrollbar('update');
                    app.trigger('doc:toggleLeftPanelPerbar');
                    //修改6833 其它导航切换到团队的时候会左侧面板空白问题
                    $(".team-list").scrollTop(0).scrollLeft(0);
                }, 100);
                setTimeout(function(){
                    if(data.moduleId == 'doc/quick/myalbum/index'){
                        navigationView("doc/navigationtab/index");
                        isMyalbum(true)
                    }else{
                        navigationView("doc/navigation/index");
                        isMyalbum(false)
                    }
                },20)
                if (data.className == 'nav-mytag') {
                    return true;
                }
                toggleLeftPanel(data.loadTree, curSelectNavType() == 0);//
                app.trigger('centerPanel:width');//修改内容区宽度
                // 如果是轻文档页面并且当前无数据，那么不展示按钮
                if (isDocflowFlag() && showDofclowEmpty()) {
                    // app.trigger('NewPreviewModelViewPanelShow:close');
                } else {
                    app.trigger('NewPreviewModelViewPanelShow:close');
                }
                if(data.moduleId == 'doc/quick/myshareme/index'){
                    app.trigger('docsetbar:ShowPreviewModelButton',false);
                }else{
                    app.trigger('docsetbar:ShowPreviewModelButton',true);
                }
                // modify 2022-06-29 liuchunli #31075最近访问、外发多次点击导航条，会出现左侧导航条收起按钮
                if (data.className == "nav-myvisit" || data.className == 'nav-mypublic') {
                    width = 0;
                    showClosePanelBtn(false);
                }
                return true;
            },
            //只用于多语言修改智能标签的title的值
            onModifyIR: function (data) {
                if (data && data.en == "IR") {
                    setTimeout(function () {
                        var list = $(".tooltip-content").length
                        for (var i = 0; i < list; i++) {
                            if ($(".tooltip-content").eq(i).text() == "IR") {
                                $(".tooltip-content").eq(i).text("Intelligent Recommendation");
                            }
                        }
                    }, 1000);
                }
            },
            onclickBottomNavItem: function (data, event) {
                alert(0);
                toggleLeftPanel(data.loadTree, curSelectNavType() == 0);//
                //刷新导航视图的滚动条
                setTimeout(function () {
                    app.trigger('doc:toggleLeftPanelPerbar');
                }, 100);
                return true;
            },
            curSelectNavType: curSelectNavType,
            curSelectNavView: curSelectNavView,
            showDocflowMenu: showDocflowMenu,
            newTagFlag: newTagFlag,
            isDocflowFlag: isDocflowFlag,
            newTagHeight: newTagHeight,
            isOpenLeftBar: isOpenLeftBar,
            rightPanelHeight: rightPanelHeight,
            isMyalbum: isMyalbum,
            toggleLeftPanel: function (open) {
                setTimeout(function () { toggleLeftPanel(open) }, 110);
            },
            sharebutShow: sharebutShow,
            sharcurentModule: sharcurentModule,
            docflowIframeUrl: docflowIframeUrl,
            docflowNowReadNum: docflowNowReadNum,
            showDocflowNowReadNum: showDocflowNowReadNum,
            showDofclowEmpty: showDofclowEmpty,
            routerTab: function(){
                app.trigger("mysharemebut:event")
            }
        }
    });

/*
    author:sailer
    create:2015-5
    desc:单文件夹
*/
define('doc/informationbar/basicinfo/singlefolder/index',["komapping", "knockout", "common/edoc2v5-qrcode", "durandal/app", lang(), '../service', "header/userinfo/constinfo", "doc/index", "doc/contextmenu/permtype", "common/utils"],
    function (komapping, ko, qrcodeHandler, app, lang, Service, constinfo,doc,permType,utilsFun) {
    var utilsFuns = new utilsFun();
    //引用父页面数据
    var infomation = require("doc/informationbar/index");
    //服务类
    var _service = new Service();
    //订阅句柄
    var subscribeHander = null;
    var parentFolderName = ko.observable();
    var parentFolderTarget = ko.observable();
        var showContains = ko.observable(false);//v6判断右侧面板包含字段是否展示
       
    //计时id
    var timeid;
    /*
     * sailer创建于2015年5月
     * 组装 文件夹的 “含”
     * @fileCount 文件个数
     * @folderCount 文件夹个数
     * @return 选中的文件文件夹描述
     */
    function conposeConclude(fileCount, folderCount) {
        return fileCount + lang.files + folderCount + lang.folders;
    }
    var selectEl = function () {
        var re = {};
        jQuery('.navbar-list a').each(function (i, item) {
            var clasname = $(this).attr('class');
            if (clasname.indexOf('selected') > -1) {
                if (clasname.indexOf('enterprise') > -1) {
                    re = [1, 0];
                }
                //if (clasname.indexOf('team') > -1) {
                //    re = [constinfo.userInfo.teamidActive.path, constinfo.userInfo.teamidActive.id];
                //}
                if (clasname.indexOf('person') > -1) {
                    re = [constinfo.userInfo.personActive.path, constinfo.userInfo.personActive.id];
                }
            }
        });
        return re;
    }
    /*
     * sailer创建于2015年5月
     * 进度条控制
     * @text 描述
     * @value 进度
     */
    function conposeProcessBar(text, value) {
        infomation.initData().showProcessBar(true);
        value = value.toFixed(0);
        $('#p').progressbar({ text: text, value: value });
    }
    var spstr = function (str, a) {
        var sp = str.split("\\");
        if (a == 1) { return sp[sp.length - 1]; }
        var va = "";
        for (var i = 0; i < sp.length - 1; i++) {
            va += sp[i] + "/";
        }
        return va;
    }
    var niceWidth = ko.computed(function () {
        return (infomation.infoWidth() - 120) + 'px';
    });
    var nameWidth = ko.computed(function () {
        return (infomation.infoWidth() - 160) + 'px';
    })
    var parentData = {
        isInOutpublish: ko.observable(false),
        isInPublishPreview: ko.observable(false)
    };
    var isPublicRoot = ko.observable(false);
        var isPersonRoot = ko.observable(false);
        var visibleRemarkEditIcon = ko.observable(false);//
        //modify:20220407 by xuying [缺陷:28466]文件夹外发结果右侧面板隐藏编辑按钮
        var visibleRemarkEditIcon1 = ko.observable(true);
    //文件夹大小计算loading效果
    var showLoading = ko.observable(false);
    var data = ko.$store.state.curentModule();
    //获取当前回收站的目录类型
    function getRecyle(infomation) {
        var data = ko.$store.state.curentModule();
        if (!data) return 2;
        var isrecyle = (data.module == "enterprise" || data.module == "person" || data.module == "team") && data.id == "-1";
        var id = infomation.initData.sourceData.id ? infomation.initData.sourceData.id : 0;
        if(data.module=="team"&&window.location.href.indexOf("recycle")>0){//团队回收站
            return 1;
        }
        /*modify: 2022-02-28 by wenyuan.niu[缺陷: 27342]团队库回收站文件夹右侧面板缺少备注字段*/
        if (id == 0 || isrecyle) {
            return 1;
        } else {
            return 2;
        }
    }
    // modify:2022-02-16 by zhengjie[26143]团队库的文件夹打开右侧面板后基本信息显示不全
    //获取是否回收站
    function getIsShowIcon(infomation) {
        var data = ko.$store.state.curentModule();
        if (!data) return 2;
        var isrecyle=false;
        isrecyle = (data.module == "enterprise" || data.module == "person") && data.id == "-1";
        if(data.module == "team"&&window.location.href.indexOf("recycle")>0)
        {
            isrecyle=true;
        }
        // if(isrecyle||isPublicRoot||isPersonRoot||infomation.initData.sourceData.parentFolderName === 'Team')
        var id = infomation.initData.sourceData.id ? infomation.initData.sourceData.id : 0;
        var result=false;
        if(id == 0 &&isrecyle){
            result= true; 
        }
        if(isPublicRoot()){
            //result= true;
        }
        if(isPersonRoot()){
            // result= true;
        }
        return result;
    }
    //获取当前是否团队库
    function getIsTeam(infomation) {
        var data = ko.$store.state.curentModule();
        if (!data) return 2;
        if( data.module == "team"){
            return 0; 
        }
        return 1;
    }
    //1为顶级回收站 2为子级文件夹
    var isrecyle = ko.observable(1);
    var isteam = ko.observable(1);
    var isShowIcon = ko.observable(false);
    
    function getNameByParentFolderName(name) {
        switch (name) {
            case "PublicRoot":
                return constinfo.productMes().DocFuncCompanyName;
                break;
            case "PersonalRoot":
                return constinfo.productMes().DocFuncPersonalName;
                break;
            case "Team":
                return constinfo.productMes().DocFuncTeamName;
                break;
        }
    }

    function getNavigatePath(path, id) {
        if (/^1\\/.test(path)) {
            return "doc/enterprise/" + id;
        }
        if (/^2\\/.test(path)) {
            return "doc/person/" + id;
        }
        if (/^7\\/.test(path)) {
            return "doc/team/" + id;
        }
    }
    //ldb 解决首次展开右侧面板默认有滚动条
    app.on('singlefolder:index').then(function (message) {
        if (message.type) {
            $(".singfolderScroll").perfectScrollbar('update').perfectScrollbar();
        }
    });
    return {
        isRecyle: isrecyle,
        parentData: parentData,
        isPublicRoot: isPublicRoot,
        isPersonRoot: isPersonRoot,
        // 2022-03-19 缺陷 28151 wei1.zhou 企业库不显示备注
        visibleRemark:ko.computed(function(){
            var isvisibleRemark=true;
            isvisibleRemark = !isPublicRoot();
            return isvisibleRemark;
        }),
        showLoading:showLoading,
        showContains:showContains,
        isTeam:isteam,
        isShowIcon: isShowIcon,
        visibleRemarkEditIcon: visibleRemarkEditIcon,
        visibleRemarkEditIcon1: visibleRemarkEditIcon1,
        /**
         * 对快捷方式的文件夹信息进行装饰
         */
        decorate: function () {
            var that = this, initData = infomation.initData();
            var sourceData = infomation.initData.sourceData; //modify: 2017-12-19 获取通知对象的原始对象。
            if (sourceData.shortcut) {
                that.referenceFolderLocation();
                _service.decorateFolderByShortCut(initData, function (folderInfo) {
                    that.sourceName(folderInfo.name);
                    return infomation.formateData(folderInfo);
                });
            }
        },
        goToSource: function () {
            var sourceData = infomation.initData.sourceData;
            if (sourceData.shortcut) {
                var perm = infomation.initData.sourceData.shortcut.entryPermission;
                if ((perm & permType.display) != permType.display) {
                    app.trigger("message:show", { type: 'error', message: lang.noPerm });
                    return;
                }
                var strId = sourceData.shortcut.entryPath.split("\\");
                strId = strId[strId.length - 2];
                var router = getNavigatePath(sourceData.shortcut.entryPath, strId);
                //导航跳转
                if (router.split('/')[1] == "person") {
                    ko.$store.dispatch('setCurrentFolderPosition', { folderId: router.split('/')[2], position: 'person' });
                }
                doc.router.navigate(router);
            }
        },
        /**
        * 获取引用文件夹所在位置信息
        */
        referenceFolderLocation: function () {

            var initData = infomation.initData();
            if (!initData.namePath()) return;
            var strParentName = (initData.namePath() ? initData.namePath().split("\\") : initData.namePath().split("\\"));
                var rootFolderKey = strParentName[0];
                if (rootFolderKey == 'PersonalRoot') {
                    //个人文档库的情况，去掉username_id的路径节点
                    strParentName.splice(1, 1);
                }
                var rootFolderName = getNameByParentFolderName(rootFolderKey);
                strParentName[0] = rootFolderName;
                //判断层级
                if (strParentName.length > 2) {
                    //超级两层的只显示 最后两层的路径
                    initData.parentFolderName(strParentName[strParentName.length - 2] + "\\" + strParentName[strParentName.length - 1]);
                } else {
                    initData.parentFolderName(strParentName.join('\\'));
                }
                return infomation.initData();
        },

        activate: function (activationData) {
            parentData.isInOutpublish(activationData.isInOutpublish());
            parentData.isInPublishPreview(activationData.isInPublishPreview());
            //infomation.initData().showBtnCalculate(true);
            var type = getRecyle(infomation);
            var isTeam=getIsTeam(infomation);
            var isShowIcon=getIsShowIcon(infomation);
            this.isShowIcon(isShowIcon);
            this.isTeam(isTeam);
            this.isRecyle(type);
            this.decorate();
            //modify: 2022.02.25 mengfangchao 专项RD - 640主列表可编辑备注、密级、标签，其他页面下均不可编辑-start
            this.visibleRemarkEditIcon(true);
            var moduleName = require('knockout').$store.state.curentModule()//获取当前模块
            if (moduleName && (moduleName.module == "favoritelist" || moduleName.module == "quick")) {
                this.visibleRemarkEditIcon(false);
            }
            var windowpath = window.location.href;
            if (windowpath.indexOf("preview.html") > -1) {
                this.visibleRemarkEditIcon(false);
            }
            // modify 2022-04-18 liuchunli #29289（UI）外发结果界面文件夹右侧面板不显示图标
            if (windowpath.indexOf("outpublish.html") > -1) {
                this.visibleRemarkEditIcon(false);
                this.visibleRemarkEditIcon1(false);
                this.isShowIcon(false);
            }
            if (windowpath.indexOf("ispublish=true") > -1) {
                this.visibleRemarkEditIcon(false);
            }

        },

        compositionComplete: function () {
            if (constInfo.getLang() == "en") {
                $("#folderCreatime").css("max-width","80px");
            }
            $(".singfolderScroll").perfectScrollbar('update').perfectScrollbar();
            var that = this;
            that.setQrcode();
            //初始化数据
            infomation.initData().folderUsedSize(" ");
            infomation.initData().conclude(" ");

             // 发送右侧面板大小变化消息
             if($(".right-panel")){
                 app.trigger('centerPanel:width');
            };
       
            if (subscribeHander) return;
            subscribeHander = app.on('upload:init').then(function (messager) {
                if (messager.module == "update"
                    || messager.module == "myfavorite"
                    || messager.module == "myshare"
                    || messager.module == "mypublish"
                    || messager.module == "mylock"
                    || messager.module == "mysubscribe"
                    || messager.module == "publish"
                    || messager.module == "outpublish"
                    || messager.module == "myvisit"
                ) {
                    return;
                }
                if (messager.module && messager.module == "null") {
                    return;
                }
                //zhifeiya-优化右侧面板无选中文件的时候报错
                if (messager.select.length != 1) {
                    return;
                }
                var messagerId = messager.select[0].data ? messager.select[0].data.id : messager.select[0][0].data.id;
                if (that.folderId == messagerId) {
                    return;
                }
                var _data = messager.select[0].data || messager.select[0][0].data;
                if ((_data.fileType == 32 && (!_data.shortcut || _data.shortcut.entryType == 2)) || !_data.folderType) {
                    //快捷方式的文件，或者是文件的通知，不用处理。
                    return;
                }
                that.decorate();
                that.folderId = messager.select[0].data.id;
            });
        },
        
        detached: function () {
            showContains(false);
        },

        //主页面返回的文件夹数据
        //parentView:infomation.initData,
        parentView: function () {
            var type = getRecyle(infomation);
            this.isRecyle(type);
            var sourceData = infomation.initData.sourceData;
            //if (sourceData.childFileCount != undefined && sourceData.childFolderCount != undefined) {
            //    infomation.initData().conclude(conposeConclude(sourceData.childFileCount, sourceData.childFolderCount));
            //}
            // modify: 2022.03.19 28139  wei1.zhou 【UI】备注过长显示...
            if(!infomation.initData().remarkdis){                 
                infomation.initData().remarkdis = ko.computed(function(){
                    if((infomation.initData().remark()||'').length>21){
                        return (infomation.initData().remark().substring(0,19)+'...');
                    }
                    return (infomation.initData().remark());
                });
            } 

            //modify: 2022-03-01 by shuanbao.wei [缺陷:26957]从企业库/个人库切换至回收站，点击F5刷新页面，界面点击其他地方无反应
            if (infomation.initData.hasOwnProperty('code') && !infomation.initData().code()) {
                infomation.initData().code("-");
            };
            
            if (sourceData.shortcut) {
                this.isRecyle(2);
            }
            //兼容左侧树点击回收站的时候
            var data = ko.$store.state.curentModule();
            //刷新时候获取不到sourceData.parentFolderName的值
            if(sourceData.id&&sourceData.id=="1"&&!sourceData.parentFolderName)
            {
                sourceData.parentFolderName = 'PublicRoot';
            }
            if(sourceData.data && sourceData.data.name && sourceData.data.name.indexOf("_")>0 && sourceData.path.substring(0,1)=="2")
            {
                // modify: 2022-04-29 shuchen.hu #29871 个人库根目录右侧面板不应该出现备注字段
                // if(data){
                //     if(data.module == "person" && data.id == 8 && sourceData.id === 8){
                //         isPublicRoot(true);
                //     }else{
                //         isPublicRoot(false);
                //     }
                // }
                isPersonRoot(true);
                var name =constinfo.productMes().DocFuncPersonalName;
                infomation.initData().name(name);
                return infomation.initData();
            }
            if(data){
                // modify: 2022-04-29 shuchen.hu #29871 个人库根目录右侧面板不应该出现备注字段 || data.module == "person" && data.id == 8 && sourceData.id === 8

                if (data.module == "enterprise" && data.id == 1 && sourceData.id === 1 ) {
                    //隐藏顶级目录企业文档库的编号
                    isPublicRoot(true);
                } else {
                    isPublicRoot(false);
                };
                if (sourceData.parentFolderName === 'PersonalRoot') {
                    isPersonRoot(true);
                } else {
                    isPersonRoot(false);
                }
                if (isPublicRoot() || isPersonRoot()) {
                    var newName = getNameByParentFolderName(sourceData.parentFolderName);
                    // 取用接口的企业名称
                    // if(sourceData.parentFolderName == "PublicRoot" && constinfo.licInfo() && constinfo.licInfo().CustomerName){ // 企业库根节点取产品名称
                    //     infomation.initData().name(constinfo.licInfo().CustomerName);
                    // } else {
                    //     infomation.initData().name(newName);
                    // };
                    infomation.initData().name(newName);
                }
            }
            //modify: 2022-03-10 by wenyuan.niu[缺陷:27418]【UI】企业库根目录右侧面板中，创建信息没有左对齐
            if (infomation.initData().creatorName instanceof Function) {
                if (infomation.initData().creatorName() == null) {
                    infomation.initData().creatorName('Administrator');
                }
            }

            return infomation.initData();
        },
        sourceData: function () {
            return infomation.initData.sourceData;
        },
        //文件新增趋势
        recentFileInfo: function () {

            alert("文件新增趋势");
        },
        lang: lang,
        sourceName: ko.observable(),
        folderId: null,
        niceWidth: niceWidth,
        nameWidth: nameWidth,
        /*
       *显示
       *zhouwei 2017-3-23
       * 设置二维码huang.hao 2017年11月27日
       */
        setQrcode: function () {
            var self = this;
            //modify 20220906 xuying 缺陷36981 在回收站打开右侧面板后刷新界面，右侧面板一直加载中且显示错误
            if (!$('.single-folder-qrcode')[0]) { return; }
            qrcodeHandler.makeHoverQrcode({
                selector: '.single-folder-qrcode',
                width: 150,
                height: 150,
                position: 'bottom',
                hideDelay: 200,
                getNewContent: function () {
                    var windowpath = window.location.href;
                    var protocol = window.location.protocol;
                    var host = window.location.host;
                    var search = window.location.search;
                    var pathname = window.location.pathname;
                    var tkn = $.cookie("tkn");
                    var folderId = infomation.initData().id();
                    var path = windowpath.substring(0, windowpath.lastIndexOf("/") + 1) + folderId;
                    return path;
                },
                onShow: function () {
                    var $this = $(this);
                    var pos = this.getBoundingClientRect();
                    $this.tooltip('tip').css({
                        left: (pos.left - 150) + 'px',
                        top: (pos.top + 30) + 'px',
                        width: '150px',
                        height: '150px'
                    })
                    $this.tooltip('arrow').css({ display: 'none' })
                }
            });
        },


        // v6文件夹-->右侧面板备注调起属性弹窗
        // modify:2022-02-28 by wenyuan.niu[缺陷:27329]打开文件夹右侧面板，点击备注修改按钮，弹出了属性弹窗
        folderHanleRemarkClick:function($data,event){
            var curentData = ko.$store.state.curentModule();

            app.trigger("component:show", {
                moduleId: "doc/informationbar/basicinfo/singlefolder/remark/index",
                datas: infomation.initData(),
                message: curentData.module + ':reload',
                moduleName: curentData.module
            });
        },

        /*
        * sailer创建于2015年5月
        * 文件夹计算
        */
        calculate: function () {
           var self=this;
            //按钮点击后隐藏
            //infomation.initData().showBtnCalculate(false);
            self.showLoading(true);
            // 判断右侧面板基本信息包含字段赋值
            showContains(infomation.initData.sourceData.conclude.length);
            //当前选中文件夹id
            var currentSelectedFld = 0;
            //path
            var currentPath = "";
            var arrparm = selectEl();
            var DeleteFolder = "GetFolderSizeChildCount";
            currentSelectedFld = infomation.initData.sourceData.id ? infomation.initData.sourceData.id : 0;
            if(infomation.initData.sourceData.shortcut){ //如果是引用，则需要查询源文件夹的Id
                currentSelectedFld = infomation.initData.sourceData.shortcut.entryId;
            }
            var data = ko.$store.state.curentModule();
            if(data){ //modify: data在外发界面 获取不到值 #4178
                var isrecyle = (data.module == "enterprise" || data.module == "person" || data.module == "team") && data.id == "-1";
                if (isrecyle||(data.module=="team" && window.location.href.indexOf("recycle")>0)) {
                    DeleteFolder = "GetDeleteFolderSizeChildCount";
                    if (currentSelectedFld == 0) {
                        if (arrparm[0]) {
                            currentPath = arrparm[0];
                            currentSelectedFld = arrparm[1];
                        } else if (data.module == "team" && window.location.href.indexOf("recycle") > 0) {
                            //截取团队ID，后边优化
                            var url = window.location.href;
                            var teamId = url.substr((url.lastIndexOf("/") + 1), (url.indexOf("?") - url.lastIndexOf("/") - 1));
                            currentPath = "7\\" + teamId;
                            currentSelectedFld = teamId;
                        }
                    }
                }
            }
            $.ajax({
                data: {
                    module: "WebClient",
                    fun: DeleteFolder,
                    folderId: currentSelectedFld,
                    path: currentPath,
                    recalculate: true
                }
            }).done(function (data) {

                self.showLoading(false);
                if (!data || data.folderSize === undefined) {
                    return;
                }
                //个人内容库刷新左侧容量图
                if (data.module == "person") {
                    app.trigger("person:updateCapacity");
                }
                //更改文件夹列表中选中文件夹的大小
                //文件夹大小计算页面去除大小显示, 此方法更新已无用处,而且会造成文件夹大小变更时,页面会显示文件夹大小,所以注释掉
                //app.trigger("folder:updateFolderSize", { folderId: currentSelectedFld, folderSize: data.folderSize, folderUsedSize: infomation.clculateSize(data.folderSize) });
                //包含
                infomation.initData().conclude(conposeConclude(data.childFiles, data.childFolders));
                //文件夹大小
                infomation.initData().folderUsedSize(infomation.clculateSize(data.folderSize));

                //有最大空间的显示进度条，否则显示实际占用 回收站显示实际占用
                if (infomation.initData().maxFolderSize == undefined || infomation.initData().maxFolderSize() == 0 || self.isRecyle()) {
                    infomation.initData().showProcessBar(false);
                    //infomation.initData().showUsedSize(true);
                    //cq.tianao  引用文件不改有大小 2018.4.3
                    if (infomation.initData.sourceData.shortcut) {
                        infomation.initData().folderUsedSize(infomation.clculateSize(0));
                    } else {
                        infomation.initData().folderUsedSize(infomation.clculateSize(data.folderSize));
                    }

                } else {
                    conposeProcessBar(
                        infomation.clculateSize
                            (infomation.initData().folderUsedSize()) +
                        "/" +
                        infomation.clculateSize(
                            infomation.initData().maxFolderSize()
                        ),
                        infomation.initData().folderUsedSize() * 100.0 / infomation.initData().maxFolderSize());
                }

            });
        }
    };


});
define('outpublish/service',["knockout", "jquery", "durandal/app", "underscore", "header/userinfo/constinfo"],
    function (ko, $, app, _, constinfo) {
        var ajax = function () {
            /**
             * ajax内部包装方法
             * @param {} conroller 控制器名
             * @param {} action    方法名
             * @param {} data      数据
             * @returns {} 
             */
            var ajaxBase = function (conroller, action, data) {
                return $.ajax({ data: $.extend({ module: conroller, fun: action }, data) });
            }
            return {
                /**
                 * 通过code获取外发信息
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                getPublishInfoByCode: function (extData) {
                    return ajaxBase("PublishManager", "GetPublishInfoByCode", extData);
                },
            /**
             * 通过code获取外发信息
             * @param {Object} extData 需要传递的参数对象(不包括module,fun)
             * @returns {Objectdeferred} jquery延迟对象
             */
                GetPublishSystemInfoByCode: function (extData) {
                    return $.ajax({ data: $.extend({ module: "SystemManager", fun: "GetPublishSystemInfoByCode" }, extData), async: false });
                },
                /**
                 * 通过code获取外发信息
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                getPublishBasicInfoByCode: function (extData) {
                    return ajaxBase("PublishManager", "GetPublishBasicInfoByCode", extData);
                },
                /**
                 * 通过code检测密码
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                checkPublishPasswordByCode: function (extData) {
                    // TODO 
                   return $.ajax({
                        type: "POST",
                        url: "/api/services/Fexmain/outPublish/checkCode",
                        data: JSON.stringify(extData),
                        dataType: 'json',
                        async: false,
                        contentType:'application/json;charset=utf-8',
                        success: function(data){

                        }
                    })
                },
                /**
                 * 改变对于外发的下载次数
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                changePublishDownloadTime: function (extData) {
                    return ajaxBase("PublishManager", "ChangePublishDownloadTime", extData);
                },
                /**
                 * 获取产品logo地址
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                getProductLogo: function (extData) {
                    return ajaxBase("WebClient", "GetProductLogo", extData);
                },
                /**
                 * 获取外发过期时间
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                getExpirationTime: function (extData) {
                    return ajaxBase("WebClient", "ExpirationTime", extData);
                },
                /**
                 * 通过code获取文件夹外发（未使用）
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                // getFolderPublishByCode: function (extData) {
                //     return ajaxBase("WebClient", "GetFolderPublishByCode", extData);
                // },
                /**
                 * 获取文件夹外发的子文件内容
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                getFolderChildren: function (extData) {
                    return ajaxBase("WebClient", "GetFolderChildren", extData);
                },
                /**
                 * 获取文件外发数据
                 * @param {Object} extData 需要传递的参数对象(不包括module,fun)
                 * @returns {Objectdeferred} jquery延迟对象
                 */
                loadPublishFile: function (extData) {
                    return ajaxBase("WebClient", "LoadPublishFile", extData);
                }
            }
        }();
        /**
         * 存放一些辅助方法
         */
        var helper = {
            /**
             * loading效果
             */
            loading: {
                /**
                 * 显示
                 * @returns {} 
                 */
                show: function () {
                    app.trigger("message:show", { type: "loading" });
                },
                /**
                 * 隐藏
                 * @returns {} 
                 */
                hide: function () {
                    app.trigger("message:show", { type: "loaded" });
                }
            },
            /**
             * 消息对象
             */
            msg: {
                /**
                 * 弹窗
                 * @param {String} content 弹窗消息
                 * @param {Boolean} type 弹窗类型 true:success | false:error(默认)
                 * @returns {} 
                 */
                alert: function (content, type) {
                    if (type) {
                        app.trigger("message:show", { type: 'success', message: content });
                    } else {
                        app.trigger("message:show", { type: 'error', message: content });
                    }
                }
            },
            /**
             * 判断是不是鼠标右键点击
             * @param {} event 
             * @returns {} 
             */
            isRightClick: function (event) {
                return event.button === 2;
            },
            /**
             * *mikahuang创建于201610
             * 根据路径返回跳转文档库类型
             * @param {} path 路径
             * @param {} id   文件id
             * @returns {} 
             */
            getNavigatePath: function (path, id) {
                if (/^1\\/.test(path)) {
                    return "doc/enterprise/" + id;
                }
                if (/^2\\/.test(path)) {
                    return "doc/person/" + id;
                }
                if (/^7\\/.test(path)) {
                    return "doc/team/" + id;
                }
                return "";
            },
            /**
             * 获取文件缩略图视图图标，优先显示图标，然后判断是否有缩略图，如果有，则替换原有的
             * @param {Object} currItem
             * @param {} fileId 
             * @param {} fileName 
             * @returns {} 
             */
            getFileThumbIcon: function (currItem, fileId, fileName) {
                var iconUrl = constInfo.getFileIconUrl(fileName, "48");
                var thumbUrl = constInfo.getThumbUrl(fileId);
                var img = new Image();
                img.onload = function () {
                    currItem.thumbUrl(thumbUrl);
                };
                img.src = thumbUrl;
                return iconUrl;
            },
            /**
             * 获取文件列表图视图图标
             * @param {} fileName 
             * @returns {} 
             */
            getFileListIcon: function (fileName) {
                var unknownIcon = "unknown.png";
                var iconUrl = constInfo.getFileIconUrl(fileName);
                /*//如果无匹配的，显示未知图标
                if (extName !== "undefined") {
                    extName = extName + ".png";
                } else {
                    extName = unknownIcon;
                }
                //var baseUrl = constInfo.getBaseUrl() + constInfo.systemInfo.appPath.substring(1) + "/Content/images/fileIcons/32/";自定义图标不再32文件夹下
                var baseUrl = constInfo.getBaseUrl() + constInfo.systemInfo.appPath.substring(1) + "/Content/images/fileIcons/";
                var iconUrl = baseUrl + extName;*/
                return iconUrl;
            },
            /**
            * 获取文件夹的图标
            */
            getFolderIcon: function () {
                return constinfo.getFolderIconUrl();
            },
            /**
             * 获取Xml格式化的查询条件字符串,不需要的值，传入null
             * @param {} pageSize 页面数据条数
             * @param {} pageIndex 页索引
             * @param {} sortField 排序字段
             * @param {} sortDesc 是否降序排序
             * @returns {} 
             */
            getArgXml: function (pageSize, pageIndex, sortField, sortDesc) {
                var pageSizeStr = "",
                    pageIndexStr = "",
                    sortFieldStr = "",
                    sortDescStr = "";

                if (pageSize !== null) {
                    pageSizeStr = "<PageSize>" + pageSize + "</PageSize>";
                }
                if (pageIndex !== null) {
                    pageIndexStr = "<PageNum>" + pageIndex + "</PageNum>";
                }
                if (sortField !== null) {
                    sortFieldStr = "<SortInfoName>" + sortField + "</SortInfoName>";
                }
                if (sortDesc !== null) {
                    sortDescStr = "<SortDesc>" + sortDesc + "</SortDesc>";
                }
                var condition = "<GetListArgs>" + pageSizeStr + pageIndexStr + sortFieldStr + sortDescStr + "</GetListArgs>";
                return encodeURIComponent(condition);
            }
        };
        var getCode = function () {
            return $.getQueryString(window.decodeURIComponent(window.location.href), "code");
        };

        /**
         * 根据URL生成Code相关参数：兼容：外发，共享等
         */
        var generalCodeParam = function () {
            var key, value, href = window.decodeURIComponent(window.location.href);
            if ($.getQueryString(href, 'code')) {
                //外发
                key = 'code';
                value = $.getQueryString(href, 'code');
            } else if ($.getQueryString(href, 'shareCode')) {
                //共享
                key = 'shareCode';
                value = $.getQueryString(href, 'shareCode');
            }
            return "&" + key + "=" + value;
        }

        return {
            helper: helper,
            getCode: getCode, //通过url获取code值
            generalCodeParam: generalCodeParam, //通过url生成code参数，兼容：外发，共享
            ajax: ajax//ajax对象
        }
    });
/*
 *author:mikahuang创建于201505
 *描述:外发路由设定
 */
define('outpublish/outpublish',['plugins/router', './service', 'cookie',lang(),"jquery",], function (router, service, cookie,lang,$,) {
    return {
        router: router,
        activate: function () {
            //没有外发code显示白页
            var code = service.getCode();
            if (!code) {
                document.writeln(lang.urlCode);
                return false;
            }
            //路由设定
            return router.map([
                { route: ['', 'view'], title: lang.publish, moduleId: 'outpublish/view/index', nav: false },
                { route: 'authentication', title: lang.vali, moduleId: 'outpublish/authentication/index', nav: false },
                { route: 'authenticationpwd', title: lang.vali, moduleId: 'outpublish/authentication/index1', nav: false },
                { route: 'expire', title: lang.expire, moduleId: 'outpublish/expire/index', nav: false }
            ]).buildNavigationModel().activate();
        },
        // modify: 2022-08-25 cuizhichao #36391 外发文件加，设置上传权限，使用未登录的浏览器访问外发连接，拖拽上传文件夹，弹出上传框，跳转登录；
        compositionComplete: function (child, parent, settings) {
           setTimeout(() => {
                $('#dropOverlay').remove()
           }, 1000);
            
        },
    };
});
/**
 * @author hao.huang@macrowing.com
 * @time 创建于2017-8-25进行二次开发
 * @desc 企业内容库列表视图功能模块
 * @description 主要实现通过外部指定一个文件夹,然后显示文件夹的子文件夹及文件
 *       列表中包括菜单的右键弹出，和工具栏的操作通知
 */
define('outpublish/plugins/publishView/list/List',[commLang(), "./../" + lang(), "knockout", "komapping", "underscore", "durandal/app", "easyuimapping", "jquery", "../../../service",
    "doc/contextmenu/index", "header/userinfo/constinfo", "doc/contextmenu/permtype"],
    function (clang, lang, ko, komapping, _, app, easyuimapping, $, service,
        contextmenu, constinfo, permType) {
        /**
         * 存放一些辅助方法
         */
        var hp = service.helper;

        /**
         * ajax请求管理对象
         */
        var ajax = service.ajax;
        /**
         * 数据项类型
         */
        var fileTypeData = {
            folder: 1,
            file: 2,
            shortCut: 32,
            docflow:128
        }
        //外发类型：文件外发和文件夹外发
        var publishTypeData = {
            file: "file",
            folder: "folder"
        }
        var columnTitleList = [
            { "Name": "basic:name", "Title": lang.name, "Width": 300, "Remark": null, "DataType": "docName", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:creator", "Title": lang.creator, "Width": 100, "Remark": null, "DataType": "string", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            /*modify: 2022-03-10 by yujianghe[缺陷:27623]外发结果界面列表优化*/
            /*{ "Name": "basic:createTime", "Title": lang.createTime, "Width": 125, "Remark": null, "DataType": "datetime", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },*/
            { "Name": "basic:editor", "Title": lang.editor, "Width": 150, "Remark": null, "DataType": "string", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:modifyTime", "Title": lang.modifyTime, "Width": 150, "Remark": null, "DataType": "datetime", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:size", "Title": lang.size, "Width": 100, "Remark": null, "DataType": "fileSize", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:version", "Title": lang.version, "Width": 150, "Remark": null, "DataType": "version", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:downloadTime", "Title": lang.downloadTime, "Width": 160, "Remark": null, "DataType": "downloadTime", "SortDesc": true, "isActive": false, "isSort": false, "minWidth": 100 },
            { "Name": "basic:previewTimes", "Title": lang.previewTimes, "Width": 160, "Remark": null, "DataType": "previewTimes", "SortDesc": true, "isActive": false, "isSort": false, "minWidth": 100 }];
            
        //var message = "enterpriselist:reload";

        /**
         * 视图模型构造函数，在视图上进行绑定的视图变量值,所有的值都进行ko绑定，统一调用格式
         * @returns {} 
         */
        function Viewmodel() {
            this.isSelectAll = ko.observable(false); //是否是全选
            this.folderList = ko.observableArray([]); //文件夹列表
            this.fileList = ko.observableArray([]); //文件列表
            this.selectedList = ko.observableArray([]); //选中文件(夹)列表
            this.columnTitleList = ko.observableArray([]);//列头列表
            this.isTriggerColumnTitleStopResize = ko.observable(false); //是否是刚触发过调整列宽事件
            this.showPaging = ko.observable(false)//是否显示分页  cq.tianao
            this.message = "";
        }
        //用于控制按钮的显示的数据
        function wrapForToolbar(self, list) {
            var vm = self.vm;
            var parent = self.parent;
            var publishType = parent.vm.publishType();
            var data = {
                isPublishFolder: false,//是否是文件夹外发
                isPublishFile: false,//是否是文件外发
                isSingleFile: false,//是否是单文件
                isMultiFile: false,//是否是多文件
                isSingleFolder: false,//
                isFileAndFolder: false,//是否是否是单文件夹
                isMultiFolder: false,//是否是多文件夹是既包含文件又包含文件夹
                isInPublishRoot: false,//是否是在外发根目录
                isInSubFolder: false,//是否是在子文件夹(仅文件夹外发存在)
                canRefresh: false,//能否进行刷新操作
                isSelectAllFolde:false,//第一加载的时候判断是否选择文件，是否给下载按钮
                isDetelefile:false,//是否是已删除文件
            };
            data.canRefresh = (0 === vm.selectedList().length);
           
            if (publishType === publishTypeData.file) {
                //文件外发没有根目录
                data.isPublishFile = true;
                var isSelectfile=false;
                if (list.length > 0) {
                    for(var i=0;i<list.length;i++){
                       
                        if(list[i].isSelected)
                        {
                           isSelectfile=true;
                           break;
                        }
                        
                    }
                    if (list.length === 1) {
                        data.isSingleFile = true;
                    } else {
                        if (isSelectfile) {
                            data.isMultiFile = true
                        }else{
                            data.isMultiFile = false;
                        }
                        
                    }
                    for(var j=0;j<list.length;j++)
                    {
                        if(list[j].data.isDeleted)
                        {
                            data.isDetelefile=true;
                            break;
                        }
                    }
                   
                }
            }
            if (publishType === publishTypeData.folder) {
                var publishRootId = parent.vm.publishInfo().publishFolderId;//文件夹外发的根文件夹
                var currentFolderId = parent.vm.currentFolder().id;//当前文件夹
                data.isPublishFolder = true;
                if (currentFolderId === publishRootId) {
                    data.isInPublishRoot = true;
                } else {
                    data.isInSubFolder = true;
                }
                if (list.length > 0) {
                    if (list.length === 1) {
                        //单个文件   ||   单个轻文挡文件  轻文挡为独立类型
                        if (list[0].data.fileType === fileTypeData.file || list[0].data.fileType === fileTypeData.docflow) {
                            data.isSingleFile = true;
                        }
                        //单个文件夹
                        if (list[0].data.folderType === fileTypeData.folder) {
                            data.isSingleFolder = true;
                        }
                        //判断是否选中文件
                        if(vm&&vm.isSelectAll&&vm.isSelectAll()||(list&&list[0]&&list[0].isSelected))
                        {
                          data.isSelectAllFolde=true;
                        }
                    } else {
                        //如果没有返回undefined
                        var countRes = _.countBy(list, function (item) {
                            var d = item.data;
                            if (d.fileType === fileTypeData.file || d.fileType === fileTypeData.docflow) {
                                return "fileNum";
                            } else if (d.folderType === fileTypeData.folder) {
                                return "folderNum";
                            }
                        });
                        if (countRes.fileNum > 0 && (!countRes.folderNum)) {
                            data.isMultiFile = true;
                        }
                        if ((!countRes.fileNum) && countRes.folderNum > 0) {
                            data.isMultiFolder = true;
                        }
                        if (countRes.fileNum > 0 && countRes.folderNum > 0) {
                            data.isFileAndFolder = true;
                        }
                    }
                }
            }
            return data;
        }
        //权限类型
        var permData = {
            download: 1 << 1,
            upload: 1 << 2,
            edit: 1 << 3,
            downloadformat: 1 << 4
        }
        /*
         * 计算外发操作权限
         * return {}
         */
        var calcPermission = function (perm) {
            return {
                canUpload: (perm & permData.upload) === permData.upload,
                canEdit: (perm & permData.edit) === permData.edit,
                canDownload: (perm & permData.download) === permData.download,
                canDownloadformat: (perm & permData.downloadformat) === permData.downloadformat
            }
        }
        /**
         *  挂载模块内执行一些核心的方法，不会与视图交互,主要供viewfunction对象调用
         */
        var core = {
            /**
             * 更新工具栏的数据
             * config { showContextMenu: true, pageX:60, pageY:10}
             * @returns {} 
             */
            updateToolbarData: function (config) {
                var self = this;
                var vm = self.vm;
                var parent = self.parent;
                var selectedList = vm.selectedList();
                var isFilePbulishSelectedListIsZero = false;
                //在文件夹外发的情况下,未选择任何文件夹，则把当前文件夹选中
                if (selectedList.length === 0 && (parent.vm.publishType() === publishTypeData.folder)) {
                    var cf = parent.vm.currentFolder();
                    cf.data = _.clone(cf);
                    if (cf.id === 1) cf.data.folderType = -1;//如果为企业内容库根文件夹
                    selectedList = [cf];
                } else if (selectedList.length === 0 && (parent.vm.publishType() === publishTypeData.file)) {
                    //如果在文件外发情况下，如果未选择任何文件，则把所有文件都传递过去
                    selectedList = vm.fileList();
                    isFilePbulishSelectedListIsZero = true;
                }
                var list = ko.toJS(selectedList);//插件需要纯js对象，且无ko监控属性，不然会有不可预知的错误
                //为空跳过
                if (list.length !== 0) {
                    setTimeout(function () {
                        list[0].operationData = wrapForToolbar(self, list);//工具栏按钮根据勾选文件类型过滤，通过消息传递数据到工具栏
                        list[0].permissionData = calcPermission(parent.vm.publishPermission());//工具栏按钮根据权限过滤
                        window.permissionData = list[0].permissionData;
                        list[0].permissionData.downloadTime = parent.vm.downloadTime();
                        if (isFilePbulishSelectedListIsZero) {
                            app.trigger("upload:init", { message: vm.message, select: [], module: parent.vm.viewName() });
                        } else {
                            app.trigger("upload:init", { message: vm.message, select: list, module: parent.vm.viewName() });
                        }
                        app.trigger("toolbar:update", { message: vm.message, select: list, module: parent.vm.viewName() });
                        //显示右键菜单
                        if (config && config.showContextMenu) {
                            contextmenu.showMenu(config.pageX, config.pageY, list, function (command) { },
                                self, vm.message, parent.vm.viewName());
                        }
                    }, 10);
                }
            },
            /**
             * 初始化所有数据为默认值
             * @returns {} 
             */
            initData: function () {
                var self = this;
                var vm = self.vm;
                vm.selectedList([]);
                vm.fileList([]);
                vm.folderList([]);
            },
            /**
             * 初始化可拖拽功能
             * @param {String} containter 容器类名不带点
             * @param {String} subItem 子项类名不带点
             * @returns {} 
             */
            initSelectable: function (containter, subItem) {
                var self = this;
                var vm = self.vm;
                //给列表添加绑定拖拉多选功能
                $(containter).selectable({
                    filter: subItem,
                    distance: 10,
                    selecting: function (event, ui) {
                        var item = ko.dataFor(ui.selecting);
                        if (!item.isSelected()) {
                            item.isSelected(true);
                            vm.selectedList.push(item);
                            self.isSelectAll();
                        }
                        self.updateSelectAll();
                    },
                    unselecting: function (event, ui) {
                        var item = ko.dataFor(ui.unselecting);
                        if (item.isSelected()) {
                            item.isSelected(false);
                            vm.selectedList.remove(item);
                        }
                        self.updateSelectAll();
                    },
                    start: function (event, ui) {
                        self.clearSelectedList();
                    },
                    stop: function (event, ui) {
                        self.updateSelectAll();
                        self.updateToolbarData();
                    }
                });
            },
            /**
             * 预览文件
             * @param {} entry 对象实体
             * @returns {} 传入的数据对象
             */
            previewFile: function (entry) {
                //lic控制
                if (constInfo.productCanPreview()) {
                    app.trigger("component:show", { moduleId: "doc/download/index", datas: [{ fileType: 2, id: entry.id }], message: "", moduleName: "" });
                    return false;
                }
                //在文档库可以走在线预览和下载预览，但是外发对下载有控制，所以外发目前只走在线预览
                if ((entry.perm & permType.preview) !== permType.preview) {
                    hp.msg.alert(lang.publish_tip);
                    return false;
                }
                //if (!constInfo.checkExtCanPreview(entry.name) || entry.id === "") {
                //    hp.msg.alert(lang.publish_tip1);
                //    return false;
                //}
                var previewUrl = constInfo.systemInfo.previewUrl;
                //var permDownload = (entry.perm & permType.download) === permType.download;
                var url = $.format(previewUrl + "?fileid={0}&ispublish=true&code={1}",
                    entry.fileGuid, service.getCode());
                /*
                 * modify: 2022-02-18 by shuanbao.wei[BUG:26224]
                 * 外发包含其他路径引用文件的文件夹，点击预览引用文件无法进入预览页面
                 * 描述：当预览文件类型为引用文件时，预览url增加linkfileid参数
                 */
                if (entry.data.fileType == 32) {
                    url = $.format(url + "&linkfileid={0}",
                        entry.data.id);
                }
                window.open(url);
                return true;
                // return constInfo.selectPreviewType(entry);
            },
            /**
             * 格式化列表视图的表头数据
             * @param {} list 
             * @returns {} 
             */
            formatColumnTitleList: function (list) {
                _.each(list, function (v, i, list) {
                    v.Title = clang[v.Name] || v.Title;
                    //默认降序图标
                    if (v.SortDesc == undefined || v.SortDesc == null) {
                        v.SortDesc = true;
                    }
                    //是否选中
                    if (v.isActive == undefined || v.isActive == null) {
                        v.isActive = false;
                    }
                    //设置列的最小宽度
                    v.minWidth = 100;
                    //宽度没有设置时，赋给列默认宽度
                    //重置宽度,暂时不用490配置宽度,v4界面宽度适合v5界面显得太挤
                    switch (v.DataType) {
                        case "docName":
                            v.Width = v.Width || 300;
                            break;
                        case "string":
                            v.Width = v.Width || 150;
                            break;
                        case "datetime":
                            v.Width = v.Width || 150;
                            break;
                        case "fileSize":
                            v.Width = v.Width || 120;
                            break;
                        case "version":
                            v.Width = v.Width || 120;
                            break;
                        case "remark":
                            v.Width = v.Width || 200;
                            break;
                        default:
                            v.Width = v.Width || 200;
                    }
                });
                return komapping.fromJS(list)();
            },
            /**
             * 更新排序字段状态
             * @returns {} 
             */
            updateSortStatus: function () {
                var self = this;
                var pvm = self.parent.vm;
                var vm = self.vm;
                var sortItem = _.find(vm.columnTitleList(), function (item) {
                    return item.Name() === pvm.sortField();
                });
                sortItem.SortDesc(pvm.sortDesc.peek());
            },
            /**
             * 刷新UI数据
             * @param {Object} docData 父级传递过来的格式化好的文档数据
             * @param {Object} data  ajax.getFolderChildren返回的数据对象
             * @returns {} 
             */
            refreshView: function (docData, data) {
                var self = this;
                var vm = self.vm;
                self.initData();
                //文件夹外发没有记录已下载次数
                // if(self.parent.vm.publishType()===publishTypeData.folder){
                //     columnTitleList.pop();
                // }
                if (vm.columnTitleList().length === 0) {
                    var titleList = self.formatColumnTitleList(columnTitleList);
                    vm.columnTitleList(titleList);
                }
                vm.fileList(docData.fileList);
                vm.folderList(docData.folderList);
                if (self.parent) {
                    self.vm.showPaging(self.parent.vm.totalCount() > 10);
                    self.updateSortStatus();
                    self.updateToolbarData();
                    var selector = '.' + self.parent.vm.viewName() + " .document-list-body";
                    $(selector).perfectScrollbar('update');
                }
                self.updateSelectAll();
            }
        }
        /**
         * 复选框勾选部分的API
         */
        var selector = {
            /**
            * 判断是否是全选
            * @param {} vm 
            * @returns {} 
            */
            isSelectAll: function () {
                var self = this;
                var vm = self.vm;
                if (vm.selectedList().length === 0 && (vm.folderList().length + vm.fileList().length) === 0) {
                    return false;
                }
                return vm.selectedList().length === (vm.folderList().length + vm.fileList().length);
            },
            /**
             * 更新全选复选框状态
             * @returns {} 
             */
            updateSelectAll: function () {
                var self = this;
                self.vm.isSelectAll(self.isSelectAll());
            },
            /**
             * 清空已选择列表
             * @param {} list 
             * @returns {} 
             */
            clearSelectedList: function () {
                var self = this;
                var vm = self.vm;
                _.each(vm.selectedList(), function (v, i, list) {
                    v.isSelected(false);
                });
                vm.selectedList([]);
            },
            /**
             * 进行全选/全不选
             * @returns {} 
             */
            selectAllOrNot: function () {
                var self = this;
                var vm = self.vm;
                if (self.isSelectAll()) {
                    //已经全选，进行取消全选操作
                    self.clearSelectedList();
                } else {
                    //未进行全选，进行全选操作
                    vm.selectedList([]);
                    _.each(vm.folderList(), function (v, i, list) {
                        v.isSelected(true);
                        vm.selectedList.push(v);
                    });
                    _.each(vm.fileList(), function (v, i, list) {
                        v.isSelected(true);
                        vm.selectedList.push(v);
                    });
                }
                self.updateSelectAll();
            }
        }
        var tempResizeElement = null;
        /**
         * 在视图上进行绑定的视图方法
         */
        var viewfunction = {
            /**
             * 全选/全不选复选框被点击的处理程序
             * @returns {} 
             */
            onCheckboxSelectAllClick: function (val, obj, event) {
                if (hp.isRightClick(event)) return;
                var self = obj;
                self.selectAllOrNot();
                self.updateToolbarData();
            },
            /**
             * 在数据项的checkbox被点击的处理程序
             * @param {} obj 
             * @param {} event 
             * @returns {} 
             */
            onItemSelectClick: function (val, obj, event) {
                if (hp.isRightClick(event)) return;
                var self = ko.contextFor(event.target).$root;
                var vm = self.vm;
                event.stopPropagation();
                if (obj.isSelected()) {
                    vm.selectedList.push(obj);
                } else {
                    vm.selectedList.remove(obj);
                }
                self.updateSelectAll();
                self.updateToolbarData();
            },
            /**
             * 文件夹项被点击执行的处理程序
             * @param {} obj 
             * @param {} event 
             * @returns {} 
             */
            onFolderClick: function (obj, event) {
                if (hp.isRightClick(event)) return;
                event.stopPropagation();
                var currItem = ko.contextFor(event.target).$parent;
                var list = ko.contextFor(event.target).$root;
                var d = currItem.data;
                list.parent.vm.pageNumber(1);
                list.parent.loadData(d.id);
            },
            /**
             * 点击文件执行的处理程序
             * @param {} obj 
             * @param {} event 
             * @returns {} 
             */
            onFileClick: function (obj, event) {
                if (hp.isRightClick(event)) return;
                var currItem = ko.contextFor(event.target).$parent;
                var self = ko.contextFor(event.target).$root;
                event.stopPropagation();
                var data = currItem.data;
                var entry = null;
                if(data.isDeleted)
                {
                    app.trigger("message:show",{type:'error',message:lang.fileDeleted});
                    return;
                }
                if(data.isMorePreviewTimes)
                {
                    app.trigger("message:show",{type:'error',message:lang.morePreviewTime});
                    return;
                }
                if (data.fileType === fileTypeData.shortCut) {
                    var sc = data.shortcut;
                    entry = {
                        id: sc.entryId,
                        type: sc.entryType,
                        name: sc.entryName,
                        path: sc.entryPath,
                        perm: sc.entryPermission,
                        data: data
                    }
                    //点击的link为文件夹的情况
                    if (sc.entryType === fileTypeData.folder) {
                        //判断文件夹权限
                        if ((entry.perm & permType.display) !== permType.display) {
                            hp.msg.alert(lang.publish_tip3);
                            return;
                        } else {
                            self.parent.loadData(entry.id);
                        }
                    } else {
                        self.previewFile(entry);
                    }
                } else {
                    entry = {
                        id: data.id,
                        type: data.fileType,
                        name: data.name,
                        path: data.path,
                        perm: data.permission,
                        data: data,
                        fileGuid: data.fileGuid
                    }
                    self.previewFile(entry);
                }
            },
            /**
             * 在某项上右键菜单执行的处理程序
             * @param {} obj 
             * @param {} e 
             * @returns {} 
             */
            onItemContextClick: function (obj, event) {
                var self = ko.contextFor(event.target).$root;
                var vm = self.vm;
                var parent = self.parent;
                event.stopPropagation();
                var selectedList = vm.selectedList();
                //如果已选中列表不包含当前右击项，则列表为当前选中项
                if (!_.contains(selectedList, obj)) {
                    if (selectedList.length !== 0) {
                        self.clearSelectedList();
                    }
                    obj.isSelected(true);
                    vm.selectedList.push(obj);
                }
                self.updateSelectAll();
                self.updateToolbarData({ showContextMenu: true, pageX: event.pageX - 2, pageY: event.pageY - 2 });
            },
            /**
             * 空白处点击右键菜单执行的处理程序
             * @param {} obj 
             * @param {} e 
             * @returns {} 
             */
            onSpaceContextClick: function (obj, event) {
                var self = ko.contextFor(event.target).$root;
                var vm = self.vm;
                var parent = self.parent;
                if (parent.vm.publishType() === publishTypeData.file) {
                    return;//文件外发右键空白处不做操作
                }
                self.clearSelectedList();
                self.updateSelectAll();
                self.updateToolbarData({ showContextMenu: true, pageX: event.pageX - 2, pageY: event.pageY - 2 });
            },
            /**
             * 在空白地方点击的时候的事件
             * @returns {} 
             */
            onSpaceClick: function (obj, event) {
                if (hp.isRightClick(event)) return;
                var self = ko.contextFor(event.target).$root;
                self.clearSelectedList();
                self.updateSelectAll();
                self.updateToolbarData();
            },
            /**
             * 开始改变列表头大小的事件
             * @returns {} 
             */
            onColumnTitleStartResize: function (event) {
                var target = event.target;
                tempResizeElement = target;
                var self = ko.contextFor(target).$root;
                if (self.parent) {
                    $('.' + self.parent.vm.viewName() + ' .document-list-header th').css('cursor', 'e-resize');
                }
            },
            /**
             * 结束改变列表头大小的事件
             * @returns {} 
             */
            onColumnTitleStopResize: function (event) {
                var target = event.target;
                var self = ko.contextFor(target).$root;
                var index = $(target).attr("index");
                if (!index) {
                    target = tempResizeElement;
                    self = ko.contextFor(target).$root;
                    index = $(target).attr("index");
                }
                var width = $(target).width();
                self.vm.columnTitleList()[index].Width(width);
                self.vm.isTriggerColumnTitleStopResize(true);
                if (self.parent) {
                    $('.' + self.parent.vm.viewName() + ' .document-list-header th').css('cursor', 'pointer');
                }
            },
            /**
             * 点击列表头事件
             * @returns {} 
             */
            onColumnTitleClick: function (obj, event) {
                if (obj.Name() == "basic:downloadTime") return;
                if (obj.Name() == "basic:previewTimes") return;
                if (hp.isRightClick(event)) return;
                var self = ko.contextFor(event.target).$root;
                var vm = self.vm;
                if (vm.isTriggerColumnTitleStopResize()) {
                    vm.isTriggerColumnTitleStopResize(false);
                    return;
                }
                _.each(self.vm.columnTitleList(), function (v, i, list) {
                    v.isActive(false);
                });
                obj.isActive(true);
                obj.SortDesc(!obj.SortDesc());
                var parent = self.parent;
                var pvm = parent.vm;
                pvm.sortField(obj.Name());
                pvm.sortDesc(obj.SortDesc.peek());//赋值非监控值
                if (pvm.currentFolder()) {//如果为文件夹外发
                    app.trigger("folder-publish:load", { text: "pagination", id: pvm.currentFolder().id });
                } else {
                    self.parent.loadData();
                }
            },
            /**
             * 点击数据网格行的事件
             * @returns {} 
             */
            onGridRowClick: function (obj, event) {

                if (hp.isRightClick(event)) return;
                //兼容对象类型
                if (obj.data && obj.data.fileType) {
                    obj.data.type = obj.data.fileType;
                }
                if (obj.data && obj.data.folderType) {
                    obj.data.type = obj.data.folderType;
                }
                event.stopPropagation();
                var self = ko.contextFor(event.target).$root;
                var vm = self.vm;
                self.clearSelectedList();
                obj.isSelected(true);
                vm.selectedList.push(obj);
                self.updateSelectAll();
                self.updateToolbarData();
            }
        };

        /**
         * 生命周期对象
         */
        var lifecycle = {
            attached: function (curr, parent, settings) {
                var parentData = settings.bindingContext.$data;
                var self = this;

                //把父级上下文挂载上
                self.parent = parentData;
                self.vm.message = parentData.vm.viewName() + ":reload";
                //超过10条才显示分页
                self.vm.showPaging(parentData.vm.totalCount() > 10);
                self.updateToolbarData();
                var pager = $(curr).find(".v5-pagination").pagination({
                    context: self,
                    showPageList: true,
                    pageList: [10, 20, 30, 40, 50],
                    total: parentData.vm.totalCount(),
                    pageNumber: parentData.vm.pageNumber(),
                    /**
                     * 点击上一页、下一页、输入页码的时候触发
                     * @param {} pageNumber 
                     * @param {} pageSize 
                     * @returns {} 
                     */
                    onSelectPage: function (pageNumber, pageSize) {
                        var opts = $(this).pagination("options");
                        var context = opts.context;
                        context.parent.vm.pageNumber(pageNumber);
                        context.parent.vm.pageSize(pageSize);
                        var id = null;
                        if (context.parent.vm.currentFolder()) {//如果为文件夹外发
                            app.trigger("folder-publish:load", { text: "pagination", id: context.parent.vm.currentFolder().id });
                        } else {
                            context.parent.loadData();
                        }
                    }
                });
                self.pager = pager;
                var selector = '.' + parentData.vm.viewName() + " .document-list-body";
                self.initSelectable(selector, ".document-list-item");
                $(selector).perfectScrollbar();
                var thy = _.throttle(function () {
                    $(selector).selectable('refresh');
                }, 250)//节流
                var thx = _.throttle(function () {
                    var newWidth = $(selector + ' table').width();
                    $('.' + parentData.vm.viewName() + ' .v5-pagination').css('width', newWidth + 'px');
                    var s = $(selector).get(0);
                    $('.' + parentData.vm.viewName() + ' .document-list-header table').css('left', (-s.scrollLeft) + 'px');
                }, 10)//节流
                var debresize = _.debounce(function () {
                    $(selector).perfectScrollbar('update');
                }, 150);
                $(selector).off('ps-scroll-y').on('ps-scroll-y', thy);
                $(selector).off('ps-scroll-x').on('ps-scroll-x', thx);
                $(window).off('resize.docView').on('resize.docView', debresize);
            }
        }
        /**
         * 传入视图类型 enterprise person team
         * @param {} viewType 
         * @returns {} 
         */
        function List(viewname, parent) {
            this.parent = parent;
            this.vm = new Viewmodel();
            this.vf = viewfunction;
            this.constinfo = constinfo;
        }

        $.extend(List.prototype, core, lifecycle, selector);
        return List;
    });
/**
 * 外发结果页
 */
define('outpublish/plugins/publishView/DocView',[commLang(), lang(), 'knockout', 'jquery', 'underscore', "../../service", "durandal/app", "./list/List", "header/userinfo/constinfo", "edoc2ErrorCode", "plugins/router"],
    function (clang, lang, ko, $, _, service, app, List, constinfo, edoc2ErrorCode, router) {
        /**
         * 子视图类型
         */
        var subViewTypeData = {
            thumbnail: "thumbnail",
            list: "list"
        }
        //外发类型：文件外发和文件夹外发
        var publishTypeData = {
            file: "file",
            folder: "folder"
        }

        //授权类型
        var authTypeData = {
            needPwd: 2,
            noPwd: 1
        }
        //对象类型
        var objTypeData = {
            folder: 1,
            file: 2,
            shortcut: 32
        };
        var hp = service.helper;
        var ajax = service.ajax;
        var columnTitleList = [
            { "Name": "basic:name", "Title": lang.name, "Width": 300, "Remark": null, "DataType": "docName", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:size", "Title": lang.size, "Width": 100, "Remark": null, "DataType": "fileSize", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:creator", "Title": lang.creator, "Width": 100, "Remark": null, "DataType": "string", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:createTime", "Title": lang.createTime, "Width": 100, "Remark": null, "DataType": "datetime", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:editor", "Title": lang.editor, "Width": 150, "Remark": null, "DataType": "string", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:modifyTime", "Title": lang.modifyTime, "Width": 150, "Remark": null, "DataType": "datetime", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:version", "Title": lang.version, "Width": 150, "Remark": null, "DataType": "version", "SortDesc": true, "isActive": false, "isSort": true, "minWidth": 100 },
            { "Name": "basic:downloadTime", "Title": lang.downloadTime, "Width": 100, "Remark": null, "DataType": "downloadTime", "SortDesc": true, "isActive": false, "isSort": false, "minWidth": 100 },
            { "Name": "basic:previewTimes", "Title": lang.previewTimes, "Width": 100, "Remark": null, "DataType": "previewTimes", "SortDesc": true, "isActive": false, "isSort": false, "minWidth": 100 }];

        /**
        * 验证文件是否可以预览
        * @param {object} data - 实体对象
        */
        function checkFileCanPreview(file) {
            var name = file.name;
            //2018-01-16 Tian.Ao 判断是否是引用文件 如果是引用文件就取源文件名称
            if (file.fileType == 32) {
                name = file.shortcut.entryName;
            }
            var ret = true;
            if (constInfo.systemInfo.instanceConfigInfo.grayFileCanNotPreview && !constInfo.checkExtCanPreview(name)) {
                ret = false;
            }
            //modify 2022-4-12 yangyang.li 缺陷27421 系统管理设置无法预览文件显示灰色,外发列表已删除源文件的引用文件应显示灰色
            if (constInfo.systemInfo.instanceConfigInfo.grayFileCanNotPreview && file.shortcut && file.shortcut.isDelete) {
                ret = false;
            }
            return ret;
        }
        /**
         * 格式化后台返回的文件/文件夹数据
         * @param {} info 
         * @param {} subViewType 
         * @returns {} 
         */
        function formatDocData(info, subViewType) {
            var attrList = _.map(columnTitleList, function (item) { return item.Name.split(":")[1]; });
            var fileList = info.FilesInfo;
            var folderList = info.FoldersInfo;
            function wrap(item) {
                var obj = {
                    data: item
                }
                _.each(attrList, function (prop, index, list) {
                    if (item[prop] != undefined) {
                        switch (prop) {
                            case "createTime":
                            case "modifyTime":
                                //时间格式化
                                obj[prop] = constInfo.getDateInterval(item[prop]);
                                break;
                            case "archiveTime":
                            case "effectiveTime":
                            case "expirationTime":
                                obj[prop] = constInfo.getDateInterval(item["effectiveTime"]);
                                //排除无效时间#gs
                                if (obj[prop].indexOf('1-') === 0) {
                                    obj[prop] = '';
                                }
                                break;
                            case "state":
                                obj[prop] = lang["DocState" + item[prop]];
                                break;
                            default:
                                //其他值原样输出
                                obj[prop] = item[prop];
                        }
                        //格式化文件大小，文件夹的信息为0，列表中显示为空
                        if (item["folderType"] != undefined && item["folderType"] == 1) {
                            //文件夹
                            obj["size"] = "-";
                        } else {
                            if (obj["size"] === "0") {
                                obj["size"] = "";
                            } else if (obj["size"]  == '') {
                                obj["size"]  = "-";
                            } else{
                                //统一格式化大小
                                obj["size"] = constInfo.formatSize(parseInt(item["size"]));
                            }
                        }

                    } else {
                        switch (prop) {
                            case "creator":
                                //创建人与列头信息做转换匹配
                                obj[prop] = item["creatorName"];
                                break;
                            case "editor":
                                //修改人与列头信息做转换匹配
                                obj[prop] = item["editorName"];
                                break;
                            case "downloadTime":
                                var c = item['nowDownloadCount'];
                                obj[prop] = ko.observable(c !== undefined ? c : '-');
                                break;
                            case "version":
                                obj[prop] = item["lastVerNumStr"] === undefined ? "-" : item["lastVerNumStr"];
                                break;
                            case "previewTimes":
                                  var p=item["nowPreviewCount"];
                                obj[prop] = ko.observable(p !== undefined ? p : '-');
                                break;

                        }
                    }
                });
                //给记录信息附加isSelected属性，并且初始化不选中
                obj.isSelected = ko.observable(false);
                return obj;
            };

            folderList = _.map(folderList, function (val) {
                return wrap(val);
            });

            fileList = _.map(fileList, function (val) {
                var obj = wrap(val);
                var url = "";
                obj.iconUrl = ko.observable();//图标文件路径,list和thumb都存在
                //如果是快捷方式
                if ((val.fileType === objTypeData.shortcut) && val.shortcut) {
                    if (val.shortcut.entryType === objTypeData.folder) {
                        //如果是文件夹快捷方式
                        url = hp.getFolderIcon();
                    } else {
                        //如果是文件快捷方式
                        url = hp.getFileListIcon(val.shortcut.entryName);
                    }
                    //处理掉引用文件扩展名
                    var index = obj.name.lastIndexOf(".link");
                    if (index > -1) {
                        obj.name = obj.name.substring(0, index);
                    }
                } else {
                    url = hp.getFileListIcon(obj.name);
                }
                obj.iconUrl(url);
                var canPreview = checkFileCanPreview(val);
                obj.canPreview = ko.observable(canPreview);
                return obj;
            });
            return {
                fileList: fileList,
                folderList: folderList
            }
        };

        function Viewmodel(viewName) {
            this.activeView = ko.observable();
            this.code = ko.observable();//外发的code
            this.publishInfo = ko.observable(); //根据code获取的当前外发信息
            this.publishType = ko.observable();//外发类型 file folder
            this.currentFolder = ko.observable(null); //当前文件夹对象
            this.publishPermission = ko.observable()//外发的权限值，控制 上传、下载、编辑、更新
            this.currentSubViewType = ko.observable(subViewTypeData.list);//当前子视图类型
            this.viewName = ko.observable(viewName);//当前视图名，如 enterprise person team publish
            this.sortDesc = ko.observable(false); //降序排序
            this.sortField = ko.observable("basic:name"); //排序字段
            this.pageSize = ko.observable(10); //页大小
            this.pageNumber = ko.observable(1); //页索引
            this.totalCount = ko.observable(1); //总记录数
            this.navPathList = ko.observableArray([{ id: -1, text: lang.publish_list }]);//导航索引
            this.downloadTime = ko.observable(0);
            this.verifyCode = ko.observable();
        }


        var core = {
            /**
             * 改变显示视图
             * @param {} viewType 
             * @returns {} 
             */
            changeActiveView: function (viewType) {
                var self = this;
                var vm = self.vm;
                vm.pageNumber(1).currentSubViewType(viewType);
                vm.currentViewId(vm.lastViewId());
                self.loadData();
            },
            registerEvents: function () {
                var self = this;
                var vm = self.vm;
                //下载成功后
                app.on("publish:download:success").then(function () {
                    self.list.updateToolbarData();
                    var list = self.list.vm.selectedList();
                    for (var i = 0; i < list.length; i++) {
                        if(list[i].data.folderType!==objTypeData.folder){
                            list[i].downloadTime(list[i].downloadTime() + 1);
                        }
                    }
                });
                //面包屑跳转
                app.on("folder-publish:load", function (node) {
                    if (node.id === -1) return;//-1为外发列表节点
                    var list = self.vm.navPathList();
                    var index = null;
                    _.each(list, function (v, i, list) {
                        if (v.id === node.id) {
                            index = i;
                        }
                    });
                    self.vm.navPathList(list.slice(0, index));
                    self.loadData(node.id);
                });
                //面包屑，返回上一级
                app.on("folder-publish:back").then(function () {
                    var last = _.last(self.vm.navPathList());
                    if (last.id !== -1) {
                        self.vm.navPathList.pop();
                    }
                    var pop = self.vm.navPathList.pop();
                    self.vm.pageNumber(1);
                    self.loadData(pop.id);
                });
                //用于刷新
                app.on(self.viewName + ":reload").then(function () {
                    if (vm.currentFolder()) {//如果为文件夹外发
                        app.trigger("folder-publish:load", { text: "pagination", id: vm.currentFolder().id });
                    } else {
                        self.loadData();
                    }
                });
                app.on("publish:downloadTime:update", function (time) {
                    vm.downloadTime(time);
                })
            },
            /**
             * 刷新子视图
             * @param {} data ajax.getFolderChildren返回的数据对象
             * @returns {} 
             */
            refreshSubView: function (data) {
                var self = this;
                var vm = self.vm;
                var docData = data.docListInfo;

                if (vm.currentSubViewType() === subViewTypeData.list) {
                    docData = formatDocData(docData, subViewTypeData.list);
                    vm.activeView(self.list);
                    self.list.refreshView(docData, data);
                } else if (vm.currentSubViewType() === subViewTypeData.thumbnail) {
                    docData = formatDocData(docData, subViewTypeData.thumbnail);
                    vm.activeView(self.thumb);
                    self.thumb.refreshView(docData, data);
                }
            },
            /*
             * 按情况加载数据
             */
            loadData: function (folderId) {
                var self = this;
                var vm = self.vm;
                hp.loading.show();
                if (vm.publishType() === publishTypeData.file) {
                    //文件外发
                    ajax.loadPublishFile({
                        code: vm.code(),
                        argsXml: hp.getArgXml(vm.pageSize(), vm.pageNumber(), vm.sortField(), vm.sortDesc())
                    }).done(function (data) {
                        if (data.result == 0) {
                            self.setPaginationData(data);
                            self.updatePagination(data);
                            self.refreshSubView(data);
                            app.trigger("navigation:init", { message: "file-publish", paths: vm.navPathList() });
                        } else {
                            hp.msg.alert(edoc2ErrorCode["ErrorCode" + data.result]);
                        }
                        hp.loading.hide();
                    })
                } else if (vm.publishType() === publishTypeData.folder) {
                    var fid
                    //第一次加载文件夹外发界面
                    if (vm.currentFolder() === null) {
                        fid = vm.publishInfo().publishFolderId;
                        //文件夹外发
                    } else {
                        fid = folderId;
                    }
                    ajax.getFolderChildren({
                        fid: fid,
                        docViewId: 0,
                        code: vm.code(),
                        argsXml: hp.getArgXml(vm.pageSize(), vm.pageNumber(), vm.sortField(), vm.sortDesc())
                    }).done(function (data) {
                        if (data.result == 0) {
                            var f = data.thisFolder;
                            if (data.verifyCode) {
                                self.parent.vm.verifyCode(data.verifyCode);
                            }
                            if (f && f.permission == 0) {
                                app.trigger("message:show", { type: 'error', message: lang.publishUserNoPerm });
                            }
                            self.updateNavbar({ id: f.id, text: f.name });
                            vm.currentFolder(f);
                            self.setPaginationData(data);
                            self.updatePagination(data);
                            self.refreshSubView(data);
                        } else {
                            //hp.msg.alert(edoc2ErrorCode["ErrorCode" + data.result]);
                            if (data.result == 1) {
                                router.navigate("expire");
                            }
                            else {
                                hp.msg.alert(edoc2ErrorCode["ErrorCode" + data.result]);
                            }
                        }
                        hp.loading.hide();
                    });
                }
            },
            /**
             * 设置分页相关的数据
             *
             * @param {} data 
             * @returns {} 
             */
            setPaginationData: function (data) {

                var self = this;
                var vm = self.vm;
                var s = data.docListInfo.Settings;
                //vm.pageNumber(s.pageNum);
                vm.totalCount(s.totalCount);
            },
            /**
             * 更新导航栏
             */
            updateNavbar: function (pathNode) {
                var self = this;
                var vm = self.vm;
                if (pathNode) {
                    vm.navPathList.push(pathNode);
                }
                app.trigger("navigation:init", { message: "folder-publish:load", paths: vm.navPathList() });
            },
            /**
             * 更新分页组件数据
             * 
             * @returns {} 
             */
            updatePagination: function () {

                var self = this;
                var vm = self.vm;
                if (self.list.pager) {
                    self.list.pager.pagination({
                        pageNumber: vm.pageNumber(),
                        pageSize: vm.pageSize(),
                        total: vm.totalCount()
                    });
                }
            }
        }

        /**
         * 生命周期对象
         */
        var lifecycle = {
            compositionComplete: function (curr, parent, settings) {
                var parentData = settings.bindingContext.$data;
                var info = parentData.vm.publishInfo;
                var self = this;
                self.parent = parentData;
                var vm = self.vm;
                vm.pageNumber(1);
                vm.code(service.getCode());
                vm.publishInfo(info);
                vm.publishType(info.publishType);
                vm.publishPermission(info.publishPermission);
                vm.downloadTime(info.publishDownTime);
                //parentData.vm.downloadTime(info.publishDownTime);
                self.loadData();
                self.registerEvents();
            }
        }
        /**
         * 
         * @param {} viewName 视图名 如 enterprise person team
         * @returns {} 
         */
        function DocView(viewName) {
            var self = this;
            this.viewName = viewName;
            self.vm = new Viewmodel(viewName);
            self.list = new List(viewName, this);
        }
        $.extend(DocView.prototype, core, lifecycle);
        return DocView;
    });
/*
   author:mikahuang创建于201505
   desc:外发主视图
   editor:huang.hao 重构于2017年11月14日
*/
define('outpublish/view/index',["jquery", "knockout", "../service", "header/userinfo/cookie", 'plugins/router',
    "common/edoc2v5-qrcode", "durandal/app", "../plugins/publishView/DocView", lang(), "doc/toolbar/index", "header/userinfo/constinfo", "md5"],
    function ($, ko, service, cookie, router, qrcodeHandler, app, DocView, lang, Toolbar, constinfo) {
        var ajax = service.ajax;
        var hp = service.hp;
        //所有选择的文件和文件夹信息数组
        var datas = ko.observableArray([]);
        //组件需要回调的消息
        var message = ko.observable();
        //调用者的模块名称
        var moduleName = ko.observable();
        //调用者的模块的备注信息
        var moduleRemark = ko.observable();




        var viewmodel = {
            activeView: ko.observable(null),
            code: null,
            publishInfo: null,
            productBase64Logo: ko.observable(""),
            navigationView: ko.observable("outpublish/plugins/navigation/index"),
            informationbarView: ko.observable(''),
            toolbarView: Toolbar.createToolbar(Toolbar.createToolbar()),
            menuView: ko.observable("doc/contextmenu/index"),
            docSetBarView: ko.observable("outpublish/plugins/docsetbar/index"),
            isVerifyOk: ko.observable(false),//验证成功后才加载相关组件，不然跳转到过期或者输入密码界面，加载的其他组件会报错
            component: ko.observable(null),
            upload: ko.observable('component/upload/index'),
            message: ko.observable('component/message/index'),
            node: null,//list视图传递过来的数据
            expireTime: ko.observable(""),//过期时间
            remainTime: ko.observable(""),//剩余时间
            downloadTime: ko.observable(0),//下载次数，默认为0，仅在有下载权限的情况下有效 ,-1表示无下载次数限制
            verifyCode: ko.observable("")
        }
        var vm = viewmodel;

        //modify: 2018-01-11 haibin.long 增加对工具栏更新事件的拦截
        function InterceptEvent() {
        }
        var pt = InterceptEvent.prototype;
        pt.intercept = function () {
            this.bindComputed();
            //捕获工具栏更新的事件响应
            app.on('toolbar:update').then(function (msg) {
                ko.$store.dispatch("setCurrent", msg.select, msg.module, msg.message);
            });
            //捕获窗口变化
            $(window).on('resize.docpanel', function () {
                if (vm.toolbarView.moduleName) {
                    vm.toolbarView.reLoadSize();
                }
            });
        }
        /**
         * 绑定$store.current的改变，用于通知toolbar的更新
         */
        pt.bindComputed = function () {
            //监听数据的变化
            ko.computed(function () {
                var cur = ko.$store.state.currentData();
                if (cur) {
                    vm.toolbarView.refreshData(cur);
                }
            });
        }

        //------------------------------------------------------------------



        var authTypeData = {
            needPwd: 2,
            noPwd: 1
        }  
      //验证code
        function verifyByCode(code) {
            //先判断是不是IE8，是的话跳转登录页
            if (!window.document.getElementsByClassName) {
                location.href = constInfo.systemInfo.loginPageUrl;
            }
            if (!code) return false;
            ajax.getPublishBasicInfoByCode({ code: code })
                .done(function (data) {
                    //已过期，跳转到过期界面
                    if (data.publishExpired === undefined || data.publishExpired === 1) {
                        router.navigate("expire");
                        return;
                    }
                    getExpirationTime();
                    var code = $.getQueryString('code');
                    //需要密码
                    if (data.authType === authTypeData.needPwd) {
                        // var email = getCodeUserAccount().trim();
                        var md5Code = window.unescape(cookie("md5Code") || "");

                        ajax.checkPublishPasswordByCode({
                            code: code,
                            md5Code: md5Code
                            // firstEntry: "1"//modify: 2022-05-30 by shuanbao.wei [RD:883]进行中ECM v6.1 外发防破解
                        }).done(function (data) {
                            if (data.code === 0 && data.data.md5Code != null && data.data.md5Code != '') {
                                // cookie(code,data.data.pwd);
                                ajax.GetPublishSystemInfoByCode({ code: code }).done(function (result) {
                                    if (result.result && result.result == 5) {
                                        return false;
                                    }
                                    constInfo.systemInfo = result.systemInfo;
                                })
                                vm.isVerifyOk(true);
                                setPublishInfo(code);
                                var curUrl=document.URL;
                                if(curUrl&&curUrl.indexOf("fileid=")>-1)
                                {
                                    curUrl=curUrl.replace("outpublish.html","preview.html");
                                    router.navigate(curUrl);
                                }
                                else{
                                    afterVerifyOk();
                                }
                            }else {
                               
                                cookie(code, null);
                                 // // TODO 密码无效，清空，跳转到输入密码界面
                                 if(data.data.pwdType == 1){
                                    router.navigate("authenticationpwd");
                                 }else if(data.data.pwdType == 2){
                                    router.navigate("authentication");
                                 }
                               
                               
                            }
                        });
                    } else {
                        ajax.GetPublishSystemInfoByCode({ code: code }).done(function (result) {
                            if (result.result && result.result == 5) {
                                return false;
                            }
                            constInfo.systemInfo = result.systemInfo;
                        })
                        setPublishInfo(code);
                        vm.isVerifyOk(true);
                        afterVerifyOk();
                    }
                });
        }
        function setPublishInfo(code) {
            ajax.getPublishInfoByCode({ code: code }).done(function (data) {
                vm.publishInfo = data;
                vm.downloadTime(data.publishDownTime);
                getExpirationTime();
            });
        }
        //验证通过后再加载子视图
        function afterVerifyOk() {
            setTimeout(function () {
                //modify 20220524 缺陷30536 外发结果页面，多语言和二维码功能失效
                initView();
                vm.informationbarView('doc/informationbar/index');
                var publishView = new DocView("outpublish");
                vm.activeView(publishView);
            }, 300);//等候上面模块加载后再加载最后的模块
        }

        /*
         *wuyou创建于11/23/2016
         *获取外发过期时间
         */
        function getExpirationTime() {
            ajax.getExpirationTime({ code: code }).done(function (data) {
                //外发过期时间
                var endTime = formatDateString(data.publishEndTime);
                var currentTime = formatDateString(data.currentTime);
                vm.expireTime(lang.expiryTime + endTime.split(' ')[0]);
                countDown(getTimeSpan(data.currentTime,data.publishEndTime));
            });
        }
        /*23/11/2016
         *create by wuyou 
         *计算外发倒计时
         */
        function countDown(timeSpan) {
            var timer = setInterval(function () {
                var timeDistance, strTime;
                var intDay, intHour, intMinute, intSecond;
                timeDistance = timeSpan;
                if (timeDistance > 0) {
                    intDay = Math.floor(timeDistance / 86400000);
                    timeDistance -= intDay * 86400000;
                    intHour = Math.floor(timeDistance / 3600000);
                    timeDistance -= intHour * 3600000;
                    intMinute = Math.floor(timeDistance / 60000);
                    timeDistance -= intMinute * 60000;
                    intSecond = Math.floor(timeDistance / 1000);
                    countDown = lang.countDown + intDay + lang.day + fillZero(intHour) + lang.hour + fillZero(intMinute) + 
                    lang.minute + fillZero(intSecond) + lang.second;
                    timeSpan = timeSpan - 1000;
                    vm.remainTime(countDown);
                } else {
                    vm.remainTime(lang.expired);
                    router.navigate("expire");
                    clearInterval(timer);
                }
            }, 1000);

        }
        function getTimeSpan(start,end) {
            var reg= /\/Date\(([0-9]+)\+([0-9]+)/;
            reg.exec(start);
            start=RegExp.$1;
            reg.lastIndex=0;
            reg.exec(end);
            end=RegExp.$1;
            return parseInt(end)-parseInt(start);
        }
        function fillZero(num){
            num=parseInt(num);
            if(num<10){
                return '0'+num;
            }else{
                return num;
            }
        }
        function formatDateString(backendDate) {
            //  /Date(1524758400000+0800)/
            /\/Date\(([0-9]+)\+([0-9]+)/.exec(backendDate);
            var stamp = RegExp.$1;
            var date = new Date(parseInt(stamp));
            return date.getFullYear()+'/'+(date.getMonth()+1)+'/'
            +date.getDate()+' '+fillZero(date.getHours())+':'+fillZero(date.getMinutes())+':'+fillZero(date.getSeconds());
        }
        var setLogo = function () {
            vm.productBase64Logo(constInfo.productInfo.ProductLogo);
        }
        var setQrcode = function () {
            var windowpath = window.location.href;
            var protocol = window.location.protocol;
            var host = window.location.host;
            var search = window.location.search;
            var pathname = window.location.pathname;
            var tkn = $.cookie("tkn");
            var path = "";
            // modify 2022-04-19 liuchunli #29167（UI）图片和其他类型文件在预览界面，二维码悬浮状态下大小不一致（KB也有此问题）
            qrcodeHandler.makeHoverQrcode({
                selector: '.publish-qrcode',
                width: 100,
                height: 100,
                position: 'bottom',
                hideDelay: 200,
                isOutpublish: false, //modify 2022-03-01 liuchunli #27037【UI】外发结果页的下载二维码按钮错位
                isMyPublish: true,
                getNewContent: function () {
                    if (!$.getQueryString('code')) {
                        path = windowpath;
                    } else {
                        path = window.location.origin + "/publish.html?code=" + code + "#publish/publish";
                    }
                    return path;
                },
                onShow: function () {
                     var $this = $(this);
                    var pos = this.getBoundingClientRect();
                    $this.tooltip('tip').css({
                        left: (pos.left - 50) + 'px',
                       // top: (pos.top-88) + 'px',
                        //width: '178px',
                       // height:'158',
                    })
                    //$this.tooltip('arrow').css({ display: 'none' });//小图标隐藏
                    //鼠标停留后能显示
                    $this.tooltip('tip').unbind().bind('mouseenter',function(){
                        $this.tooltip('show');
                    }).bind('mouseleave',function(){
                        $this.tooltip('hide');
                        //$this.tooltip('destroy');
                    });
                    
                }
            });
            // qrcodeHandler.makeHoverQrcode({
            //     selector: '.publish-qrcode',
            //     width: 150,
            //     height: 150,
            //     //text: window.location.href,
            //     position: 'bottom',//modify: 2018.3.29 by tao.jiang  二维码跳转到h5
            //     getNewContent: function () {
            //         if (!$.getQueryString('code')) {
            //             path = windowpath;
            //         } else {
            //             path = window.location.origin + "/publish.html?code=" + code + "#publish/publish";
            //         }
            //         return path;
            //     },
            // })
        }
        function registerEvents() {
            app.on('templatelist:reload').then(function (action) {
                if (action == "Refresh") {
                    app.trigger("outpublishlist:reload", "");
                }
            });
            $("#publishlang").mousemove(function (e) {
                $("#changeIcon").attr("class", "logo-view-up");
                $("#publish-isshow-Langer").css('display', 'block');
            });
            $("#publishlang").mouseout(function (e) {
                $("#changeIcon").attr("class", "logo-view-down");
                $("#publish-isshow-Langer").css('display', 'none');
            });
            $("#Multilingualism").text(lang.Multilingualism);
            switch ($.cookie('lang')) {
                case "zh-cn":
                    $("#Language").attr("class", "log-icon-zh-cn");
                    $("#changelogo").text("简体中文");
                    break;
                case "en":
                    $("#Language").attr("class", "log-icon-en");
                    $("#changelogo").text("English");
                    break;

                case "ja":
                    $("#Language").attr("class", "log-icon-ja");
                    $("#changelogo").text("日本語");
                    break;
                case "zh-tw":
                    $("#Language").attr("class", "log-icon-zh-tw");
                    $("#changelogo").text("繁體中文");
                    break;
            }
            app.on("component:show").then(function (node) {
                datas(node.datas);//用于工具栏按钮获取数据
                moduleName(node.moduleName);
                message(node.message);
                moduleRemark(node.moduleRemark || "")
                vm.node = node;
                switch (node.moduleId) {
                    //上传的守护模块，即界面加载之后，可以通过拖拽进上传文件
                    case "component/upload/index":
                        //加入外发界面标记
                        node.datas.showWindow = true;
                        node.datas.isOutpublish = true;
                        app.trigger("upload:show", node.datas);
                        break;
                    default:
                        //这样写是为了防止两次相等
                        vm.component(null);
                        vm.component(node.moduleId);
                        break;
                }
            });
            app.on("publish:downloadTime:update", function (time) {
                vm.downloadTime(time);
            })
        }
        function initView() {
            setLogo();
            setQrcode();
            registerEvents()
        }

        return {
            vm: vm,
            datas: datas,
            moduleName: moduleName,
            message: message,
            lang:lang,
            moduleRemark: moduleRemark,
            binding: function () {
                var intercept = new InterceptEvent();
                intercept.intercept();
                return {
                    cacheViews: true
                }
            },
            attached: function () {
                //此处可以获取到所有dom
                code = service.getCode();
                verifyByCode(code);
            },
            compositionComplete: function (child, parent, settings) {
               
            },
            ChangeLanguage: function (text) {
                // <!-- modify 2022-03-08 liuchunli 多语言点击事件修改 -->
                // var theEvent = window.event || arguments.callee.caller.arguments[0];
                // var $target = $(theEvent.target);
                // var lng = $target.data("lang");
                var lng = text;
                var oUrl = location.href.toString();
                var nUrl = "";
                $.cookie("lang", lng);
                if (oUrl.indexOf("lang=") > 0) {
                    var re = eval('/(lang=)([^&]*)/gi');
                    nUrl = oUrl.replace(re, 'lang=' + lng +'#view');
                } else {
                    nUrl = oUrl.replace("#view", '&lang=' + lng + '#view');
                }
                location.href = nUrl;
            },
            do_about: function ($data, event) {
                //点击时触发,弹出具体窗体
                app.trigger("component:show", {
                    moduleId: "../header/about/about",
                    moduleName: "about",
                    datas: "",
                    message: ""
                });
            }
        };
    }
);

/*
    author:mikahuang
    create:201505
    desc:外发验证登录
*/
define('outpublish/authentication/index',['plugins/router', 'durandal/app', 'knockout', "header/userinfo/cookie", lang(), "../service", "jquery", "header/userinfo/constinfo", "md5"],
    function (router, app, ko, cookie, lang, service, $, constinfo) {
        var ajax = service.ajax;
        //发布信息
        var infoHtml = lang.ok + app.title + lang.sendFile;
        //头像src地址
        var userFace = ko.observable();
        if(constInfo.getLang()&&constInfo.getLang()=="en")
        {
            infoHtml=" sent you a file or files ";
        }
        //发布信息
        var publishInfoHtml = ko.observable(infoHtml);
        //密码框值
        var passwordText = ko.observable("");
        //发布code
        var code = "";
        //产品logo
        var logo = ko.observable();
        var backgroundImage = ko.observable();
        var publishUserName = ko.observable();
        var getCodeUserAccount =  ko.observable("");
        
        
        /*
         * mikahuang创建于2015年7月
         * 验证密码
         * @return deferred
         */
        var verifyPassword = function () {
            var pwd = passwordText().trim();
            var email = getCodeUserAccount().trim();
            ajax.checkPublishPasswordByCode({
                emailCode: pwd,
                code: code,
                userAccount: email
            }).done(function (data) {
                if (data.code === 0 && (data.data.md5Code != undefined && data.data.md5Code != '')) {
                    //modify: 2022-05-30 by shuanbao.wei [RD:883]进行中ECM v6.1 外发防破解,处理外发密码为特殊字符文件无法预览问题
                    cookie(code, window.decodeURI(data.data.pwd));
                    cookie("md5Code",data.data.md5Code);
                    router.navigate("view"); 
                    location.reload();
                } else {
                    cookie(code, null);
                    passwordText(""); 
                    if (data.result == 2) {
                        $.messager.alert(lang.tip, $.format(lang.authlock_tip,data.locktime), "warning");
                    }
                    else {
                        $.messager.alert(lang.tip, lang.auth_tip, "warning");
                    }
                }
            });

        }
        /**光标离开事件，恢复输入边框颜色****/
        var passwordOnblur = function () {
            $(".pwd-input").removeClass('focus');
        }
        /****光标进入事件，改变输入边框颜色***/
        var passwordonFocus = function () {
            $(".pwd-input").addClass('focus');
        }
        var onKeydown = function (obj, event) {
            if (event.keyCode === 13) {//enter
                verifyPassword();
            }
            return true;//允许默认行为
        }
        /*
      * mikahuang创建于2015年7月
      * 获取文档发布信息
      * @return deferred
      */
        var getPublishInfo = function (code, callback) {
            return ajax.getPublishBasicInfoByCode({ code: code }).done(function (data) {
                var url = "/ImageType/GetUserAvatar?&userId=" + data.publishUserId;
                userFace('url(' + url + ') no-repeat center center transparent');
                publishUserName(data.publishUserName);
                publishInfoHtml(infoHtml);
                if (data.authType === 1) {
                    router.navigate("view");
                }
            });
        };
        var updateCodeTask;
        var count;
        const emailRegex = "^[A-Za-z0-9\\u4e00-\\u9fa5]+(\\.[A-Za-z0-9\\u4e00-\\u9fa5]+)*@[A-Za-z0-9\\u4e00-\\u9fa5]+(\\.[A-Za-z0-9\\u4e00-\\u9fa5]+)*(\\.[A-Za-z\\u4e00-\\u9fa5]{2,})$";
       var getEmailCode = function(){
        var email = getCodeUserAccount().trim();
        if(!email.match(emailRegex)){
            $.messager.alert(lang.tip, lang.emailTip, "warning");
            return;
        }else{
            // $.getQueryString('code')
            // $.messager.alert(lang.tip, $.getQueryString('code') + "," + email, "warning");
            var data = {code:$.getQueryString('code'),userAccount: email};
            $.ajax({
                type: "POST",
                url: "/api/services/Fexmain/outPublish/getEmailCode",
                data: JSON.stringify(data),
                dataType: 'json',
                async: false,
                contentType:'application/json;charset=utf-8',
                success: function (data) {
                    if (data.code == 0) {
                        // 解除验证按钮
                        $("#passwordButton").attr("style", "width: 100px;");
                        $("#passwordButton").removeAttr("disabled");
                    } else {
                        $.messager.alert(lang.tip, data.message, "warning");
                    }
                }
            })
        }
        count = 60;
        
            // 禁用获取验证按钮 并修改样式
            $("#getcodebutton").attr("disabled","true");
            $("#getcodebutton").attr("style","width: 100px; background-color: gray;border-color: gray;");
            $("#getcodebutton").text(count + "s");
            updateCodeTask = setInterval(updateCountdown, 1000);
        };

        function updateCountdown(){
            count --;
            if(count == 0){
                clearInterval(updateCodeTask);
                $("#getcodebutton").text(lang.getcode);
                $("#getcodebutton").attr("style", "width: 100px;");
                $("#getcodebutton").removeAttr("disabled");
                return;
            }
            $("#getcodebutton").text(count + "s");
           
        }


        return {
            activate: function () {
                var def=$.Deferred();
                code = service.getCode();
                if (!code) return false;
                cookie(code, null);
                getPublishInfo(code);
                logo(constinfo.productInfo.ProductLogo);
                $.ajax({
                    data: {
                        module: 'PublishManager',
                        fun: 'GetLoginBackground'

                    }
                }).done(function (data) {
                    backgroundImage('url(' + data.loginBackground + ')');
                    def.resolve();
                });
                return def.promise();
            },
            binding: function () {
                return { cacheViews: true };
            },
            compositionComplete: function (child, parent, settings) {
                $('.password').focus();
                $("#publishlang").mousemove(function (e) {
                    $("#changeIcon").attr("class", "logo-view-up");
                    $("#isshow-Langer").css('visibility', 'visible');
                });
                $("#publishlang").mouseout(function (e) {
                    $("#changeIcon").attr("class", "logo-view-down");
                    $("#isshow-Langer").css('visibility', 'hidden');
                });
                $("#Multilingualism").text(lang.Multilingualism);
                switch ($.cookie('lang')) {
                    case "zh-cn":
                        $("#Language").attr("class", "log-icon-zh-cn");
                        $("#changelogo").text("简体中文");
                        break;
                    case "en":
                        $("#Language").attr("class", "log-icon-en");
                        $("#changelogo").text("English");
                        break;

                    case "ja":
                        $("#Language").attr("class", "log-icon-ja");
                        $("#changelogo").text("日本語");
                        break;
                    case "zh-tw":
                        $("#Language").attr("class", "log-icon-zh-tw");
                        $("#changelogo").text("繁體中文");
                        break;
                }

            },
            passwordText: passwordText,
            verifyPassword: verifyPassword,
            publishInfoHtml: publishInfoHtml,
            publishUserName:publishUserName,
            passwordOnblur: passwordOnblur,
            passwordonFocus: passwordonFocus,
            logo: logo,
            backgroundImage: backgroundImage,
            userFace: userFace,
            lang: lang,
            onKeydown: onKeydown,
            getEmailCode: getEmailCode,
            getCodeUserAccount: getCodeUserAccount,
            ChangeLanguage: function(text) {
                // <!-- modify 2022-03-08 liuchunli 多语言点击事件修改 -->
                // var theEvent = window.event || arguments.callee.caller.arguments[0];
                // var $target = $(theEvent.target);
                // var lng = $target.dataset.lang;
                var lng = text;
                var oUrl = location.href.toString();
                var nUrl = "";
                $.cookie("lang", lng);
                if(oUrl.indexOf("lang=")>0){
                    var re = eval('/(lang=)([^&]*)/gi');
                    nUrl = oUrl.replace(re, 'lang=' + lng);
                }else{
                    nUrl = oUrl.replace("#authentication", '&lang=' + lng);
                }
                location.href = nUrl;
            }

        };
     
    });

define('outpublish/authentication/index1',['plugins/router', 'durandal/app', 'knockout', "header/userinfo/cookie", lang(), "../service", "jquery", "header/userinfo/constinfo", "md5"],
    function (router, app, ko, cookie, lang, service, $, constinfo) {
        var ajax = service.ajax;
        //发布信息
        var infoHtml = lang.ok + app.title + lang.sendFile;
        //头像src地址
        var userFace = ko.observable();
        if(constInfo.getLang()&&constInfo.getLang()=="en")
        {
            infoHtml=" sent you a file or files ";
        }
        //发布信息
        var publishInfoHtml = ko.observable(infoHtml);
        //密码框值
        var passwordText = ko.observable("");
        //发布code
        var code = "";
        //产品logo
        var logo = ko.observable();
        var backgroundImage = ko.observable();
        var publishUserName = ko.observable();
        var getCodeUserAccount =  ko.observable("");
        
        
        /*
         * mikahuang创建于2015年7月
         * 验证密码
         * @return deferred
         */
        var verifyPassword = function () {
            var pwd = passwordText().trim();
            ajax.checkPublishPasswordByCode({
                code: code,
                emailCode: pwd
            }).done(function (data) {
                if (data.code === 0 && (data.data.md5Code != undefined && data.data.md5Code != '')) {
                    //modify: 2022-05-30 by shuanbao.wei [RD:883]进行中ECM v6.1 外发防破解,处理外发密码为特殊字符文件无法预览问题
                    cookie(code, window.decodeURI(data.data.pwd));
                    cookie("md5Code",data.data.md5Code);
                    router.navigate("view"); 
                    location.reload();
                } else {
                    cookie(code, null);
                    passwordText(""); 
                    if (data.result == 2) {
                        $.messager.alert(lang.tip, $.format(lang.authlock_tip,data.locktime), "warning");
                    }
                    else {
                        $.messager.alert(lang.tip, lang.auth_tip, "warning");
                    }
                }
            });

        }
        /**光标离开事件，恢复输入边框颜色****/
        var passwordOnblur = function () {
            $(".pwd-input").removeClass('focus');
        }
        /****光标进入事件，改变输入边框颜色***/
        var passwordonFocus = function () {
            $(".pwd-input").addClass('focus');
        }
        var onKeydown = function (obj, event) {
            if (event.keyCode === 13) {//enter
                verifyPassword();
            }
            return true;//允许默认行为
        }
        /*
      * mikahuang创建于2015年7月
      * 获取文档发布信息
      * @return deferred
      */
        var getPublishInfo = function (code, callback) {
            return ajax.getPublishBasicInfoByCode({ code: code }).done(function (data) {
                var url = "/ImageType/GetUserAvatar?&userId=" + data.publishUserId;
                userFace('url(' + url + ') no-repeat center center transparent');
                publishUserName(data.publishUserName);
                publishInfoHtml(infoHtml);
                if (data.authType === 1) {
                    router.navigate("view");
                }
            });
        };
        var updateCodeTask;
        var count;
        const emailRegex = "^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,}$";
       var getEmailCode = function(){
        var email = getCodeUserAccount().trim();
        if(!email.match(emailRegex)){
            $.messager.alert(lang.tip, lang.emailTip, "warning");
            return;
        }else{
            // $.getQueryString('code')
            // $.messager.alert(lang.tip, $.getQueryString('code') + "," + email, "warning");
            var data = {code:$.getQueryString('code'),userAccount: email};
            $.ajax({
                type: "POST",
                url: "/api/services/Fexmain/outPublish/getEmailCode",
                data: JSON.stringify(data),
                dataType: 'json',
                async: false,
                contentType:'application/json;charset=utf-8',
                success: function (data) {
                    if (data.code == 0) {
                        // 解除验证按钮
                        $("#passwordButton").attr("style", "width: 100px;");
                        $("#passwordButton").removeAttr("disabled");
                    } else {
                        $.messager.alert(lang.tip, data.message, "warning");
                    }
                }
            })
        }
        count = 60;
        
            // 禁用获取验证按钮 并修改样式
            $("#getcodebutton").attr("disabled","true");
            $("#getcodebutton").attr("style","width: 100px; background-color: gray;border-color: gray;");
            $("#getcodebutton").text(count + "s");
            updateCodeTask = setInterval(updateCountdown, 1000);
        };

        function updateCountdown(){
            count --;
            if(count == 0){
                clearInterval(updateCodeTask);
                $("#getcodebutton").text(lang.getcode);
                $("#getcodebutton").attr("style", "width: 100px;");
                $("#getcodebutton").removeAttr("disabled");
                return;
            }
            $("#getcodebutton").text(count + "s");
           
        }


        return {
            activate: function () {
                var def=$.Deferred();
                code = service.getCode();
                if (!code) return false;
                cookie(code, null);
                getPublishInfo(code);
                logo(constinfo.productInfo.ProductLogo);
                $.ajax({
                    data: {
                        module: 'PublishManager',
                        fun: 'GetLoginBackground'

                    }
                }).done(function (data) {
                    backgroundImage('url(' + data.loginBackground + ')');
                    def.resolve();
                });
                return def.promise();
            },
            binding: function () {
                return { cacheViews: true };
            },
            compositionComplete: function (child, parent, settings) {
                $('.password').focus();
                $("#publishlang").mousemove(function (e) {
                    $("#changeIcon").attr("class", "logo-view-up");
                    $("#isshow-Langer").css('visibility', 'visible');
                });
                $("#publishlang").mouseout(function (e) {
                    $("#changeIcon").attr("class", "logo-view-down");
                    $("#isshow-Langer").css('visibility', 'hidden');
                });
                $("#Multilingualism").text(lang.Multilingualism);
                switch ($.cookie('lang')) {
                    case "zh-cn":
                        $("#Language").attr("class", "log-icon-zh-cn");
                        $("#changelogo").text("简体中文");
                        break;
                    case "en":
                        $("#Language").attr("class", "log-icon-en");
                        $("#changelogo").text("English");
                        break;

                    case "ja":
                        $("#Language").attr("class", "log-icon-ja");
                        $("#changelogo").text("日本語");
                        break;
                    case "zh-tw":
                        $("#Language").attr("class", "log-icon-zh-tw");
                        $("#changelogo").text("繁體中文");
                        break;
                }

            },
            passwordText: passwordText,
            verifyPassword: verifyPassword,
            publishInfoHtml: publishInfoHtml,
            publishUserName:publishUserName,
            passwordOnblur: passwordOnblur,
            passwordonFocus: passwordonFocus,
            logo: logo,
            backgroundImage: backgroundImage,
            userFace: userFace,
            lang: lang,
            onKeydown: onKeydown,
            getEmailCode: getEmailCode,
            getCodeUserAccount: getCodeUserAccount,
            ChangeLanguage: function(text) {
                // <!-- modify 2022-03-08 liuchunli 多语言点击事件修改 -->
                // var theEvent = window.event || arguments.callee.caller.arguments[0];
                // var $target = $(theEvent.target);
                // var lng = $target.dataset.lang;
                var lng = text;
                var oUrl = location.href.toString();
                var nUrl = "";
                $.cookie("lang", lng);
                if(oUrl.indexOf("lang=")>0){
                    var re = eval('/(lang=)([^&]*)/gi');
                    nUrl = oUrl.replace(re, 'lang=' + lng);
                }else{
                    nUrl = oUrl.replace("#authentication", '&lang=' + lng);
                }
                location.href = nUrl;
            }

        };
     
    });

    
/*
    author:mikahuang创建于201505
    desc:外发验证过期
*/
define('outpublish/expire/index',['durandal/app', 'knockout',"../"+lang(),"../service","header/userinfo/constinfo"], function (app, ko,lang,service,constinfo) {
    return {
        lang: lang,
        logo:ko.observable(),
        activate: function (id) {
        },
        binding: function () {
            return { cacheViews: true };
        },
        compositionComplete: function (child, parent, settings) {
            var self = this;
            self.logo(constInfo.productInfo.ProductLogo);
        }
    };
});
/*
 *xietangyu 创建于2016年9月18日
 *描述:返回上一级
*/
define('outpublish/plugins/back/index',["durandal/app"], function (app) {
    return {
        attached: function (view, current) {
            app.trigger('folder-publish:back', "");
        }
    };
});
define('outpublish/plugins/docsetbar/service',["underscore", lang()], function (_, lang) {
    /**
     * 菜单配置项
     */
    var menuConfig = [
        {
            id: "toright",
            command: "toright",
            perm: 1,
            title: lang.toggleRightBord,
            text: '',
            icon: 'icon-toright',
            isShow: true
        }
    ];
    var helper = {
        /**
         * 根据id获取菜单项
         * @param {} list 菜单列表
         * @param {} id  
         * @returns {} 
         */
        getMenuItem: function (list, id) {
            return _.find(list, function (item) {
                return item.id() === id;
            });
        }
    }

    return {
        helper: helper,
        menuConfig: menuConfig
    }
});
/*
  mikahuang创建于2015年5月
  描述:文档设置bar
*/
define('outpublish/plugins/docsetbar/index',["underscore", "komapping", "header/userinfo/constinfo",
        "knockout", "durandal/app", "./service",lang()],
function (_, komapping, constInfo,
          ko, app, service,lang) {
    var helper = service.helper;
    var viewmodel = {
        menuList: komapping.fromJS(service.menuConfig),
        canShow: ko.observable(false) 
    };
    var vm = viewmodel;
    var core = {
        /**
         * 注册一些事件
         * @returns {} 
         */
        registerEvents: function () {
            app.on('docsetbar:showinformationbar').then(function (messager) {
                helper.getMenuItem(vm.menuList(), "toright").isShow(true);
            });
            app.on('docsetbar:hideinformationbar').then(function (messager) {
                helper.getMenuItem(vm.menuList(), "toright").isShow(false);
                setTimeout(function () { app.trigger("informationbar:toggleinformationBar", { type: 0 }); }, 10);
            });
        }
    }

    var temCount = 0;
    /**
     * 命令行子方法
     */
    var subFunction = {
        doToright: function () {
            app.trigger("informationbar:toggleinformationBar", {});
        }
    }
    /**
     * 视图事件函数
     */
    var viewfunction = {
        /**
         * 当按钮点击触发
         * @returns {} 
         */
        doBtnClick: function (obj, event) {
            var self = ko.contextFor(event.target).$root;
            var command = obj.command();
            command = "do"+command.substr(0, 1).toUpperCase() + command.substring(1);
            subFunction[command].call(self,obj,event);
            if ($(".docmain-panel .right-panel").width() > 40) {
                $("#outpublish-page .outpbulish-right > span").removeClass("icon-toright")
            }else{
                $(".outpbulish-right > span").addClass("icon-toright")
                $(".outpbulish-right > span").removeClass("icon-toright-selected")
            }
        }
    }
    var vf = viewfunction;
    return {
        vm: vm,
        vf: vf,
        lang: lang,
        compositionComplete: function () {
            setTimeout(function () {
                if (constInfo && constInfo.systemInfo && constInfo.systemInfo.instanceConfigInfo) {
                    vm.canShow(constInfo.systemInfo.instanceConfigInfo.publishViewInforBar);
                }
            }, 1000)
           
            core.registerEvents();
        }
    };
});
/*
 * xietangyu 创建于2016年9月18日
 * @editor huang.hao 2017-11-20
 * 外发下载模块
 * 
 */
define('outpublish/plugins/download/index',["knockout", "durandal/app", "../../service", lang(), "edoc2ErrorCode","header/userinfo/constinfo"], function (ko, app, service, lang, edoc2ErrorCode,constInfo) {
    //选中的文件夹
    var folders = "";
    //选中的文件
    var files = "";
    //是否是下载文件版本
    var isDownLoadFileVer = false;
    //版本ID
    var fileVerId = ko.observable();
    //是否只选择了文件
    var justSingleFile = false;
    //文件个数
    var fileCount = 0;
    var publishCode = "";//外发编码
    var verifyCode = "";//外发验证code
    //进度条相关参数
    var content = ko.observable();
    var type = ko.observable();
    var canDownload = ko.observable(false);//判断能否进行下载
    //文件类型
    var fileType = {
        folder: 1,
        file: 2,
        shortCut: 32
    }
    function IsIE() {
        var userAgent = navigator.userAgent; //取得浏览器的userAgent字符串  
        var isIE = userAgent.indexOf("compatible") > -1 && userAgent.indexOf("MSIE") > -1; //判断是否IE<11浏览器  
        var isEdge = userAgent.indexOf("Edge") > -1 && !isIE; //判断是否IE的Edge浏览器  
        var isIE11 = userAgent.indexOf('Trident') > -1 && userAgent.indexOf("rv:11.0") > -1;
        if (isIE || isEdge || isIE11) {
            return true
        } else {
            return false;//不是ie浏览器
        }
    }
    /*
     * sailer创建于2015年5月
     * 获取压缩进度
     * @taskId 任务id
     */
    function getZipProgress(taskId, regionType, regionUrl, msg) {
        var config = {};
        //if (regionType === 1) { // 从主区域获得下载进度
        //    config.data = {
        //        module: "WebClient",
        //        fun: "GetZipProgress",
        //        pTaskId: taskId
        //    };
        //} else { // 从分区域获得下载进度
        
        //}

        var checkmsg = msg;
        $.ajax({data:{
                url:regionUrl + "WebCore/Index",
                module: "WebClient",
                fun: "GetZipProgress",
                pTaskId: taskId
            }   
        }).done(function (result) {
            // 0  进度状态获取正常
            if (result.result == 0) {
                if (result.status != "Error" && result.status != "Complete") {
                    if (result.status == "Ziping") {
                    }
                    else if (result.status != "Error") {
                    }
                    var currentDoc = result.currentFileName;
                    //截取过长文件名
                    if (currentDoc.length > 20) {
                        currentDoc = currentDoc.substring(0, 10) + ".." + currentDoc.substring(currentDoc.length - 9);
                    }
                    var remainTime = result.remainTime == -1 ? "00:00:00" : result.remainTime;
                    var showText = lang.compressedFile + "'" + currentDoc + "'," + lang.remainingTime + ":" + remainTime;
                    type('loading');
                    content(showText);
                    $(".publishDownload-progressbar-value").css("width", "" + result.percent + "");
                    //当前下载未完成，继续获取进度
                    setTimeout(function () { getZipProgress(result.pTaskId, regionType, regionUrl, checkmsg); }, 1000);
                }
                else {
                    if (result.status == "Complete") {
                        type('loading');
                        content(lang.download_tip);
                        $(".publishDownload-progressbar-value").css("width", "100%");
                        $(".downloadBox").hide();
                        //if (result.ex!="") {//如果成功，但是有提示信息，表示有部分文件无法下载
                        if (checkmsg) {
                            var msg = checkmsg.replace("etc..", lang.etc) + lang.download_tip3;
                            app.trigger("message:show", { type: 'warning', message: msg, delay: 5000 });//默认显示5秒
                        }
                        else{
                            app.trigger("message:show", { type: 'success', message: lang.download_ok });
                        }
                        //app.trigger("publish:download:success");
                        //ldb #6493去掉压缩成功提示
                        //setTimeout(function () {
                        //    app.trigger("message:show", { type: 'success', message: lang.fileCompressing });
                        //}, 800);
                        var url = "downLoad/GetFile?fileName=" + result.zipName;
                        //if (regionType !== 1) {
                            url = regionUrl + url;
                        //}
                        //跳转进行下载
                        $(".download-frame").eq(0).attr("src", url);
                    } else if (result.status == "Error") {
                        var delay=3000;
                        setTimeout(function(){
                            type('loaded');
                            $(".downloadBox").hide();
                            //601错误：文件不存在
                            if (result.ex == "ErrorCode601") {
                                var currentDoc = result.currentFileName;
                                if (currentDoc.length > 20) {
                                    currentDoc = currentDoc.substring(0, 10) + ".." + currentDoc.substring(currentDoc.length - 9);
                                }
                                var showText = currentDoc;
                                app.trigger("message:show", { type: 'error', message: showText, delay:delay});
                            } else if (result.ex == "ErrorCode632") {
                                app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.outBatchDownoadCountLimit, delay:delay});
                            }
                            else if (result.ex == "ErrorCode8") {//超出下载次数限制
                                app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.download_tip1, delay:delay});
                            }
                            else if (result.ex == "ErrorCode631") {
                                app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.outBatchDownoadSizeLimit, delay:delay});
                            } else if (result.ex == "ErrorCode634") {
                                $(".downloadBox").hide();
                                app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.diffRegion, delay: delay });
                            } else {
                                app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + result.currentFileName, delay:delay});
                            }
                        },200);
                        
                    }
                }
            }
            //当前下载未完成，继续获取进度
            else if (result.result == -2) {
                setTimeout(function () { getZipProgress(result.pTaskId, regionType, regionUrl, checkmsg); }, 2000);
            }
        });
    }


    function rTrim(str, flag) {
        return str.replace(new RegExp((flag || " ") + "$"), "");
    }

    /*
     * sailer创建于2015年5月
     * 获取选中
     * @data  页面传递的选中项
	 * wuyou修改于2016年12月13日
     */
    function initSelectedInfo(data) {
        folders = "";
        files = "";
        fileCount = 0;
        justSingleFile = false;
        $.each(data, function (n, obj) {
            //如果没有data，就从本身取
            var doc;
            if (obj.data) {
                if (obj.data.data) {
                    doc = obj.data.data;
                }
                else {
                    doc = obj.data;
                }
            }
            else {
                doc = obj;
            }
            if (doc.folderType) {
                folders += doc.id + ",";
            }
            else if (doc.fileType) {
                files += doc.id + ",";
                fileCount++;
            }
        });
        if (fileCount == 1 && folders == "") {
            justSingleFile = true;
        }

        folders = rTrim(folders, ",");
        files = rTrim(files, ",");
    }

    var parserRegionUrl = function (checkResult) {
        var regionUrl = checkResult.RegionUrl;
        var regionId = checkResult.RegionId;
        var regionType = checkResult.RegionType;

        //if (regionType === 1) { // 主区域
        //    return "";
        //} else {
            if (regionUrl.lastIndexOf("/") === regionUrl.length - 1)
                return regionUrl;
            else
                return regionUrl + "/";
        //}
    }

    var showErrorTip = function (nResult, delay) {
        if (nResult == "4127") {
            app.trigger("message:show", { type: 'error', message: lang.fileStorageInfoNotFound, delay: delay }); // 文件存储信息不存在
        } else if (nResult == "4215") {
            app.trigger("message:show", { type: 'error', message: lang.ruleParserFailed, delay: delay });  // 规则解析失败
        } else if (nResult == "8") {
            app.trigger("message:show", { type: 'error', message: lang.download_tip1, delay: delay });//次数超限
        } else if (nResult == "614" || nResult == "-1") {
            app.trigger("message:show", { type: 'error', message: lang.downLoadFileError }); // 下载文件时发生异常
        } else if (nResult == 634) {
            app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.diffRegion, delay: delay });
        } else if (nResult === 632) {
            app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.outBatchDownoadCountLimit });
        } else if (nResult == 631) {
            app.trigger("message:show", { type: 'error', message: lang.zipFail + "：" + lang.outBatchDownoadSizeLimit });
        }else if (nResult == 633) {
            app.trigger("message:show", { type: 'error', message: lang.regionHashInvalid, delay: delay });
        } else if (nResult == 653) {
            app.trigger("message:show", { type: 'error', message: lang.downloadToEncrypt });
        } else if (nResult == 654) {
            app.trigger("message:show", { type: 'error', message: lang.downloadFileShouldBeEncrypted });
        } else if (nResult == 655) {
            app.trigger("message:show", { type: 'error', message: lang.downloadFileCannotBeEncryptedFailed });
        }
        else if (nResult == 5709) {//杀毒不允许下载
            app.trigger("message:show", { type: 'error', message: lang.securityscanNotAllowDownload });
        }
        else {
            app.trigger("message:show", { type: 'error', message: constInfo.getedoc2ErrorCodeLang()["ErrorCode" + nResult], delay: delay });
        }
    }

    return {
        lang: lang,
        //进度条相关
        content: content,
        type: type,
        compositionComplete: function (child, parent) {
            app.trigger("message:hide");//隐藏提示消息，避免快速多次点击消息显示错乱问题
            var $parent = ko.dataFor(parent);
            var datas = $parent.datas();

            if (window.ImagePerviewCurData != undefined) {
                datas = window.ImagePerviewCurData;
                $parent.datas(datas);
            }

            publishCode = service.getCode();
            verifyCode = $parent.datas()[0].data.verifyCode;
            if (!verifyCode && $parent.vm && $parent.vm.verifyCode) {
                verifyCode = $parent.vm.verifyCode();
            }
            //modify: 2017-11-28 haibin.long 增加code参数生成，实现外发 与 共享的兼容
            var codeParam = service.generalCodeParam();
            var isAttach = datas[0].isAttach;
            if (publishCode !== null && publishCode.length > 0 && !isAttach) {
                service.ajax.getPublishInfoByCode({ code: publishCode }).done(function (data) {
                    execDownload();
                    // var dt = data.publishDownTime;
                    // if (dt !== -1 && dt === 0) {
                        // app.trigger("message:show", { type: 'error', message: lang.download_tip1 });
                        // canDownload(false);
                        // return;
                    // } else if (dt === -1) {
                        
                    // } else {
                    //     execDownload();
                        /*
                        service.ajax.changePublishDownloadTime({
                            code: publishCode,
                            publishType: data.publishType
                        }).done(function (result) {
                            //判断下载次数是否用完
                            if (result.result == 1) {
                                app.trigger("message:show", { type: 'error', message: lang.download_tip1 });
                            }
                            if (result && result.result === 0) {
                                var time = dt - 1;
                                app.trigger("publish:downloadTime:update", time);
                                execDownload();
                            }
                        });
                        */
                    // }
                })
            } else {
                //modify; 20220219 by xuying 缺陷26313 验证分享是否已取消
                if (codeParam.indexOf("&shareCode=") > -1) {
                    $.ajax({
                        data: {
                            module: "DocShareManager",
                            fun: "GetShareById",
                            existenceShareRecord: true,
                            shareId: codeParam.substring(11, codeParam.length)
                        }
                    }).done(function (nresult) {
                        if (nresult.result != 0) {
                            $.messager.alert(lang.tip, lang.noDownloadPermission, "warning");
                            return;
                        } else {
                            execDownload();
                        }
                    });
                } else {
                    execDownload();
                }

               
            }
            function execDownload() {
                $(".messageBox").hide();//初次加载时清除messageBox提示框
                $(".downloadBox").hide().parent().parent().css("display", "block");
                isDownLoadFileVer = datas[0].fileVerId ?  datas[0].fileVerId != datas[0].curVerId : false;
                if (isDownLoadFileVer) {
                    fileVerId(datas[0].fileVerId);
                }
                //isDownLoadFileVer = datas[0].data.curVerId > 0;
                //if (isDownLoadFileVer) {
                //    fileVerId($parent.datas()[0].data.curVerId);
                //}
                initSelectedInfo($parent.datas());
                content(lang.download_tip2);
                var randomValue = window.parseInt(Math.random() * 100);//模拟进度条
                $(".publishDownload-progressbar-value").css("width", "" + randomValue + "%");
                $(".downloadBox").show();
                type('loading');
                var code = $.getQueryString("code");
                canDownload(true);
                //if(code.length!==0){
                //    data.code = code;
                //}
                //$.ajax({
                //   data: data
                //}).done(function (data) {
                    //if (data.state == false) {
                    //    $(".downloadBox").hide();
                    //    setTimeout(function () {
                    //        //app.trigger("message:show", { type: 'success', message: lang.download_tip3 });
                    //    }, 2000);
                    //    return;
                    //}
                //mac 兼容
                    if (window.DMSClientCommObj) {
                        $(".downloadBox").hide();
                        app.trigger("message:show", { type: 'success', message: lang.download_tip4 });
                        if (justSingleFile) {
                            window.DMSClientCommObj.DownloadFile(files, isDoloadFileVer ? fileVerId() : 0);
                        } else {
                            window.DMSClientCommObj.DownloadFilesFolders(files, folders);
                        }
                    } else {
                        var checkUrl = "downLoad/DownLoadCheck";
                        var urlParams = "";
                        var dataParams = {};
                        if (justSingleFile) {
                            files = parseInt(files);
                            var verifyCodeStr = "";
                            if (verifyCode) {
                                verifyCodeStr = "&verify=" + verifyCode;
                            }
                            urlParams = "fileIds=" + files + codeParam + verifyCodeStr + "&r=" + Math.floor(Math.random() * 100000);
                            if (isDownLoadFileVer) {
                                urlParams = "ver_id=" + fileVerId() + codeParam + verifyCodeStr + "&r=" + Math.floor(Math.random() * 100000);
                            }
                            if (isAttach) {
                                urlParams = "ver_id=" + fileVerId() + "&r=" + Math.floor(Math.random() * 100000);
                            }

                            var isIe = IsIE();
                            checkUrl += "?" + urlParams + "&isIe=" + isIe;
                        } else {
                            checkUrl += "?r=" + Math.floor(Math.random() * 100000);
                            dataParams = {
                                folderIds: folders,
                                fileIds: files,
                                code: publishCode,
                                async: true
                            };
                            if (verifyCode) {
                                dataParams.verify = verifyCode;
                            }
                        }
                        $.ajax({
                            type: "GET",
                            url: checkUrl,
                            contentType: false,
                            data: dataParams,
                            success: function (checkResult) {
                                // 验证失败
                                if (checkResult.nResult !== 0) {
                                    $(".downloadBox").hide();
                                    showErrorTip(checkResult.nResult);
                                    return;
                                }
                                // 验证成功并获取到了规则
                                var regionType = checkResult.RegionType;
                                var regionUrl = parserRegionUrl(checkResult);
                                var downloadUrl = regionUrl + "downLoad/index";
                                if (urlParams) {
                                    downloadUrl += "?" + urlParams + "&regionHash=" + checkResult.RegionHash;
                                } else {
                                    downloadUrl += "?regionHash=" + checkResult.RegionHash;
                                }

                                if (justSingleFile) { //单文件,直接下载文件实体
                                    $(".publishDownload-progressbar-value").css("width", "100%");
                                    setTimeout(function () {
                                        $(".downloadBox").hide();
                                        // app.trigger("message:show", { type: 'success', message: lang.download_ok });
                                        //因为初始化时$(".messageBox")被隐藏，此时app.trigger无法显示，所以添加if条件，针对快速多次下载时两种提示方式重叠错位
                                        if ($(".messageBox").css("display") == "none") {
                                            $(".messageBox").show();
                                        }
                                        //showWindow(true);
                                    }, 200);
                                    // 请求单文件下载action, src的赋值要放在绑定load事件之后
                                    var fream = $(".download-frame").eq(0).unbind("load").load(function () {
                                        var freamWin = fream[0].contentWindow;
                                        if (freamWin) {
                                            var body = freamWin.document.getElementsByTagName("body");
                                            var returnMsg = body[0].innerText;
                                            if (!returnMsg) {
                                                returnMsg = freamWin.document.getElementsByTagName("pre")[0].innerHTML;
                                            }
                                            if (returnMsg) {
                                                var nResult = $.parseJSON(returnMsg).nResult;
                                                setTimeout(function () {
                                                    if (nResult != "0") {
                                                        $(".downloadBox").hide();
                                                        var delay = 4000;
                                                        showErrorTip(nResult, delay);
                                                    }
                                                }, 200);
                                            }
                                        }
                                    }).attr("src", downloadUrl);
                                } else {
                                    type('loading');
                                    content(lang.download_tip6);
                                    $(".publishDownload-progressbar-value").css("width", "1%");

                                    //其它情况，先压缩再下载
                                    $.ajax({
                                        type: "POST",
                                        url: downloadUrl,
                                        data: dataParams,
                                        success: function (data) {
                                            if (data.nResult == 0) {

                                                getZipProgress(data.pTaskId, regionType, regionUrl, checkResult.msg);
                                                //app.trigger("message:show", { type: 'success', message: lang.download_ok });
                                                //app.trigger("publish:download:success");
                                            }
                                            else {
                                                var nResult = data.nResult;
                                                var delay = 4000;
                                                setTimeout(function () {
                                                    $(".downloadBox").hide();
                                                    showErrorTip(nResult, delay);
                                                }, 200);
                                            }
                                        }
                                    }).fail(function (data) {
                                        $(".downloadBox").hide();
                                    });
                                }

                            }
                        });
                    }
                //});
            }
        },
        cancelBox: function () {
            $(".downloadBox").hide();
            return;
        },
        canDownload: canDownload
    }
});
/*
 author:mikahuang
 create:2015-5-10
 desc:导航栏
*/
define('outpublish/plugins/navigation/index',["knockout", "durandal/app", 'underscore'], function (ko, app, _) {
    //缺陷26545 外发结果页面，面包屑与企业库一样
    var subscription;
    var pathObj;
    var sharePerm;
    var pathNodes = ko.observableArray();
    var orgPathNodes = ko.observableArray();
    var omitHideList = ko.observableArray();
    var _sourceNodes = null;
    var selectedItem = ko.observable('');
    var omit = { id: -2, text: '···', crumbsOmit: true };
    var currentPath;
    var currentNode = ko.observable(1);
    var omitHideLeft = ko.observable();
    var currentPageMsg = ko.observable(); // 当前页标识
    var omitIs = ko.observable(false);
    var isShowInnerChain = ko.observable(false);//v6是否展示面包屑内链（ture:展示，false不展示）

    function textSize(fontSize, fontFamily, text, textNum, isReturnSize = false) {
        var span = document.createElement("span");
        var result = {};
        var textWidth = 0;
        var additionalWidth = (textNum * 16) + ((textNum - 1) * 18) || 0;
        result.width = span.offsetWidth;
        result.height = span.offsetHeight;
        span.style.visibility = "hidden";
        span.style.fontSize = fontSize;
        span.style.fontFamily = fontFamily;
        span.style.display = "inline-block";
        document.body.appendChild(span);
        if (typeof span.textContent != "undefined") {
            span.textContent = text;
        } else {
            span.innerText = text;
        }
        textWidth = window.getComputedStyle(span).width;
        //modify by haiwei.cui 20220309  动态增加的span标签用完及时清理
        if (isReturnSize) {
            span.remove();
            return parseInt(textWidth); // 直接返回文字长度
        }
        result.width = parseFloat(textWidth) + additionalWidth - result.width;
        result.height = parseFloat(window.getComputedStyle(span).height) - result.height;
        span.remove();
        return result;
    }

    //视图设置图标 是否显示
    function showViewSetting() {
        //视图设置要根据权限显示
        //if (window.location.hash.indexOf("enterprise") > -1) {
        //    app.trigger('docsetbar:showsetup', { isneed: true });
        //} else {
        //    app.trigger('docsetbar:showsetup', { isneed: false });
        //}
    }

    function deDuplicationArr(allList = [], remList = []) {
        var cloneAllList = _.clone(allList);
        var cloneRemList = _.clone(remList);
        for (var i = 0; i < cloneAllList.length; i++) {
            for (var j = 0; j < cloneRemList.length; j++) {
                if (cloneAllList[i].id == cloneRemList[j].id && !cloneRemList[j].crumbsOmit) {
                    cloneAllList.splice(i, 1);
                }
            }
        };
        return cloneAllList;
    }

    // 计算显示 4 个面包屑
    // modify: shuchen.hu 2022-04-28 #29761（UI）外发多级文件夹，外发结果界面进入至面包屑出现...，所有路径都显示在...下
    function calculateCrumbsShow(crumbsList = [], message, omitList = [], omitLeft = 0) {
        var cloneCrumbsList = _.clone(crumbsList);
        var nwidth = $(".navigation-list").width();
        var textLenWidth = 0;
        var truncateIndex = 2;
        var itemTxtWidth = 0;


         for (var index = 0; index < cloneCrumbsList.length; index++) {
            itemTxtWidth = textSize("12px", '"Microsoft YaHei", arial', cloneCrumbsList[index].text, index, true);
            // 加上每一项的内边距宽度
            textLenWidth += 16;
            // 加上文字宽度
            textLenWidth += (itemTxtWidth > 400 ? 400 : itemTxtWidth);
            // 加上箭头宽度
            if(index > 0){
                textLenWidth += 18;
            };
            if(omitList.length < 1 && (index == 0 || index == 1 )) {
                // omitLeft += (itemTxtWidth + (18 * 2));
                omitLeft += ((itemTxtWidth > 400 ? 400 : itemTxtWidth) + (18 * 2));
            };
        };

        if((nwidth - textLenWidth) < 50){
            if(cloneCrumbsList.findIndex(item => item.id == -2) == -1){
                cloneCrumbsList.splice(truncateIndex, 0, omit);
            };
            omitList.push(...cloneCrumbsList.splice(truncateIndex + 1, 1));
            calculateCrumbsShow(cloneCrumbsList, message,  omitList, omitLeft);
        }else{
            pathNodes(cloneCrumbsList);
            omitHideList(omitList)
            omitHideLeft(omitLeft)
        }

        // if (cloneCrumbsList.length > 4) {
        //     if (message == 'teamlist:load') { // 团队库特殊处理
        //         omitHideList(cloneCrumbsList.splice(2, cloneCrumbsList.length - 4));
        //         cloneCrumbsList[1].moreTeams = true;
        //         showCrumbsList = [cloneCrumbsList[0], cloneCrumbsList[1]].concat(omit).concat(cloneCrumbsList.splice(-2));
        //     } else {
        //         omitHideList(cloneCrumbsList.splice(1, cloneCrumbsList.length - 4)); // 隐藏的面包屑
        //         showCrumbsList = [cloneCrumbsList[0]].concat(omit).concat(cloneCrumbsList.splice(-3)); // 显示的面包屑
        //     };
        //     // omitHideList(cloneCrumbsList.splice(1, cloneCrumbsList.length - 4)); // 隐藏的面包屑
        //     // showCrumbsList = [cloneCrumbsList[0]].concat(omit).concat(cloneCrumbsList.splice(-3)); // 显示的面包屑
        //     try { // 拖拽时动态计算显示面包屑
        //         showCrumbsList.forEach(function (res, index) {
        //             showCrumbsTextLen += res.text;
        //             var nodeBody = textSize("12px", '"Microsoft YaHei", arial', showCrumbsTextLen, index + 1);
        //             var hwidth = Math.ceil(nodeBody.width);
        //             if ((nwidth - hwidth) < 50) {
        //                 if (showCrumbsList.length > index) {
        //                     var concatList = omitHideList().concat(showCrumbsList.splice(2, showCrumbsList.length - index));
        //                     omitHideList(concatList);
        //                 };
        //                 throw Error();
        //             }
        //         });
        //     } catch (error) { };
        //     return showCrumbsList;
    };


    return {
        compositionComplete: function (element) {
            /*
             反调用消息  例如:app.trigger("navigation:init", { message: "enterprise:tree", paths: [{id:1,text:""}] })
            */
            subscription = app.on("navigation:init").then(function (messager) {
                if (messager.message != currentPageMsg()) { // 切换页面清除选中状态
                    selectedItem("");
                };
                ele_font_size = $('input').css('font-size');
                ele_font_family = $('input').css('font-family');
                //左侧面板拖动大小走此逻辑
                if (messager.message == "leftPanelResize") {
                    var nodes = _.clone(orgPathNodes());
                    // nodes = calculateCrumbsShow(nodes, messager.message);
                    // pathNodes(nodes);
                    // modify: shuchen.hu 2022-04-28 #29761（UI）外发多级文件夹，外发结果界面进入至面包屑出现...，所有路径都显示在...下
                    calculateCrumbsShow(nodes, messager.message)
                    currentPageMsg(messager.message);
                    omitHideList(deDuplicationArr(orgPathNodes(), pathNodes())); // 隐藏的面包屑
                    return;
                }
                pathObj = messager;
                sharePerm = messager && messager.sharePerm;
                var nodes = messager.paths;
                orgPathNodes(messager.paths);
                _sourceNodes = _.clone(nodes);

                // nodes = calculateCrumbsShow(nodes, messager.message);

                // pathNodes(nodes);
                // modify: shuchen.hu 2022-04-28 #29761（UI）外发多级文件夹，外发结果界面进入至面包屑出现...，所有路径都显示在...下
                calculateCrumbsShow(nodes, messager.message)
                currentPageMsg(messager.message);
                currentPath = _sourceNodes[_sourceNodes.length - 1];
                currentNode(currentPath);
                //视图设置图标 是否 显示
                showViewSetting();
                // modify: shuchen.hu 2022-04-28 #29761（UI）外发多级文件夹，外发结果界面进入至面包屑出现...，所有路径都显示在...下
                // // 设置隐藏的面包屑距右距离
                // if (orgPathNodes()[0]) {
                //     var orgNodes = _.clone(orgPathNodes());
                //     if (messager.message == 'teamlist:load') {
                //         omitHideLeft(textSize("12px", '"Microsoft YaHei", arial', orgNodes.splice(0, 2), 0, true) + 15);
                //     } else {
                //         omitHideLeft(textSize("12px", '"Microsoft YaHei", arial', orgNodes[0], 0, true) + 15);
                //     };
                // };
                // v6 只有企业和团队库展示面包屑内链
                if (messager.message.indexOf("enterprise") > -1 || messager.message.indexOf("teamlist") > -1) {
                    isShowInnerChain(true)
                } else {
                    isShowInnerChain(false)
                }

                // 26141企业库回收站的面包屑不应显示复制内链按钮
                if (messager.paths.length > 0) {
                    for (var i = 0; i < messager.paths.length; i++) {
                        if (messager.paths[i].id == -1) {
                            isShowInnerChain(false)
                        }
                    }
                }

            });
        },
        currentNode: currentNode,
        pathNodes: pathNodes,
        selectedItem: selectedItem,
        omitHideLeft: omitHideLeft,
        omitHideList: omitHideList,
        //转到
        gotoPath: function (data) {
            // 我的共享 ##gs  我的共享面包屑优化
            if (currentPageMsg() == 'shareNav' && data.id <= 0 && data.type == 'shareBase') {
                return false;
            }

            // 我的收藏
            if (currentPageMsg() == 'favoritenavigation' && data.id <= 0) {
                return false;
            }


            if (data.id == -1) {//回收站点了也不跳它
                return;
            };
            // 选中后再次点击去掉选中状态
            if (((selectedItem() || selectedItem() === 0) && selectedItem() === data.id)
                || (data.crumbsOmit && selectedItem() === 'omit' + data.id)
                // ||(data.moreTeams && selectedItem() === 'teamlist:load' + data.id)
            ) {
                selectedItem("");
                return;
            };
            // 更多
            if (data.crumbsOmit) {
                selectedItem('omit' + data.id);
                return;
            };
            // 团队库更多团队额外处理
            // if(data.moreTeams){
            //     selectedItem('teamlist:load' + data.id);
            //     return;
            // };
            selectedItem(data.id);
            currentNode(data);
            data.sharePerm = sharePerm;
            app.trigger(pathObj.message, data);
        },
        //刷新
        refresh: function () {
            currentPath = pathNodes();
            currentNode(currentPath[currentPath.length - 1]);
            app.trigger(pathObj.message, currentNode(), true);
        },
        handleMouseoverOmit: function () {
            selectedItem('omit-2');
            omitIs(true);
        },
        handleMouseover: function (data) {
            if (data == "omit") {
                selectedItem('');
                return;
            };
            if (omitIs()) {
                omitIs(false);
            };
            if (data.crumbsOmit && !selectedItem()) {
                selectedItem('omit' + data.id);
            };
            if (!data.crumbsOmit && selectedItem()) {
                selectedItem("");
            };
        },
        handleMouseout: function () {
            if (selectedItem()) {
                var timeName = setTimeout(function () {
                    if (!omitIs()) {
                        selectedItem("");
                    };
                    clearTimeout(timeName);
                }, 100)
            };
        }
    }
});
define('outpublish/plugins/outpublishlog/opttype',[lang()], function (lang) {
    return lang;
});
/*
*mikahuang创建于201611
*/
define('outpublish/plugins/outpublishlog/index',["durandal/app", "knockout", "header/userinfo/constinfo", "./opttype",lang()], function (app, ko, constInfo, optType,lang) {
    //窗体显示
    var showWindow = ko.observable(1);
    //操作类型
    var entryType = ko.observable();
    //操作ID
    // var entryId = ko.observable();
    //操作日志信息
    var optLogList = ko.observableArray();
    //当前页面编号
    var currentPage = ko.observable(1);
    //总页面数
    var totalPages = ko.observable(1);
    //每页大小，默认为30
    var pageSize = ko.observable(10);
    //总记录数
    var totalCount = ko.observable(0);
    //筛选时间
    var optTimeType = ko.observableArray();
    //选中的筛选时间
    var selectOptTime = ko.observable("today");
    //操作大类
    var optClass = ko.observableArray();
    //选中操作大类
    var selectOptClass = ko.observable("0");
    var optTypearry = "'322','345','406','423','1201'";
    var sourceIdArry = ko.observable("'3640'");

    var publishCode = null;//外发码
    /*
    * mikahuang创建于2015-9
    * 数据加载
    */
    function loadOptLogData(success) {
        $.mask.show();
        //如果外部日志是在外发结果页打开，则token使用外发人的token
        //如果在我的外发打开，则使用当前token即可
        var token = '';
        if (window.location.href.toLowerCase().indexOf('outpublish.html') > -1) {
            token = $.cookie('token-publish');
        }
        //请求日志记录信息
        //$.ajax(data)
        $.ajax({ async: false, data: {
                module: "LogOperationManager",
                    fun: "LoadLogOperationByCondition",
                    entryType: "-1",
                    optUserIdstr: "",
                    pageNum: currentPage(),
                    pageSize: pageSize(),
                    destName: publishCode,
                    OptSourceName: "",
                    optTimeStartStr: "",
                    optTimeEndStr: "",
                    sourceId: "",
                    userRealName: "",
                    userIdArray: "-1",
                    optType: "",
                    messageFrom: "1",
                    // modify 2022-07-11 tangning #33789 [安全]越权查看日志
                    isPublish: "1",
                    token:token
            } 
        }).done(function (result) {
            $.mask.hide();
            var logs = result.logOpteration;
            $.each(logs, function (index, log) {
                // if (constInfo.userInfo.topPersonalFolderId == entryId()) {
                //     log.OptSourceName = constInfo.replaceSystemPath("PersonalRoot");
                // }
                log.OptTime = constInfo.getDateInterval(log.OptTime);
                log.OptTypeExt = optType["optTypeName_" + log.OptType] || log.OptType;
                log.OptTypeExt += "  " + log.OperatorRemark + "  " + log.OptSourceName;
                log.OptIp = log.IP;
                // log.UserPhotoExt = constInfo.getAvatar("", log.UserId);
                log.EntryIcon = '';
                /**
                 * cq.liangyao注释于2017-9-27 外发日志不显示时间/
                 */
                log.Remarks = log.OptIp + " " +/* log.OptTime + " " +*/ (optType["optTypeName_" + log.OptType] || log.OptType) + "  " + log.OptSourceName + " " + log.OperatorRemark;
                //判断图标类型
                //if (entryType() == 1) {
                //    log.EntryIcon = './Content/images/fileIcons/folder32.png';
                //}
                //else {
                //    log.EntryIcon = './Content/images/fileIcons/' + constInfo.getFileIcon(log.OptSourceName) + '32.png';
                //}
            });
            optLogList(logs);
            totalCount(result.totalCount);
            var total = Math.ceil(totalCount() / pageSize());
            totalPages(total);
            success && success()
        });
    }
    var pagesizes = ko.observableArray([
        { value: "10" },
        { value: "20" },
        { value: "30" }
    ]);
    var nowpagesize = ko.observable(10);
    //更改每页数据条数
    return {
        lang:lang,
        nowpagesize: nowpagesize,
        pagesizes: pagesizes(),
        //窗体显示
        showWindow: showWindow,
        //操作日志信息
        optLogList: optLogList,
        //总页面数
        totalPages: totalPages,
        //当前页面编号
        currentPage: currentPage,
        //每页大小
        pageSize: pageSize,
        //总记录数
        totalCount: totalCount,
        //公用操作
        constInfo: constInfo,
        //筛选时间
        optTimeType: optTimeType,
        //选中的筛选时间
        selectOptTime: selectOptTime,
        //操作大类
        optClass: optClass,
        attached: function (view, parent) {
            var $parent = ko.dataFor(parent);
            // entryId($parent.datas()[0].data.id);
            var datas = $parent.datas();
            //我的外发
            if (datas[0].data && datas[0].data.Publish_Code) {
                publishCode = datas[0].data.Publish_Code;
            } else {
                //外发结果页
                publishCode = $.getQueryString(window.decodeURIComponent(window.location.href), "code");
            }
            var sources = "";
            for (var i in datas) {
                //缺陷34038我的外发中外部日志点击无反应  modify by zhengting 2022-07-11
                if (datas[i].data != undefined) {
                    if (i < (datas.length - 1)) {
                        sources += "'" + datas[i].data.id + "',";
                    } else {
                        sources += "'" + datas[i].data.id + "'";
                    }
                }
              
            }
            sourceIdArry(sources);
            if (constInfo.getObjType($parent.datas()[0]) == "folder") {
                entryType(1);
            } else {
                entryType(2);
            }
            currentPage(1);
            totalPages(1);
            pageSize(10);
            totalCount(0);
            // <!-- modify 2022-09-22 liuchunli #37689【产品张欢已确认】对文件/文件夹进行外发，进入外发链接中，对文件进行操作，产生外部日志后，对外发进行取消外发，外部日志显示错乱 -->
            optLogList([]);
            showWindow(false);
            $('.outloglist-box').perfectScrollbar();
            $(".optloglist-pager .v5-pagination").pagination({
                total: totalCount(),
                pageSize: pageSize(),
                pageList: [10, 20, 30, 40, 50],
                pageNumber: currentPage(),
                type: "small",
                onSelectPage: function (pageNumber, Size) {
                    currentPage(pageNumber);
                    pageSize(Size);
                    loadOptLogData(function () {
                        $('.outloglist-box').perfectScrollbar('update');
                        $('.outloglist-box').get(0).scrollTop = 0;
                    });
                },
                showPageList: true,
            });
            loadOptLogData(function () {
                //缺陷31100：解决5条日志记录时出现滚动条的问题，modify by jiali.yao 2022.05.31
                $('.outloglist-box').perfectScrollbar('update');
                $('.outloglist-box').get(0).scrollTop = 0;
                var opts = $(".optloglist-pager .v5-pagination").pagination('options');
                $(".optloglist-pager .v5-pagination").pagination($.extend(opts,{
                    total: totalCount(),
                    pageSize: pageSize(),
                    pageNumber: currentPage()
                }));
            });

        }
    };
});
define('outpublish/plugins/publishView/lang/zh-cn',{
    "DocState0": "正常",
    "DocState2": "正在上传",
    "DocState4": "已锁定",
    "DocState8": "正在更新",
    "DocState16": "正在审核",
    "DocState32": "被锁定",
    "DocState64": "已归档",
    "DocState512": "编辑中",
    "name": "名称",
    "size": "大小",
    "creator": "创建人",
    "createTime": "创建时间",
    "editor": "修改人",
    "modifyTime": "修改时间",
    "version": "版本",
    "downloadTime": "已下载次数",
    "publish_tip": "您没有预览该文件的权限",
    "publish_tip1": "这种类型的文件无法预览",
    "publish_tip2": "文件预览个数超过系统限制, 不能预览",
    "publish_tip3": "您没有访问该文件夹的权限",
    "publish_tip4": "文件预览个数超过系统限制,不能预览",
    "publish_list":"外发列表",
    "fileDeleted":"文件已删除",
    "previewTimes":"已预览次数",
    "morePreviewTime": "预览次数超出限制，请联系外发用户",
    "publishUserNoPerm":"外发人已无该文档权限"
});
define('outpublish/lang/zh-cn',[],function(){
    return {
        "enterPassword": "请输入密码",
        determine: "确定",
        prompt: "提示",
        verifyPasswordfailed: "验证密码失败.",
        downloadProgress: "下载进度",
        compressedFile: "正在压缩文件",
        fileCompressing: "文件压缩完成",
        StartTransmission: "即将开始传输",
        remainingTime: "剩余时间",
        compressionFailure: "压缩失败",
        filePreparation: "文件准备完成",
        outpublishExpiredOrNoNumber: "外发已过期或无此外发编号",
        urlCode: "访问地址没有code参数",
        publish: "外发",
        vali: "验证",
        expire:"外发过期"
    }
});
define('outpublish/plugins/docsetbar/lang/zh-cn',{
    "teamContentBase": "团队内容库",
    "enterpriseContentBase": "企业内容库",
    "personalContentBase": "个人内容库",
    "teamRecycle":"团队回收站",
    "recycle": "回收站",
    "listView": "列表视图",
    "thumbnail": "缩略图视图",
    "viewSet": "视图设置",
    "toggleLeftBord": "收缩展开左侧面板",
    "toggleRightBord":"收缩展开右侧面板"
});
define('outpublish/view/lang/zh-cn',{
    expiryTime: "到期时间：",
    countDown: "倒计时：",
    about: "关于",
    day: "天",
    hour: "小时",
    minute: "分钟",
    second: "秒",
    expired: "已过期",
    Multilingualism:"多语言"
});
